#!/usr/bin/python
# -*- encoding: utf-8; py-indent-offset: 4 -*-
# .------------------------------------------------------------------------.
# |                ____ _               _        __  __ _  __              |
# |               / ___| |__   ___  ___| | __   |  \/  | |/ /              |
# |              | |   | '_ \ / _ \/ __| |/ /   | |\/| | ' /               |
# |              | |___| | | |  __/ (__|   <    | |  | | . \               |
# |               \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\              |
# |                                        |_____|                         |
# |             _____       _                       _                      |
# |            | ____|_ __ | |_ ___ _ __ _ __  _ __(_)___  ___             |
# |            |  _| | '_ \| __/ _ \ '__| '_ \| '__| / __|/ _ \            |
# |            | |___| | | | ||  __/ |  | |_) | |  | \__ \  __/            |
# |            |_____|_| |_|\__\___|_|  | .__/|_|  |_|___/\___|            |
# |                                     |_|                                |
# |                     _____    _ _ _   _                                 |
# |                    | ____|__| (_) |_(_) ___  _ __                      |
# |                    |  _| / _` | | __| |/ _ \| '_ \                     |
# |                    | |__| (_| | | |_| | (_) | | | |                    |
# |                    |_____\__,_|_|\__|_|\___/|_| |_|                    |
# |                                                                        |
# | mathias-kettner.com                                 mathias-kettner.de |
# '------------------------------------------------------------------------'
#  This file is part of the Check_MK Enterprise Edition (CEE).
#  Copyright by Mathias Kettner and Mathias Kettner GmbH.  All rights reserved.
#
#  Distributed under the Check_MK Enterprise License.
#
#  You should have  received  a copy of the Check_MK Enterprise License
#  along with Check_MK. If not, email to mk@mathias-kettner.de
#  or write to the postal address provided at www.mathias-kettner.de

import rrdtool, select

default_rrd_format = "pnp_multiple"

# RRD creation is taking place via two entry points
# 1. called via CMC in keepalive mode
# 2. called via cmk --convert-rrds
# This module does the following operations
# - Creating new RRDs or extending existing RRDs with new metrics
# - Splitting up old-school PNP SINGLE RRDs into PNP MULTIPLE
# - Converting existing RRDs in format PNP MULTIPLE or CMC SINGLE to
#   match a changed RRA configuration
# - Converting existing RRDs in formatn PNP MULTIPLE into CMC SINGLE

#   .--Helpers-------------------------------------------------------------.
#   |                  _   _      _                                        |
#   |                 | | | | ___| |_ __   ___ _ __ ___                    |
#   |                 | |_| |/ _ \ | '_ \ / _ \ '__/ __|                   |
#   |                 |  _  |  __/ | |_) |  __/ |  \__ \                   |
#   |                 |_| |_|\___|_| .__/ \___|_|  |___/                   |
#   |                              |_|                                     |
#   +----------------------------------------------------------------------+
#   |  General functions and helpers                                       |
#   '----------------------------------------------------------------------'

def rrd_log(text):
    queue_rrd_helper_response(text + "\n")


rra_default_config = [
    "RRA:AVERAGE:0.50:1:2880",
    "RRA:AVERAGE:0.50:5:2880",
    "RRA:AVERAGE:0.50:30:4320",
    "RRA:AVERAGE:0.50:360:5840",
    "RRA:MAX:0.50:1:2880",
    "RRA:MAX:0.50:5:2880",
    "RRA:MAX:0.50:30:4320",
    "RRA:MAX:0.50:360:5840",
    "RRA:MIN:0.50:1:2880",
    "RRA:MIN:0.50:5:2880",
    "RRA:MIN:0.50:30:4320",
    "RRA:MIN:0.50:360:5840",
]

rrd_heartbeat = 8460

def get_rrd_conf(hostname, servicedesc="_HOST_"):
    if servicedesc != "_HOST_":
        rrdconf = service_extra_conf(hostname, servicedesc, cmc_service_rrd_config)
    else:
        rrdconf = host_extra_conf(hostname, cmc_host_rrd_config)

    if rrdconf:
        rrdconf = rrdconf[0]
        rra_config = []
        for cf in rrdconf["cfs"]:
            for xfs, step, rows in rrdconf["rras"]:
                rra_config.append("RRA:%s:%.2f:%d:%d" % (cf, xfs / 100, step, rows))
        step = rrdconf["step"]
        rrd_format = rrdconf.get("format", default_rrd_format)
        rra_config.sort()

    else:
        rra_config = sorted(rra_default_config)
        step = 60
        rrd_format = default_rrd_format

    return rrd_format, rra_config, step, rrd_heartbeat

def rrd_pnp_host_dir(hostname):
    # We need /opt here because of bug in rrdcached
    return "/opt/omd/sites/" + omd_site + "/var/pnp4nagios/perfdata/" + pnp_cleanup(hostname)

def rrd_cmc_host_dir(hostname):
    # We need /opt here because of bug in rrdcached
    return "/opt/omd/sites/" + omd_site + "/var/check_mk/rrd/" + pnp_cleanup(hostname)


def read_existing_metrics(info_file_path):
    return parse_cmc_rrd_info(info_file_path)["metrics"]


def parse_cmc_rrd_info(info_file_path):
    info = {}
    try:
        for line in file(info_file_path):
            keyword, rest = line.strip().split(None, 1)
            if keyword == "METRICS":
                value = rest.split(";")
            else:
                value = rest
            info[keyword.lower()] = value

        return info

    except Exception, e:
        if opt_debug:
            raise
        raise Exception("Invalid RRD info file %s: %s" % (info_file_path, e))


def float_or_nan(s):
    try:
        float(s)
        return s
    except:
        return "U"


def render_rrd_size(x):
    return str(int(round(float(x)/1024))) + " KB"


#.
#   .--Creation------------------------------------------------------------.
#   |                ____                _   _                             |
#   |               / ___|_ __ ___  __ _| |_(_) ___  _ __                  |
#   |              | |   | '__/ _ \/ _` | __| |/ _ \| '_ \                 |
#   |              | |___| | |  __/ (_| | |_| | (_) | | | |                |
#   |               \____|_|  \___|\__,_|\__|_|\___/|_| |_|                |
#   |                                                                      |
#   +----------------------------------------------------------------------+
#   |  Creation of new RRD files, triggered by CMC                         |
#   '----------------------------------------------------------------------'

# TODO: Make use of the generic do_keepalive() mechanism
def do_create_rrd(args):
    if opt_keepalive:
        register_keepalive_sigint_handler()
        if args:
            sys.stderr.write("You cannot specify arguments when using --keepalive mode.\n")
            sys.exit(1)
        create_rrds_keepalive()
    else:
        for spec in args:
            create_rrd_from_spec(spec)


def create_rrds_keepalive():
    input_buffer = ""
    global rrd_helper_output_buffer
    rrd_helper_output_buffer = ""

    job_queue = []
    read_packed_config()

    verbose("Started Check_MK RRD creator.\n")
    try:
        # We read asynchronously from stdin and put the jobs into a queue.
        # That way the cmc main process will not be blocked by IO wait.
        while True:
            if job_queue:
                timeout = 0
            else:
                timeout = None

            if rrd_helper_output_buffer:
                wait_for_write = [1]
            else:
                wait_for_write = []
            readable, writeable, exceptions = select.select([0], wait_for_write, [], timeout)
            if 1 in writeable:
                write_rrd_helper_response()

            if 0 in readable:
                try:
                    new_bytes = os.read(0, 4096)
                except:
                    new_bytes = ""
                if not new_bytes and not job_queue:
                    verbose("Core closed stdin, all jobs finished. Exiting.\n")
                    break

                input_buffer += new_bytes
                parts = input_buffer.split("\n")
                if parts[-1] != "": # last job not terminated
                    input_buffer = parts[-1]
                else:
                    input_buffer = ""
                parts = parts[:-1]
                job_queue += parts

            # Create *one* RRD file
            if job_queue:
                if job_queue[0] == "*":
                    verbose("Reloading configuration.\n")
                    read_packed_config()
                    reset_global_caches()
                else:
                    try:
                        create_rrd_from_spec(job_queue[0])
                    except Exception, e:
                        if opt_debug:
                            raise
                        rrd_log("Error creating RRD for %s: %s" % (job_queue[0], e))
                del job_queue[0]

    except Exception, e:
        if opt_debug:
            raise
        import traceback
        rrd_log("Check_MK RRD creator failed: %s" % traceback.format_exc())

    verbose("Stopped Check_MK RRD creator.\n")


def queue_rrd_helper_response(response):
    if opt_keepalive:
        global rrd_helper_output_buffer
        rrd_helper_output_buffer += response
    else:
        sys.stdout.write(response)
        sys.stdout.flush()


def write_rrd_helper_response():
    global rrd_helper_output_buffer
    size = min(4096, len(rrd_helper_output_buffer))
    written = os.write(1, rrd_helper_output_buffer[:size])
    rrd_helper_output_buffer = rrd_helper_output_buffer[written:]


def create_rrd_from_spec(spec):
    # Examples for spec (sent by CMC when RRDs for metrics are not present)
    # cmc_single;heute;Check_MK;execution_time;0.141;user_time;0.020;system_time;0.000;children_user_time;0.000;children_system_time;0.000
    # pnp_multiple;heute;_HOST_;rta;0.141
    try:
        parts = spec.split(';')
        rrd_format = parts[0]
        host = parts[1]

        if len(parts) % 2 == 1: # service
            service = parts[2]
            metric_specs = parts[3:]
            specname = host + ";" + service
        else: # host
            service = "_HOST_"
            metric_specs = parts[2:]
            specname = host

        metrics = zip(metric_specs[::2], metric_specs[1::2])
        metric_names = [ e[0] for e in metrics ]
        rrd_file_name = create_rrd(host, service, rrd_format, metric_names)

        # Do first update right now
        now = time.time()

        args = [ rrd_file_name, "%d:%s" % (now, ":".join([
            float_or_nan(first_value) for (varname, first_value) in metrics ]))]
        rrdtool.update(*args)

        queue_rrd_helper_response("CREATED %s %s;%s;%s\n" % (rrd_format, host, service, ";".join(metric_names)))

    except Exception, e:
        if opt_debug:
            raise
        rrd_log("Error creating RRD for %s: %s" % (spec, e))


# Create a new RRD. Automatically reuses data from an existing RRD if the
# type is CMC SINGLE. This mode is for extending existing RRDs by new metrics.
def create_rrd(hostname, servicedesc, rrd_format, metric_names):
    # We get the configured rrd_format here as well. But we rather trust what CMC
    # specifies.
    configured_rrd_format, rra_config, step, heartbeat = get_rrd_conf(hostname, servicedesc)

    if rrd_format == "pnp_multiple":
        host_dir = rrd_pnp_host_dir(hostname)
        base_file_name = host_dir + "/" + pnp_cleanup(servicedesc) + "_" + pnp_cleanup(metric_names[0])

    else: # cmc_single
        host_dir = rrd_cmc_host_dir(hostname)
        base_file_name = host_dir + "/" + pnp_cleanup(servicedesc)

    rrd_file_name = base_file_name + ".rrd"

    migration_arguments = []
    migration_mapping = {}
    if os.path.exists(rrd_file_name):
        if rrd_format == "pnp_multiple":
            raise Exception("Tried to create %s, but this RRD exists." % rrd_file_name)
        else:
            # Need to migrate data from existing RRD
            existing_metrics = read_existing_metrics(base_file_name + ".info")
            migration_arguments = [ "--source", rrd_file_name ]
            for nr, varname in enumerate(existing_metrics):
                migration_mapping[varname] = nr+1

    if not os.path.exists(host_dir):
        os.makedirs(host_dir)

    if cmc_log_rrdcreation:
        rrd_log("Creating %s" % rrd_file_name);
        if cmc_log_rrdcreation == "full":
            for entry in rra_config:
                rrd_log("    %s" % entry)

    args = [ rrd_file_name, "--step", str(step) ]
    for nr, varname in enumerate(metric_names):
        if varname in migration_mapping:
            source_spec = "=%d" % migration_mapping[varname]
        else:
            source_spec = ""
        args.append("DS:%d%s:GAUGE:%d:U:U" % (nr+1, source_spec, heartbeat))
    args += rra_config
    args = migration_arguments + args

    # Note: rrdtool.create bails out with a Bus error if the disk is full. There
    # is no way to handle this here. Or can we catch the signal 6? In any case it does not
    # make sense to check the size of the RRD for 0 after this command since our process
    # will not exist anymore by then...
    rrdtool.create(*args)

    if rrd_format == "cmc_single":
        create_cmc_rrd_info_file(hostname, servicedesc, metric_names)

    return rrd_file_name


#.
#   .--Conversion----------------------------------------------------------.
#   |           ____                              _                        |
#   |          / ___|___  _ ____   _____ _ __ ___(_) ___  _ __             |
#   |         | |   / _ \| '_ \ \ / / _ \ '__/ __| |/ _ \| '_ \            |
#   |         | |__| (_) | | | \ V /  __/ |  \__ \ | (_) | | | |           |
#   |          \____\___/|_| |_|\_/ \___|_|  |___/_|\___/|_| |_|           |
#   |                                                                      |
#   +----------------------------------------------------------------------+
#   |  Conversion of existing RRD files to the new configured structure.   |
#   '----------------------------------------------------------------------'

def do_convert_rrds(hostnames):

    if rrdtool.__version__ == "1.4.8":
        sys.stderr.write("Sorry, your RRDTool version is 1.4.8. This does not support conversion.\n"
                         "You are probably running RedHat/CentOS 5.X. Please upgrade to version 6\n"
                         "in order to have support for the new RRDTool version.\n")
        sys.exit(1)

    if not hostnames:
        hostnames = all_hosts_untagged + all_active_clusters()
    for hostname in hostnames:
        convert_rrds_of_host(hostname)


def convert_rrds_of_host(hostname):
    verbose("%s%s%s:\n" % (tty_bold + tty_yellow, hostname, tty_normal))

    try:
        existing_rrds = find_host_rrd_services(hostname)
        convert_cmc_versus_pnp(hostname, existing_rrds)
        convert_pnp_rrds(hostname, existing_rrds)
        convert_cmc_rrds(hostname, existing_rrds)
    except Exception, e:
        if opt_debug:
            raise
        if not opt_verbose:
            sys.stderr.write("  HOST: %s\n" % hostname)
        sys.stderr.write("      %sERROR: %s%s\n" % (tty_red + tty_bold, e, tty_normal))

    verbose("\n")


def convert_pnp_rrds(hostname, existing_rrds):
    host_dir = rrd_pnp_host_dir(hostname)
    for servicedesc, existing_rrd_formats in existing_rrds.items():
        if "pnp_multiple" in existing_rrd_formats:
            verbose("  %s (%sPNP%s)...\n" % (servicedesc, tty_bold + tty_cyan, tty_normal))
            xmlinfo = read_pnp_xml_for(hostname, servicedesc)
            target_rrdconf = get_rrd_conf(hostname, servicedesc)[1:]
            convert_pnp_rrds_of(host_dir, xmlinfo, pnp_cleanup(servicedesc), target_rrdconf)


def convert_pnp_rrds_of(host_dir, xmlinfo, file_prefix, rrdconf):
    need_split = False
    for ds in xmlinfo["ds"]:
        old_ds_name = ds["ds"]
        old_rrd_path = ds["rrdfile"]
        if old_rrd_path.startswith("/opt/omd/"):
            old_rrd_path = old_rrd_path[4:] # drop the /opt, otherwise conflict with new path
        new_rrd_path = host_dir + "/" + file_prefix + "_" + pnp_cleanup(ds["name"]) + ".rrd"
        need_split = ds["rrd_storage_type"] == "SINGLE"
        old_size = os.stat(old_rrd_path).st_size
        if need_split:
            old_size /= len(xmlinfo["ds"])

            if not opt_split_rrds:
                verbose("    old: %s\n" % old_rrd_path)
                verbose("    new: %s\n" % new_rrd_path)
                raise Exception("storage type single, use --split-rrds to split this up.")

        verbose("    - %s%s.." % (ds["name"], need_split and "(split)" or ""))
        result = convert_pnp_rrd(old_rrd_path, new_rrd_path, old_ds_name, rrdconf)
        if result == True:
            new_size = os.stat(new_rrd_path).st_size
            verbose("..%sconverted%s, %s -> %s\n" % (tty_green + tty_bold, tty_normal,
                    render_rrd_size(old_size),
                    render_rrd_size(new_size)))
        elif result == None:
           verbose(".." + tty_red + tty_bold + "failed\n" + tty_normal)
        else:
           verbose(".." + tty_blue + tty_bold + "uptodate\n" + tty_normal)

    if need_split:
        fixup_pnp_xml_file(host_dir + "/" + file_prefix + ".xml")
        os.remove(old_rrd_path)
        verbose("    deleted %s\n" % old_rrd_path)


def convert_pnp_rrd(old_rrd_path, new_rrd_path, old_ds_name, new_rrdconf):
    if not os.path.exists(old_rrd_path):
        raise Exception("RRD %s is missing" % old_rrd_path)

    # Our problem here: We must not convert files that already
    # have the correct configuration. We we try to extract
    # the existing configuration and compare with the new one.
    try:
        old_rrdconf = get_old_rrd_config(old_rrd_path, old_ds_name)
        if old_rrdconf == None:
            return None # DS not contained in old RRD

    except Exception, e:
        if opt_debug:
            raise
        raise Exception("Existing RRD %s is incompatible: %s" % (old_rrd_path, e))

    # Beware: we use /opt/omd always because of bug in rrdcached
    if old_rrd_path.startswith("/omd"):
        old_rrd_path = "/opt" + old_rrd_path

    if old_rrdconf == new_rrdconf and old_rrd_path == new_rrd_path:
        return False # Nothing to do
    else:
        new_rra_config, new_step, new_heartbeat = new_rrdconf
        args = [ new_rrd_path, "--step", str(new_step), "DS:1=%s:GAUGE:%d:U:U" % (old_ds_name, new_heartbeat),
                 "--source", old_rrd_path ] + new_rra_config
        try:
            rrdtool.create(*args)
        except Exception, e:
            if opt_debug:
                sys.stderr.write("COMMAND: rrdtool create %s\n" % (" ".join(args)))
                raise
            raise Exception("Error on running rrdtool create %s: %s" % (" ".join(args), e))
        return True


def convert_cmc_rrds(hostname, existing_rrds):
    host_dir = rrd_cmc_host_dir(hostname)
    for servicedesc, existing_rrd_formats in existing_rrds.items():
        if "cmc_single" in existing_rrd_formats:
            verbose("  %s (%sCMC%s)..." % (servicedesc, tty_bold + tty_bold, tty_normal))
            base_path = host_dir + "/" + pnp_cleanup(servicedesc)
            existing_metrics = read_existing_metrics(base_path + ".info")
            target_rrdconf = get_rrd_conf(hostname, servicedesc)[1:]
            rrd_file_path = base_path + ".rrd"
            convert_cmc_rrd_of(hostname, servicedesc, rrd_file_path, existing_metrics, target_rrdconf)


def convert_cmc_rrd_of(hostname, servicedesc, rrd_file_path, existing_metrics, target_rrdconf):
    old_rrdconf = get_old_rrd_config(rrd_file_path, "1")
    if old_rrdconf == target_rrdconf:
        verbose(".." + tty_blue + tty_bold + "uptodate\n" + tty_normal)
    else:
        try:
            old_size = os.stat(rrd_file_path).st_size
            create_rrd(hostname, servicedesc, "cmc_single", existing_metrics)
            new_size = os.stat(rrd_file_path).st_size
            verbose("..%sconverted%s, %s -> %s\n" % (tty_green + tty_bold, tty_normal,
                    render_rrd_size(old_size),
                    render_rrd_size(new_size)))
        except Exception, e:
            if opt_debug:
                raise
            verbose(".." + tty_red + tty_bold + "failed\n" + tty_normal)


def convert_cmc_versus_pnp(hostname, existing_rrds):
    # Find services with RRDs in the two possible formats "cmc" and "pnp". "_HOST_" means
    # host metrics.
    for servicedesc, existing_rrd_formats in existing_rrds.items():
        target_rrdconf = get_rrd_conf(hostname, servicedesc)
        target_rrd_format = target_rrdconf[0]
        if target_rrd_format not in existing_rrd_formats:
            if target_rrd_format == "pnp_multiple":
                queue_rrd_helper_response("WARNING: Converting RRD format CMC into PNP not implemented (%s/%s)\n" % (hostname, servicedesc))
                # convert_cmc_to_pnp(hostname, servicedesc)
            else:
                convert_pnp_to_cmc(hostname, servicedesc)
                existing_rrd_formats.append(target_rrd_format)

        if len(existing_rrd_formats) > 1:
            if opt_delete_rrds:
                for rrd_format in existing_rrd_formats:
                    if rrd_format != target_rrd_format:
                        delete_rrds(hostname, servicedesc, rrd_format)
                existing_rrd_formats[:] = [ target_rrd_format ]
            else:
                queue_rrd_helper_response("  WARNING: Dupliate RRDs for %s/%s. "
                      "Use --delete-rrds for cleanup.\n" % (hostname, servicedesc))


def delete_rrds(hostname, servicedesc, rrd_format):
    def try_delete(path):
        try:
            os.remove(path)
            verbose("Deleted %s.\n" % path)
        except:
            pass

    if rrd_format == "cmc_single":
        host_dir = rrd_cmc_host_dir(hostname)
        base_file_name = host_dir + "/" + pnp_cleanup(servicedesc)
        try_delete(base_file_name + ".rrd")
        try_delete(base_file_name + ".info")
    else:
        host_dir = rrd_pnp_host_dir(hostname)
        base_file_name = host_dir + "/" + pnp_cleanup(servicedesc)
        try_delete(base_file_name + ".xml")
        for filename in sorted(os.listdir(host_dir)):
            if filename.startswith(pnp_cleanup(servicedesc) + "_"):
                try_delete(host_dir + "/" + filename)



def convert_pnp_to_cmc(hostname, servicedesc):
    verbose("   %s %sPNP%s -> %sCMC%s" % (servicedesc, tty_bold + tty_cyan, tty_normal, tty_bold, tty_normal))

    # We get the configured rrd_format here as well. But we rather trust what CMC
    # specifies.
    configured_rrd_format, rra_config, step, heartbeat = get_rrd_conf(hostname, servicedesc)

    host_dir = rrd_cmc_host_dir(hostname)
    base_file_name = host_dir + "/" + pnp_cleanup(servicedesc)
    rrd_file_name = base_file_name + ".rrd"

    args = [ rrd_file_name, "--step", str(step) ]
    xml_info = read_pnp_xml_for(hostname, servicedesc)
    metric_names = []
    for nr, ds in enumerate(xml_info["ds"]):
        varname = ds["name"]
        metric_names.append(varname)
        pnp_rrd_filename = rrd_pnp_host_dir(hostname) + "/" + pnp_cleanup(servicedesc) + "_" + pnp_cleanup(varname) + ".rrd"

        args += [
            "--source",
            pnp_rrd_filename,
            "DS:%d=1[%d]:GAUGE:%d:U:U" % (nr+1, nr+1, heartbeat),
        ]

    if not os.path.exists(host_dir):
        os.makedirs(host_dir)

    args += rra_config

    # Note: rrdtool.create bails out with a Bus error if the disk is full. There
    # is no way to handle this here. Or can we catch the signal 6? In any case it does not
    # make sense to check the size of the RRD for 0 after this command since our process
    # will not exist anymore by then...
    rrdtool.create(*args)

    # Create information file for CMC format RRDs. Problem is that RRD
    # limits variable names to 19 characters and to just alphanumeric
    # characters. We cannot savely put our variablenames into the RRDs.
    # So we do it like PNP and use 1, 2, 3... as DS names and keep the
    # actual real names in a separate file with the extension ".info"
    create_cmc_rrd_info_file(hostname, servicedesc, metric_names)
    verbose("..%sconverted.%s\n" % (tty_bold + tty_green, tty_normal))
    vverbose("    (rrdtool create %s)\n" % " ".join(args))


# Create information file for CMC format RRDs. Problem is that RRD
# limits variable names to 19 characters and to just alphanumeric
# characters. We cannot savely put our variablenames into the RRDs.
# So we do it like PNP and use 1, 2, 3... as DS names and keep the
# actual real names in a separate file with the extension ".info"
def create_cmc_rrd_info_file(hostname, servicedesc, metric_names):
    base_file_name = rrd_cmc_host_dir(hostname) + "/" + pnp_cleanup(servicedesc)
    file(base_file_name + ".info", "w").write("HOST %s\nSERVICE %s\nMETRICS %s\n" % (
        hostname, servicedesc, ";".join(metric_names)))


def find_host_rrd_services(hostname):
    rrd_services = {}
    for service in find_pnp_rrds(hostname):
        rrd_services.setdefault(service, []).append("pnp_multiple")
    for service in find_cmc_rrds(hostname):
        rrd_services.setdefault(service, []).append("cmc_single")
    return rrd_services


def find_pnp_rrds(hostname):
    host_dir = rrd_pnp_host_dir(hostname)
    if not os.path.exists(host_dir):
        return

    if os.path.exists(host_dir + "/_HOST_.xml"):
        yield "_HOST_"

    for xml_file in sorted(os.listdir(host_dir)):
        if xml_file.endswith(".xml") and xml_file != "_HOST_.xml":
            xmlinfo = parse_pnp_xml_file(host_dir + "/" + xml_file)
            servicedesc = xmlinfo["service"]
            yield servicedesc


def find_cmc_rrds(hostname):
    host_dir = rrd_cmc_host_dir(hostname)
    if not os.path.exists(host_dir):
        return
    for info_file in sorted(os.listdir(host_dir)):
        if info_file.endswith(".info"):
            rrd_info = parse_cmc_rrd_info(host_dir + "/" + info_file)
            yield rrd_info["service"]


def parse_pnp_simple_line(line):
    key = line.split(">")[0][1:]
    value = line.split(">")[1].split("<")[0]
    return key.lower(), value


def read_pnp_xml_for(hostname, servicedesc="_HOST_"):
    host_dir = rrd_pnp_host_dir(hostname)
    return parse_pnp_xml_file(host_dir + "/" + pnp_cleanup(servicedesc) + ".xml")


# Note: this is no true XML parser. The exact syntax of the
# files are known.
def parse_pnp_xml_file(xml_path):

    xmlinfo = {
        "ds" : []
    }

    mode = None
    ds = None
    for line in file(xml_path):
        line = line.strip()
        if mode == "ds":
            if line == "</DATASOURCE>":
                mode = None
            else:
                key, value = parse_pnp_simple_line(line)
                if key in [ "name", "rrdfile", "ds", "rrd_storage_type" ]:
                    ds[key] = value
        elif line == "<DATASOURCE>":
            ds = {}
            xmlinfo["ds"].append(ds)
            mode = "ds"
        elif not mode:
            key, value = parse_pnp_simple_line(line)
            if key == "nagios_auth_hostname":
                xmlinfo["host"] = value
            elif key == "nagios_auth_servicedesc":
                xmlinfo["service"] = value
            elif key == "nagios_rrdfile":
                xmlinfo["rrdfile"] = value

    return xmlinfo


# Convert a PNP XML file from SINGLE to MULTIPLE
def fixup_pnp_xml_file(xml_path):
    new_lines = []
    mode = None
    ds_name = None
    for line in file(xml_path):
        ls = line.strip()
        if ls.startswith("<DATASOURCE>"):
            datasource = []
            mode = "ds"
            new_lines.append(line)
        elif ls.startswith("</DATASOURCE>"):
            mode = None
            for l in datasource:
                if "RRDFILE" in l:
                    l = l.replace(".rrd</RRDFILE>\n", "_" + pnp_cleanup(ds_name) + ".rrd</RRDFILE>\n")
                new_lines.append(l)
            new_lines.append(line)
        elif mode == "ds":
            if ls.startswith("<DS>"):
                line = "    <DS>1</DS>\n"
            elif ls.startswith("<NAME>"):
                key, ds_name = parse_pnp_simple_line(line)
            elif ls.startswith("<RRD_STORAGE_TYPE>"):
                line = "    <RRD_STORAGE_TYPE>MULTIPLE</RRD_STORAGE_TYPE>\n"
            datasource.append(line)
        else:
            new_lines.append(line)
    file(xml_path, "w").writelines(new_lines)



def get_old_rrd_config(rrd_file_path, old_ds_name):
    old_config_raw = rrdtool.info([rrd_file_path])
    rra_defs = {}
    heartbeat = None
    step = None
    for varname, value in old_config_raw.items():
        if varname == "ds[%s].minimal_heartbeat" % old_ds_name:
            heartbeat = value
        elif varname.startswith("ds["):
            pass # other values are not relevant
        elif varname == "step":
            step = value
        elif varname.startswith("rra["):
            conf_nr = int(varname[4:].split("]")[0])
            conf = rra_defs.setdefault(conf_nr, {})
            subvar = varname.split(".")[1]
            conf[subvar] = value

    rra_items = rra_defs.items()
    rra_items.sort()
    rra_config = []
    for nr, rra in rra_items:
        rra_config.append("RRA:%(cf)s:%(xff).2f:%(pdp_per_row)d:%(rows)d" % rra)

    rra_config.sort()
    if heartbeat == None or step == None:
        verbose("(%smissing %s:%s%s)" % (tty_red, rrd_file_path.split("/")[-1], old_ds_name, tty_normal))
        return None
    return rra_config, step, heartbeat
