// +------------------------------------------------------------------+
// |             ____ _               _        __  __ _  __           |
// |            / ___| |__   ___  ___| | __   |  \/  | |/ /           |
// |           | |   | '_ \ / _ \/ __| |/ /   | |\/| | ' /            |
// |           | |___| | | |  __/ (__|   <    | |  | | . \            |
// |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
// |                                                                  |
// | Copyright Mathias Kettner 2014             mk@mathias-kettner.de |
// +------------------------------------------------------------------+
//
// This file is part of Check_MK.
// The official homepage is at http://mathias-kettner.de/check_mk.
//
// check_mk is free software;  you can redistribute it and/or modify it
// under the  terms of the  GNU General Public License  as published by
// the Free Software Foundation in version 2.  check_mk is  distributed
// in the hope that it will be useful, but WITHOUT ANY WARRANTY;  with-
// out even the implied warranty of  MERCHANTABILITY  or  FITNESS FOR A
// PARTICULAR PURPOSE. See the  GNU General Public License for more de-
// ails.  You should have  received  a copy of the  GNU  General Public
// License along with GNU Make; see the file  COPYING.  If  not,  write
// to the Free Software Foundation, Inc., 51 Franklin St,  Fifth Floor,
// Boston, MA 02110-1301 USA.


// Styling. Please note that for each visible pixel our canvas
// has two pixels. This improves the resolution when zooming
// in with your browser.
var v_label_margin   = 10; // pixels between vertical label and v axis
var t_label_margin   = 10;  // pixels between time label and t axis
var v_axis_width     = 100;
var bottom_margin    = 0;
var axis_color       = "#000000";
var canvas_color     = "#ffffff";
var background_color = "#f8f4f0"; // If you change this, also update check_mk.css
var axis_over_width  = 5; // pixel that the axis is longer for optical reasons
var color_gradient   = 0.2; // ranges from 0 to 1
var curve_line_width = 2.0;
var rule_line_width  = 2.0;
var v_line_color     = [ null, "#a0a0a0", "#a0a0a0" ];
var v_line_dash      = [ null, [1,5], [1] ];
var t_line_color     = [ "#000000", "#a0a0a0", "#666666" ];
var t_line_dash      = [ null, [1,5], [1] ];

// FIXME: Move other global style vars to this construct
var g_graph_style = {
    top_margin: 10
}

// Global graph constructs to store the graphs etc.
var g_graphs            = {};
var g_current_graph_id  = 0;
var g_page_update_delay = 60; // prevent page update for X seconds

//#   .-Creation-----------------------------------------------------------.
//#   |              ____                _   _                             |
//#   |             / ___|_ __ ___  __ _| |_(_) ___  _ __                  |
//#   |            | |   | '__/ _ \/ _` | __| |/ _ \| '_ \                 |
//#   |            | |___| | |  __/ (_| | |_| | (_) | | | |                |
//#   |             \____|_|  \___|\__,_|\__|_|\___/|_| |_|                |
//#   |                                                                    |
//#   +--------------------------------------------------------------------+
//#   | Graphs are created by a javascript function. The unique graph id   |
//#   | is create solely by the javascript code. This function is being    |
//#   | called by web/plugins/graphs.py:render_graphs_htmls()              |
//#   '--------------------------------------------------------------------'

function get_id_of_graph(ajax_context)
{
    // Return the graph_id for and eventual existing graph
    for (var graph_id in g_graphs) {
        // JSON.stringify seems to be the easiest way to compare the both dicts
        if (JSON.stringify(ajax_context.definition.specification) ==
            JSON.stringify(g_graphs[graph_id].ajax_context.definition.specification)
            && JSON.stringify(ajax_context.render_options) ==
               JSON.stringify(g_graphs[graph_id].ajax_context.render_options)) {
            return graph_id;
        }
    }

    // Otherwise create a new graph
    return "graph_" + g_current_graph_id++;
}

function create_graph(html_code, abstract_graph, graph_render_options, ajax_context)
{
    // Detect whether or not a new graph_id has to be calculated. During the view
    // data reload create_graph() is called again for all already existing graphs.
    // In this situation the graph_id needs to be detected and reused instead of
    // calculating a new one. Otherwise e.g. g_graphs will grow continously.
    var graph_id = get_id_of_graph(ajax_context);

    // create container div that contains the graph.
    var container_div = document.createElement("div");
    container_div.setAttribute("id", graph_id);
    container_div.innerHTML = html_code;
    if (graph_render_options.show_timeranges)
        container_div.className = "graph_container timeranges";
    else
        container_div.className = "graph_container";

    var embedded_script = get_current_script();

    // Insert the new container right after the script tag
    embedded_script.parentNode.insertBefore(container_div, embedded_script.nextSibling);

    // No register and paint the graph
    ajax_context["graph_id"] = graph_id;
    g_graphs[graph_id] = abstract_graph;
    g_graphs[graph_id]["ajax_context"] = ajax_context;
    g_graphs[graph_id]["render_options"] = graph_render_options;
    g_graphs[graph_id]["id"] = graph_id;
    render_graph(g_graphs[graph_id]);
}

// determine DOM node of the <javascript> that called us. It's
// parent will get the graph node attached.
function get_current_script()
{
    var embedded_script = g_current_script;
    if (embedded_script)
        return embedded_script;

    // The function fixes IE compatibility issues
    return document.currentScript || (function() {
        var scripts = document.getElementsByTagName("script");
        return scripts[scripts.length - 1];
    })()
}

//#.
//#   .-Painting-------------------------------------------------------------.
//#   |                ____       _       _   _                              |
//#   |               |  _ \ __ _(_)_ __ | |_(_)_ __   __ _                  |
//#   |               | |_) / _` | | '_ \| __| | '_ \ / _` |                 |
//#   |               |  __/ (_| | | | | | |_| | | | | (_| |                 |
//#   |               |_|   \__,_|_|_| |_|\__|_|_| |_|\__, |                 |
//#   |                                               |___/                  |
//#   +----------------------------------------------------------------------+
//#   |  Paint the graph into the canvas object.                             |
//#   '----------------------------------------------------------------------'

// Keep draw contex as global variable for conveniance
var ctx = null;

// Notes:
// - In JS canvas 0,0 is at top left
// - We paint as few padding as possible. Additional padding is being
//   added via CSS
// NOTE: If you change something here, then please check if you also need to
// adapt the Python code that creates that abstract_graph
function render_graph(graph)
{
    // First find the canvas object and add a reference to the graph dict
    // If the initial rendering failed then any later update does not
    // make any sense.
    var container = document.getElementById(graph["id"]);
    var canvas = container.childNodes[0].getElementsByTagName("canvas")[0];
    if (canvas == null)
        return;
    graph["canvas_obj"] = canvas;

    ctx = canvas.getContext('2d'); // Create one ctx for all operations

    var font_size = graph["render_options"]["font_size"] * 2;
    ctx.font = font_size + "px sans-serif";

    var width = canvas.width;
    var height = canvas.height;

    var top_margin = graph_top_margin(graph);
    var bottom_border = graph_bottom_border(graph, font_size);

    // Prepare position and translation of origin
    var t_range_from = graph["time_axis"]["range"][0];
    var t_range_to = graph["time_axis"]["range"][1];
    var t_range = t_range_to - t_range_from;
    var t_pixels = width;
    if (!graph.render_options.preview)
        t_pixels -= v_axis_width;
    var t_pixels_per_second = t_pixels / t_range;
    graph["time_axis"]["pixels_per_second"] = t_pixels_per_second; // store for dragging

    var v_range_from = graph["vertical_axis"]["range"][0];
    var v_range_to = graph["vertical_axis"]["range"][1];
    var v_range = v_range_to - v_range_from;
    var v_pixels = height - top_margin - bottom_border;
    var v_pixels_per_unit = v_pixels / v_range;
    graph["vertical_axis"]["pixels_per_unit"] = v_pixels_per_unit; // store for dragging

    if (!graph.render_options.preview)
        var t_orig = v_axis_width;
    else
        t_orig = 0;
    graph["time_origin"] = t_orig; // for dragging

    var v_orig = height - bottom_border;
    graph["vertical_origin"] = v_orig; // for dragging

    var v_axis_orig = v_range_from;

    // paint graph background
    paint_rect([t_orig, top_margin], t_pixels, v_pixels, canvas_color);

    // Now transform the whole chooridate system to our real t and v choords
    // so if we paint something at (0, 0) it will correctly represent a
    // value of 0 and a time point of time_start.
    var trans_t = function(t)    { return (t - t_range_from) * t_pixels_per_second + t_orig; };
    var trans_v = function(v)    { return v_orig - ((v - v_axis_orig) * v_pixels_per_unit); };
    var trans   = function(t, v) { return [ trans_t(t), trans_v(v) ]; };

    // Paint curves
    var curves = graph["curves"];
    var step = graph["step"] / 2.0;
    for (var i=0; i<curves.length; i++)
    {
        var t = graph["start_time"];
        var curve = curves[i];
        if (curve["dont_paint"])
            continue;

        var points = curve["points"];
        var color = curve["color"];

        if (curve["type"] == "area") {
            var prev_lower = null;
            var prev_upper = null;
            ctx.save();
            var gradient = ctx.createLinearGradient(t_orig, top_margin + v_pixels, t_orig, top_margin - v_pixels);
            gradient.addColorStop(0, render_color(darken_color(parse_color(color), color_gradient)));
            gradient.addColorStop(0.5, color);
            gradient.addColorStop(1, render_color(lighten_color(parse_color(color), color_gradient)));
            ctx.fillStyle = gradient;
            // <F12><F3><F12>ctx.fillStyle = color;
            ctx.strokeStyle = "#000";
            ctx.imageSmoothingEnabled = true; // seems no difference on FF
            for (var j=0; j<points.length; j++) {
                var point = points[j];
                var lower = point[0];
                var upper = point[1];
                if (lower != null && upper != null && prev_lower != null && prev_upper != null) {
                    ctx.beginPath();
                    ctx.moveTo(trans_t(t - step)-1, trans_v(prev_lower));
                    ctx.lineTo(trans_t(t - step)-1, trans_v(prev_upper));
                    ctx.lineTo(trans_t(t),          trans_v(upper));
                    ctx.lineTo(trans_t(t),          trans_v(lower));
                    ctx.lineTo(trans_t(t - step)-1, trans_v(prev_lower));
                    ctx.closePath();
                    ctx.fill();
                    // paint_rect(trans(t, upper), step * t_pixels_per_second, (upper - lower) * v_pixels_per_unit, color);
                }
                prev_lower = lower;
                prev_upper = upper;
                t += step;
            }
            ctx.restore();
        }

        else { // "line"
            ctx.save();
            ctx.beginPath();
            ctx.strokeStyle = color;
            ctx.lineWidth = curve_line_width;
            var last_value = null;
            for (var j=0; j<points.length; j++) {
                var value = points[j];
                if (value != null) {
                    var p = trans(t, value);
                    if (last_value != null)
                        ctx.lineTo(p[0], p[1]);
                    else
                        ctx.moveTo(p[0], p[1]);
                }
                last_value = value;
                t += step;
            }
            ctx.stroke();
            ctx.closePath();
            ctx.restore();
        }
    }

    // Clear areas where values have been painted out of range. This is
    // At top and bottom
    paint_rect([t_orig, 0], t_pixels, top_margin,           background_color);
    paint_rect([t_orig, v_orig], t_pixels, height - v_orig, background_color);

    // Paint axes and a strong line at 0, if that is in the range
    paint_line([t_orig, v_orig + axis_over_width], [t_orig, top_margin],     axis_color);
    paint_line([t_orig - axis_over_width, v_orig], [width, v_orig], axis_color);
    if (v_range_from <= 0 && v_range_to >= 0)
       paint_line(trans(graph["start_time"], 0), trans(graph["end_time"], 0), "#000");

    if (!graph.render_options.preview)
    {
        // Paint the vertical axis
        var labels = graph["vertical_axis"]["labels"];
        ctx.save();
        ctx.textAlign = "end";
        ctx.textBaseline = "middle";
        for (var i=0; i<labels.length; i++)
        {
            var position   = labels[i][0];
            var label      = labels[i][1];
            var line_width = labels[i][2];
            if (line_width > 0) {
                paint_line(trans(t_range_from, position), trans(t_range_to, position),
                           v_line_color[line_width], v_line_dash[line_width]);
            }

            if (label != null)
                ctx.fillText(label, t_orig - v_label_margin, trans(t_range_from, position)[1]);
        }
        ctx.restore();

        // Paint time axis
        labels = graph["time_axis"]["labels"];
        ctx.save();
        ctx.textAlign = "center";
        ctx.textBaseline = "top";
        for (var i=0; i<labels.length; i++)
        {
            var position   = labels[i][0];
            var label      = labels[i][1];
            var line_width = labels[i][2];
            if (line_width > 0) {
                paint_line(trans(position, v_range_from), trans(position, v_range_to),
                           t_line_color[line_width], t_line_dash[line_width]);
            }
            if (label != null)
                ctx.fillText(label, trans(position, 0)[0], v_orig + t_label_margin);
        }
        ctx.restore();
    }

    // Paint horizontal rules like warn and crit
    ctx.save();
    ctx.lineWidth = rule_line_width;
    var rules = graph["horizontal_rules"];
    for (var i=0; i<rules.length; i++)
    {
        var position   = rules[i][0];
        var label      = rules[i][1];
        var color      = rules[i][2];
        var title      = rules[i][3];
        if (position >= v_range_from && position <= v_range_to) {
            paint_line(trans(t_range_from, position), trans(t_range_to, position), color);
        }
    }
    ctx.restore();

    // paint the optional pin
    if (graph.pin_time != null) {
        var pin_x = trans_t(graph.pin_time);
        if (pin_x >= t_orig) {
            paint_line([pin_x, v_orig + axis_over_width], [pin_x, top_margin], "#000");
            paint_dot([pin_x, top_margin], "#000");
        }
    }

    // Enable interactive mouse control of graph
    graph_activate_mouse_control(graph);
}

function to_display_coord(canvas_coord)
{
    return canvas_coord / 2;
}

function graph_top_margin(graph)
{
    if (!graph.render_options.preview)
        return g_graph_style.top_margin;
    else
        return 0;
}

function graph_bottom_border(graph, pixel_per_ex)
{
    if (!graph.render_options.preview)
        return pixel_per_ex + t_label_margin + bottom_margin;
    else
        return 0;
}

function paint_line(p0, p1, color, dash)
{
    ctx.save()

    if (typeof(dash) !== 'undefined') {
        if (ctx.setLineDash) {
            ctx.setLineDash(dash);
        }
        else {
            // Alpha color
            // FIXME: Currently not taking the base color into account, but this
            // fits our needs for the moment.
            if (dash[0] == 1 && dash[1] == 5) {
                color = "rgba(0, 0, 0, 0.1)";
            } else if (dash[0] == 1) {
                color = "rgba(0, 0, 0, 0.2)";
            }
        }
    }

    ctx.strokeStyle = color;
    ctx.beginPath();
    ctx.moveTo(p0[0], p0[1]);
    ctx.lineTo(p1[0], p1[1]);
    ctx.stroke();
    ctx.closePath();
    ctx.restore()
}

function paint_rect(p, width, height, color)
{
    ctx.save()
    ctx.fillStyle = color;
    ctx.fillRect(p[0], p[1], width, height);
    ctx.restore()
}

function paint_point(p, color)
{
    ctx.save();
    ctx.beginPath();
    ctx.lineWidth = 4;
    ctx.strokeStyle = color;
    ctx.moveTo(p[0]-6, p[1]-6);
    ctx.lineTo(p[0]+6, p[1]+6);
    ctx.moveTo(p[0]+6, p[1]-6);
    ctx.lineTo(p[0]-6, p[1]+6);
    ctx.stroke();
    ctx.closePath();
    ctx.restore();
}

function paint_dot(p, color)
{
    ctx.save();
    ctx.beginPath();
    ctx.arc(p[0], p[1], 5, 0, 2 * Math.PI);
    ctx.fillStyle = color;
    ctx.fill();
    ctx.closePath();
    ctx.restore();
}

function parse_color(hexcolor)
{
    var bits = parseInt(hexcolor.substr(1), 16);
    var r = ((bits >> 16) & 255) / 255.0;
    var g = ((bits >> 8) & 255) / 255.0;
    var b = (bits & 255) / 255.0;
    return [ r, g, b ];
}

function render_color(rgb)
{
    var r = rgb[0];
    var g = rgb[1];
    var b = rgb[2];
    var bits = parseInt(b * 255) + 256 * parseInt(g * 255) + 65536 * parseInt(r * 255);
    var hex = bits.toString(16);
    while (hex.length < 6)
        hex = "0" + hex;
    return "#" + hex
}

function lighten_color(rgb, v)
{
    var lighten = function(x, v) {
        return x + ((1.0 - x) * v);
    }
    return [
        lighten(rgb[0], v),
        lighten(rgb[1], v),
        lighten(rgb[2], v)
    ];
}

function darken_color(rgb, v)
{
    var darken = function(x, v) {
        return x * (1.0 - v);
    }
    return [
        darken(rgb[0], v),
        darken(rgb[1], v),
        darken(rgb[2], v)
    ];
}



//#.
//#   .-Mouse Control--------------------------------------------------------.
//#   |   __  __                         ____            _             _     |
//#   |  |  \/  | ___  _   _ ___  ___   / ___|___  _ __ | |_ _ __ ___ | |    |
//#   |  | |\/| |/ _ \| | | / __|/ _ \ | |   / _ \| '_ \| __| '__/ _ \| |    |
//#   |  | |  | | (_) | |_| \__ \  __/ | |__| (_) | | | | |_| | | (_) | |    |
//#   |  |_|  |_|\___/ \__,_|___/\___|  \____\___/|_| |_|\__|_|  \___/|_|    |
//#   |                                                                      |
//#   +----------------------------------------------------------------------+
//#   |  Code for handling dragging and zooming via the scroll whell.        |
//#   '----------------------------------------------------------------------'

g_dragging_graph = null;
g_resizing_graph = null;

// Is set to True when one graph is started being updated via AJAX.
// It is set to False when the update has finished.
g_graph_update_in_process = false;

// Is set to True when one graph is started being updated via AJAX. It is
// set to False after 100 ms to prevent too often graph rendering updates.
g_graph_in_cooldown_period = false;

// Holds the timeout object which triggers an AJAX update of all other graphs
// on the page 500ms after the last mouse wheel zoom step.
g_graph_wheel_timeout = null;

// Returns the graph container node. Can be called with any DOM node as
// parameter which is part of a graph
function get_graph_container(obj)
{
    while (obj && !has_class(obj, "graph_container"))
        obj = obj.parentNode;
    return obj;
}

function get_main_graph_container(obj)
{
    while (obj && !has_class(obj, "graph_with_timeranges"))
        obj = obj.parentNode;
    return obj.childNodes[1];
}

function get_graph_graph_node(obj)
{
    while (obj && !has_class(obj, "graph"))
        obj = obj.parentNode;
    return obj;
}

// Walk up DOM parents to find the graph container, then walk down to
// find the canvas element which has the graph_id in it's id attribute.
// Strip off the graph_id and return it.
function get_graph_id_of_dom_node(target)
{
    var graph_container = get_graph_container(target);
    if (!graph_container)
        return null;

    return graph_container.id;
}

function graph_global_mouse_wheel(event)
{
    event = event || window.event; // IE FIX

    var obj = getTarget(event);
    // prevent page scrolling when making wheelies over graphs
    while (obj && !obj.className)
        obj = obj.parentNode;
    if (obj && obj.tagName == 'DIV' && obj.className == 'graph_container')
        return prevent_default_events(event);
}

function graph_activate_mouse_control(graph)
{
    var canvas = graph["canvas_obj"];
    add_event_handler('mousemove', function(event) {
        return graph_mouse_move(event, graph);
    }, canvas);

    add_event_handler('mousedown', function(event) {
        return graph_mouse_down(event, graph);
    }, canvas);

    var on_wheel = function(event) {
        return graph_mouse_wheel(event, graph);
    };

    add_event_handler(wheel_event_name(), on_wheel, canvas);
    add_event_handler(wheel_event_name(), graph_global_mouse_wheel);

    add_event_handler('mouseup', global_graph_mouse_up);

    if (graph.ajax_context.render_options.resizable) {
        // Find resize img element
        var container = get_graph_container(canvas);
        var resize_img = container.getElementsByClassName('resize')[0]
        add_event_handler('mousedown', function(event) {
            return graph_start_resize(event, graph);
        }, resize_img);

        add_event_handler('mousemove', global_graph_mouse_move);
    }
}

function global_graph_mouse_move(event)
{
    event = event || window.event; // IE FIX
    update_mouse_indicator(event);
    graph_mouse_resize(event);
}

function graph_start_resize(event, graph)
{
    event = event || window.event; // IE FIX
    g_resizing_graph = {
        pos:   [ event.clientX, event.clientY ],
        graph: graph
    }
    return prevent_default_events(event);
}

function graph_mouse_resize(event)
{
    if (!g_resizing_graph)
        return true;

    if (g_graph_update_in_process || g_graph_in_cooldown_period)
        return prevent_default_events(event);

    var new_x = event.clientX;
    var new_y = event.clientY;
    var delta_x = new_x - g_resizing_graph.pos[0];
    var delta_y = new_y - g_resizing_graph.pos[1];
    g_resizing_graph.pos = [ new_x, new_y ];

    var graph = g_resizing_graph.graph;
    var post_data = "context=" + encodeURIComponent(JSON.stringify(graph.ajax_context))
                  + "&resize_x="  + delta_x
                  + "&resize_y="  + delta_y;

    start_graph_update(graph["canvas_obj"], post_data);
    return prevent_default_events(event);
}


// Get the mouse position of an event in choords of the
// shown time/value system. Return null if the choords
// lie outside.
function graph_get_mouse_position(event, graph)
{
    var time = graph_get_click_time(event, graph);
    if (time < graph["time_axis"]["range"][0] || time > graph["time_axis"]["range"][1])
        return null; // out of range

    var value = graph_get_click_value(event, graph);
    if (value < graph["vertical_axis"]["range"][0] || value > graph["vertical_axis"]["range"][1])
        return null; // out of range

    return [ time, value ];
}


function graph_mouse_down(event, graph)
{
    event = event || window.event; // IE FIX
    var pos = graph_get_mouse_position(event, graph);
    if (!pos)
        return;

    // Store information needed for update globally
    g_dragging_graph = {
        pos:   pos,
        graph: graph
    };
    g_graph_update_in_process = false;

    return prevent_default_events(event);
}


function has_mouse_moved(pos1, pos2)
{
    if (Math.abs(pos1[0] - pos2[0]) < 1)
        return false;
    else
        return true;
}


function global_graph_mouse_up(event)
{
    event = event || window.event; // IE FIX

    if (g_dragging_graph) {
        var graph = g_dragging_graph.graph;
        var pos = graph_get_mouse_position(event, graph);
        if (pos) {
            var graph_id = graph["id"];

            // When graph has not been dragged, the user did a simple click
            // Fire the graphs click action or, by default, positions the pin
            if (!has_mouse_moved(g_dragging_graph.pos, pos)) {
                handle_graph_clicked(graph);
                set_pin_position(event, graph, pos[0]);
            }

            if (graph.render_options.interaction)
                sync_all_graph_timeranges(graph_id);
        }
    } else if (!g_resizing_graph) {
        var target = getTarget(event);
        if (target.tagName == "TH" && has_class(target, "scalar") && has_class(target, "inactive")) {
            // Click on inactive scalar title: Change graph consolidation function to this one
            var graph_id = get_graph_id_of_dom_node(target);
            if (graph_id) {
                var graph = g_graphs[graph_id];

                var consolidation_function = "";
                if (has_class(target, "min"))
                    consolidation_function = "min";
                else if (has_class(target, "max"))
                    consolidation_function = "max";
                else
                    consolidation_function = "average";

                handle_graph_clicked(graph);
                set_consolidation_function(event, graph, consolidation_function);
            }

        } else if (target.tagName != "IMG" && target.tagName != "A") {
            var graph_id = get_graph_id_of_dom_node(target);
            if (graph_id) {
                var graph = g_graphs[graph_id];

                // clicked out of graphical area but on graph: remove the pin
                handle_graph_clicked(graph);
                remove_pin(event, graph);
            }
        }
    }

    g_dragging_graph = null;
    g_resizing_graph = null;
    g_graph_update_in_process = false;
    return true;
}

function handle_graph_clicked(graph)
{
    if (graph.render_options.onclick) {
        eval(graph.render_options.onclick);
    }
}

function set_consolidation_function(event, graph, consolidation_function)
{
    if (graph.render_options.interaction) {
        update_graph(event, graph, 0.0, null, null, null, null, consolidation_function);
        sync_all_graph_timeranges(graph.id);
    }
}

function remove_pin(event, graph)
{
    // Only try to remove the pin when there is currently one
    if (graph.render_options.interaction && graph.pin_time !== null) {
        set_pin_position(event, graph, -1);
        sync_all_graph_timeranges(graph.id);
    }
}

function set_pin_position(event, graph, timestamp)
{
    if (graph.render_options.interaction)
        return update_graph(event, graph, 0.0, null, null, null, parseInt(timestamp), null);
}


// move is used for dragging and also for resizing
function graph_mouse_move(event, graph)
{
    event = event || window.event; // IE FIX

    if (!graph.render_options.interaction)
        return; // don't do anything when this graph is not allowed to set the pin

    if (g_graph_update_in_process || g_graph_in_cooldown_period)
        return false;

    if (g_dragging_graph == null
        || g_dragging_graph.graph["canvas_obj"].id != graph["canvas_obj"].id)
        return false; // Not dragging or dragging other graph

    // Compute new time range
    var time_shift = g_dragging_graph.pos[0] - graph_get_click_time(event, graph);

    // Compute vertical zoom
    var value = graph_get_click_value(event, graph);
    var vertical_zoom = value / g_dragging_graph.pos[1];
    if (vertical_zoom <= 0)
        vertical_zoom = null; // No mirroring, no zero range

    update_graph(event, graph, time_shift, null, null, vertical_zoom, null, null);

    return prevent_default_events(event);
}


function update_mouse_indicator(event)
{
    remove_all_mouse_indicators();

    var canvas = getTarget(event);
    if (canvas.tagName != "CANVAS")
        return;

    var graph_node = get_graph_graph_node(canvas);
    var graph_id = get_graph_id_of_dom_node(graph_node);
    var graph = g_graphs[graph_id];

    if (!graph.render_options.interaction)
        return; // don't do anything when this graph is not allowed to set the pin

    var x = get_event_offset_x(event);
    var area_offset = !graph.render_options.preview ? to_display_coord(v_axis_width) : 0;
    if (x <= area_offset + 1)
        return;

    var indicator = document.createElement("div");
    add_class(indicator, "indicator");
    graph_node.appendChild(indicator);

    indicator.style.left = (x + 4) + "px";
    indicator.style.top    = (canvas.offsetTop + to_display_coord(graph_top_margin(graph))) + "px";
    indicator.style.height = (canvas.clientHeight - to_display_coord(graph_top_margin(graph)
                              + graph_bottom_border(graph, graph["render_options"]["font_size"]))) + "px";
}

function remove_all_mouse_indicators()
{
    var indicators = document.getElementsByClassName("indicator");
    for (var i = 0, len = indicators.length; i < len; i++) {
        indicators[i].parentNode.removeChild(indicators[i]);
    }
}

function graph_mouse_wheel(event, graph)
{
    event = event || window.event; // IE FIX

    if (!graph.render_options.interaction)
        return; // don't do anything when this graph is not allowed to set the pin

    if (g_graph_update_in_process)
        return prevent_default_events(event);

    var time_zoom_center = graph_get_click_time(event, graph);
    var delta = wheel_event_delta(event);

    var zoom = null;
    if (delta > 0) {
        zoom = 1.1;
    }
    else {
        // Do not zoom further in if we already display only 10 points or less
        var curves = graph["curves"];
        if (curves.length == 0)
            return true;
        var curve = curves[0];
        var points = curve["points"];
        if (points.length <= 10)
            return true;

        zoom = 1 / 1.1;
    }

    if (!update_graph(event, graph, 0.0, zoom, time_zoom_center, null, null, null))
        return false;

    /* Also zoom all other graphs on the page */
    if (g_graph_wheel_timeout)
        clearTimeout(g_graph_wheel_timeout);
    g_graph_wheel_timeout = setTimeout("sync_all_graph_timeranges('" + graph.id + "')", 500);

    return prevent_default_events(event);
}


function graph_get_click_time(event, graph)
{
    var canvas = getTarget(event);

    // Get X position of mouse click, converted to canvas pixels
    var x = get_event_offset_x(event) * canvas.width / canvas.clientWidth;

    // Convert this to a time value and check if its within the visible range
    var t_offset = (x - graph["time_origin"]) / graph["time_axis"]["pixels_per_second"];
    return graph["time_axis"]["range"][0] + t_offset;
}


function graph_get_click_value(event, graph)
{
    var canvas = getTarget(event);

    // Get Y position of mouse click, converted to canvas pixels
    var y = get_event_offset_y(event) * canvas.height / canvas.clientHeight;

    // Convert this to a vertical value and check if its within the visible range
    var v_offset = - (y - graph["vertical_origin"]) / graph["vertical_axis"]["pixels_per_unit"];
    return graph["vertical_axis"]["range"][0] + v_offset;
}

//#.
//#   .-Graph-Update-------------------------------------------------------.
//#   |   ____                 _           _   _           _       _       |
//#   |  / ___|_ __ __ _ _ __ | |__       | | | |_ __   __| | __ _| |_ ___ |
//#   | | |  _| '__/ _` | '_ \| '_ \ _____| | | | '_ \ / _` |/ _` | __/ _ \|
//#   | | |_| | | | (_| | |_) | | | |_____| |_| | |_) | (_| | (_| | ||  __/|
//#   |  \____|_|  \__,_| .__/|_| |_|      \___/| .__/ \__,_|\__,_|\__\___||
//#   |                 |_|                     |_|                        |
//#   +--------------------------------------------------------------------+
//#   | Handles re-rendering of graphs after user actions                  |
//#   '--------------------------------------------------------------------'

// TODO: Refactor the arguments to use something like call_ajax(). Makes things much clearer.
function update_graph(event, graph, time_shift, time_zoom, time_zoom_center, vertical_zoom, pin_timestamp, consolidation_function)
{
    var canvas = graph["canvas_obj"];

    var start_time;
    var end_time;

    // Time zoom
    if (time_zoom != null) {
        // The requested start/end time can differ from the real because
        // RRDTool align the times as it needs. The graph always is align
        // to the RRDTool data, but the zooming into small time intervals
        // does not work correctly if we do not base this on the requested start_time.
        start_time = time_zoom_center - ((time_zoom_center - graph["requested_start_time"]) * time_zoom);
        end_time   = time_zoom_center + ((graph["requested_end_time"] - time_zoom_center) * time_zoom);

        // Sanity check
        if (end_time < start_time) {
            end_time = start_time + 60;
            start_time -= 60;
        }

        // Do not allow less than 120 secs.
        var range = end_time - start_time;
        if (range < 120) {
            var diff = 120 - range;
            start_time -= (time_zoom_center - start_time) / 120 * diff;
            end_time   += (end_time - time_zoom_center)   / 120 * diff;
        }
    }

    // Time shift
    else {
        start_time = graph["start_time"] + time_shift;
        end_time   = graph["end_time"]   + time_shift;
    }

    // Check for range
    if (start_time < 0 || end_time < 0
        || start_time > 2147483646 || end_time > 2147483646
        || start_time > end_time) {
        return true;
    }

    // Vertical zoom
    var range_from = null;
    var range_to = null;
    if (vertical_zoom != null) {
        var old_range_from = graph["vertical_axis"]["range"][0];
        var old_range_to = graph["vertical_axis"]["range"][1];
        var range_from = old_range_from / vertical_zoom;
        var range_to = old_range_to / vertical_zoom;
    }
    else if (graph["requested_vrange"] != null) {
        range_from = graph["requested_vrange"][0];
        range_to = graph["requested_vrange"][1];
    }

    // Recompute step
    var step = (end_time - start_time) / canvas.width / 2;

    // wenn er einmal grob wurde, nie wieder fein wird, auch wenn man in
    // einen Bereich draggt, der wieder fein vorhanden wäre? Evtl. müssen
    // wir den Wunsch-Step neu berechnen. Oder sicher speichern, also
    // den ursprügnlichen Wunsch-Step anders als den vom RRD zurückgegebene.

    var post_data = "context="     + encodeURIComponent(JSON.stringify(graph.ajax_context))
                  + "&start_time=" + encodeURIComponent(start_time)
                  + "&end_time="   + encodeURIComponent(end_time)
                  + "&step="       + encodeURIComponent(step);

    if (range_from != null) {
        post_data +=   "&range_from=" + encodeURIComponent(range_from)
                  + "&range_to="   + encodeURIComponent(range_to);
    }

    if (pin_timestamp != null) {
        post_data += "&pin="+encodeURIComponent(pin_timestamp);
    }

    if (consolidation_function != null) {
        post_data += "&consolidation_function="+encodeURIComponent(consolidation_function);
    }

    if (g_graph_update_in_process)
        return prevent_default_events(event);

    start_graph_update(canvas, post_data);
    return true;
}


function start_graph_update(canvas, post_data)
{
    g_graph_update_in_process = true;

    set_graph_update_cooldown();
    pause_reload(g_page_update_delay);

    call_ajax("ajax_graph.py", {
        method           : "POST",
        response_handler : handle_graph_update,
        handler_data     : get_graph_container(canvas),
        post_data        : post_data
    });
}

function set_graph_update_cooldown()
{
    g_graph_in_cooldown_period = true;
    setTimeout(function() {
        g_graph_in_cooldown_period = false;
    }, 100);
}


function handle_graph_update(graph_container, ajax_response, http_code)
{
    if (http_code !== undefined) {
        console.log("Error calling AJAX web service for graph update: " + ajax_response);
        g_graph_update_in_process = false;
        return;
    }

    try {
        var response = JSON.parse(ajax_response);
    } catch (e) {
        console.log(e);
        alert("Failed to parse graph update response: "+ajax_response);
        return;
    }
    // Structure of response:
    // {
    //     "html" : html_code,
    //     "graph" : abstract_graph,
    //     "context" : {
    //         "graph_id"       : context["graph_id"],
    //         "definition"     : graph_definition,
    //         "data_range"     : graph_data_range,
    //         "render_options" : graph_render_options,
    // }
    var graph_id = response.context.graph_id;
    var graph = response.graph;
    graph["id"] = graph_id;
    graph["ajax_context"] = response.context;
    graph["render_options"] = graph["ajax_context"]["render_options"];
    g_graphs[graph_id] = graph;

    // replace eventual references
    if (g_dragging_graph && g_dragging_graph.graph.id == graph.id)
        g_dragging_graph.graph = graph;
    if (g_resizing_graph && g_resizing_graph.graph.id == graph.id)
        g_resizing_graph.graph = graph;

    graph_container.innerHTML = response["html"];

    render_graph_and_subgraphs(graph);
    g_graph_update_in_process = false;
}

// re-render the given graph and check whether or not there are subgraphs
// which need to be re-rendered too.
function render_graph_and_subgraphs(graph)
{
    render_graph(graph);

    for (var graph_id in g_graphs) {
        if (graph_id != graph.id && graph_id.substr(0, graph.id.length) == graph.id) {
            render_graph(g_graphs[graph_id]);
        }
    }
}

// Is called on the graph overview page when clicking on a timerange
// graph to change the timerange of the main graphs.
function change_graph_timerange(graph, duration)
{
    // Find the main graph by DOM tree:
    // <div class=graph_with_timeranges><div container of maingraph></td><table><tr><td>...myself
    var maingraph_container = get_main_graph_container(graph["canvas_obj"]);

    var main_graph_id = maingraph_container.id;
    var main_graph = g_graphs[main_graph_id];

    var now = Math.floor(new Date().getTime() / 1000);

    main_graph.start_time = now - duration;
    main_graph.end_time = now;

    sync_all_graph_timeranges(main_graph_id, false);
}

function update_pdf_export_link_timerange(start_time, end_time)
{
    var context_button = document.getElementById("context_pdf_export");
    if (!context_button)
        return;

    var link = context_button.getElementsByTagName("a")[0];
    link.href = makeuri({"start_time": start_time, "end_time": end_time}, link.href);
}

g_timerange_update_queue = [];

// Syncs all graphs on this page to the same time range as the selected graph.
// Be aware: set_graph_timerange triggers an AJAX request. Most browsers have
// a limit on the concurrent AJAX requests, so we need to slice the requests.
function sync_all_graph_timeranges(graph_id, skip_origin)
{
    if (skip_origin === undefined)
        skip_origin = true;

    g_timerange_update_queue = []; // abort all pending requests

    var graph = g_graphs[graph_id];
    for (var name in g_graphs) {
        // only look for the other graphs. Don't update graphs having fixed
        // time ranges, like the timerange chooser graphs on the overview page
        if ((!skip_origin || name != graph_id) && !g_graphs[name].render_options.fixed_timerange) {
            g_timerange_update_queue.push([name, graph.start_time, graph.end_time]);
        }
    }

    update_pdf_export_link_timerange(graph.start_time, graph.end_time);

    // Kick off 4 graph timerange updaters (related to the number of maximum
    // parallel AJAX request)
    for (var i = 0; i < 4; i++)
        update_next_graph_timerange();
}

function update_next_graph_timerange()
{
    var job = g_timerange_update_queue.pop();
    if (job)
        set_graph_timerange(job[0], job[1], job[2]);
}

function set_graph_timerange(graph_id, start_time, end_time)
{
    var graph = g_graphs[graph_id];
    var canvas = graph["canvas_obj"];
    if (canvas !== null) {
        var step = (end_time - start_time) / canvas.width / 2;

        // wenn er einmal grob wurde, nie wieder fein wird, auch wenn man in
        // einen Bereich draggt, der wieder fein vorhanden wäre? Evtl. müssen
        // wir den Wunsch-Step neu berechnen. Oder sicher speichern, also
        // den ursprügnlichen Wunsch-Step anders als den vom RRD zurückgegebene.

        var post_data = "context="     + encodeURIComponent(JSON.stringify(graph.ajax_context))
                      + "&start_time=" + encodeURIComponent(start_time)
                      + "&end_time="   + encodeURIComponent(end_time)
                      + "&step="       + encodeURIComponent(step);

        call_ajax("ajax_graph.py", {
            method           : "POST",
            post_data        : post_data,
            response_handler : handle_graph_timerange_update,
            handler_data     : get_graph_container(canvas)
        });
    }
}

// First updates the current graph and then continues with the next graph
function handle_graph_timerange_update(graph_container, ajax_response, http_code)
{
    handle_graph_update(graph_container, ajax_response, http_code);
    update_next_graph_timerange();
}

//#.
//#   .-Designer--------------------------------------------------------------.
//#   |               ____            _                                       |
//#   |              |  _ \  ___  ___(_) __ _ _ __   ___ _ __                 |
//#   |              | | | |/ _ \/ __| |/ _` | '_ \ / _ \ '__|                |
//#   |              | |_| |  __/\__ \ | (_| | | | |  __/ |                   |
//#   |              |____/ \___||___/_|\__, |_| |_|\___|_|                   |
//#   |                                 |___/                                 |
//#   +-----------------------------------------------------------------------+
//#   |  Code for the interactive graph designed                              |
//#   '-----------------------------------------------------------------------'

function at_least_one_graph_designed_checkbox_checked()
{
    var o_form = document.getElementById("form_graph");
    var childs = o_form.getElementsByTagName('input');
    var checkboxes = [];
    for (var i = 0; i < childs.length; i++) {
        if (childs[i].type == 'checkbox') {
            if (childs[i].checked)
                return true
        }
    }
    return false;
}


function fix_graph_designer_operator_visibiliy(check_box)
{
    var o_operators = document.getElementById("graph_designer_operators");
    var o_operators_off = document.getElementById("graph_designer_operators_off");
    if (at_least_one_graph_designed_checkbox_checked()) {
        o_operators.style.display = "";
        o_operators_off.style.display = "none";
    }
    else {
        o_operators.style.display = "none";
        o_operators_off.style.display = "";
    }
}

/* ValueSpec MetricChoice
This function is called for several purposes:
1. Updating the list of services when the host is changed
2. Updating the list of metrics when the service is changed
3. Submitting the form when something is changed in an existing
   metric (not [Add metric] case) *and* all four choices
   have a valid value.
*/
function metric_choice_changed(varprefix, o_input)
{
    var o_metric_choice = o_input.parentNode;
    var o_host = o_metric_choice.childNodes[0];
    var o_service = o_metric_choice.childNodes[2];
    var o_metric = o_metric_choice.childNodes[4];
    var o_cf = o_metric_choice.childNodes[6];

    // Do completion / updating of selections according to
    // changed host/service. but not for changes in the metric
    // or cf.
    if (o_input !== o_cf && o_input !== o_metric)
    {
        var url = "ajax_metric_choice.py?"
            + "varprefix=" + encodeURIComponent(varprefix)
            + "&host="     + encodeURIComponent(o_host.value)
            + "&service="  + encodeURIComponent(o_service.value)
            + "&metric="   + encodeURIComponent(o_metric.value)
            + "&cf="       + encodeURIComponent(o_cf.value);

        response = get_url_sync(url);
        if (response)
             o_metric_choice.innerHTML = response;
        else
            return; // Do not submit - host name is incomplete
    }

    // Modify existing formula? Submit, but only if
    // Service and metric selection are not empty.
    if (varprefix.indexOf("formula_") === 0) {
        var o_service = o_metric_choice.childNodes[2];
        var o_metric = o_metric_choice.childNodes[4];
        if (o_service.value && o_metric.value)
            submit_graph_designer_form();
    }

}

// Shall we do async updates?
// function metric_choice_update(o_metric_choice, ajax_response, http_code)
// {
//     if (ajax_response)
//         o_metric_choice.innerHTML = ajax_response;
// }

function submit_graph_designer_form()
{
    document.getElementById('form_graph').submit();
}
