#!/usr/bin/python
# -*- encoding: utf-8; py-indent-offset: 4 -*-
# .------------------------------------------------------------------------.
# |                ____ _               _        __  __ _  __              |
# |               / ___| |__   ___  ___| | __   |  \/  | |/ /              |
# |              | |   | '_ \ / _ \/ __| |/ /   | |\/| | ' /               |
# |              | |___| | | |  __/ (__|   <    | |  | | . \               |
# |               \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\              |
# |                                        |_____|                         |
# |             _____       _                       _                      |
# |            | ____|_ __ | |_ ___ _ __ _ __  _ __(_)___  ___             |
# |            |  _| | '_ \| __/ _ \ '__| '_ \| '__| / __|/ _ \            |
# |            | |___| | | | ||  __/ |  | |_) | |  | \__ \  __/            |
# |            |_____|_| |_|\__\___|_|  | .__/|_|  |_|___/\___|            |
# |                                     |_|                                |
# |                     _____    _ _ _   _                                 |
# |                    | ____|__| (_) |_(_) ___  _ __                      |
# |                    |  _| / _` | | __| |/ _ \| '_ \                     |
# |                    | |__| (_| | | |_| | (_) | | | |                    |
# |                    |_____\__,_|_|\__|_|\___/|_| |_|                    |
# |                                                                        |
# | mathias-kettner.com                                 mathias-kettner.de |
# '------------------------------------------------------------------------'
#  This file is part of the Check_MK Enterprise Edition (CEE).
#  Copyright by Mathias Kettner and Mathias Kettner GmbH.  All rights reserved.
#
#  Distributed under the Check_MK Enterprise License.
#
#  You should have  received  a copy of the Check_MK Enterprise License
#  along with Check_MK. If not, email to mk@mathias-kettner.de
#  or write to the postal address provided at www.mathias-kettner.de

import defaults, pdf, time, config, StringIO, copy, subprocess, livestatus
import forms, table, visuals, views, default_permissions, wato, userdb, metrics
import availability

from lib import *
from valuespec import *
from PIL import PngImagePlugin, JpegImagePlugin, PdfImagePlugin

#   .--Declarations--------------------------------------------------------.
#   |       ____            _                 _   _                        |
#   |      |  _ \  ___  ___| | __ _ _ __ __ _| |_(_) ___  _ __  ___        |
#   |      | | | |/ _ \/ __| |/ _` | '__/ _` | __| |/ _ \| '_ \/ __|       |
#   |      | |_| |  __/ (__| | (_| | | | (_| | |_| | (_) | | | \__ \       |
#   |      |____/ \___|\___|_|\__,_|_|  \__,_|\__|_|\___/|_| |_|___/       |
#   |                                                                      |
#   +----------------------------------------------------------------------+
#   |  A few declarations that are used in more than one code section.     |
#   '----------------------------------------------------------------------'

def placement_choices():
    return  [
        ( "n",  _("From the top of the page"),             LengthDown() ),
        ( "s",  _("From the bottom of the page"),          LengthUp() ),
        ( "w",  _("From left of the page"),                LengthRight() ),
        ( "e",  _("From right of the page"),               LengthLeft() ),
        ( "nw", _("From the top left of the page"),        Tuple(orientation="horizontal", elements=[LengthRight(), LengthDown()]) ),
        ( "ne", _("From the top right of the page"),       Tuple(orientation="horizontal", elements=[LengthLeft(), LengthDown()]) ),
        ( "sw", _("From the bottom left of the page"),     Tuple(orientation="horizontal", elements=[LengthRight(), LengthUp()]) ),
        ( "se", _("From the bottom right of the page"),    Tuple(orientation="horizontal", elements=[LengthLeft(), LengthUp()]) ),
        ( "c",  _("At the center of the page") ),
    ]

possible_anchors = {
    "vertical"   : [ "n", "s", "c" ],
    "horizontal" : [ "w", "e", "c" ],
    "free"       : [ "nw", "ne", "sw", "se", "n", "s", "w", "e", "c" ],
    "none"       : [ "c" ],
}

def page_choices():
    return [
       ( "all",    _("All pages") ),
       ( "first",  _("First page") ),
       ( "rest",   _("All but first page") ),
       ( "odd",    _("All odd pages") ),
       ( "even",   _("All even pages") ),
    ]

#.
#   .--Plugins-------------------------------------------------------------.
#   |                   ____  _             _                              |
#   |                  |  _ \| |_   _  __ _(_)_ __  ___                    |
#   |                  | |_) | | | | |/ _` | | '_ \/ __|                   |
#   |                  |  __/| | |_| | (_| | | | | \__ \                   |
#   |                  |_|   |_|\__,_|\__, |_|_| |_|___/                   |
#   |                                 |___/                                |
#   +----------------------------------------------------------------------+
#   |  Load plugins (builtin reports), declare permissions                 |
#   '----------------------------------------------------------------------'

loaded_with_language = False
builtin_reports = {}
fixel_types = {}
element_types = {}
paper_sizes = {}
painter_printers = {}

def load_plugins(force):
    global loaded_with_language, builtin_reports
    if loaded_with_language == current_language and not force:
        return

    # Load plugins for reports. Currently these files
    # just may add custom reports by adding to builtin_reports.
    load_web_plugins("reporting", globals())

    # This must be set after plugin loading to make broken plugins raise
    # exceptions all the time and not only the first time (when the plugins
    # are loaded).
    loaded_with_language = current_language

    # Clear this structure to prevent users accessing report structures created
    # by other users, make them see these reports
    reports = {}

    # General reporting permissions
    default_permissions.declare_visual_permissions('reports', _("reports"))

    config.declare_permission("general.reporting", _("Use Reporting"),
              _("Allows the user to use the reporting, i.e. generate reports. For creating own report "
                "templates or customizing existing ones a dedicated permission is required."),
              [ "user", "admin", "guest" ])

    config.declare_permission("general.instant_reports", _("Instant Reports - PDF Export"),
              _("Allows the user the do instant reporting. At the top of each "
                "view a button will be visible for exporting the current view's data "
                "as a PDF file."), [ "user", "admin", "guest" ])

    config.declare_permission("general.schedule_reports", _("Manage Own Scheduled Reports"),
              _("Allows a user to create or modify entries in the report scheduler."),
              [ "user", "admin" ])

    config.declare_permission("general.schedule_reports_all", _("Manage All Scheduled Reports"),
              _("Allows a user to modify scheduled reports of other users and also to specify "
                "arbitrary Email addresses as report destination."),
              [ "admin" ])

    # Declare permissions for all reports
    config.declare_permission_section("report", _("Reports"), do_sort = True)
    for name, rep in builtin_reports.items():
        config.declare_permission("report.%s" % name,
                rep["title"],
                rep.get("description", ""),
                config.builtin_role_ids)

    # Make sure that custom reports also have permissions
    config.declare_dynamic_permissions(lambda: visuals.declare_custom_permissions('reporting'))


def load_reports():
    global reports, available_reports
    reports = visuals.load('reports', builtin_reports)
    available_reports = visuals.available('reports', reports)

def get_report(name):
    load_reports()
    report = available_reports.get(name)
    if not report:
        raise MKGeneralException(_("No report defined with the name '%s'.") % html.attrencode(name))
    return report

def permitted_reports():
    return available_reports


#.
#   .--ValueSpecs----------------------------------------------------------.
#   |        __     __    _            ____                                |
#   |        \ \   / /_ _| |_   _  ___/ ___| _ __   ___  ___ ___           |
#   |         \ \ / / _` | | | | |/ _ \___ \| '_ \ / _ \/ __/ __|          |
#   |          \ V / (_| | | |_| |  __/___) | |_) |  __/ (__\__ \          |
#   |           \_/ \__,_|_|\__,_|\___|____/| .__/ \___|\___|___/          |
#   |                                       |_|                            |
#   +----------------------------------------------------------------------+
#   |  A few special ValueSpecs that are used in this module and/or in     |
#   |  some fixels or elements.                                            |
#   '----------------------------------------------------------------------'

def text_macro_help():
    return _("You can use macros like <tt>$PAGE$</tt> here. "
             "For a complete list of all available text macros "
             "you can add the report element <i>Table of available text macros</i> to your report.")


def Fontsize(**args):
    args.setdefault("title", _("Font size"))
    args.setdefault("default_value", 10)
    args["size"] = 5
    args["unit"] = _("pt")
    return Float(**args)


def Color(**args):
    defval = args.get("default_value", (1.0, 1.0, 1.0))
    args["elements"] = [
        Percentage(title = colname, default_value = defval[nr] * 100.0)
        for (nr, colname)
        in enumerate([_("Red"), _("Green"), _("Blue")]) ]
    args.setdefault("title", _("Color"))
    args["orientation"] = "horizontal"
    return Transform(
        Tuple(**args),
        forth = lambda outter: tuple(map(lambda x: x*100.0, outter)),
        back = lambda inner: tuple(map(lambda y: y/100.0, inner)),
    )


def Length(**args):
    args["unit"] = _("mm")
    args["size"] = 6
    args["display_format"] = "%." + str(args.get("precision", 1)) + "f"
    if "whence" in args:
        args["unit"] += " " + args["whence"]
    return Float(**args)


def LengthUp(**args):
    args["whence"] = _("up")
    return Length(**args)


def LengthDown(**args):
    args["whence"] = _("down")
    return Length(**args)


def LengthLeft(**args):
    args["whence"] = _("left")
    return Length(**args)


def LengthRight(**args):
    args["whence"] = _("right")
    return Length(**args)


def ReportSelection(**args):
    args["choices"] = get_report_choices
    return DropdownChoice(**args)

def GraphLayout(**args):
    args["elements"] = [
        ( "font_size",
          Fontsize(default_value = 8.0)
        ),
        ( "show_title",
          Checkbox(
            title = _("Title"),
            label = _("Show graph title"),
            default_value = True,
        )),
        ( "title_format",
          DropdownChoice(
              title = _("Format of graph title"),
              choices = [
                ( "plain",          _("Just show graph title") ),
                ( "add_host_name",  _("Add host name to graph title") ),
                ( "add_host_alias", _("Add host alias to graph title") ),
              ]
        )),
        ( "show_legend",
          Checkbox(
            title = _("Legend"),
            label = _("Show graph legend"),
            default_value = True,
        )),
        ( "border_width",
          Length(
              title = _("Border width"),
              help = _("Width of the black border around the whole graph. Enter 0 for no border here."),
              default_value = 0.05,
        )),
        ( "background_color",
          Color(
              title = _("Background color"),
              default_value = (0.95, 0.93, 0.91),
        )),
        ( "canvas_color",
          Color(
              title = _("Canvas color"),
              default_value = (1.0, 1.0, 1.0),
        )),
        ( "color_gradient",
          Percentage(
              title = _("Color gradient"),
              help = _("Slight gradient in the colors of the colored areas in the graphs. 0% turns off the gradient, 100% makes "
                       "it the strongest possible."),
              default_value = 20.0,
              max_value = 100.0,
        )),
    ]
    args.setdefault("title", _("Graph layout"))
    return Dictionary(**args)


def TableLayout(**args):
    args["elements"] = [
        ( "font_size",
          Fontsize(default_value = 8.0)
        ),
        ( "show_headings",
          Checkbox(
            title = _("Headings"),
            label = _("Show column headings"),
            default_value = True,
        )),
        ( "hrules",
          Checkbox(
            title = _("Horizontal rules"),
            label = _("Render horizontal rules between rows"),
            default_value = True,
        )),
        ( "vrules",
          Checkbox(
            title = _("Vertical rules"),
            label = _("Render vertical rules between columns"),
            default_value = False,
        )),
        ( "rule_width",
           Length(
             title = _("Width of the rules"),
             precision = 2,
             default_value = 0.05,
        )),
        ( "padding",
          Tuple(
              title = _("Cell padding"),
              elements = [
                  Length(title = _("Horizontal padding"), default_value=2),
                  Length(title = _("Vertical padding"), default_value=0.5),
              ],
              help = _("The cell padding is only applied when rules are being rendered.")
        )),
        ( "spacing",
          Tuple(
              title = _("Cell spacing"),
              elements = [
                  Length(title = _("Horizontal spacing"), default_value=4),
                  Length(title = _("Vertical spacing"), default_value=1),
              ],
              help = _("The cell spacing is only applied when <b>no</b> rules are being rendered.")
        )),
        ( "row_shading",
           Dictionary(
               title = _("Background shading for rows"),
               elements = [
                   ( "enabled", Checkbox(title = _("Active"), label=_("Enable row shading")) ),
                   ( "odd",     Color(title = _("Odd rows"),  default_value=(0.97, 0.97, 0.97))),
                   ( "even",    Color(title = _("Even rows"), default_value=(0.94, 0.94, 0.94))),
                   ( "heading", Color(title = _("Headings"),  default_value=(0.70, 0.70, 0.70))),
               ],
               columns = 2,
               optional_keys = False,
        )),
    ]
    args.setdefault("title", _("Table Layout"))
    return Dictionary(**args)

# Options for views and availability
def ViewOptions(element, **args):
    args.setdefault("title", _("Parameters"))
    args["required_keys"] = ["single_infos"]
    args["elements"] = [
        visuals.single_infos_spec(element['single_infos']),
        ( "title",
          TextUnicode(
              title = _("A heading for the view"),
              size = 64,
          )
        ),
        ( "limit",
          Integer(
              title = _("Limit the number of rows"),
              help = _("If you set a limit here then there will never be fetched and "
                       "rendered more data rows than you specify here. This can save you "
                       "against reports will a thousend pages in cases of an invalid "
                       "filter specification or the like.") + " " + \
                     _("Note: If you also set a limit in the report, then the limit set here "
                       "will replace that one."),
              default_value = 1000,
        )),
        ( "layout",
          CascadingDropdown(
              title = _("Type of table"),
              orientation = "horizontal",
              choices = [
                  ( "table",   _("Headers at the top (normal table)")),
                  ( "dataset", _("Headers at the left (single dataset)")),
                  ( "matrix",  _("Matrix layout"),
                    Integer(
                        label = _("Number of columns: "),
                        default_value = 12,
                        minvalue = 1,
                        maxvalue = 50,
                  ))
              ]
        )),
        ( "table_layout",
          TableLayout(),
        ),
    ]
    return Dictionary(**args)


def ElementTypeSelection(what): # what is "fixel" or "element"
    if what == "fixel":
        part_types = fixel_types
        title = _("Select the type of fixed page element")
    else:
        part_types = element_types
        title = _("Select the type of content element")

    return DropdownChoice(
        title = title,
        sorted = True,
        choices = [ (name, part_type["title"]) for (name, part_type) in part_types.items() ],
    )


def SchedulePeriod():
    return \
        CascadingDropdown(
            title = _("Period"),
            orientation = "horizontal",
            choices = [
                ( "day",   _("Every day"), ),
                ( "week",  _("Every week on..."),
                  Weekday(title = _("Day of the week"))),
                ( "month_begin", _("At the beginning of every month at day"),
                  Integer(minvalue=1, maxvalue=28)),
                ( "month_end", _("At the end of every month at day"),
                  Integer(minvalue=1, maxvalue=28, unit=_("from the end"))),
            ]
        )


def ReportEmailOptions():
    return Dictionary(
        title = _("Options for report emails"),
        elements = [
            ( "from",
              EmailAddress(title = _("From: email address")),
            ),
            ( "reply_to",
              EmailAddress(title = _("Reply-To: email address")),
            ),
            ( "subject",
              TextUnicode(
                  title = _("Subject"),
                  default_value = _("Check_MK $PERIODNAME$ '$TITLE$'"),
                  size = 64,
            )),
            ( "filename",
              TextAscii(
                  title = _("Filename for attached PDF"),
                  default_value = "report-$DATE$.pdf",
                  size = 64,
                  forbidden_chars = "/\\",
            )),
        ]
    )

#.
#   .--Configuration-------------------------------------------------------.
#   |    ____             __ _                       _   _                 |
#   |   / ___|___  _ __  / _(_) __ _ _   _ _ __ __ _| |_(_) ___  _ __      |
#   |  | |   / _ \| '_ \| |_| |/ _` | | | | '__/ _` | __| |/ _ \| '_ \     |
#   |  | |__| (_) | | | |  _| | (_| | |_| | | | (_| | |_| | (_) | | | |    |
#   |   \____\___/|_| |_|_| |_|\__, |\__,_|_|  \__,_|\__|_|\___/|_| |_|    |
#   |                          |___/                                       |
#   +----------------------------------------------------------------------+
#   |  Multisite-pages for creating and editing reports                    |
#   '----------------------------------------------------------------------'

def page_edit_reports():
    def check_report_deletable(reports, user_id, delname):
        if ('', delname) in reports:
            return True # builtin version exists. No dependency problems

        # Note: we might be too strict. If user A let's depend his report on
        # report X of user B. And also user C as a report X. And both are
        # visible. Then user B could delete his version.
        del_report = reports[(user_id, delname)]
        for (owner_id, name), report in reports.items():
            if report != del_report and report.get('use') == del_report['name']:
                raise MKUserError(None, _('Another report (<tt>%s</tt>) uses the settings and page layout '
                                          'of this report, so it can not be deleted. In order to be able to '
                                          'delete this report, you need to remove the dependency.') % name)

        if report_used_in_scheduler(delname):
            raise MKUserError(None, _("The report is used in the report scheduler. You first have to delete "
                                      "all schedules refering this report to be able to delete the report."))

    load_reports()
    visuals.page_list('reports', _("Edit Reports"), reports,
         render_custom_buttons = render_report_buttons,
         render_custom_columns = render_custom_columns,
         render_custom_context_buttons = render_custom_context_buttons,
         check_deletable_handler = check_report_deletable)


def report_used_in_scheduler(delname):
    for schedule in load_report_schedule():
        if schedule["report"] == delname:
            return True
    return False


def render_report_buttons(report_name, report):
    if report["owner"] == config.user_id:
        html.icon_button("edit_report_content.py?name=" + report_name, _("Edit report content"), "report_element")
    html.icon_button("report.py?name=" + report_name, _("Generate this report"), "pdf")
    html.icon_button("report_scheduler_edit.py?report=" + report_name, _("Create regular time-schedule for this report"), "reportscheduler")


def render_custom_columns(report_name, report):
    table.cell(_("Elements"), len(report.get("elements", 0)), css="number")
    table.cell(_("Fixed el."), len(report.get("fixels", 0)), css="number")
    if report.get("use") and report["use"] in available_reports:
        use_name = available_reports[report["use"]]["title"]
    else:
        use_name = ""
    table.cell(_("Based on"), use_name)


def render_custom_context_buttons():
    html.context_button(_("Scheduler"), "report_scheduler.py", "reportscheduler")


def page_create_report():
    visuals.page_create_visual('reports', visuals.infos.keys())


def page_edit_report():
    load_reports()

    # Specific options for reports
    # TODO: This is ugly. Do we really need a global variable here?
    # Anyway we will migrate this to pagetypes.py...
    global vs_report
    vs_report = Dictionary(
        title = _('Report Properties'),
        render = 'form',
        elements = report_options(False),
    )
    visuals.page_edit_visual('reports', reports,
        create_handler = create_report,
        custom_field_handler = custom_field_handler,
	sub_pages = [
	   (_("Elements"), "edit_report_content", "report_element"),
           (_("Generate"), "report", "pdf"),
	]
    )

# The options for reports are put into this function so that they
# can be used both as global options by WATO and as options directly
# in the report
def report_options(global_settings):
    return [
        ( 'use',
          ReportSelection(
              title = _("Use settings and page layout from"),
              help = _("You can specify another report here. All global settings from that "
                       "report are takes as default values. Fixed page elements from that report "
                       "are added to the fixed elements defined in this report."),
              default_value = "default",
        )),
        ( 'rangespec',
          Timerange(
              title = _("Default time range"),
              help = _("If no explicit time range is being provided then the report "
                       "defaults to this range."),
        )),
        ( "view_limit",
          Integer(
              title = _("Limit the number of rows in View tables"),
              help = _("If you set a limit here then there will never be fetched and "
                       "rendered more data rows than you specify here. This can save you "
                       "against reports will a thousend pages in cases of an invalid "
                       "filter specification or the like."),
              default_value = 1000,
        )),
        ( 'filename',
          TextUnicode(
              title = _("Download filename"),
              help = _("When creating a report PDF then the browser will "
                       "suggest to save the file with this filename. The "
                       "extension should always be <tt>pdf</tt>."),
              default_value = "report.pdf",
              regex = r"^[^/\\:]+$",
              regex_error = _("No colon, slash or backslash is allowed in the filename."),
        )),
        ( 'font_family',
          DropdownChoice(
              title = _("Font family"),
              choices = [
                  ( "Helvetica", _("Helvetica") ),
                  ( "Times",     _("Times") ),
              ],
              default_value = "Helvetica",
        )),
        ( 'font_size',
          Fontsize()
        ),
        ( 'lineheight',
          Float(
              title = _("Line spacing"),
              help = _("This setting controls the distance of text lines, in relation to "
                       "the font width. A setting of 1.2 thus means that the line spacing is 120% "
                       "of the font height."),
              default_value = 1.2
        )),
        ( 'pagesize',
          OptionalDropdownChoice(
              title = _("Page size"),
              choices = lambda: paper_sizes.items(),
              sorted = True,
              explicit = Tuple(
                 elements = [
                    Length(title=_("Width"), default_value=210.0),
                    Length(title=_("Height"), default_value=297.0),
                 ]
              ),
              default_value = (210.0, 297.0), # A4
        )),
        ( 'margins',
          Tuple(
              title = _("Paper margins"),
              elements = [
                  Length(title=_("Top"), default_value = 10.0),
                  Length(title=_("Right"), default_value = 10.0),
                  Length(title=_("Bottom"), default_value = 10.0),
                  Length(title=_("Left"), default_value = 10.0),
              ]
        )),
        ( 'mirror_margins',
          Checkbox(
              title = _("Mirror margins"),
              label = _("Swap left and right margins on pages with even numbers"),
        )),
        ( 'date_format',
          DateFormat(),
        ),
        ( 'time_format',
          TimeFormat(),
        ),
        ( 'table_layout',
          TableLayout(title = _("Default table layout"), optional_keys=not global_settings,
        )),
        ( 'graph_layout',
          GraphLayout(title = _("Default Graph layout"), optional_keys=not global_settings,
        )),
    ]


def custom_field_handler(report):
    vs_report.render_input('report', report and report or None)


def create_report(old_report, report):
    rep_properties = vs_report.from_html_vars('report')
    vs_report.validate_value(rep_properties, 'report')
    report.update(rep_properties)

    # Do not remove the elements
    report['elements'] = old_report.get('elements', [])
    report['fixels'] = old_report.get('fixels', [])
    report['mtime'] = int(time.time())

    # Make sure that thumbnail is refreshed after any change
    remove_report_preview(report["name"])
    return report


def page_edit_content():
    load_reports()
    report_name = html.var("name")
    if (config.user_id, report_name) not in reports:
        raise MKGeneralException(_("The requested report does not exist."))
    report = reports[(config.user_id, report_name)]

    html.header(_("Content of report %s") % report["title"],
                stylesheets=["pages", "views", "status", "bi", "reporting"])

    html.begin_context_buttons()
    html.context_button(_("All Reports"),        "edit_reports.py", "back")
    html.context_button(_("General Options"), "edit_report.py?load_name=" + report_name, "edit")
    html.context_button(_("Add Content"), "edit_report_element.py?name=" + report_name + "&mode=new", "new")
    html.context_button(_("Add Page Element"),   "edit_report_fixel.py?name=" + report_name + "&mode=new", "new")
    html.context_button(_("Generate"), "report.py?name=" + report_name, "pdf")
    html.end_context_buttons()

    # Deletion
    if html.var("_delete") and html.transaction_valid():
        delnr = int(html.var("_delete"))
        if html.var("_delwhat") == "fixel":
            things = report["fixels"]
            thing_types = fixel_types
        else:
            things = report["elements"]
            thing_types = element_types
        thing = things[delnr]
        deltitle = thing_types[thing["type"]]["title"]
        c = html.confirm(_("Delete %s number %d?") % (deltitle, delnr + 1))
        if c:
            del things[delnr]
            visuals.save("reports", reports)
            remove_report_preview(report_name)
        elif c == False:
            html.footer()
            return

    # Moving elements around
    elements = report.get("elements", [])
    if html.var("_move") and html.check_transaction():
        from_pos = int(html.var("_move"))
        to_pos = int(html.var("_where"))
        element = elements[from_pos]
        del elements[from_pos] # make to_pos now match!
        elements[to_pos:to_pos] = [element]
        visuals.save("reports", reports)
        remove_report_preview(report_name)

    html.write("<table class='report_contents'><tr>")

    # Show thumbnail of this report, if possible
    if pdf.is_pdf2png_possible():
        html.write("<td class=preview>")
        html.write("<a href='report.py?name=%s'><img src='report_thumbnail.py?name=%s'></a>" %
                                                                (report_name, report_name))
        html.write("</td>")

        html.write("<td class=table>")


    table.begin(title = _("Content elements"))
    for nr, element in enumerate(elements):
        element_type = element_types[element["type"]]
        table.row()
        table.cell(_("Actions"), css="buttons")
        html.icon_button("edit_report_element.py?name=%s&edit=%d" % (report_name, nr), _("Edit this schedule entry"), "edit")
        html.icon_button("edit_report_element.py?name=%s&clone=%d" % (report_name, nr), _("Clone this schedule entry"), "clone")
        html.icon_button(html.makeactionuri([("_delete", str(nr)), ("_delwhat", "element")]), _("Delete this schedule entry"), "delete")

        # Buttons for moving elements around
        top_url    = html.makeactionuri([("_move", nr), ("_where", 0)])
        bottom_url = html.makeactionuri([("_move", nr), ("_where", len(elements)-1)])
        up_url     = html.makeactionuri([("_move", nr), ("_where", nr-1)])
        down_url   = html.makeactionuri([("_move", nr), ("_where", nr+1)])
        if not element is elements[0]:
            html.icon_button(top_url, _("Move this element to the top"), "top")
            html.icon_button(up_url, _("Move this element one position up"), "up")
        else:
            html.empty_icon_button()
            html.empty_icon_button()

        if not element is elements[-1]:
            html.icon_button(down_url, _("Move this element one position down"), "down")
            html.icon_button(bottom_url, _("Move this element to the bottom"), "bottom")
        else:
            html.empty_icon_button()
            html.empty_icon_button()

        table.cell(_("Type"), element_type["title"])
        table.cell(_("Description"))
        if "to_text" in element_type:
            html.write(element_type["to_text"](element))
    table.end()

    table.begin(title = _("Fixed page elements"))
    for nr, fixel in enumerate(report.get("fixels", [])):
        fixel_type = fixel_types[fixel["type"]]
        table.row()
        table.cell(_("Actions"), css="buttons")
        html.icon_button("edit_report_fixel.py?name=%s&edit=%d" % (report_name, nr), _("Edit this fixel element"), "edit")
        html.icon_button("edit_report_fixel.py?name=%s&clone=%d" % (report_name, nr), _("Clone this fixel element"), "clone")
        html.icon_button(html.makeactionuri([("_delete", str(nr)), ("_delwhat", "fixel")]), _("Delete this fixel element"), "delete")

        table.cell(_("Type"), fixel_type["title"])

        table.cell(_("Position"))
        for entry in placement_choices():
            if entry[0] == fixel["placement"]["position"][0]:
                html.write(entry[1])
                if len(entry) > 2:
                    vs = entry[2]
                    html.write(" " + vs.value_to_text(fixel["placement"]["position"][1]))

        table.cell(_("Pages"))
        html.write(dict(page_choices())[fixel["placement"]["pages"]])

        table.cell(_("Parameters"))
        if "parameters" in fixel_type:
            html.write(fixel_type["parameters"].value_to_text(fixel["parameters"]))
    table.end()

    if pdf.is_pdf2png_possible():
        html.write("</td>")
        html.write("</tr></table>")

    html.footer()


def get_report_choices():
    load_reports()
    choices = [ (n, r["title"]) for (n,r) in available_reports.items() ]
    choices.sort(key = lambda x: x[1])
    choices = [ (None, _("(no report)")) ] + choices
    return choices


def page_edit_fixel():
    page_edit_reportpart("fixels", what_title=_("fixed page element"), part_types=fixel_types)


def page_edit_element():
    page_edit_reportpart("elements", what_title=_("report element"), part_types=element_types)


def page_edit_reportpart(what, what_title, part_types):
    load_reports()
    report_name = html.var("name")

    try:
        report = reports[(config.user_id, report_name)]
    except KeyError:
        raise MKUserError("name", _("This report does not exist."))

    if html.var("edit"):
        mode = "edit"
	title = _("Edit %s") % what_title

        try:
            part_nr = int(html.var("edit"))
            part = report[what][part_nr]
        except (ValueError, IndexError):
            raise MKUserError("edit", _("This report element does not exist."))

    elif html.var("clone"):
        mode = "clone"
        title = _("Clone %s") % what_title

        try:
            clone_nr = int(html.var("clone"))
            part = report[what][clone_nr].copy()
        except (ValueError, IndexError):
            raise MKUserError("edit", _("This report element does not exist."))
    else:
        mode = "new"
        title = _("Create new %s") % what_title
        part = {}

    html.header(title, stylesheets=["pages", "views", "status", "bi"])

    html.begin_context_buttons()
    html.context_button(_("Abort"), "edit_report_content.py?name=" + report_name, "abort")
    html.end_context_buttons()

    # When creating a new element, we need to know a few things, before the
    # actional element parameters can be shown. One of them is the type of
    # the element. Some element types need further information. These is
    # being specified in "creation_steps"
    def do_creation_step(nr, varname, vsinfo):
        if type(vsinfo) == type(lambda: None):
            valuespec = vsinfo(part)
        else:
            valuespec = vsinfo

        # Check if this step already has been done. We detect this according
        # to set HTML variables - which refer to the variables set by the
        # according ValueSpec. Note: complex ValueSpecs might not set the
        # variable directly but only ones with suffixes.
        varprefix = "step%d_" % (nr + 1)
        if html.has_var(varprefix + varname) or html.has_var_prefix(varprefix + varname + "_"):
            output = valuespec.from_html_vars(varprefix + varname)
        else:
            output = forms.edit_dictionary(
                [( varname, valuespec )],
                part,
                title = _("Step %d") % (nr + 1),
                buttontext = _("Continue"),
                varprefix = varprefix,
                formname = "step%d" % (nr + 1)
            )

        if output == None:
            html.footer()
            return False
        else:
            part[varname] = output
            return True

    if mode == "new":
        if not do_creation_step(0, "type", ElementTypeSelection(what[:-1])):
            return

    # Now we have the type. Otherwise the HTML would already have ended.
    part_type = part_types[part["type"]]

    if mode == "new":
        for nr, (varname, vsinfo) in enumerate(part_type.get("creation_steps", [])):
            if not do_creation_step(nr+1, varname, vsinfo):
                return

    # Now edit its properties
    entries = []

    # For fixed elements ask for placement and pages. This is done in the subkey "placement",
    if what == "fixels":
        allowed_placement_choices = [ c for c in placement_choices() if c[0] in possible_anchors[part_type["position"]]]

        entries.append(("placement", Dictionary(
            title = _("Placement"),
                elements = [
                    ( "pages",
                      DropdownChoice(
                          title = _("Put this element on"),
                          choices = page_choices(),
                    )),
                    ( "position",
                      CascadingDropdown(
                          title = _("Position"),
                          choices = allowed_placement_choices,
                    )),
                ],
                optional_keys = False,
            )))

    # Does this element type have parameters to edit? Parameters can either be a single
    # entry (a valuespec) or a function that returns a list of entries of the type
    # ("dictkey", entry), where entry is a valuespec.
    if part_type.get("parameters"):
        if type(part_type["parameters"]) == type(lambda: None):
            entries += part_type["parameters"](part)
        else:
            entries.append(("parameters", part_type["parameters"]))

    if part_type.get("has_context"):
        # All parts having a context are visuals. And all visuals need to have the
        # "single_infos" key defined. If it is not set yet, set it.
        part.setdefault("single_infos", part_type.get("single_infos", []))

        single_infos = part.get("single_infos", [])

    # The element might want to be transformed (needed for views)
    if "transform" in part_type:
        part_type["transform"][0](part)

    # Does this element need a filter context?
    if part_type.get("has_context"):
        part["context"] = visuals.pack_context_for_editing(part, report_context_info_handler)

        entries.append(
            ( "context",
              Dictionary(
                  title = _("Context / Search Filters"),
                  elements = visuals.get_context_specs(part, report_context_info_handler),
                  optional_keys = None,
            ))
        )

    # Do we have something to edit? Some elements to not and are simply added
    # without any further questions.
    if not entries:
        new_part = {}
    else:
        new_part = forms.edit_dictionaries(entries, part, method="POST", title=_("Properties for %s") % part_type["title"])

    if new_part != None:
        # Remove one dict level from context entry
        if "context" in new_part:
            new_part["context"] = visuals.unpack_context_after_editing(new_part["context"])

        # Add keys that have been set in prior steps
        for key, value in part.items():
            new_part.setdefault(key, value)

        # The element might want to be transformed (needed for views)
        if "transform" in part_type:
            part_type["transform"][1](new_part)

        parts = report.setdefault(what, [])
        if mode in [ "new", "clone" ]:
            parts.append(new_part)
        else:
            parts[part_nr] = new_part
        visuals.save("reports", reports)
        remove_report_preview(report_name)
        html.immediate_browser_redirect(0.1, "edit_report_content.py?name=" + report_name)
        html.message(_('Your %s element has been saved.') % what_title)

    html.footer()

def report_context_info_handler(element):
    # The element either needs to refer to a datasource (view, availability_)
    # or it explicitely defines single infos (PNP graphs)
    if "datasource" in element:
        return views.multisite_datasources[element["datasource"]]["infos"]
    else:
        element_type = element_types[element["type"]]
        return element_type["single_infos"]



#.
#   .--Scheduler-----------------------------------------------------------.
#   |            ____       _              _       _                       |
#   |           / ___|  ___| |__   ___  __| |_   _| | ___ _ __             |
#   |           \___ \ / __| '_ \ / _ \/ _` | | | | |/ _ \ '__|            |
#   |            ___) | (__| | | |  __/ (_| | |_| | |  __/ |               |
#   |           |____/ \___|_| |_|\___|\__,_|\__,_|_|\___|_|               |
#   |                                                                      |
#   +----------------------------------------------------------------------+
#   |  Multisite pages and logic for the report scheduler                  |
#   '----------------------------------------------------------------------'

def page_scheduler():
    load_reports()
    schedule = load_report_schedule()

    is_admin = config.may("general.schedule_reports_all")
    if not is_admin:
        config.need_permission("general.schedule_reports")

    html.header(_("Report Scheduler"), stylesheets=["pages", "views", "status", "reporting"])

    html.begin_context_buttons()
    html.context_button(_("New Entry"), "report_scheduler_edit.py", "new")
    html.context_button(_("Reports"), "edit_reports.py", "report")
    html.end_context_buttons()

    if html.transaction_valid():

        # Deletion
        if html.var("_delete"):
            delnr = int(html.var("_delete"))
            try:
                entry = schedule[delnr]
            except IndexError:
                raise MKUserError("_delete", _("The schedule you are trying to delete "
                                               "does not exist."))

            c = html.confirm(_("Delete report schedule entry '%s'?") % (entry["title"]))
            if c:
                del schedule[delnr]
                save_report_schedule(schedule)
            elif c == False:
                html.footer()
                return

        # Email right now
        elif html.var("_email"):
            delnr = int(html.var("_email"))
            try:
                entry = schedule[delnr]
            except IndexError:
                raise MKUserError("_email", _("The schedule you are trying to send "
                                              "an email for does not exist."))

            targets = create_scheduler_preview(entry, "email")
            html.message(_("Emailed report to %s.") % ", ".join(targets))



    table.begin(_("Scheduled reports"))
    for nr, entry in enumerate(schedule):
        table.row()

        # Actions
        table.cell(_("Actions"), css="buttons")
        if is_admin or entry["owner"] == config.user_id:
            html.icon_button("report_scheduler_edit.py?edit=%d" % nr, _("Edit this element"), "edit")
        html.icon_button("report_scheduler_edit.py?clone=%d" % nr, _("Clone this element"), "clone")
        if is_admin or entry["owner"] == config.user_id:
            html.icon_button(html.makeactionuri([("_delete", str(nr))]), _("Delete this element"), "delete")
            html.icon_button(html.makeactionuri([("_email", str(nr))]), _("Create and email this report right now"), "email")
        html.icon_button("report_scheduler_preview.py?id=%d&mode=pdf" % nr, _("Preview this report"), "pdf")

        # Disabled
        table.cell("", css="buttons")
        if entry["disabled"]:
            html.icon(_("This entry is currently disabled."), "disabled")

        # Owner
        table.cell(_("Owner"), entry["owner"])

        # Comment
        table.cell(_("Title"), html.attrencode(entry["title"]))

        # Report template
        table.cell(_("Report"))
        report_name = entry["report"]
        if report_name in available_reports:
            html.write(html.attrencode(available_reports[report_name]["title"]))
        else:
            html.write(report_name)

        # Range
        table.cell(_("Time Range"))
        html.write(Timerange().value_to_text(entry["rangespec"]))

        # Context
        table.cell(_("Context"))
        html.write(_("%d Filter settings") % len(entry["context"]))

        # Period and time of day
        table.cell(_("Period"), SchedulePeriod().value_to_text(entry["period"]))
        table.cell(_("Time"), Timeofday().value_to_text(entry["timeofday"]))

        # Last and Next time
        # table.cell(_("Last Period"))
        # html.write(time.strftime("%Y-%m-%d %H:%M", time.localtime(last_scheduled_time(entry))))
        table.cell(_("Last Run"))
        if entry.get("last_run"):
            html.write(time.strftime("%Y-%m-%d %H:%M", time.localtime(entry["last_run"])))
        table.cell(_("Last Error"))
        html.write(entry.get("last_error", "") or "")
        table.cell(_("Next Run"))
        html.write(time.strftime("%Y-%m-%d %H:%M", time.localtime(next_scheduled_time(entry))))


    table.end()

    html.footer()


def choose_report_name_for_schedentry():
    load_reports()

    html.header(_("Please choose a report to schedule"), stylesheets=["pages", "views", "status", "bi"])
    html.begin_context_buttons()
    html.context_button(_("Abort"), "report_scheduler.py", "abort")
    html.end_context_buttons()

    html.begin_form("report")
    forms.header("")
    forms.section(_("Report to schedule"))
    choices = [ (k, v["title"]) for (k,v) in available_reports.items() ]
    html.sorted_select("report", choices)
    forms.end()
    html.button("save", _("Continue"))
    html.hidden_fields()
    html.end_form()

    html.footer()


def page_scheduler_edit():
    config.need_permission("general.reporting")
    is_admin = config.may("general.schedule_reports_all")
    if not is_admin:
        config.need_permission("general.schedule_reports")

    load_reports()
    schedule = load_report_schedule()

    if html.var("edit"):
        mode = "edit"
        entry_nr = int(html.var("edit"))
	title = _("Edit report schedule entry %d") % entry_nr
        try:
            entry = schedule[entry_nr]
        except IndexError:
            raise MKUserError("edit", _("The schedule you are trying to edit does not exist."))

    elif html.var("clone"):
        mode = "clone"
        entry_nr = int(html.var("clone"))
	title = _("Clone report schedule entry %d") % entry_nr
        try:
            entry = schedule[entry_nr].copy()
        except IndexError:
            raise MKUserError("clone", _("The schedule you are trying to clone does not exist."))

    else:
        mode = "new"
        report_name = html.var("report")
        if not report_name:
            choose_report_name_for_schedentry()
            return

        try:
            report = available_reports[report_name]
        except KeyError:
            raise MKGeneralException(_("Cannot edit schedule entry for non-existing report %s") % report_name)

        title = _("Create new report schedule entry")
        entry = {
            "report"       : report_name,
            "single_infos" : report["single_infos"],
            "context"      : report["context"],
        }

    report_name = entry["report"]
    if not report_name in available_reports:
        raise MKGeneralException(_("Cannot edit schedule entry for non-existing report %s") % report_name)

    report = available_reports[report_name]

    html.header(title, stylesheets=["pages", "views", "status", "bi"])
    html.begin_context_buttons()
    html.context_button(_("Abort"), "report_scheduler.py", "abort")
    html.end_context_buttons()

    entries = []

    general_elements = [
        ( "report",
          FixedValue(
             report_name,
             title = _("Report to create"),
             totext = html.attrencode(report["title"])
        )),
        ( "title",
          TextUnicode(
              title = _("Title"),
              size = 64,
              allow_empty = False,
        )),
        ( "disabled",
          Checkbox(
              title = _("Activation"),
              help = _("Disabled schedule entries are not being executed."),
              label = _("do not create this report"),
          )
        ),
        ( "period",
          SchedulePeriod()
        ),
        ( "timeofday",
          Timeofday(
              title = _("Time of day to create report at"),
              default_value = (0,0),
        )),
        ( "rangespec",
          Timerange(
              title = _("Time range to create report for"),
              help = _("Please note: the time range selected here is relative to the time when the "
                       "report is being created. For example if you create a weekly report at Monday 00:00 "
                       "you need to select <i>last week</i>."),
              default_value = "m1", # last week
        )),
    ]

    if is_admin:
        general_elements += [
            (  "targets",
               ListOfStrings(
                   title = _("Email-Addresses to mail PDF reports to"),
                   valuespec = EmailAddress()
            )),
            (  "owner",
               wato.UserSelection(
                   title = _("Run report as this user"),
            )),
        ]

    general_elements.append((
      "email_options",
      ReportEmailOptions(),
    ))


    entries.append(
        ( "general",
          Dictionary(
              title = _("General Options"),
              optional_keys = None,
              elements = general_elements,
        ))
    )

    entries.append(
        ( "context",
          Dictionary(
              title = _("Context / Search Filters"),
              elements = visuals.get_context_specs(report, None),
              optional_keys = None,
        ))
    )

    edit_entry = {
        "context" : visuals.pack_context_for_editing(entry, None),
        "general" : entry,
    }
    dict_entry = forms.edit_dictionaries(entries, edit_entry, method="POST")

    if dict_entry != None:
        new_entry = dict_entry["general"]
        new_entry["context"] = visuals.unpack_context_after_editing(dict_entry["context"])

        if not is_admin:
            new_entry["owner"] = config.user_id
            new_entry["targets"] = []
        if mode in [ "new", "clone" ]:
            schedule.append(new_entry)
        else:
            schedule[entry_nr] = new_entry
        save_report_schedule(schedule)
        html.immediate_browser_redirect(0.1, "report_scheduler.py")
        html.message(_('Your schedule task has been saved.'))

    html.footer()



schedule_file_path = defaults.var_dir + "/report_schedule.py"

def load_report_schedule():
    if not os.path.exists(schedule_file_path):
        save_report_schedule([])

    aquire_lock(schedule_file_path)
    return eval(file(schedule_file_path).read())

def save_report_schedule(schedule):
    file(schedule_file_path, 'a') # ensure the file exists before trying to lock
    aquire_lock(schedule_file_path)
    file(schedule_file_path, "w").write("%s\n" % pprint.pformat(schedule))


def page_scheduler_preview():
    config.need_permission("general.reporting")
    mode = html.var("mode")
    entry_nr = int(html.var("id"))
    schedule = load_report_schedule()
    entry = schedule[entry_nr]
    return create_scheduler_preview(entry, mode)


def create_scheduler_preview(entry, mode):

    owner = entry["owner"]
    # We need to create that report with the permissions of its owner

    if owner != config.user_id:
        config.need_permission("general.schedule_reports_all")
        old_user = config.user_id
        config.login(owner)
    else:
        old_user = None

    load_reports()
    report_name = entry["report"]

    try:
        report = available_reports[report_name]
    except KeyError:
        raise MKUserError(None, _("The requested report does not exist."))

    do_send = mode == "pdf"

    # Set context by setting URI filter variables, just like
    # a user clicking on a context button for a report
    html.stash_vars()
    # add_context_to_uri_vars needs the key "single_infos"
    entry["single_infos"] = report["single_infos"]
    visuals.add_context_to_uri_vars(entry)
    pdf_source = render_report(report_name, report, do_send=do_send,
                               rangespec = entry["rangespec"])
    html.unstash_vars()

    if not do_send:
        return email_scheduled_report(entry, report, pdf_source)

    if old_user:
        config.login(old_user)
        load_reports()



# Computes for a scheduling entry the last/next time that this entry
# should have run / will be run.
def last_scheduled_time(entry):
    return scheduled_time(entry, "last")

def next_scheduled_time(entry):
    return scheduled_time(entry, "next")

def scheduled_time(entry, how):
    if how == "last":
        comp = lambda a, b: a > b
        add = 1
    else:
        comp = lambda a, b: a < b
        add = -1

    timeofday = entry["timeofday"]
    period = entry["period"]
    now = time.time()

    year, month, mday, hour, minute, second, wday = range(7)

    # Get the current time according to our timezone
    brokentime = list(time.localtime(now))

    # Enter the time of the day into the struct
    brokentime[hour]   = timeofday[0] # hours
    brokentime[minute] = timeofday[1] # minutes
    brokentime[second] = 0            # seconds

    if period == "day":
        ref_time = time.mktime(brokentime)
        if comp(ref_time, now): # in the future: substract one day
            ref_time -= 24 * 3600 * add

    elif period[0] == "week":
        ref_time = time.mktime(brokentime)
        daydiff = period[1] - brokentime[wday] # wday
        ref_time += daydiff * 24 * 3600
        if comp(ref_time, now): # in the future: substract one week
            ref_time -= 7 * 24 * 3600 * add

    elif period[0] == "month_begin":
        brokentime[mday] = period[1] # mday
        ref_time = time.mktime(brokentime)
        if comp(ref_time, now): # in the future: go back to previous month
            brokentime[month] -= 1 * add
            if brokentime[month] == 0:
                brokentime[month] = 12
                brokentime[year] -= 1
            elif brokentime[month] == 13:
                brokentime[month] = 1
                brokentime[year] += 1
            ref_time = time.mktime(brokentime)

    elif period[0] == "month_end":
        minus_mday = period[1]
        # Find last day in this month.
        brokentime[mday] = 1
        brokentime[month] += 1
        if brokentime[month] == 13:
            brokentime[month] = 1
            brokentime[year] += 1
        ref_time = time.mktime(brokentime) - minus_mday * 24 * 3600
        if comp(ref_time, now): # switch to previous/next month
            brokentime = list(time.localtime(ref_time))
            brokentime[mday] = 1
            if how == "next":
                brokentime[month] += 1
                if brokentime[month] == 13:
                    brokentime[year] += 1

            ref_time = time.mktime(brokentime) - minus_mday * 24 * 3600 * add

    # Due to the date shift a change in the timezone could have
    # happened. Make sure hour is correctly set again
    brokentime = list(time.localtime(ref_time))
    brokentime[hour] = timeofday[0]
    ref_time = time.mktime(brokentime)
    return ref_time


# This is registered as a Multisite cron job
def do_scheduled_reports():
    now = time.time()
    schedule = load_report_schedule()
    modified = False
    for entry in schedule:
        if entry["disabled"]:
            continue

        last_run = entry.get("last_run", 0)
        last_time = last_scheduled_time(entry)
        # Run report if last_time > last_run and if last_time is not more
        # recent then one hour.
        if last_time > last_run and now - last_time < 3600:
            modified = True
            html.write(_("Creating scheduled report '%s'...") % entry["title"])
            try:
                create_scheduled_report(entry)
                entry["last_run"] = now
                entry["last_error"] = None
                html.write("OK\n")
            except Exception, e:
                if config.debug:
                    raise
                html.write("failed: %s\n" % e)
                try:
                    entry["last_error"] = e.reason
                except:
                    entry["last_error"] = str(e)

    if modified:
        save_report_schedule(schedule)


def create_scheduled_report(entry):
    owner = entry["owner"]

    # We need to create that report with the permissions of its owner
    config.login(owner)
    load_reports()

    report_name = entry["report"]

    try:
        report = available_reports[report_name]
    except KeyError:
        raise MKUserError(None, _("The requested report does not exist."))

    # Set context by setting URI filter variables, just like
    # a user clicking on a context button for a report
    html.stash_vars()
    # add_context_to_uri_vars needs the key "single_infos"
    entry["single_infos"] = report["single_infos"]
    visuals.add_context_to_uri_vars(entry)
    pdf_source = render_report(report_name, report, do_send=False,
                               rangespec = entry["rangespec"])
    html.unstash_vars()

    email_scheduled_report(entry, report, pdf_source)


def email_scheduled_report(entry, report, pdf_source):
    filename = report.get("filename", config.reporting_filename)
    email_addresses = entry["targets"]
    if not email_addresses:
        address = None
        users = userdb.load_users(lock = False)
        user = users.get(entry["owner"])
        if user:
            address = user.get("email")
        if not address:
            raise MKGeneralException(_("Cannot email report: The owner %s does not have an email "
                                       "address and no explicit addresses are configured.") % str(entry["owner"]))
        email_addresses = [ address ]

    title = entry["title"]

    email_options = {}
    email_options.update(config.reporting_email_options)
    email_options.update(entry.get("email_options", {}))

    # Construct subject
    period = entry["period"]
    if period == "day":
        periodname = _("Daily Report")

    elif period[0] == "week":
        periodname = _("Weekly Report")

    else:
        periodname = _("Monthly Report")

    subject_template = email_options.get("subject", _("Check_MK $PERIODNAME$ '$TITLE$'"))
    subject = subject_template.replace("$TITLE$", title).replace("$PERIODNAME$", periodname)

    # Construct text body
    text_body = "Scheduled Check_MK Report %s.\r\n" % (title)

    # Filename of the report
    filename_template = email_options.get("filename")
    if not filename_template:
        # Get filename from the report
        trunk = report.get("filename", "report.pdf")
        if trunk.lower().endswith(".pdf"):
            trunk = trunk.rsplit(".", 1)[0] # omit *.pdf
        filename_template = trunk + "-$DATE$.pdf"
    filename = filename_template.replace("$DATE$", time.strftime("%Y-%m-%d"))

    send_report_email(email_addresses, email_options.get("from"), email_options.get("reply_to"),
                      subject, text_body, pdf_source, filename)
    return email_addresses



#.
#   .--Emails--------------------------------------------------------------.
#   |                    _____                 _ _                         |
#   |                   | ____|_ __ ___   __ _(_) |___                     |
#   |                   |  _| | '_ ` _ \ / _` | | / __|                    |
#   |                   | |___| | | | | | (_| | | \__ \                    |
#   |                   |_____|_| |_| |_|\__,_|_|_|___/                    |
#   |                                                                      |
#   +----------------------------------------------------------------------+
#   |  Code for sending emails. This could be put into a library as soon   |
#   |  as we need the code a second time.                                  |
#   '----------------------------------------------------------------------'

try:
    from email.mime.multipart import MIMEMultipart
    from email.mime.text import MIMEText
    from email.mime.application import MIMEApplication
except ImportError:
    # python <2.5 compat
    from email.MIMEMultipart import MIMEMultipart
    from email.MIMEText import MIMEText
    from email.MIMEBase import MIMEBase
    from email import Encoders
    MIMEApplication = None

def send_report_email(recipients, from_address, reply_to_address, subject, text_body, pdf_source, filename):
    if not os.path.exists("/usr/sbin/sendmail"):
        raise MKGeneralException(_("You have no mail transfer agent installed. "
             "<tt>/usr/sbin/sendmail</tt> is missing. Please install Postfix, Qmail, "
             "Nullmailer or Sendmail."))

    mail = MIMEMultipart('mixed', _charset='utf-8')
    mail.attach(MIMEText(text_body, 'plain', _charset='utf-8'))

    if MIMEApplication != None:
        pdf_att = MIMEApplication(pdf_source, "pdf; name=\"%s\"" % filename)
    else:
        # python <2.5 compat
        pdf_att = MimeBase('application', 'octet-stream')
        pdf_att.set_payload(pdf_source)
        Encoders.encode_base64(pdf_att)

    pdf_att.add_header('Content-Disposition', 'attachment', filename=filename)
    mail.attach(pdf_att)

    mail['Subject'] = subject
    mail['To'] = ",".join(recipients)

    if reply_to_address:
        mail['Reply-To'] = reply_to_address

    cmd = ["/usr/sbin/sendmail"]
    if from_address:
        mail['From'] = from_address
        cmd += ['-f', from_address]
    cmd += [ "-i" ] # Do not need '.' at end of email
    cmd += recipients
    p = subprocess.Popen(cmd, stdin = subprocess.PIPE)
    p.communicate(mail.as_string())
    return True


#.
#   .--Adding views to reports---------------------------------------------.
#   |             _       _     _   ____                                   |
#   |            / \   __| | __| | |  _ \ ___  _ __  _   _ _ __            |
#   |           / _ \ / _` |/ _` | | |_) / _ \| '_ \| | | | '_ \           |
#   |          / ___ \ (_| | (_| | |  __/ (_) | |_) | |_| | |_) |          |
#   |         /_/   \_\__,_|\__,_| |_|   \___/| .__/ \__,_| .__/           |
#   |                                         |_|         |_|              |
#   +----------------------------------------------------------------------+
#   |  Add views and PNP graphs to reports via a popup.                    |
#   '----------------------------------------------------------------------'

def popup_list_reports():
    if not config.may("general.edit_reports"):
        return []

    load_reports()
    return [ (name, report["title"])
             for (name, report)
             in available_reports.items() ]


def popup_add_element(report_name, element_type_name, context, parameters):
    if not config.may("general.edit_reports"):
        return

    load_reports()
    report = available_reports[report_name]

    # If the user has not yet an own copy of this report, create one.
    if report['owner'] != config.user_id:
        report = copy.deepcopy(report)
        report['owner'] = config.user_id

        reports[(config.user_id, report_name)] = report
        available_reports[report_name] = report

    element_type = element_types[element_type_name]
    element = element_type["clone_from_gui"](parameters, context)
    report.setdefault("elements", []).append(element)
    visuals.save("reports", reports)
    remove_report_preview(report_name)

    # We need to return the URL where the user shall do to after
    # adding. A good idea is the configuration page of the report's
    # contents - if you ask me.
    html.write("OK edit_report_content.py?name=" + report_name)


#.
#   .--Preview-(Thumbnails)------------------------------------------------.
#   |                ____                 _                                |
#   |               |  _ \ _ __ _____   _(_) _____      __                 |
#   |               | |_) | '__/ _ \ \ / / |/ _ \ \ /\ / /                 |
#   |               |  __/| | |  __/\ V /| |  __/\ V  V /                  |
#   |               |_|   |_|  \___| \_/ |_|\___| \_/\_/                   |
#   |                                                                      |
#   +----------------------------------------------------------------------+
#   |  Code for rendering thumbnails of a report                           |
#   '----------------------------------------------------------------------'


def preview_filename(report_name, pageno=1):
    preview_dir = config.user_confdir + "/report-thumbnails"
    if not os.path.exists(preview_dir):
        make_nagios_directory(preview_dir)
    return "%s/%s-%d.png" % (preview_dir, report_name, pageno)


def have_report_preview(report_name):
    return os.path.exists(preview_filename(report_name))


def remove_report_preview(report_name):
    if have_report_preview(report_name):
        os.remove(preview_filename(report_name))


def update_report_preview(report_name, report):
    if not pdf.is_pdf2png_possible():
        return

    # Create the data of a PDF document (of course without creating a temporary
    # file ;-) ) and limit the number of rows in each view to 50. This prevents
    # us from going crazy if the report has no intrinsic filter and would e.g.
    # show *all* services.
    pdf_source = render_report(report_name, report, min_pages=2, do_send=False, view_limit=50)
    file(preview_filename(report_name), "w").write(pdf.pdf2png(pdf_source))


def page_thumbnail():
    load_reports()
    report_name = html.var("name")
    report = reports[(config.user_id, report_name)] # makes sure that the name is valid!
    path = preview_filename(report_name)
    if not os.path.exists(path):
        update_report_preview(report_name, report)

    html.set_content_type("image/png")
    html.set_http_header("Content-Disposition", "attachment; filename=\"%s.png\"" % report_name)
    content = file(path).read()
    html.write(content)



#.
#   .--Rendering-----------------------------------------------------------.
#   |            ____                _           _                         |
#   |           |  _ \ ___ _ __   __| | ___ _ __(_)_ __   __ _             |
#   |           | |_) / _ \ '_ \ / _` |/ _ \ '__| | '_ \ / _` |            |
#   |           |  _ <  __/ | | | (_| |  __/ |  | | | | | (_| |            |
#   |           |_| \_\___|_| |_|\__,_|\___|_|  |_|_| |_|\__, |            |
#   |                                                    |___/             |
#   +----------------------------------------------------------------------+
#   |  Code that actually create PDF report documents                      |
#   '----------------------------------------------------------------------'


def page_report():
    report_name = html.var("name")
    report = get_report(report_name)
    render_report(report_name, report)


def page_instant_graph_collection():
    collection = metrics.GraphCollection.requested_page()

    elements = []
    for element in collection.elements():
        # Transform graph collection element to report element
        graph_spec = collection.graph_specification_from_element(element)
        if graph_spec[0] == "template":
            if graph_spec[1]["service_description"] == "_HOST_":
                datasource   = "hosts"
                single_infos = ["host"]
            else:
                datasource = "services"
                single_infos = ["host", "service"]

            report_element = {
                "type"         : "pnpgraph",
                "context"      : {
                    "host"    : graph_spec[1]["host_name"],
                    "service" : graph_spec[1]["service_description"],
                },
                "datasource"   : datasource,
                "single_infos" : single_infos,
                "parameters"   : {
                    "source": graph_spec[1]["graph_index"] + 1,
                },
            }

        elif graph_spec[0] == "custom":
            report_element = {
                "type"         : "custom_graph",
                "context"      : {},
                "datasource"   : None,
                "single_infos" : [],
                "parameters"   : {
                    "name": graph_spec[1],
                },
            }
        else:
            raise MKGeneralException(_("Invalid graph type: %s") % graph_spec[0])

        elements.append(report_element)

    instant_report = {
        "name"         : "instant",
        "use"          : "instant_graph_collection",
        "context"      : {},
        "single_infos" : [],
        "elements"     : elements,
        "title"        : collection.title(),
    }

    if html.var("start_time") and html.var("end_time"):
        try:
            start_time = int(html.var("start_time"))
            end_time = int(html.var("end_time"))
        except ValueError:
            raise MKGeneralException(_("Invalid start_time or end_time given"))

        rangespec = ('time', (start_time, end_time))
    else:
        graph_data_range = collection.get_graph_data_range()
        rangespec = ('time', tuple(graph_data_range["time_range"]))

    render_report("instant", instant_report, rangespec=rangespec)


def get_availability_type_from_view(view):
    infos = views.multisite_datasources[view["datasource"]]["infos"]
    if "aggr" in infos:
        return "bi"
    elif "service" in infos:
        return "service"
    else:
        return "host"


def page_instant_view():
    do_availability = html.var("mode") == "availability"
    view_name = html.var("view_name")
    view = views.get_view_by_name(view_name)
    title = visuals.visual_title("view", view)
    datasource_name = view["datasource"]
    datasource = views.multisite_datasources[datasource_name]
    view_infos = datasource["infos"]
    single_infos = view.get("single_infos", [])
    context = visuals.get_context_from_uri_vars(view_infos, single_infos)
    layout = view["layout"]
    if layout == "matrix":
        layout = ("matrix", view["num_columns"])

    rangespec = None

    if do_availability:
        element = {
            "type"         : "availability",
            "avoptions"    : views.get_availability_options_from_url(get_availability_type_from_view(view)),
            "datasource"   : datasource_name,
            "context"      : {},
            "single_infos" : single_infos,
            "parameters"   : {},
        }
        if html.var("av_mode") == "timeline":
            element["avoptions"]["elements"] = [ "timebar", "timeline" ]
        else:
            element["avoptions"]["elements"] = [ "table" ]
        elements = [ element ]
        rangespec = element["avoptions"]["range"]

    else:
        view_element = {
            "type"         : "view",
            "datasource"   : datasource_name,
            "context"      : {},
            "single_infos" : single_infos,
            "parameters"   : {
                "layout" : layout,
            },
        }
        for key in [ "painters", "group_painters", "sorters" ]:
            view_element[key] = view[key]

        elements = [view_element]

        # Determine whether or not the view has time related filters and use the
        # configuration of these filters as rangespec for the report.
        rangespec = get_time_range_from_view_filters(view)

    instant_report = {
        "name"         : "instant",
        "use"          : "instant_" + (do_availability and "availability" or "view"),
        "context"      : context,
        "single_infos" : single_infos,
        "elements"     : elements,
        "title"        : title,
    }

    render_report("instant", instant_report, rangespec=rangespec)


def get_time_range_from_view_filters(view):
    # First try if the view has some active filters specifying a
    # certain time range (like log views)
    datasource = views.multisite_datasources[view["datasource"]]
    use_filters = visuals.filters_of_visual(view, datasource['infos'])
    start_times, end_times = [], []
    for filt in visuals.visible_filters_of_visual(view, use_filters):
        if isinstance(filt, visuals.FilterTime):
            start_time, end_time = filt.get_time_range()
            if start_time:
                start_times.append(start_time)
            if end_time:
                end_times.append(end_time)

    if not end_times: # Default end is "now"
        end_times.append(int(time.time()))

    if start_times and end_times:
        return ("time", (min(start_times), max(end_times)))


    # Now look for the painter option for graphs
    if html.has_var_prefix("po_pnp_timerange"):
        vs = views.multisite_painter_options["pnp_timerange"]["valuespec"]
        return vs.from_html_vars("po_pnp_timerange")


def render_report(report_name, report, min_pages=0, do_send=True, view_limit=None,
                  rangespec=None):

    config.need_permission("general.reporting")

    # Optionally use general options and fixed elements from other report.
    use = report.get("use", config.reporting_use)
    if use and use != report_name: # Do not import ourselves if we are the default template!
        base_report = get_report(use)
        report = use_report(report, base_report)

    # instance encapsulate all information about how the report should
    # be created, which is not part of the report template itself.
    # instance is a dictionary that is just being used during the creation
    # of the report PDF. It contains
    # - "report" : The report object (e.g. for getting its context)
    # - "range" : information about the time range to cover
    # - "macros" : text macros for replacements like $DATE$ and $HOSTNAME$
    # - "options" : global reporting settings, overridden by settings in this report
    instance = {}

    # Compute options for this report from global settings, overridden
    # by settings in the report itself.
    instance["options"] = {}
    for varname, valuespec in report_options(False):
        global_value = eval("config.reporting_%s" % varname)
        if type(global_value) == dict: # merge, e.g. table_layout
            value = {}
            value.update(global_value)
            value.update(report.get(varname, {}))
        else:
            value = report.get(varname, global_value)
        instance["options"][varname] = value

    # Force a view limit if the caller wants us to. This is being used
    # in thumbnail mode
    if view_limit != None:
        instance["options"]["view_limit"] = view_limit

    # Get timerange. It can be specified (from least to most significant)
    # - given by the calling function (in availability instant reports)
    # - as a factory default (today)
    # - as a global option
    # - as a user setting via the sidebar snapin "reports"
    # - as a global option of this report
    if type(rangespec) == tuple and type(rangespec[0][0]) in (int, float):
        from_until, range_title = rangespec
    else:
        if rangespec == None:
            if "rangespec" in report:
                rangespec = report["rangespec"]
            else:
                rangespec = config.load_user_file("reporting_timerange",
                            config.reporting_rangespec)

        from_until, range_title = Timerange().compute_range(rangespec)

    instance["range"] = from_until
    instance["range_title"] = range_title
    # TODO: context specific macros
    instance["macros"] = create_report_macros(report, from_until, range_title)
    instance["report"] = report

    document = pdf.Document(
        font_family         = instance["options"]["font_family"],
        font_size           = instance["options"]["font_size"],
        lineheight          = instance["options"]["lineheight"],
        pagesize            = instance["options"]["pagesize"],
        margins             = instance["options"]["margins"],
        mirror_margins      = instance["options"]["mirror_margins"],
        pagebreak_function  = do_pagebreak,
        pagebreak_arguments = (report, instance),
    )

    # Store reference to the document in our current request.
    # This is better than a global variable. It makes us thread save
    html.pdf_document = document

    # Now render fixed and floating parts of the report
    render_fixels(instance, report.get("fixels", []), 1)
    render_report_elements(report, instance, report.get("elements", []))

    # In fixel preview mode we make shure that at least a couple
    # of pages are being rendered
    while document.page_number() < min_pages:
        document.do_pagebreak()

    filename = report.get("filename", config.reporting_filename)

    # PDF source is either being sent to browser or returned as string
    pdf_source = document.end(sendas=filename, do_send=do_send)
    return pdf_source


# Return a new report by combining one report with
# a list of further reports.
def use_report(report, used_report, max_depth = 10):
    if max_depth == 0:
        raise MKGeneralException(_("There is a cycle in the imports of the report %s.") % html.attrencode(report["title"]))

    new_report = {}
    new_report.update(report) # Create a new shallow copy

    # Override/set all globals from the imported report. Recursively
    # handle imports. Fixed elements are being pre-pended. Floating
    # elements are also being prepended.

    for key, value in used_report.items():
        if key not in [ "elements", "use", "fixels" ]:
            if type(value) == dict: # Merge an a per-key-basis
                new_dict = {}
                new_dict.update(value)
                new_dict.update(new_report.get(key, {}))
                new_report[key] = new_dict
            else:
                new_report.setdefault(key, value)

    # Fixed and liquid elements are prepended
    for listkey in [ "elements", "fixels" ]:
        new_report[listkey] = used_report.get(listkey, []) + new_report.get(listkey, [])

    # Recursively interpret "use"
    use = used_report.get("use")
    if use:
        new_report = use_report(new_report, get_report(use), max_depth - 1)

    return new_report


def replace_macros(instance, text):
    for varname, value in instance["macros"].items():
        text = text.replace("$" + varname + "$", value)
    return text


def create_report_macros(report, from_until, range_title):
    now = time.localtime(time.time())
    macros = {
        "REPORT_NAME"      : report["name"],
        "REPORT_TITLE"     : report["title"],
        "CHECK_MK_VERSION" : defaults.check_mk_version,
        "OMD_SITE"         : defaults.omd_site,
        "PAGE"             : "1",
        "RANGE_TITLE"      : range_title,
        "USER_ALIAS"       : config.user_alias,
        "USER_ID"          : config.user_id,
    }
    for name, ts in [
        ( "NOW",  time.time() ),
        ( "FROM", from_until[0], ),
        ( "UNTIL", from_until[1], ),
    ]:
        macros.update(timestamp_macros(name, ts,
           report.get("date_format", config.reporting_date_format),
           report.get("time_format", config.reporting_time_format)
        ))

    return macros


def timestamp_macros(name, ts, date_format, time_format):
    broken = time.localtime(ts)
    return {
        name + "_EPOCH" : str(int(ts)),
        name + "_DATE"  : time.strftime(date_format, broken).strip(),
        name + "_TIME"  : time.strftime(time_format, broken).strip(),
        name            : time.strftime(date_format + " " + time_format, broken).strip(),
    }


def do_pagebreak(report, instance):
    page_number = html.pdf_document.page_number()
    instance["macros"]["PAGE"] = str(page_number)
    render_fixels(instance, report.get("fixels", []), page_number)


# Returns True or False depending on whether a page number matches
# a page specifier like "first" or "odd".
def on_page(how, pageno):
    if how == "first":
        return pageno == 1
    elif how == "rest":
        return pageno != 1
    elif how == "odd":
        return pageno % 2 == 1
    elif how == "even":
        return pageno % 2 == 0
    else:
        return True # should be "all"


def render_fixels(instance, fixels, pageno):
    for fixel in fixels:
        if on_page(fixel["placement"]["pages"], pageno):
            render_fixel(instance, fixel)


def render_fixel(instance, fixel):
    fixel_type_name = fixel["type"]
    if fixel_type_name not in fixel_types:
        if config.debug:
            raise MKGeneralException(_("Invalid fixel element type '%s'") % fixel_type_name)
        return

    fixel_type = fixel_types[fixel_type_name]
    fixel_type["render"](fixel["placement"]["position"], fixel.get("parameters"), instance)


def render_report_elements(report, instance, elements):
    for element in elements:
        # Prepare filter-context for elements that need such
        element_type = element_types[element["type"]]
        if element_type.get("has_context"):
            context = visuals.get_context_from_uri_vars(single_infos=report["single_infos"])
            context.update(report["context"])
            context.update(element["context"])
            instance["context"] = context

            # Now check if all single infos from the element are known. Either they
            # are contained in the context, or they are defined in the element itself.
            have_error = False
            for info_key in element.get("single_infos", []):
                needed_variables = visuals.info_params(info_key)
                for varname in needed_variables:
                    if varname not in context:
                        info_name = visuals.infos[info_key]["title"]
                        render_report_error(_("Missing %s context") % info_name,
                                            _("The content element \"%s, %s\" cannot be "
                                              "rendered, because it is missing the following context information: "
                                              "\"%s\". You can either add this as a fixed setting to the element "
                                              "in the report, or call the report from a context where a value for "
                                              "\"%s\" is known.") %
                                              (element_type["title"], element_type["to_text"](element), info_name, info_name))
                        have_error = True
                        break

            if have_error:
                continue

        render_report_element(report, instance, element)


def render_report_error(title, message, wrap_message=True):
    html.pdf_document.add_hrule(margin=2, width=3, color=pdf.red)
    html.pdf_document.set_font_bold()
    html.pdf_document.add_text_line(title)
    html.pdf_document.set_font_bold(False)
    if not wrap_message:
        for line in message.strip().split('\n'):
            html.pdf_document.add_text_line(line)
    else:
        html.pdf_document.add_paragraph(message)
    html.pdf_document.add_hrule(margin=2, width=3, color=pdf.red)

def render_report_element(report, instance, element):
    element_type_name = element["type"]
    if element_type_name not in element_types:
        if config.debug:
            raise MKGeneralException(_("Invalid report element type '%s'") % element_type_name)
        return

    element_type = element_types[element_type_name]
    try:
        element_type["render"](element, instance)
    except Exception, e:
        if config.debug:
            raise
        render_report_error(_("Error while rendering element type '%s'") % element_type["title"],
                                                        format_exception(), wrap_message=False)


#.
#   .--Images--------------------------------------------------------------.
#   |                 ___                                                  |
#   |                |_ _|_ __ ___   __ _  __ _  ___  ___                  |
#   |                 | || '_ ` _ \ / _` |/ _` |/ _ \/ __|                 |
#   |                 | || | | | | | (_| | (_| |  __/\__ \                 |
#   |                |___|_| |_| |_|\__,_|\__, |\___||___/                 |
#   |                                     |___/                            |
#   +----------------------------------------------------------------------+
#   |  Some generic functions for handling images in reports               |
#   '----------------------------------------------------------------------'

supported_image_extensions = "png", "jpg", "jpeg"


def image_dirs():
    subdir = "/share/check_mk/reporting/images"
    for subpath in ("/local", ""):
        yield defaults.omd_root + subpath + subdir


def find_image(filename):
    for image_dir in image_dirs():
        if os.path.exists(image_dir) and filename in os.listdir(image_dir):
            return image_dir + "/" + filename


def list_image_names():
    filenames = []
    for image_dir in image_dirs():
        try:
            for filename in os.listdir(image_dir):
                if not filename.startswith(".") and \
                    filename.rsplit(".",1)[-1].lower() in supported_image_extensions:
                    filenames.append(filename)
        except:
            pass
    return filenames


def load_image_as_pil(filename):
    path = find_image(filename)
    if not path:
        raise MKGeneralException(_("Cannot find image \"%s\" in %s") %
                                 (filename, _(" or ").join(image_dirs())))

    image_bytes = file(path).read()
    extension = filename.rsplit(".", 1)[-1].lower()
    if extension == "png":
        return PngImagePlugin.PngImageFile(fp = StringIO.StringIO(image_bytes))
    elif extension in ("jpeg", "jpg"):
        return JpegImagePlugin.JpegImageFile(fp = StringIO.StringIO(image_bytes))
    else:
        raise MKGeneralException(_("Unsupported image format of \"%s\" (invalid file extension)") % filename)


def compute_image_size(size_param):
    width, height, resolution = None, None, None
    if size_param != "pagewidth":
        size_how, size_info = size_param
        if size_how == "resolution":
            resolution = size_info
        elif size_how == "width":
            width = size_info
        elif size_how == "height":
            height = size_info
        else:
            width, height = size_info
    return width, height, resolution


def ImageFilename(**args):
    args["choices"] = lambda: [ (n,n) for n in list_image_names() ]
    args["help"] = _("You can select either from the shipped images or upload your "
                     "own images. Copy your own images to the monitoring server into "
                     "the directory <tt>%s</tt>. Supported formats: %s") % (
                        list(image_dirs())[0], ", ".join(supported_image_extensions))
    return DropdownChoice(**args)


def ImageParameters(**args):
    args = args.copy()

    args["elements"] = [
        ( "filename",
          ImageFilename(
              title = _("Filename"),
        )),
        ( "size",
          CascadingDropdown(
              title = _("Size"),
              orientation = "horizontal",
              choices  = [
                  ( "pagewidth",
                    _("Scale to width of page"),
                  ),
                  ( "resolution",
                    _("Set resolution"),
                    Integer(
                        title = _("Set resolution"),
                        minvalue = 10,
                        maxvalue = 10000,
                        default_value = 150,
                        unit = _("DPI"),
                  )),
                  ( "width",
                    _("Set width"),
                    Length(
                        title = _("Set the width"),
                        help = _("The aspect of the image is preserved."),
                        default_value = 50,
                  )),
                  ( "height",
                    _("Set height"),
                    Length(
                        title = _("Set the height"),
                        help = _("The aspect of the image is preserved."),
                        default_value = 50,
                  )),
                  ( "size",
                    _("Image size"),
                    Tuple(
                        title = _("Set the absolute size"),
                        help = _("The original aspect of the image will be ignored."),
                        orientation = "horizontal",
                        elements = [
                            Length(
                                title = _("width"),
                                default_value = 50,
                            ),
                            Length(
                                title = _("height"),
                                default_value = 50,
                            ),
                        ]
                  )),
              ],
              default_value = ('resolution', 150),
        )),
    ]

    args["required_keys"] = [ "filename", "size", ]
    return Dictionary(**args)
