#!/usr/bin/python
# -*- encoding: utf-8; py-indent-offset: 4 -*-
# .------------------------------------------------------------------------.
# |                ____ _               _        __  __ _  __              |
# |               / ___| |__   ___  ___| | __   |  \/  | |/ /              |
# |              | |   | '_ \ / _ \/ __| |/ /   | |\/| | ' /               |
# |              | |___| | | |  __/ (__|   <    | |  | | . \               |
# |               \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\              |
# |                                        |_____|                         |
# |             _____       _                       _                      |
# |            | ____|_ __ | |_ ___ _ __ _ __  _ __(_)___  ___             |
# |            |  _| | '_ \| __/ _ \ '__| '_ \| '__| / __|/ _ \            |
# |            | |___| | | | ||  __/ |  | |_) | |  | \__ \  __/            |
# |            |_____|_| |_|\__\___|_|  | .__/|_|  |_|___/\___|            |
# |                                     |_|                                |
# |                     _____    _ _ _   _                                 |
# |                    | ____|__| (_) |_(_) ___  _ __                      |
# |                    |  _| / _` | | __| |/ _ \| '_ \                     |
# |                    | |__| (_| | | |_| | (_) | | | |                    |
# |                    |_____\__,_|_|\__|_|\___/|_| |_|                    |
# |                                                                        |
# | mathias-kettner.com                                 mathias-kettner.de |
# '------------------------------------------------------------------------'
#  This file is part of the Check_MK Enterprise Edition (CEE).
#  Copyright by Mathias Kettner and Mathias Kettner GmbH.  All rights reserved.
#
#  Distributed under the Check_MK Enterprise License.
#
#  You should have  received  a copy of the Check_MK Enterprise License
#  along with Check_MK. If not, email to mk@mathias-kettner.de
#  or write to the postal address provided at www.mathias-kettner.de

import metrics

try:
    import simplejson as json
except ImportError:
    import json

dashlet_types["custom_graph"] = dashlet_types["pnpgraph"].copy()
del dashlet_types["custom_graph"]["infos"]
del dashlet_types["custom_graph"]["single_infos"]

def dashlet_custom_graph_reload_js(nr, dashlet):
    custom_graph_id = dashlet['custom_graph']
    if not custom_graph_id:
        raise MKUserError("custom_graph", _("Missing needed custom_graph parameter."))

    metrics.CustomGraph.load()
    graph = metrics.CustomGraph.find_page(custom_graph_id)
    if not graph:
        raise MKUserError("custom_graph", _("The configured custom graph does not exist."))

    # New graphs which have been added via "add to visual" option don't have a timerange
    # configured. So we assume the default timerange here by default.
    timerange = dashlet.get('timerange', '1')

    graph_specification = ("custom", graph.name())
    graph_render_options = {
        "show_legend": dashlet.get("show_legend", False),
    }

    return "dashboard_render_graph(%d, %s, %s, '%s')" % \
            (nr, json.dumps(graph_specification), json.dumps(graph_render_options), timerange)


dashlet_types["custom_graph"].update({
    "title"       : _("Custom Graph"),
    "description" : _("Displays a custom graph designed with the graph designer."),
    "on_resize"   : dashlet_custom_graph_reload_js,
    "on_refresh"  : dashlet_custom_graph_reload_js,
    "parameters"  : [
        ("custom_graph", DropdownChoice(
            title = _('Custom Graph'),
            choices = lambda: metrics.CustomGraph.page_choices(),
            no_preselect = True,
        )),
        ("timerange", DropdownChoice(
            title = _('Timerange'),
            default_value = '1',
            choices= [
                ("0", _("4 Hours")),  ("1", _("25 Hours")),
                ("2", _("One Week")), ("3", _("One Month")),
                ("4", _("One Year")),
            ],
        )),
        ("show_legend", Checkbox(
            title = _("Show Legend"),
            label = _("Show the legend area of the graph"),
            default_value = False,
        )),
    ],
    "styles": """
.dashlet.custom_graph .dashlet_inner {
    background-color: #f8f4f0;
    color: #000;
    text-align: center;
}
.dashlet.custom_graph .graph {
    border: none;
    box-shadow: none;
}
.dashlet.custom_graph .container {
    background-color: #f8f4f0;
}
""",
})
