#!/usr/bin/python
# -*- encoding: utf-8; py-indent-offset: 4 -*-
# .------------------------------------------------------------------------.
# |                ____ _               _        __  __ _  __              |
# |               / ___| |__   ___  ___| | __   |  \/  | |/ /              |
# |              | |   | '_ \ / _ \/ __| |/ /   | |\/| | ' /               |
# |              | |___| | | |  __/ (__|   <    | |  | | . \               |
# |               \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\              |
# |                                        |_____|                         |
# |             _____       _                       _                      |
# |            | ____|_ __ | |_ ___ _ __ _ __  _ __(_)___  ___             |
# |            |  _| | '_ \| __/ _ \ '__| '_ \| '__| / __|/ _ \            |
# |            | |___| | | | ||  __/ |  | |_) | |  | \__ \  __/            |
# |            |_____|_| |_|\__\___|_|  | .__/|_|  |_|___/\___|            |
# |                                     |_|                                |
# |                     _____    _ _ _   _                                 |
# |                    | ____|__| (_) |_(_) ___  _ __                      |
# |                    |  _| / _` | | __| |/ _ \| '_ \                     |
# |                    | |__| (_| | | |_| | (_) | | | |                    |
# |                    |_____\__,_|_|\__|_|\___/|_| |_|                    |
# |                                                                        |
# | mathias-kettner.com                                 mathias-kettner.de |
# '------------------------------------------------------------------------'
#  This file is part of the Check_MK Enterprise Edition (CEE).
#  Copyright by Mathias Kettner and Mathias Kettner GmbH.  All rights reserved.
#
#  Distributed under the Check_MK Enterprise License.
#
#  You should have  received  a copy of the Check_MK Enterprise License
#  along with Check_MK. If not, email to mk@mathias-kettner.de
#  or write to the postal address provided at www.mathias-kettner.de


def render_fixel_hrule(position, parameters, instance):
    html.pdf_document.place_hrule(position, parameters.get("width", 0.01), color=parameters.get("color"))

fixel_types["hrule"] = {
   "title"     : _("Horizontal rule"),
   "position" : "vertical",
   "parameters" : Dictionary(
                    elements = [
                        ( "width", Length(title = _("Width"), precision = 2, default_value = 0.01) ),
                        ( "color", Color() ),
                    ]
		 ),
   "render"    : render_fixel_hrule,
}

def render_fixel_text(position, parameters, instance):
    html.pdf_document.save_state()
    if "font_size" in parameters:
        html.pdf_document.set_font_size(parameters["font_size"])
    if "color" in parameters:
        html.pdf_document.set_font_color(parameters["color"])
    if parameters.get("bold"):
        html.pdf_document.set_font_bold()
    html.pdf_document.place_text(position, replace_macros(instance, parameters["text"]))
    html.pdf_document.restore_state()


fixel_types["text"] = {
   "title"     : _("One line of text"),
   "position" : "free",
   "parameters" : Dictionary(
                     title = _("Text to place"),
                     elements = [
                        ( "text",
                           TextUnicode(
                            title = _("The text"),
                            help = text_macro_help(),
                            size = 64,
                        )),
                        ( "font_size", Fontsize() ),
                        ( "color",     Color() ),
                        ( "bold",      FixedValue(True, title = _("Bold"), totext = _("Use bold font")) ),
                     ],
                     required_keys = [ "text" ],
                 ),
   "render"    : render_fixel_text,
}

def render_fixel_pageframe(position, parameters, instance):
    html.pdf_document.place_page_box()

fixel_types["pageframe"] = {
   "title"     : _("A frame around the page"),
   "position" : "none",
   "render"    : render_fixel_pageframe,
}

#.
#   .--Image---------------------------------------------------------------.
#   |                   ___                                                |
#   |                  |_ _|_ __ ___   __ _  __ _  ___                     |
#   |                   | || '_ ` _ \ / _` |/ _` |/ _ \                    |
#   |                   | || | | | | | (_| | (_| |  __/                    |
#   |                  |___|_| |_| |_|\__,_|\__, |\___|                    |
#   |                                       |___/                          |
#   +----------------------------------------------------------------------+
#   |  An embedded PNG, JPG or similar image.                              |
#   '----------------------------------------------------------------------'


def render_image(position, parameters, instance):
    try:
        pil = load_image_as_pil(parameters["filename"])
    except MKGeneralException, e:
        render_report_error(_("Cannot render image"), "%s" % e)
        return

    width, height, resolution = compute_image_size(parameters["size"])
    html.pdf_document.place_pil_image(position, pil, width, height, resolution)


fixel_types["image"] = {
    "title"      : _("Embedded image"),
    "position"   : "free",
    "render"     : render_image,
    "to_text"    : lambda e: e["parameters"]["filename"],
    "parameters" : ImageParameters(),
}

