#!/usr/bin/python
# -*- encoding: utf-8; py-indent-offset: 4 -*-
# .------------------------------------------------------------------------.
# |                ____ _               _        __  __ _  __              |
# |               / ___| |__   ___  ___| | __   |  \/  | |/ /              |
# |              | |   | '_ \ / _ \/ __| |/ /   | |\/| | ' /               |
# |              | |___| | | |  __/ (__|   <    | |  | | . \               |
# |               \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\              |
# |                                        |_____|                         |
# |             _____       _                       _                      |
# |            | ____|_ __ | |_ ___ _ __ _ __  _ __(_)___  ___             |
# |            |  _| | '_ \| __/ _ \ '__| '_ \| '__| / __|/ _ \            |
# |            | |___| | | | ||  __/ |  | |_) | |  | \__ \  __/            |
# |            |_____|_| |_|\__\___|_|  | .__/|_|  |_|___/\___|            |
# |                                     |_|                                |
# |                     _____    _ _ _   _                                 |
# |                    | ____|__| (_) |_(_) ___  _ __                      |
# |                    |  _| / _` | | __| |/ _ \| '_ \                     |
# |                    | |__| (_| | | |_| | (_) | | | |                    |
# |                    |_____\__,_|_|\__|_|\___/|_| |_|                    |
# |                                                                        |
# | mathias-kettner.com                                 mathias-kettner.de |
# '------------------------------------------------------------------------'
#  This file is part of the Check_MK Enterprise Edition (CEE).
#  Copyright by Mathias Kettner and Mathias Kettner GmbH.  All rights reserved.
#
#  Distributed under the Check_MK Enterprise License.
#
#  You should have  received  a copy of the Check_MK Enterprise License
#  along with Check_MK. If not, email to mk@mathias-kettner.de
#  or write to the postal address provided at www.mathias-kettner.de

class PainterPrinterPerfometer:
    def __init__(self, row, time_range):
        perf_data_text = row["service_perf_data"]
        if perf_data_text:
            perf_data, check_command = metrics.parse_perf_data(perf_data_text, row["service_check_command"])
            self._translated_metrics = metrics.translate_metrics(perf_data, check_command)
        else:
            self._translated_metrics = None

    def minimal_width(self, pdfdoc):
        return 30 # mm

    def maximal_width(self, pdfdoc):
        return 30 # mm

    def can_add_dynamic_width(self):
        return True

    def height(self, pdfdoc):
        return pdfdoc.get_line_skip()

    def set_width(self, pdfdoc, width):
        pass

    # Render itself at left/top into direction right/down by width/height
    def render(self, pdfdoc, left, top, width, height, x_padding, y_padding, row_oddeven):
        perfometer_definitions = list(metrics.get_perfometers(self._translated_metrics))
        if perfometer_definitions:
            label, stack = metrics.build_perfometer(perfometer_definitions[0], self._translated_metrics)

            # Now paint the thing. Height might be more than our actual height
            paint_height = self.height(pdfdoc) + y_padding
            y_space = height - paint_height
            paint_top = top - y_space/2
            layer_top = paint_top
            for layer in stack:
                layer_height = paint_height / len(stack)
                segment_left = left
                for segment_perc, segment_color in layer:
                    # Make color lighter by ca. 20% - better for printing
                    segment_color_rgb = metrics.lighten_color(metrics.parse_color(segment_color), 0.2)
                    segment_width = width / 100.0 * segment_perc
                    pdfdoc.render_rect(segment_left, layer_top - layer_height, segment_width, layer_height, line_width=0.05, fill_color=segment_color_rgb)
                    segment_left += segment_width
                layer_top -= layer_height

            pdfdoc.save_state()
            pdfdoc.set_font_zoom(0.85)
            pdfdoc.render_aligned_text(left, paint_top-paint_height, width, paint_height, label, valign="center")
            pdfdoc.restore_state()


painter_printers["perfometer"] = PainterPrinterPerfometer
