#!/usr/bin/python
# -*- encoding: utf-8; py-indent-offset: 4 -*-
# .------------------------------------------------------------------------.
# |                ____ _               _        __  __ _  __              |
# |               / ___| |__   ___  ___| | __   |  \/  | |/ /              |
# |              | |   | '_ \ / _ \/ __| |/ /   | |\/| | ' /               |
# |              | |___| | | |  __/ (__|   <    | |  | | . \               |
# |               \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\              |
# |                                        |_____|                         |
# |             _____       _                       _                      |
# |            | ____|_ __ | |_ ___ _ __ _ __  _ __(_)___  ___             |
# |            |  _| | '_ \| __/ _ \ '__| '_ \| '__| / __|/ _ \            |
# |            | |___| | | | ||  __/ |  | |_) | |  | \__ \  __/            |
# |            |_____|_| |_|\__\___|_|  | .__/|_|  |_|___/\___|            |
# |                                     |_|                                |
# |                     _____    _ _ _   _                                 |
# |                    | ____|__| (_) |_(_) ___  _ __                      |
# |                    |  _| / _` | | __| |/ _ \| '_ \                     |
# |                    | |__| (_| | | |_| | (_) | | | |                    |
# |                    |_____\__,_|_|\__|_|\___/|_| |_|                    |
# |                                                                        |
# | mathias-kettner.com                                 mathias-kettner.de |
# '------------------------------------------------------------------------'
#  This file is part of the Check_MK Enterprise Edition (CEE).
#  Copyright by Mathias Kettner and Mathias Kettner GmbH.  All rights reserved.
#
#  Distributed under the Check_MK Enterprise License.
#
#  You should have  received  a copy of the Check_MK Enterprise License
#  along with Check_MK. If not, email to mk@mathias-kettner.de
#  or write to the postal address provided at www.mathias-kettner.de


#   .--Common Code---------------------------------------------------------.
#   |  ____                                         ____          _        |
#   | / ___|___  _ __ ___  _ __ ___   ___  _ __    / ___|___   __| | ___   |
#   || |   / _ \| '_ ` _ \| '_ ` _ \ / _ \| '_ \  | |   / _ \ / _` |/ _ \  |
#   || |__| (_) | | | | | | | | | | | (_) | | | | | |__| (_) | (_| |  __/  |
#   | \____\___/|_| |_| |_|_| |_| |_|\___/|_| |_|  \____\___/ \__,_|\___|  |
#   |                                                                      |
#   +----------------------------------------------------------------------+
#   | Code used by all types of graphs                                     |
#   '----------------------------------------------------------------------'

graph_offcut_width = 20.0 # total width - this = width of canvas in mm
mm_per_step = 0.5         # approx. one datapoint per 0.5 mm

def get_mm_per_ex(font_size):
    return font_size / 3.0


def mm_per_ex_by_render_options(graph_render_options):
    return get_mm_per_ex(graph_render_options["font_size"])


# Estimate step. It is depended on width of the graph in mm.
def compute_pdf_graph_data_range(width, start_time, end_time):
    available_width = width - graph_offcut_width
    number_of_steps = available_width / mm_per_step
    step = float(end_time - start_time) / number_of_steps / 2.0
    return {
        "time_range" : (start_time, end_time),
        "step"       : step,
    }


def render_element_graph(element, instance, graph_specification):
    parameters = element["parameters"]

    # TODO: For new graphs do not use number of source, but introduce
    # a new unique key per graph template. For the while we simply
    # enumerate the possible templates of a service.

    # Distill options from global and per-element parameters
    graph_layout = {}
    graph_layout.update(instance["options"]["graph_layout"])
    graph_layout.update(parameters.get("graph_layout", {}))

    try:
        start_time, end_time = instance["range"]
        margin_top, margin_right, margin_bottom, margin_left = instance["options"]["margins"]
        width = parameters.get("width", instance["options"]["pagesize"][0] - margin_left - margin_right)
        height = parameters.get("height", 60.0) # Default to height of 60 mm for actual graph

        graph_definitions = metrics.create_graph_definitions_from_specification(graph_specification)
        graph_data_range = compute_pdf_graph_data_range(width, start_time, end_time)

        graph_render_options = metrics.add_default_render_options(graph_layout.copy())
        mm_per_ex = get_mm_per_ex(graph_render_options["font_size"])
        graph_render_options["size"] = (width / mm_per_ex, height / mm_per_ex)

        for graph_definition in graph_definitions:
            abstract_graph = metrics.compute_abstract_graph(graph_definition, graph_data_range, graph_render_options)
            abstract_graph["title"] = reformat_graph_title(abstract_graph["title"], graph_layout, graph_specification)
            render_graph_pdf(abstract_graph, graph_data_range, graph_render_options)

    except Exception, e:
        if config.debug:
            raise
        onfail = parameters.get("onfail", "message")
        if onfail == "ignore":
            return
        elif onfail == "message":
            html.pdf_document.add_paragraph(_("No graph available: %s") % e)
        else:
            raise

def reformat_graph_title(plain_title, graph_layout, graph_specification):
    title_format = graph_layout.get("title_format", "plain")
    if title_format == "plain" or graph_specification[0] != "template":
        return plain_title

    host_name = graph_specification[1]["host_name"]
    if title_format == "add_host_name":
        return host_name + " - " + plain_title
    elif title_format == "add_host_alias":
        site = graph_specification[1]["site"]
        alias = get_alias_of_host(site, host_name)
        return alias + " - " + plain_title
    else:
        raise MKGeneralException("Invalid title_format '%s' for graph" % title_format)


def get_alias_of_host(site, host_name):
    html.live.set_only_sites([site])
    query = "GET hosts\nCache: reload\nColumns: alias\nFilter: name = " + lqencode(host_name)
    try:
        alias = html.live.query_value(query)
    except Exception, e:
        html.log("Could not determine alias of host %s on site %s: %s" % (host_name, site, e))
        if config.debug:
            raise
        alias = host_name

    html.live.set_only_sites(None)
    return alias


def render_graph_pdf(abstract_graph, graph_data_range, graph_render_options,
                     pos_left=None, pos_top=None, total_width=None, total_height=None):

    start_time, end_time = graph_data_range["time_range"]

    if pos_left == None: # floating element
        html.pdf_document.margin(2.5)

    # Styling for PDF graphs. Note: We could make some of these
    # configurable
    font_size             = graph_render_options["font_size"]
    mm_per_ex             = mm_per_ex_by_render_options(graph_render_options)
    v_label_margin        = 1.0 # mm
    t_label_margin        = 1.0 # mm
    left_border           = 13.0 # mm
    bottom_margin         = 4.0 # mm
    top_margin            = 1.0 # mm
    right_border          = 2.5 # mm
    axis_color            = metrics.parse_color("#000000")
    zero_rule_color       = metrics.parse_color("#000000")
    canvas_color          = graph_render_options["canvas_color"]
    background_color      = graph_render_options["background_color"]
    axis_over_width       = 0.5 # mm
    color_gradient        = graph_render_options["color_gradient"] / 100.0
    curve_line_width      = 0.1 # mm
    rule_line_width       = 0.1 # mm
    label_line_width      = 0.04 # mm
    v_line_color          = map(metrics.parse_color, [ "#000000", "#000000", "#000000" ])
    v_line_dash           = [ None, [0.2, 0.4], None ]
    t_line_color          = map(metrics.parse_color, [ "#000000", "#808080", "#000000" ])
    t_line_dash           = [ None, [0.2, 0.2], None ]
    legend_box_line_width = 0.1

    html.pdf_document.save_state()
    html.pdf_document.set_font_size(font_size)

    legend_box_size = mm_per_ex

    title_height = graph_title_height(graph_render_options)
    legend_height = graph_legend_height(abstract_graph, graph_render_options)

    if pos_left != None:
        # Absolute placement of graph
        height = total_height - title_height - legend_height
        width = total_width

    else:
        # Place graph in page flow
        width_ex, height_ex = graph_render_options["size"]
        width = width_ex * mm_per_ex
        height = height_ex * mm_per_ex

    left, top, width, total_height = html.pdf_document.add_canvas(width, height + title_height + legend_height,
              border_width = graph_render_options["border_width"], left_mm=pos_left)

    # From here width, height, total_height, left and top are in "mm".

    right = left + width - right_border
    total_bottom = top - total_height
    bottom = top - height - title_height

    # Fill canvas with background color
    html.pdf_document.render_rect(left, total_bottom, width, total_height, fill_color=background_color)

    # Title
    if graph_render_options["show_title"]:
        html.pdf_document.render_aligned_text(left, top - title_height, width, title_height,
                                              abstract_graph["title"], align="center", bold=True)

    # The following code is inspired by htdocs/js/graphs.js:render_graph(). Whenever
    # you change something there, the change should also be reflected here!

    bottom_border = mm_per_ex + t_label_margin + bottom_margin

    # Prepare position and translation of origin
    t_range_from = abstract_graph["time_axis"]["range"][0]
    t_range_to = abstract_graph["time_axis"]["range"][1]
    t_range = t_range_to - t_range_from
    t_mm = width - left_border - right_border
    t_mm_per_second = t_mm / t_range

    v_range_from = abstract_graph["vertical_axis"]["range"][0]
    v_range_to = abstract_graph["vertical_axis"]["range"][1]
    v_range = v_range_to - v_range_from
    v_mm = height - top_margin - bottom_border
    v_mm_per_unit = v_mm / v_range

    t_orig = left + left_border
    v_orig = bottom + bottom_border
    v_axis_orig = v_range_from

    # paint graph background
    html.pdf_document.render_rect(t_orig, v_orig, t_mm, v_mm, fill_color=canvas_color)

    # Now transform the whole chooridate system to our real t and v choords
    # so if we paint something at (0, 0) it will correctly represent a
    # value of 0 and a time point of time_start.
    trans_t = lambda t: (t - t_range_from) * t_mm_per_second + t_orig
    trans_v = lambda v: v_orig + ((v - v_axis_orig) * v_mm_per_unit)
    trans   = lambda t, v: (trans_t(t), trans_v(v))

    # Paint curves
    html.pdf_document.save_state()
    html.pdf_document.add_clip_rect(t_orig, v_orig, t_mm, v_mm)
    step = abstract_graph["step"] / 2.0;
    for curve in abstract_graph["curves"]:
        if curve.get("dont_paint"):
            continue

        t = abstract_graph["start_time"]
        points = curve["points"]
        color = metrics.parse_color(curve["color"])

        if curve["type"] == "area":
            prev_lower = None
            prev_upper = None

            gradient = (t_orig, v_orig, t_orig, v_orig + v_mm,
                        (metrics.darken_color(color, color_gradient),
                         color,
                         metrics.lighten_color(color, color_gradient)),
                        (0.0, 0.5, 1.0))

            for lower, upper in points:
                if lower != None and upper != None and prev_lower != None and prev_upper != None:
                    html.pdf_document.begin_path()
                    html.pdf_document.move_to(trans_t(t - step) - 0.01, trans_v(prev_lower))
                    html.pdf_document.line_to(trans_t(t - step) - 0.01, trans_v(prev_upper))
                    html.pdf_document.line_to(trans_t(t),        trans_v(upper))
                    html.pdf_document.line_to(trans_t(t),        trans_v(lower))
                    html.pdf_document.line_to(trans_t(t - step) - 0.01, trans_v(prev_lower))
                    html.pdf_document.close_path()
                    html.pdf_document.fill_path(color, gradient=gradient)

                prev_lower = lower
                prev_upper = upper
                t += step

        else: # "line"
            last_value = None
            html.pdf_document.begin_path()
            for value in points:
                if value != None:
                    p = trans(t, value)
                    if last_value != None:
                        html.pdf_document.line_to(p[0], p[1])
                    else:
                        html.pdf_document.move_to(p[0], p[1])
                last_value = value
                t += step
            html.pdf_document.stroke_path(color = color, width = curve_line_width)

    html.pdf_document.restore_state() # Remove clipping

    # Now we use these four dimensions for drawing into the canvas using render_...
    # functions from pdf. Note: top > bottom.

    # Clear areas where values have been painted out of range. This is
    # At top and bottom
    html.pdf_document.render_rect(t_orig, v_orig + v_mm, t_mm, top_margin, fill_color=background_color)
    html.pdf_document.render_rect(t_orig, bottom, t_mm, v_orig - bottom, fill_color=background_color)

    # Paint axes and a strong line at 0, if that is in the range
    html.pdf_document.render_line(t_orig, v_orig - axis_over_width, t_orig, v_orig + v_mm, color = axis_color)
    html.pdf_document.render_line(t_orig - axis_over_width, v_orig, right, v_orig, color = axis_color)
    if v_range_from <= 0 and v_range_to >= 0:
        html.pdf_document.render_line(t_orig, trans_v(0), right, trans_v(0), color = zero_rule_color)


    # Render optional vertical axis label
    vertical_axis_label = abstract_graph["vertical_axis"]["axis_label"]
    if vertical_axis_label:
        html.pdf_document.render_aligned_text(left, top - title_height, left_border, title_height,
                                              vertical_axis_label, align="center", valign="middle")

    # Paint the vertical axis
    for position, label, line_width in abstract_graph["vertical_axis"]["labels"]:
        if line_width > 0:
            html.pdf_document.render_line(t_orig, trans_v(position),
                    right, trans_v(position), width = label_line_width,
                    color = v_line_color[line_width], dashes = v_line_dash[line_width])

        if label:
            html.pdf_document.render_aligned_text(t_orig - v_label_margin - left_border, trans_v(position),
                                                  left_border, mm_per_ex, label, align="right", valign="middle")

    # Paint time axis
    for position, label, line_width in abstract_graph["time_axis"]["labels"]:
        t_pos_mm = trans_t(position)
        if line_width > 0:
            html.pdf_document.render_line(t_pos_mm, v_orig, t_pos_mm, trans_v(v_range_to),
                    width = label_line_width, color = t_line_color[line_width],
                    dashes = t_line_dash[line_width])
        if label:
            html.pdf_document.render_aligned_text(t_pos_mm, v_orig - t_label_margin - mm_per_ex, 0, mm_per_ex, label, align="center")

    # Paint horizontal rules like warn and crit
    rules = abstract_graph["horizontal_rules"]
    for position, label, color, title in rules:
        if position >= v_range_from and position <= v_range_to:
            html.pdf_document.render_line(t_orig, trans_v(position), right, trans_v(position),
                               width = rule_line_width, color = metrics.parse_color(color))

    # Paint legend
    if graph_render_options["show_legend"]:
        legend_lineskip = get_graph_legend_lineskip(graph_render_options)
        legend_top = bottom - legend_lineskip + bottom_margin / 2.0
        legend_column_width = (width - left_border - right_border) / 8

        def paint_legend_line(color, texts):
            l = t_orig
            if color:
                html.pdf_document.render_rect(l, legend_top + mm_per_ex * 0.2, legend_box_size, legend_box_size,
                        fill_color = color, line_width=legend_box_line_width)
            for nr, text in enumerate(texts):
                if text:
                    html.pdf_document.render_aligned_text(l + (color and nr == 0 and legend_box_size + 0.8 or 0), legend_top,
                                                          legend_column_width, legend_lineskip, text, align = nr == 0 and "left" or "right")
                if nr == 0:
                    l += legend_column_width * 3
                else:
                    l += legend_column_width

        scalars = [
            ( "first",   _("First")),
            ( "min",     _("Minimum")),
            ( "max",     _("Maximum")),
            ( "average", _("Average")),
            ( "last",    _("Last")),
        ]

        paint_legend_line(None, [None] + [x[1] for x in scalars])
        html.pdf_document.render_line(t_orig, legend_top, t_orig + t_mm, legend_top)

        for curve in abstract_graph["curves"]:
            legend_top -= legend_lineskip
            texts = [ curve["title"] ]
            for scalar, title in scalars:
                texts.append(curve["scalars"][scalar][1])
            paint_legend_line(metrics.parse_color(curve["color"]), texts)

        if abstract_graph["horizontal_rules"]:
            html.pdf_document.render_line(t_orig, legend_top, t_orig + t_mm, legend_top)
            for value, readable, color, title in abstract_graph["horizontal_rules"]:
                legend_top -= legend_lineskip
                paint_legend_line(metrics.parse_color(color), [title] + [None] * 4 + [readable])

    html.pdf_document.restore_state()
    if left == None: # floating element
        html.pdf_document.margin(2.5)


def get_graph_legend_lineskip(graph_render_options):
    return mm_per_ex_by_render_options(graph_render_options) * 1.5


def graph_title_height(graph_render_options):
    mm_per_ex = mm_per_ex_by_render_options(graph_render_options)
    return graph_render_options["show_title"] and (mm_per_ex * 2) or 0


def graph_legend_height(abstract_graph, graph_render_options):
    if graph_render_options["show_legend"]:
        legend_lineskip = get_graph_legend_lineskip(graph_render_options)

        num_painted_curves = 0
        for curve in abstract_graph["curves"]:
            if not curve.get("dont_paint"):
                num_painted_curves += 1

        return legend_lineskip * (1 + num_painted_curves + len(abstract_graph["horizontal_rules"]))
    else:
        return 0


def clone_pnpgraph_from_gui(parameters, context):
    # parameters hold the graph specification
    graph_specification = parameters
    if graph_specification[0] == "template":
        return clone_template_graph_from_gui(graph_specification[1])
    elif graph_specification[0] == "custom":
        return clone_custom_graph_from_gui(graph_specification[1])
    else:
        raise MKGeneralException(_("Invalid graph type '%s'") % parameters[0])


generic_graph_element_parameters = [
    ( "onfail",
      DropdownChoice(
          title = _("In case of an error"),
          help = _("Specify what we shall do if the selected graph is missing or cannot be retrieved."),
          choices = [
             ( "error",
               _("Render an error message into the document")),
             ( "message",
               _("Display a simple message that the graph is missing")),
             ( "ignore",
               _("Silently ignore the problem, skip this graph")),
          ]
    )),
    ( "graph_layout",
      GraphLayout(),
    ),
]


#.
#   .--Host/Service Graphs (Template graphs)-------------------------------.
#   |      _   _           _      ______                  _                |
#   |     | | | | ___  ___| |_   / / ___|  ___ _ ____   _(_) ___ ___       |
#   |     | |_| |/ _ \/ __| __| / /\___ \ / _ \ '__\ \ / / |/ __/ _ \      |
#   |     |  _  | (_) \__ \ |_ / /  ___) |  __/ |   \ V /| | (_|  __/      |
#   |     |_| |_|\___/|___/\__/_/  |____/ \___|_|    \_/ |_|\___\___|      |
#   |                                                                      |
#   |                    ____                 _                            |
#   |                   / ___|_ __ __ _ _ __ | |__  ___                    |
#   |                  | |  _| '__/ _` | '_ \| '_ \/ __|                   |
#   |                  | |_| | | | (_| | |_) | | | \__ \                   |
#   |                   \____|_|  \__,_| .__/|_| |_|___/                   |
#   |                                  |_|                                 |
#   +----------------------------------------------------------------------+
#   |  Graphs of the type "template" (normal host/service graphs)          |
#   '----------------------------------------------------------------------'

def render_element_template_graph(element, instance):
    # We are of type single_info [ "host", "service" ]. So we have
    # the *right* to assume that one host and one service is contained
    # in our context.
    context = instance["context"]
    host_name = context["host"]
    service_description = context["service"]

    parameters = element["parameters"]
    if parameters.get("source") != None:
        graph_index = parameters["source"] - 1
    else:
        graph_index = None # meaning: all graphs of this host/service

    # TODO: site hint should be available here. But isn't as it
    # seems. at least not in all cases
    try:
        row  = metrics.get_graph_data_from_livestatus(context.get("site"), host_name, service_description)
        site = row["site"]

        graph_identification = (
            "template", {
                "site"                : site,
                "host_name"           : host_name,
                "service_description" : service_description,
                "graph_index"         : graph_index,
        })
        render_element_graph(element, instance, graph_identification)

    except livestatus.MKLivestatusNotFoundError, e:
        if config.debug:
            raise
        onfail = parameters.get("onfail", "message")
        if onfail == "ignore":
            return
        elif onfail == "message":
            html.pdf_document.add_paragraph(_("No graph available: %s") % e)
        else:
            raise



def clone_template_graph_from_gui(graph_info):
    # Example for graph_info:
    # {'service_description': 'CPU load', 'site': 'mysite', 'graph_index': 0, 'host_name': 'server123'}
    single_infos = [ "host" ]
    context = { "host" : graph_info["host_name"] }

    if graph_info.get("service_description") != "_HOST_":
        single_infos.append("service")
        datasource = "services"
        context["service"] = graph_info["service_description"]
    else:
        datasource = "hosts"

    new_element = {
        "type" : "pnpgraph",
        "parameters" : {
            "source" : graph_info["graph_index"] + 1,
        },
        "context" : context,
        "single_infos" : single_infos,
        "datasource" : datasource,
    }
    return new_element


def template_graph_to_text(element):
    source = element["parameters"].get("source")
    if source != None:
        return _("graph number %d") % source
    else:
        return _("all graphs")



element_types["pnpgraph"] = {
    "title"        : _("Metrics graph of a single service"),
    "single_infos" : [ "host", "service" ],
    "has_context"  : True,
    "render"       : render_element_template_graph,
    "to_text"      : template_graph_to_text,
    "parameters"   :
        Dictionary(
            elements = [
                # TODO: Where can we set the width and height, which are being tested
                # in the parameters in render_element_template_graph?
                ( "source",
                  Integer(
                      title = _("Number of the graph template"),
                      minvalue = 1,
                )),
            ] + generic_graph_element_parameters,
            optional_keys = [ "source" ],
        ),
    "clone_from_gui" : clone_pnpgraph_from_gui,
}


#.
#   .--Custom Graphs-------------------------------------------------------.
#   |                   ____          _                                    |
#   |                  / ___|   _ ___| |_ ___  _ __ ___                    |
#   |                 | |  | | | / __| __/ _ \| '_ ` _ \                   |
#   |                 | |__| |_| \__ \ || (_) | | | | | |                  |
#   |                  \____\__,_|___/\__\___/|_| |_| |_|                  |
#   |                                                                      |
#   |                    ____                 _                            |
#   |                   / ___|_ __ __ _ _ __ | |__  ___                    |
#   |                  | |  _| '__/ _` | '_ \| '_ \/ __|                   |
#   |                  | |_| | | | (_| | |_) | | | \__ \                   |
#   |                   \____|_|  \__,_| .__/|_| |_|___/                   |
#   |                                  |_|                                 |
#   +----------------------------------------------------------------------+
#   |  Code for custom graphs (type "custom")                              |
#   '----------------------------------------------------------------------'


def render_element_custom_graph(element, instance):
    graph_specification = ( "custom", element["parameters"]["name"] )
    render_element_graph(element, instance, graph_specification)


def clone_custom_graph_from_gui(custom_graph_name):
    return {
        "type" : "custom_graph",
        "parameters" : {
            "name" : custom_graph_name,
        },
        "context" : {},
        "single_infos" : [],
        "datasource" : None,
    }


def custom_graph_to_text(element):
    graph_name = element["parameters"]["name"]
    metrics.CustomGraph.load()
    custom_graph = metrics.CustomGraph.find_page(graph_name)
    if custom_graph:
        return custom_graph.title()
    else:
        return _("Undefined custom graph '%s'") % html.attrencode(graph_name)


element_types["custom_graph"] = {
    "title"        : _("Custom Graph"),
    "single_infos" : [],
    "has_context"  : False,
    "render"       : render_element_custom_graph,
    "to_text"      : custom_graph_to_text,
    "parameters"   :
        Dictionary(
            elements = [
                ( "name",
                  DropdownChoice(
                      title = _("Custom Graph"),
                      choices = lambda: metrics.CustomGraph.page_choices(),
                )),
            ] + generic_graph_element_parameters,
            optional_keys = None,
        ),
}

#.
#   .--Painter Printer-----------------------------------------------------.
#   |  ____       _       _              ____       _       _              |
#   | |  _ \ __ _(_)_ __ | |_ ___ _ __  |  _ \ _ __(_)_ __ | |_ ___ _ __   |
#   | | |_) / _` | | '_ \| __/ _ \ '__| | |_) | '__| | '_ \| __/ _ \ '__|  |
#   | |  __/ (_| | | | | | ||  __/ |    |  __/| |  | | | | | ||  __/ |     |
#   | |_|   \__,_|_|_| |_|\__\___|_|    |_|   |_|  |_|_| |_|\__\___|_|     |
#   |                                                                      |
#   +----------------------------------------------------------------------+
#   |  Graphs that are embedded in a view (as a column painter)            |
#   '----------------------------------------------------------------------'

class PainterPrinterTimeGraph:
    def __init__(self, row, time_range):
        graph_specification = (
            "template", {
                "site"                : row["site"],
                "host_name"           : row["host_name"],
                "service_description" : row.get("service_description", "_HOST_"),
        })
        self._graph_definitions = metrics.create_graph_definitions_from_specification(graph_specification)
        self._height = 0
        self._time_range = time_range
        assumed_width = 100

        self._graph_data_range = compute_pdf_graph_data_range(assumed_width, self._time_range[0], self._time_range[1])

        self._graph_render_options = metrics.add_default_render_options(config.reporting_graph_layout.copy())
        font_size = self._graph_render_options["font_size"]
        mm_per_ex = get_mm_per_ex(font_size)
        height_ex = 50 / mm_per_ex
        width_ex = assumed_width / mm_per_ex
        self._graph_render_options["size"] = (width_ex, height_ex)

        self._graphs = []
        top_offset_mm = 0
        for graph_definition in self._graph_definitions:
            abstract_graph = metrics.compute_abstract_graph(graph_definition, self._graph_data_range, self._graph_render_options)
            total_height_ex = height_ex + 3 + len(abstract_graph["curves"]) + len(abstract_graph["horizontal_rules"])
            height = total_height_ex * mm_per_ex
            top_offset_mm += height
            self._graphs.append((abstract_graph, height, top_offset_mm))
        self._height = top_offset_mm

    def minimal_width(self, pdfdoc):
        return 80 # mm

    def maximal_width(self, pdfdoc):
        return 120 # mm

    def can_add_dynamic_width(self):
        return True

    def height(self, pdfdoc):
        return self._height

    def set_width(self, pdfdoc, width):
        pass

    # Render itself at left/top into direction right/down by width/height
    def render(self, pdfdoc, left, top, width, height, x_padding, y_padding, row_oddeven):
        for abstract_graph, graph_height, top_offset_mm in self._graphs:
            render_graph_pdf(abstract_graph, self._graph_data_range, self._graph_render_options,
                         pos_left=left, pos_top=top, total_width=width, total_height=graph_height)
            top -= graph_height
            if top < graph_height:
                pdfdoc.do_pagebreak()
                top = pdfdoc._top / pdf.mm


painter_printers["time_graph"] = PainterPrinterTimeGraph
