#!/usr/bin/python
# -*- encoding: utf-8; py-indent-offset: 4 -*-
# .------------------------------------------------------------------------.
# |                ____ _               _        __  __ _  __              |
# |               / ___| |__   ___  ___| | __   |  \/  | |/ /              |
# |              | |   | '_ \ / _ \/ __| |/ /   | |\/| | ' /               |
# |              | |___| | | |  __/ (__|   <    | |  | | . \               |
# |               \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\              |
# |                                        |_____|                         |
# |             _____       _                       _                      |
# |            | ____|_ __ | |_ ___ _ __ _ __  _ __(_)___  ___             |
# |            |  _| | '_ \| __/ _ \ '__| '_ \| '__| / __|/ _ \            |
# |            | |___| | | | ||  __/ |  | |_) | |  | \__ \  __/            |
# |            |_____|_| |_|\__\___|_|  | .__/|_|  |_|___/\___|            |
# |                                     |_|                                |
# |                     _____    _ _ _   _                                 |
# |                    | ____|__| (_) |_(_) ___  _ __                      |
# |                    |  _| / _` | | __| |/ _ \| '_ \                     |
# |                    | |__| (_| | | |_| | (_) | | | |                    |
# |                    |_____\__,_|_|\__|_|\___/|_| |_|                    |
# |                                                                        |
# | mathias-kettner.com                                 mathias-kettner.de |
# '------------------------------------------------------------------------'
#  This file is part of the Check_MK Enterprise Edition (CEE).
#  Copyright by Mathias Kettner and Mathias Kettner GmbH.  All rights reserved.
#
#  Distributed under the Check_MK Enterprise License.
#
#  You should have  received  a copy of the Check_MK Enterprise License
#  along with Check_MK. If not, email to mk@mathias-kettner.de
#  or write to the postal address provided at www.mathias-kettner.de

#   .--Heading-------------------------------------------------------------.
#   |                _   _                _ _                              |
#   |               | | | | ___  __ _  __| (_)_ __   __ _                  |
#   |               | |_| |/ _ \/ _` |/ _` | | '_ \ / _` |                 |
#   |               |  _  |  __/ (_| | (_| | | | | | (_| |                 |
#   |               |_| |_|\___|\__,_|\__,_|_|_| |_|\__, |                 |
#   |                                               |___/                  |
#   +----------------------------------------------------------------------+
#   |  Headings in various levels                                          |
#   '----------------------------------------------------------------------'

def render_element_heading(element, instance):
    parameters = element["parameters"]
    text = replace_macros(instance, parameters["text"])
    level = parameters["level"]
    if level:
        html.pdf_document.add_heading(level, text, numbers=True)
    else:
        html.pdf_document.save_state()
        html.pdf_document.set_font_bold()
        html.pdf_document.set_font_zoom(2.0)
        html.pdf_document.add_text_line(text)
        html.pdf_document.add_margin(5)
        html.pdf_document.restore_state()


element_types["heading"] = {
    "title"      : _("Heading"),
    "render"     : render_element_heading,
    "parameters"  :
        Dictionary(
            elements = [
                ( "level",
                  Integer(
                      title = _("Level (0...9)"),
                      minvalue = 0,
                      maxvalue = 9,
                )),
                ( "text",
                   TextUnicode(
                       help = text_macro_help(),
                       title = _("The text"),
                       size = 64,
                       allow_empty = False,
                )),
            ],
            required_keys = [ "level", "text" ],
        ),
    "to_text" : lambda element: element["parameters"]["text"],
}


#.
#   .--Paragraph-----------------------------------------------------------.
#   |          ____                                       _                |
#   |         |  _ \ __ _ _ __ __ _  __ _ _ __ __ _ _ __ | |__             |
#   |         | |_) / _` | '__/ _` |/ _` | '__/ _` | '_ \| '_ \            |
#   |         |  __/ (_| | | | (_| | (_| | | | (_| | |_) | | | |           |
#   |         |_|   \__,_|_|  \__,_|\__, |_|  \__,_| .__/|_| |_|           |
#   |                               |___/          |_|                     |
#   +----------------------------------------------------------------------+
#   |  A floating, auto-wrapping paragraph of text.                        |
#   '----------------------------------------------------------------------'

def render_element_paragraph(element, instance):
    parameters = element["parameters"]
    text = replace_macros(instance, parameters["text"])
    paragraphs = text.split("\n\n")
    for paragraph in paragraphs:
        html.pdf_document.add_paragraph(paragraph)

element_types["paragraph"] = {
    "title"      : _("Paragraph of text"),
    "render"     : render_element_paragraph,
    "parameters"  :
        Dictionary(
            title = _("Text"),
            elements = [
                ( "text",
                   TextAreaUnicode(
                       title = _("The text"),
                       help = text_macro_help() + \
                            _(" The text will automatically be wrapped. An empty line will "
                              "start a new paragraph."),

                )),
            ],
            required_keys = [ "text" ],
        ),
    "to_text" : lambda element: element["parameters"]["text"][:100] + "...",
}


#.
#   .--Paragraph via URL---------------------------------------------------.
#   |  ____                                       _             _          |
#   | |  _ \ __ _ _ __ __ _  __ _ _ __ __ _ _ __ | |__   __   _(_) __ _    |
#   | | |_) / _` | '__/ _` |/ _` | '__/ _` | '_ \| '_ \  \ \ / / |/ _` |   |
#   | |  __/ (_| | | | (_| | (_| | | | (_| | |_) | | | |  \ V /| | (_| |   |
#   | |_|   \__,_|_|  \__,_|\__, |_|  \__,_| .__/|_| |_|   \_/ |_|\__,_|   |
#   |                       |___/          |_|                             |
#   |                          _   _ ____  _                               |
#   |                         | | | |  _ \| |                              |
#   |                         | | | | |_) | |                              |
#   |                         | |_| |  _ <| |___                           |
#   |                          \___/|_| \_\_____|                          |
#   |                                                                      |
#   +----------------------------------------------------------------------+
#   | Some text that is fetched via an URL.                                |
#   '----------------------------------------------------------------------'

def render_element_paragraph_via_url(element, instance):
    import urllib2
    parameters = element["parameters"]
    url = replace_macros(instance, parameters["url"])
    try:
        raw_text = urllib2.urlopen(url).read()
    except urllib2.URLError, e:
        render_report_error(_("Cannot fetch URL '%s'") % url, "%s" % e)
        return

    if parameters["replace_macros"]:
        text = replace_macros(instance, raw_text)
    else:
        text = raw_text

    paragraphs = text.split("\n\n")
    for paragraph in paragraphs:
        html.pdf_document.add_paragraph(paragraph)


element_types["paragraph_via_url"] = {
    "title"      : _("Paragraph of text fetched via HTTP(s)"),
    "render"     : render_element_paragraph_via_url,
    "parameters"  :
        Dictionary(
            elements = [
                ( "url",
                  HTTPUrl(
                      title = _("URL to fetch text from"),
                      help = _("Report macros such as <tt>$HOSTNAME$</tt> will be replaced if found in the URL. "
                               "That way you can create dynamic URLs."),
                )),
                ( "replace_macros",
                  Checkbox(
                      title = _("Macro Replacement"),
                      label = _("Replace macros in fetched text body"),
                )),
            ],
            required_keys = [ "url", "replace_macros"],
        ),
    "to_text" : lambda element:
             _("Text from URL %s") % html.attrencode(element["parameters"]["url"]),
}


#.
#   .--Macro-Help----------------------------------------------------------.
#   |        __  __                            _   _      _                |
#   |       |  \/  | __ _  ___ _ __ ___       | | | | ___| |_ __           |
#   |       | |\/| |/ _` |/ __| '__/ _ \ _____| |_| |/ _ \ | '_ \          |
#   |       | |  | | (_| | (__| | | (_) |_____|  _  |  __/ | |_) |         |
#   |       |_|  |_|\__,_|\___|_|  \___/      |_| |_|\___|_| .__/          |
#   |                                                      |_|             |
#   +----------------------------------------------------------------------+
#   |  A table of all text macros that are available at that place.        |
#   |  This helps the user.                                                |
#   '----------------------------------------------------------------------'

def render_element_macro_help(element, instance):
    items = instance["macros"].items()
    items.sort(key = lambda a:a[1])

    html.pdf_document.add_margin()
    html.pdf_document.set_tabstops(["b0", "b60"])
    html.pdf_document.add_text_line("\t%s\t%s" % (_("Macro name"), _("Value")))
    html.pdf_document.add_hrule()
    html.pdf_document.set_tabstops(["tb0", "60"])
    for varname, value in items:
        html.pdf_document.add_text_line("\t$%s$\t%s" % (varname, value))
    html.pdf_document.clear_tabstops()
    html.pdf_document.add_margin()

element_types["macro_help"] = {
    "title"      : _("Table of available text macros"),
    "render"     : render_element_macro_help,
}

#.
#   .--Vertical Spacing----------------------------------------------------.
#   |                __     ______                                         |
#   |                \ \   / / ___| _ __   __ _  ___ ___                   |
#   |                 \ \ / /\___ \| '_ \ / _` |/ __/ _ \                  |
#   |                  \ V /  ___) | |_) | (_| | (_|  __/                  |
#   |                   \_/  |____/| .__/ \__,_|\___\___|                  |
#   |                              |_|                                     |
#   +----------------------------------------------------------------------+
#   |  Simply skip some space                                              |
#   '----------------------------------------------------------------------'

def render_element_vertical_space(element, instance):
    html.pdf_document.add_margin(element["parameters"]["height"], element["parameters"]["force"])

element_types["vspace"] = {
    "title"      : _("Vertical space"),
    "render"     : render_element_vertical_space,
    "to_text"    : lambda e: _("%.1f mm") % e["parameters"]["height"],
    "parameters" : Dictionary(
        elements = [
            ( "height",
              Length(
                  title = _("Height"),
                  default_value = 10.0
            )),
            ( "force",
              Checkbox(
                  title = _("Top of page handling"),
                  label = _("Force margin even at top of a page"),
                  default_value = False,
            )),
        ],
        optional_keys = None,
    ),
}


#.
#   .--Image---------------------------------------------------------------.
#   |                   ___                                                |
#   |                  |_ _|_ __ ___   __ _  __ _  ___                     |
#   |                   | || '_ ` _ \ / _` |/ _` |/ _ \                    |
#   |                   | || | | | | | (_| | (_| |  __/                    |
#   |                  |___|_| |_| |_|\__,_|\__, |\___|                    |
#   |                                       |___/                          |
#   +----------------------------------------------------------------------+
#   |  Image as floating content object (as opposed to a fixed image)      |
#   '----------------------------------------------------------------------'

def render_element_image(element, instance):
    parameters = element["parameters"]

    try:
        pil = load_image_as_pil(parameters["filename"])
    except MKGeneralException, e:
        render_report_error(_("Cannot render image"), "%s" % e)
        return

    width, height, resolution = compute_image_size(parameters["size"])
    html.pdf_document.add_pil_image(pil, width, height, resolution, border=False)


element_types["image"] = {
    "title"      : _("Embedded image"),
    "render"     : render_element_image,
    "to_text"    : lambda e: e["parameters"]["filename"],
    "parameters" : ImageParameters(),
}
