#!/usr/bin/python
# -*- encoding: utf-8; py-indent-offset: 4 -*-
# .------------------------------------------------------------------------.
# |                ____ _               _        __  __ _  __              |
# |               / ___| |__   ___  ___| | __   |  \/  | |/ /              |
# |              | |   | '_ \ / _ \/ __| |/ /   | |\/| | ' /               |
# |              | |___| | | |  __/ (__|   <    | |  | | . \               |
# |               \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\              |
# |                                        |_____|                         |
# |             _____       _                       _                      |
# |            | ____|_ __ | |_ ___ _ __ _ __  _ __(_)___  ___             |
# |            |  _| | '_ \| __/ _ \ '__| '_ \| '__| / __|/ _ \            |
# |            | |___| | | | ||  __/ |  | |_) | |  | \__ \  __/            |
# |            |_____|_| |_|\__\___|_|  | .__/|_|  |_|___/\___|            |
# |                                     |_|                                |
# |                     _____    _ _ _   _                                 |
# |                    | ____|__| (_) |_(_) ___  _ __                      |
# |                    |  _| / _` | | __| |/ _ \| '_ \                     |
# |                    | |__| (_| | | |_| | (_) | | | |                    |
# |                    |_____\__,_|_|\__|_|\___/|_| |_|                    |
# |                                                                        |
# | mathias-kettner.com                                 mathias-kettner.de |
# '------------------------------------------------------------------------'
#  This file is part of the Check_MK Enterprise Edition (CEE).
#  Copyright by Mathias Kettner and Mathias Kettner GmbH.  All rights reserved.
#
#  Distributed under the Check_MK Enterprise License.
#
#  You should have  received  a copy of the Check_MK Enterprise License
#  along with Check_MK. If not, email to mk@mathias-kettner.de
#  or write to the postal address provided at www.mathias-kettner.de

import livestatus

def render_cmc_stats():
    def write_line(left, right):
        html.write("<tr><td class=left>%s</td>"
                   "<td class=right><strong>%s</strong></td></tr>" % (left, right))

    try:
        data = html.live.query("GET status\nColumns: service_checks_rate host_checks_rate "
                               "connections_rate average_latency_generic "
                               "average_latency_cmk helper_usage_generic "
                               "helper_usage_cmk livestatus_usage\n")
        if len(data) == 0:
            raise Exception(_("No data."))
    except Exception, e:
        html.write(_('Unable to fetch CMC stats via livestatus. Maybe you are not using CMC '
                     'as your core in all of your sites. In this case this snapin is useless.'))
        return

    num_cmcs  = sum(row[3] != None and 1 for row in data)
    if num_cmcs == 0:
        html.write("No Micro Core found.<br>Please use snapin Server performance.")
    else:
        html.write("<table class=\"content_center cmc_stats\">\n")
        for what, col, format, factor in [
            (_("Service checks"),              0, "%.2f/s", 1),
            (_("Host checks and smart pings"), 1, "%.2f/s", 1),
            (_("Livestatus-connections"),      2, "%.2f/s", 1),
            (_("Average check latency"),       3, "%.3fs",  1.0 / num_cmcs),
            (_("Average Check_MK latency"),    4, "%.3fs",  1.0 / num_cmcs),
            (_("Check helper usage"),          5, "%.1f%%", 100.0 / num_cmcs),
            (_("Check_MK helper usage"),       6, "%.1f%%", 100.0 / num_cmcs),
            (_("Livestatus usage"),            7, "%.1f%%", 100.0 / num_cmcs),
            ]:
            write_line(what + ":", format % (factor * sum([row[col] or 0 for row in data])))
        html.write("</table>\n")

sidebar_snapins["cmc_stats"] = {
    "title" : _("Micro Core Statistics"),
    "description" : _("Live monitor of the performance statistics of the Check_MK "
                      "Micro Core. Note: this snapin only works if "
                      "<b>all</b> of your sites are running the Check_MK Cores"),
    "refresh" : True,
    "render" : render_cmc_stats,
    "allowed" : [ "admin", ],
    "styles" : """
#snapin_cmc_stats {
    color: white;
}
table.cmc_stats {
    width: %dpx;
    -moz-border-radius: 5px;
    background-color: #589;
    /* background-color: #6da1b8;*/
    border-style: solid;
    border-color: #444 #bbb #eee #666;
    /* The border needs to be substracted from the width */
    border-width: 1px;
}
table.cmc_stats td {
    padding: 0px 2px;
    font-size: 8pt;
}
table.cmc_stats td.right {
    text-align: right;
    padding: 0px;
    padding-right: 1px;
    white-space: nowrap;
}

""" % (snapin_width - 2)
}

