#!/usr/bin/python
# -*- encoding: utf-8; py-indent-offset: 4 -*-
# .------------------------------------------------------------------------.
# |                ____ _               _        __  __ _  __              |
# |               / ___| |__   ___  ___| | __   |  \/  | |/ /              |
# |              | |   | '_ \ / _ \/ __| |/ /   | |\/| | ' /               |
# |              | |___| | | |  __/ (__|   <    | |  | | . \               |
# |               \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\              |
# |                                        |_____|                         |
# |             _____       _                       _                      |
# |            | ____|_ __ | |_ ___ _ __ _ __  _ __(_)___  ___             |
# |            |  _| | '_ \| __/ _ \ '__| '_ \| '__| / __|/ _ \            |
# |            | |___| | | | ||  __/ |  | |_) | |  | \__ \  __/            |
# |            |_____|_| |_|\__\___|_|  | .__/|_|  |_|___/\___|            |
# |                                     |_|                                |
# |                     _____    _ _ _   _                                 |
# |                    | ____|__| (_) |_(_) ___  _ __                      |
# |                    |  _| / _` | | __| |/ _ \| '_ \                     |
# |                    | |__| (_| | | |_| | (_) | | | |                    |
# |                    |_____\__,_|_|\__|_|\___/|_| |_|                    |
# |                                                                        |
# | mathias-kettner.com                                 mathias-kettner.de |
# '------------------------------------------------------------------------'
#  This file is part of the Check_MK Enterprise Edition (CEE).
#  Copyright by Mathias Kettner and Mathias Kettner GmbH.  All rights reserved.
#
#  Distributed under the Check_MK Enterprise License.
#
#  You should have  received  a copy of the Check_MK Enterprise License
#  along with Check_MK. If not, email to mk@mathias-kettner.de
#  or write to the postal address provided at www.mathias-kettner.de

import reporting

def render_reports():
    config.need_permission("general.reporting")
    reporting.load_reports()

    def render_topic(topic, s):
        first = True
        for t, title, name, _i in s:
            if t == topic:
                if first:
                    html.begin_foldable_container("reports", topic, False, topic, indent=True)
                    first = False
                bulletlink(title, 'report.py?name=%s' % name)
        if not first: # at least one item rendered
            html.end_foldable_container()

    for topic, s in visuals_by_topic(reporting.permitted_reports().items()):
        render_topic(topic, s)

    html.write("<br>")

    # Select the current reporting time range
    html.write("<div class=timerange>%s: " % _("Default reporting timerange"))
    from valuespec import Timerange

    if html.form_submitted("reporting_timerange") and html.check_transaction():
	try:
	    new_timerange = Timerange().from_html_vars("reporting_timerange")
	    config.save_user_file("reporting_timerange", new_timerange)
            html.write("<br><i>%s</i>" % _("Changed to: <b>%s</b>") % Timerange().value_to_text(new_timerange))
	except MKUserError, e:
	    html.show_error("%s" % e)

    html.begin_form("reporting_timerange")
    current_timerange = config.load_user_file("reporting_timerange", config.reporting_rangespec)
    Timerange().render_input("reporting_timerange", current_timerange)
    html.write("</div>")
    html.end_form()

    begin_footnote_links()
    html.write(link(_("Change Timerange"), "javascript:reporting_timerange.submit();", target=None))
    if config.may("general.schedule_reports") or config.may("general.schedule_reports_all"):
	html.write(link(_("Scheduler"), "report_scheduler.py"))
    if config.may("general.edit_reports"):
	html.write(link(_("Edit"), "edit_reports.py"))
    end_footnote_links()

sidebar_snapins["reports"] = {
    "title"       : _("Reporting"),
    "description" : _("Direct access to global reports, selection "
                      "for the default reporting time range, link to report editor."),
    "render"      : render_reports,
    "allowed"     : [ "user", "admin", "guest" ],
    "styles"      : """

#snapin_reports i {
    color: black;
}

#snapin_reports input {
    font-size: 9px;
}

#snapin_reports div.timerange {
    color: white;
    margin-bottom: 10px;
}

#snapin_reports div.timerange table {
    display: inline-block;
}

#snapin_reports select {
    float: left;
    padding: 0;
    width: 230px;
    height: 19px;
    margin-top: 3px;
    margin-bottom: 5px;
    background-color: #6da1b8;
    color: #fff;
    border-color: #123a4a;
    font-size: 8pt;
}
"""
}

