#!/usr/bin/python
# -*- encoding: utf-8; py-indent-offset: 4 -*-
# .------------------------------------------------------------------------.
# |                ____ _               _        __  __ _  __              |
# |               / ___| |__   ___  ___| | __   |  \/  | |/ /              |
# |              | |   | '_ \ / _ \/ __| |/ /   | |\/| | ' /               |
# |              | |___| | | |  __/ (__|   <    | |  | | . \               |
# |               \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\              |
# |                                        |_____|                         |
# |             _____       _                       _                      |
# |            | ____|_ __ | |_ ___ _ __ _ __  _ __(_)___  ___             |
# |            |  _| | '_ \| __/ _ \ '__| '_ \| '__| / __|/ _ \            |
# |            | |___| | | | ||  __/ |  | |_) | |  | \__ \  __/            |
# |            |_____|_| |_|\__\___|_|  | .__/|_|  |_|___/\___|            |
# |                                     |_|                                |
# |                     _____    _ _ _   _                                 |
# |                    | ____|__| (_) |_(_) ___  _ __                      |
# |                    |  _| / _` | | __| |/ _ \| '_ \                     |
# |                    | |__| (_| | | |_| | (_) | | | |                    |
# |                    |_____\__,_|_|\__|_|\___/|_| |_|                    |
# |                                                                        |
# | mathias-kettner.com                                 mathias-kettner.de |
# '------------------------------------------------------------------------'
#  This file is part of the Check_MK Enterprise Edition (CEE).
#  Copyright by Mathias Kettner and Mathias Kettner GmbH.  All rights reserved.
#
#  Distributed under the Check_MK Enterprise License.
#
#  You should have  received  a copy of the Check_MK Enterprise License
#  along with Check_MK. If not, email to mk@mathias-kettner.de
#  or write to the postal address provided at www.mathias-kettner.de


# TODO: We could/should add the deployment status information as column
# into each row of the datasource hosts. This is similar to the "host_inventory"
# columns. But the later one is added via a hard coded hack in the views
# code. The new Elements-based code can do this more elegant. We can change
# this later. By now we simply load the information on the fly and use
# the caching of the html object.

import agent_bakery

multisite_builtin_views['allhosts_deploy'] = {
     'browser_reload': 30,
     'column_headers': 'pergroup',
     'datasource': 'hosts',
     'description': _('Current agent update status of all registered hosts.'),
     'hard_filters': [ "deployment_is_registered" ],
     'hard_filtervars': [ ("is_deployment_is_registered", "1") ],
     'hidden': False,
     'hide_filters': [],
     'layout': 'table',
     'mustsearch': False,
     'num_columns': 1,
     'group_painters': [],
     'painters': [
         ('host',                       'host'),
         # ('host_icons',                 None),
         ('deployment_icons',           None),
         ('deployment_last_contact',    None),
         ('deployment_last_download',   None),
         ('deployment_last_error',      None),
         ('deployment_target_hash',     None),
         ('deployment_downloaded_hash', None),
         ('deployment_installed_hash',  None),
         ('service_state', 'service', None, u'Check_MK Agent', u'Update Check'),
         ('svc_plugin_output', 'service', None, u'Check_MK Agent', u'Update Check Output'),
     ],
     'play_sounds': False,
     'public': True,
     'show_filters': [
         'hoststate',
         'siteopt',
         'hostregex',
         'hostgroups',
         'opthostgroup',
         'opthost_contactgroup',
         'hostalias',
         'host_tags',
         # Filter für Deployment
         "deployment_is_registered",
         "deployment_need_update",
         "deployment_has_agent",
         "deployment_has_downloaded",
         "deployment_host_has_error",
         "deployment_last_registered",
         "deployment_last_contact",
         "deployment_last_download",
     ],
     'sorters': [('site', False), ('site_host', False)],
     'title': _('Agent update status'),
     'topic': _('Other'),
}

def paint_deployment_timestamp(row, what):
    status = agent_bakery.get_cached_deployment_status(row["host_name"])
    last_contact = status.get(what)
    if last_contact:
        return paint_age(last_contact, True, 0)
    else:
        return "", _("(never)")


multisite_painters["deployment_last_contact"] = {
    "title" : _("Agent Deployment: Last Contact"),
    "short" : _("Last Contact"),
    "options" : [ "ts_format", "ts_date" ],
    "columns" : [ "host_name" ],
    "paint" : lambda row: paint_deployment_timestamp(row, "last_contact"),
}


multisite_painters["deployment_last_download"] = {
    "title" : _("Agent Deployment: Last Download"),
    "short" : _("Last Download"),
    "options" : [ "ts_format", "ts_date" ],
    "columns" : [ "host_name" ],
    "paint" : lambda row: paint_deployment_timestamp(row, "last_download"),
}

def paint_deployment_last_error(row):
    status = agent_bakery.get_cached_deployment_status(row["host_name"])
    if "last_error" in status:
        error = status.get("last_error")
        if error:
            return "state2", error
        else:
            return "", _("no error")
    else:
        return "stale", _("(no contact)")


multisite_painters["deployment_last_error"] = {
    "title" : _("Agent Deployment: Last Error"),
    "short" : _("Last Error"),
    "columns" : [ "host_name" ],
    "paint" : paint_deployment_last_error,
}

def paint_deployment_hash(row, key):
    status = agent_bakery.get_cached_deployment_status(row["host_name"])
    return "", (status.get(key) or "")[:8]

multisite_painters["deployment_downloaded_hash"] = {
    "title" : _("Agent Deployment: Downloaded Agent Hash"),
    "short" : _("Downloaded Agent"),
    "columns" : [ "host_name" ],
    "paint" : lambda row: paint_deployment_hash(row, "downloaded_aghash"),
}

multisite_painters["deployment_installed_hash"] = {
    "title" : _("Agent Deployment: Installed Agent Hash"),
    "short" : _("Installed Agent"),
    "columns" : [ "host_name" ],
    "paint" : lambda row: paint_deployment_hash(row, "installed_aghash"),
}


multisite_painters["deployment_target_hash"] = {
    "title" : _("Agent Deployment: Target Agent Hash"),
    "short" : _("Target Agent"),
    "columns" : [ "host_name" ],
    "paint" : lambda row: paint_deployment_hash(row, "target_aghash"),
}


def paint_deployment_icons(row):
    code = ""
    host_name = row["host_name"]
    status = agent_bakery.get_cached_deployment_status(host_name)
    target_hash = agent_bakery.target_agent_hash(row["host_name"])

    if agent_bakery.host_has_error(status):
        code += html.render_icon("deployment_error", _("An error occurred during the last update"))

    if agent_bakery.agent_needs_update(status):
        code += html.render_icon("wrong_agent", _("The installed agent needs an update"))

    if agent_bakery.has_baked_agent(status):
        url = html.makeuri_contextless([("mode", "agent_of_host"), ("host", host_name)], "wato.py")
        code += html.render_icon_button(url, _("This host has a baked agent"), "agents")

    return "icons", code



multisite_painters["deployment_icons"] = {
    "title" : _("Agent Deployment: Icons"),
    "short" : "",
    "columns" : [ "host_name" ],
    "paint" : paint_deployment_icons,
}



