#!/usr/bin/python
# -*- encoding: utf-8; py-indent-offset: 4 -*-
# .------------------------------------------------------------------------.
# |                ____ _               _        __  __ _  __              |
# |               / ___| |__   ___  ___| | __   |  \/  | |/ /              |
# |              | |   | '_ \ / _ \/ __| |/ /   | |\/| | ' /               |
# |              | |___| | | |  __/ (__|   <    | |  | | . \               |
# |               \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\              |
# |                                        |_____|                         |
# |             _____       _                       _                      |
# |            | ____|_ __ | |_ ___ _ __ _ __  _ __(_)___  ___             |
# |            |  _| | '_ \| __/ _ \ '__| '_ \| '__| / __|/ _ \            |
# |            | |___| | | | ||  __/ |  | |_) | |  | \__ \  __/            |
# |            |_____|_| |_|\__\___|_|  | .__/|_|  |_|___/\___|            |
# |                                     |_|                                |
# |                     _____    _ _ _   _                                 |
# |                    | ____|__| (_) |_(_) ___  _ __                      |
# |                    |  _| / _` | | __| |/ _ \| '_ \                     |
# |                    | |__| (_| | | |_| | (_) | | | |                    |
# |                    |_____\__,_|_|\__|_|\___/|_| |_|                    |
# |                                                                        |
# | mathias-kettner.com                                 mathias-kettner.de |
# '------------------------------------------------------------------------'
#  This file is part of the Check_MK Enterprise Edition (CEE).
#  Copyright by Mathias Kettner and Mathias Kettner GmbH.  All rights reserved.
#
#  Distributed under the Check_MK Enterprise License.
#
#  You should have  received  a copy of the Check_MK Enterprise License
#  along with Check_MK. If not, email to mk@mathias-kettner.de
#  or write to the postal address provided at www.mathias-kettner.de


def vs_downtime_time():
    return AbsoluteDate(
        include_time = True,
        default_value = None,
        allow_empty = True,
        none_means_empty = True,
    )


def paint_edit_downtime():

    html.write(_("Extend downtimes by...") + "  ")
    for title, minutes in [
        ( _("30 Minutes"), 30 ),
        ( _("One hour"), 60 ),
        ( _("Two hours"), 120 ),
    ]:
        html.button("_down_extend_secs_%d" % (minutes * 60), title)

    html.write(" ")
    html.button("_down_extend_by_span", _("Custom Time Span:"))
    html.write(" ")
    html.time_input("_down_extend_span", 4, 0)
    html.write(_("(hours:minutes)"))

    html.write("<hr>")

    html.write("<table class=modifydowntimes>")
    html.write("<tr><td>")
    html.write(_('Change downtime comment') + ": ")
    html.write("</td><td>")
    html.text_input("_down_comment", "", size=60, submit="")
    html.write("</td></tr>")

    html.write("<tr><td>")
    html.write(_("Change downtime start") + ": ")
    html.write("</td><td>")
    vs_downtime_time().render_input("_down_from", None)
    html.write("</td></tr>")

    html.write("<tr><td>")
    html.write(_("Change downtime end") + ": ")
    html.write("</td><td>")
    vs_downtime_time().render_input("_down_to", None)
    html.write("</td></tr>")


    html.write("<tr><td></td><td>")
    html.button("_down_edit", _("Modify"))
    html.write("</td></tr></table>")


def command_edit_downtime(cmdtag, spec, row):

    extend_by = None
    comment = None
    down_from = None
    down_to = None

    if html.has_var("_down_extend_by_span"):
        extend_by = html.get_time_input("_down_extend_span", _("The custom timerange"))
    else:
        for varname in html.all_varnames_with_prefix("_down_extend_secs_"):
            extend_by = int(varname[18:])

    if html.has_var("_down_edit") and extend_by == None:
        comment = html.get_unicode_input("_down_comment")
        time_spec = vs_downtime_time()

        down_from = time_spec.from_html_vars("_down_from")
        time_spec.validate_value(down_from, "_down_from")
        if down_from == None:
            down_from = ""
        else:
            down_from = str(int(down_from))

        down_to = time_spec.from_html_vars("_down_to")
        time_spec.validate_value(down_to, "_down_to")
        if down_to == None:
            down_to = ""
        else:
            down_to = str(int(down_to))

    if comment != None or extend_by != None:
        if extend_by != None:
            down_from = ""
            down_to = "+%d" % extend_by

        if comment == None:
            comment = ""

        command = ("MODIFY_" + cmdtag + "_DOWNTIME;%s;" % spec) \
                   + ("%s;%s;;;;%s;" % (lqencode(down_from), lqencode(down_to), config.user_id)) \
                   + lqencode(comment)

        return command, _("modify")


# EDIT DOWNTIMES (table downtimes)
multisite_commands.append({
    "tables"      : [ "downtime" ],
    "permission"  : "action.downtimes",
    "title"       : _("Edit Downtimes"),
    "render"      : paint_edit_downtime,
    "action"      : command_edit_downtime,
})
