#!/usr/bin/python
# -*- encoding: utf-8; py-indent-offset: 4 -*-
# .------------------------------------------------------------------------.
# |                ____ _               _        __  __ _  __              |
# |               / ___| |__   ___  ___| | __   |  \/  | |/ /              |
# |              | |   | '_ \ / _ \/ __| |/ /   | |\/| | ' /               |
# |              | |___| | | |  __/ (__|   <    | |  | | . \               |
# |               \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\              |
# |                                        |_____|                         |
# |             _____       _                       _                      |
# |            | ____|_ __ | |_ ___ _ __ _ __  _ __(_)___  ___             |
# |            |  _| | '_ \| __/ _ \ '__| '_ \| '__| / __|/ _ \            |
# |            | |___| | | | ||  __/ |  | |_) | |  | \__ \  __/            |
# |            |_____|_| |_|\__\___|_|  | .__/|_|  |_|___/\___|            |
# |                                     |_|                                |
# |                     _____    _ _ _   _                                 |
# |                    | ____|__| (_) |_(_) ___  _ __                      |
# |                    |  _| / _` | | __| |/ _ \| '_ \                     |
# |                    | |__| (_| | | |_| | (_) | | | |                    |
# |                    |_____\__,_|_|\__|_|\___/|_| |_|                    |
# |                                                                        |
# | mathias-kettner.com                                 mathias-kettner.de |
# '------------------------------------------------------------------------'
#  This file is part of the Check_MK Enterprise Edition (CEE).
#  Copyright by Mathias Kettner and Mathias Kettner GmbH.  All rights reserved.
#
#  Distributed under the Check_MK Enterprise License.
#
#  You should have  received  a copy of the Check_MK Enterprise License
#  along with Check_MK. If not, email to mk@mathias-kettner.de
#  or write to the postal address provided at www.mathias-kettner.de

import metrics

multisite_builtin_views.update({
    'service_graphs': {
        'browser_reload': 30,
        'column_headers': 'off',
        'datasource': 'services',
        'description': _('Shows all graphs including timerange selections '
                         'of a collection of services.'),
        'group_painters': [
            ('sitealias', 'sitehosts'),
            ('host_with_state', 'host'),
            ('service_description', 'service'),
        ],
        'hard_filters': [],
        'hard_filtervars': [],
        'hidden': True,
        'hide_filters': ['siteopt', 'service', 'host'],
        'layout': 'boxed_graph',
        'mustsearch': False,
        'name': 'service_graphs',
        'num_columns': 1,
        'owner': '',
        'painters': [
            ('service_graphs', None),
        ],
        'public': True,
        'show_filters': [],
        'sorters': [],
        'linktitle': _('Graphs'),
        'icon': 'pnp',
        'title': _('Service Graphs'),
    },
    'host_graphs': {
        'browser_reload': 30,
        'column_headers': 'off',
        'datasource': 'hosts',
        'description': _('Shows host graphs including timerange selections '
                         'of a collection of hosts.'),
        'group_painters': [
            ('sitealias', 'sitehosts'),
            ('host_with_state', 'host'),
        ],
        'hard_filters': [],
        'hard_filtervars': [],
        'hidden': True,
        'hide_filters': ['siteopt', 'host'],
        'layout': 'boxed_graph',
        'mustsearch': False,
        'name': 'host_graphs',
        'num_columns': 1,
        'owner': '',
        'painters': [
            ('host_graphs', None),
        ],
        'public': True,
        'show_filters': [],
        'sorters': [],
        'linktitle': _('Graphs'),
        'icon': 'pnp',
        'title': _('Host Graphs'),
    },
})

service_graph_overview_settings = {
    "painters": [
        ("service_graphs", None),
    ],
    "num_columns": 1,
    "layout": "boxed_graph",
}

host_graph_overview_settings = {
    "painters": [
        ("host_graphs", None),
    ],
    "num_columns": 1,
    "layout": "boxed_graph",
}

# Change the hosts graph view to use the normal time graphs
multisite_builtin_views["hostpnp"].update(host_graph_overview_settings)
# Change the "graphs of all services with this description" view
multisite_builtin_views["servicedescpnp"].update(service_graph_overview_settings)
# Change the "graph search" view
multisite_builtin_views["searchpnp"].update(service_graph_overview_settings)


def paint_cmk_graphs_with_timeranges(row):
    if not metrics.cmk_graphs_possible(row["site"]):
        return "", _("Sorry, you can not view the new style graphs in your browser.")

    return paint_time_graph_cmk(row, show_timeranges=True)


multisite_painters["service_graphs"] = {
    "title"     : _("Service Graphs with Timerange Previews"),
    "columns"   : [ "host_name", "service_description", "service_perf_data",
                    "service_metrics", "service_check_command" ],
    "options"   : [ "pnp_timerange" ],
    "paint"     : paint_cmk_graphs_with_timeranges,
    "printable" : "time_graph",
}


multisite_painters["host_graphs"] = {
    "title"     : _("Host Graphs with Timerange Previews"),
    "columns"   : [ "host_name", "host_perf_data",
                    "host_metrics", "host_check_command" ],
    "options"   : [ "pnp_timerange" ],
    "paint"     : paint_cmk_graphs_with_timeranges,
    "printable" : "time_graph",
}
