#!/usr/bin/python
# -*- encoding: utf-8; py-indent-offset: 4 -*-
# .------------------------------------------------------------------------.
# |                ____ _               _        __  __ _  __              |
# |               / ___| |__   ___  ___| | __   |  \/  | |/ /              |
# |              | |   | '_ \ / _ \/ __| |/ /   | |\/| | ' /               |
# |              | |___| | | |  __/ (__|   <    | |  | | . \               |
# |               \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\              |
# |                                        |_____|                         |
# |             _____       _                       _                      |
# |            | ____|_ __ | |_ ___ _ __ _ __  _ __(_)___  ___             |
# |            |  _| | '_ \| __/ _ \ '__| '_ \| '__| / __|/ _ \            |
# |            | |___| | | | ||  __/ |  | |_) | |  | \__ \  __/            |
# |            |_____|_| |_|\__\___|_|  | .__/|_|  |_|___/\___|            |
# |                                     |_|                                |
# |                     _____    _ _ _   _                                 |
# |                    | ____|__| (_) |_(_) ___  _ __                      |
# |                    |  _| / _` | | __| |/ _ \| '_ \                     |
# |                    | |__| (_| | | |_| | (_) | | | |                    |
# |                    |_____\__,_|_|\__|_|\___/|_| |_|                    |
# |                                                                        |
# | mathias-kettner.com                                 mathias-kettner.de |
# '------------------------------------------------------------------------'
#  This file is part of the Check_MK Enterprise Edition (CEE).
#  Copyright by Mathias Kettner and Mathias Kettner GmbH.  All rights reserved.
#
#  Distributed under the Check_MK Enterprise License.
#
#  You should have  received  a copy of the Check_MK Enterprise License
#  along with Check_MK. If not, email to mk@mathias-kettner.de
#  or write to the postal address provided at www.mathias-kettner.de

import agent_bakery

class DeploymentTristateFilter(FilterTristate):
    def __init__(self, name, title, match_function):
        FilterTristate.__init__(self, name, title, "host", None, -1)
        self._match_function = match_function

    def filter(self, infoname):
	return ""

    def filter_table(self, rows):
        current = self.tristate_value()
        if current == -1:
	    return rows
        new_rows = []
        for row in rows:
            status = agent_bakery.get_cached_deployment_status(row["host_name"])
            if self._match_function(status) == (current == 1):
                new_rows.append(row)
        return new_rows

declare_filter(500, DeploymentTristateFilter(
    "deployment_is_registered",
    _("Host has registered for deployment"),
    agent_bakery.host_is_registered))

declare_filter(501, DeploymentTristateFilter(
    "deployment_need_update",
    _("Host needs agent update"),
    agent_bakery.agent_needs_update))


declare_filter(502, DeploymentTristateFilter(
    "deployment_has_agent",
    _("Host has a baked agent"),
    agent_bakery.has_baked_agent))


declare_filter(503, DeploymentTristateFilter(
    "deployment_has_downloaded",
    _("Host has downloaded an agent"),
    agent_bakery.has_downloaded_agent))


declare_filter(504, DeploymentTristateFilter(
    "deployment_host_has_error",
    _("Host has deployment error"),
    agent_bakery.host_has_error))


class DeploymentTimestampFilter(FilterTime):
    def __init__(self, name, title, key_in_status):
        FilterTime.__init__(self, "host", name, title, None)
        self._key_in_status = key_in_status

    def filter(self, infoname):
        return ""

    def filter_table(self, rows):
        from_secs, until_secs = self.get_time_range()
        if from_secs == None and until_secs == None:
            return rows

        new_rows = []
        for row in rows:
            status = agent_bakery.get_cached_deployment_status(row["host_name"])
            timestamp = status.get(self._key_in_status)
            if timestamp == None:
                continue

            if from_secs != None and timestamp < from_secs:
                continue
            if until_secs != None and timestamp > until_secs:
                continue
            new_rows.append(row)
        return new_rows


declare_filter(504, DeploymentTimestampFilter(
    "deployment_last_registered",
    _("Time of registration"),
    "last_registered"))


declare_filter(505, DeploymentTimestampFilter(
    "deployment_last_contact",
    _("Last contact of agent updater"),
    "last_contact"))


declare_filter(506, DeploymentTimestampFilter(
    "deployment_last_download",
    _("Last download of agent updater"),
    "last_download"))

