#!/usr/bin/python
# -*- encoding: utf-8; py-indent-offset: 4 -*-
# .------------------------------------------------------------------------.
# |                ____ _               _        __  __ _  __              |
# |               / ___| |__   ___  ___| | __   |  \/  | |/ /              |
# |              | |   | '_ \ / _ \/ __| |/ /   | |\/| | ' /               |
# |              | |___| | | |  __/ (__|   <    | |  | | . \               |
# |               \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\              |
# |                                        |_____|                         |
# |             _____       _                       _                      |
# |            | ____|_ __ | |_ ___ _ __ _ __  _ __(_)___  ___             |
# |            |  _| | '_ \| __/ _ \ '__| '_ \| '__| / __|/ _ \            |
# |            | |___| | | | ||  __/ |  | |_) | |  | \__ \  __/            |
# |            |_____|_| |_|\__\___|_|  | .__/|_|  |_|___/\___|            |
# |                                     |_|                                |
# |                     _____    _ _ _   _                                 |
# |                    | ____|__| (_) |_(_) ___  _ __                      |
# |                    |  _| / _` | | __| |/ _ \| '_ \                     |
# |                    | |__| (_| | | |_| | (_) | | | |                    |
# |                    |_____\__,_|_|\__|_|\___/|_| |_|                    |
# |                                                                        |
# | mathias-kettner.com                                 mathias-kettner.de |
# '------------------------------------------------------------------------'
#  This file is part of the Check_MK Enterprise Edition (CEE).
#  Copyright by Mathias Kettner and Mathias Kettner GmbH.  All rights reserved.
#
#  Distributed under the Check_MK Enterprise License.
#
#  You should have  received  a copy of the Check_MK Enterprise License
#  along with Check_MK. If not, email to mk@mathias-kettner.de
#  or write to the postal address provided at www.mathias-kettner.de

import agent_bakery
import urllib

try:
    from hashlib import md5
except ImportError:
    from md5 import md5 # deprecated with python 2.5

#   .--Rulesets------------------------------------------------------------.
#   |                ____        _                _                        |
#   |               |  _ \ _   _| | ___  ___  ___| |_ ___                  |
#   |               | |_) | | | | |/ _ \/ __|/ _ \ __/ __|                 |
#   |               |  _ <| |_| | |  __/\__ \  __/ |_\__ \                 |
#   |               |_| \_\\__,_|_|\___||___/\___|\__|___/                 |
#   |                                                                      |
#   +----------------------------------------------------------------------+
#   |  Rulesets for configuring the various agent properties.              |
#   '----------------------------------------------------------------------'

group = _("Administration Tool (WATO)")
register_configvar(group,
    "bake_agents_on_restart",
    Checkbox(title = _("Automatically create monitoring agents"),
             label = _("Automatically update agents at <i>Activate Changes</i>, if neccessary"),
             help = _("When this option is enabled then updated monitoring agents "
                      "are automatically being created whenever configuration changes are "
                      "activated and the agents are outdated. Agents are being updated "
                      "if their configuration has changed (due to some change in the rules "
                      "of the monitoring agents)."),
            default_value = False),
)

register_rulegroup("agents", _("Monitoring Agents"),
   _("Configuration of monitoring agents for Linux, Windows and Unix"))

group = "agents/" + _("Generic Options")

register_rule(group,
    "agent_config:only_from",
    ListOfStrings(
        valuespec = IPv4Network(),
        title = _("Restrict agent access via IP address"),
        help = _("This rule allows you to restrict the access to the "
                 "Check_MK agent to certain IP addresses and networks. "
                 "Usually you configure just the IP addresses of your "
                 "Check_MK servers here. You can enter either IP addresses "
                 "in the form <tt>1.2.3.4</tt> or networks in the style "
                 "<tt>1.2.0.0/16</tt>. If you leave this configuration empty "
                 "or create no rule then <b>all</b> addresses are allowed to "
                 "access the agent."),
    ),
    match = "first"
)

register_rule(group,
    "agent_config:agent_paths",
    Dictionary(
        title = _("Installation paths for agent files (Linux, UNIX)"),
        elements = [
            ( "bin",
              AbsoluteDirname(
                  title = _("Directory for binaries (executables)"),
                  default_value = "/usr/bin",
                  help = _("In this directory will be installed <tt>check_mk_agent</tt>, <tt>waitmax</tt> and "
                           "possibly other binaries that are needed by the agent."),
            )),
            ( "config",
              AbsoluteDirname(
                  title = _("Directory for configuration files"),
                  default_value = "/etc/check_mk",
            )),
            ( "lib",
              AbsoluteDirname(
                  title = _("Base directory for <tt>plugins</tt> and <tt>local</tt>"),
                  default_value = "/usr/lib/check_mk_agent",
            )),
            ( "var",
              AbsoluteDirname(
                  title = _("Base directory for variable data (caches, state files)"),
                  default_value = "/var/lib/check_mk_agent",
                  help = _("If you change this paths away from its default then the package "
                           "will <b>not</b> delete the contents of that directory when uninstalling."),
            )),
        ]
    ),
    match = "dict")


def get_possible_real_time_check_choices():
    return check_mk_local_automation("get-real-time-checks")


register_rule(group,
    "agent_config:real_time_checks",
    Dictionary(
        title = _("Perform Real-Time Checks"),
	help = _("The Check_MK agent supports a feature called Real-Time Checks where it sends status updates "
                 "of specific monitoring information more often than the regular check interval of the host, "
                 "for example in an interval of 1 second. Use this rule to configure the baked agents to send "
                 "Real-Time Check data to the Check_MK server."),
        elements = [
            ("timeout", Integer(
                title = _("Timeout after last agent contact"),
                help = _("The agent stops sending Real-Time Check data to the Monitoring Server after the configured "
                         "time. Normally the monitoring server is contacting the agent in a regular interval, "
                         "for example 60 seconds, so this timeout will never be reached. Except when the "
                         "monitoring server is stopping the monitoring for some reason, then the agent "
                         "will stop sending data to the Monitoring Server. You need to change this option "
                         "according to the check interval of the Check_MK service of this host, if you "
                         "changed the check interval."),
                unit = _("Seconds"),
                minvalue = 1,
                maxvalue = 86400,
                default_value = 90,
                allow_empty = False,
            )),
            ("checks", ListChoice(
                choices = get_possible_real_time_check_choices,
                title = _("Checks to send data for"),
                help = _("Choose the checks to send the monitoring data as Real-Time Checks for. These choices "
                         "control which and how many packets will be send from the agent to the Check_MK server."),
                allow_empty = False,
            )),
            ("port", Integer(
                  title = _("UDP port the Microcore is listening on"),
                  help = _("Configure the port to send the Real-time data to. The Microcore needs "
                           "to be <a href=\"wato.py?mode=edit_configvar&varname=cmc_real_time_checks\">configured to listen on this port</a>."),
                  default_value = 6559,
                  minvalue = 1025, # don't allow well known ports
                  maxvalue = 65535,
                  allow_empty = False,
            )),
            ("secret", Password(
                title = _("Secret to encrypt UDP traffic"),
                allow_empty = False,
                help = _("This secret is shared between the agent and the Check_MK server and "
                         "used to encrypt the UDP package contents. It needs to match "
                         "the secret <a href=\"wato.py?mode=edit_configvar&varname=cmc_real_time_checks\">configured in the Microcore</a>."),
            )),
        ],
    ),
    match = "dict",
)


def transform_mrpe_forth(x):
    res = []
    for line in x:
        line = list(line)
        line[0] = urllib.unquote(line[0])
        if len(line) == 2:
            res.append((line[0], line[1], None))
        else:
            res.append(tuple(line))

    return res

def transform_mrpe_back(x):
    res = []
    for line in x:
        line = list(line)
        line[0] = urllib.quote(line[0])
        res.append(tuple(line))
    return res



register_rule(group,
    "agent_config:mrpe",
    Transform(
        ListOf(
            Tuple(
                elements = [
                    TextAscii(
                        title = _("Service description"),
                        help = _("This will be used as service description within "
                                "Check_MK. It must be a unique name per host."),
                        allow_empty = False,
                        regex = "[A-Za-z0-9-_\. ]+$",
                        regex_error = _("Valid service descriptions may only contain "
                                        "letters, numbers, dashes, underscores, spaces and dots."),
                    ),
                    TextAscii(
                        title = _("Command line to execute"),
                        help = _("This command will be executed on the target host."),
                        size = 80,
                    ),
                    Optional(
                        Integer(
                            title = _("Max Cache Age"),
                            unit = _("Seconds"),
                            default_value = 60,
                        ),
                        title = _("Cache output"),
                        help = _("If this is set, the MRPE output will be cached and will only "
                                "be refreshed once the age of the cache has exceeded "
                                "this value."
                                ),
                        label = _("Enable"),
                    ),
                ],
                orientation = "horizontal",
            ),
            title = _("Execute MRPE checks"),
            help = _("The Check_MK agent supports executing Nagios plugins on the remote host like you might "
                    "have done it before when monitoring via NRPE. This feature is called MRPE (MK's Remote Plugin Executor). "
                    "This rule can be used to configure the agent to execute plugins of your choice using the given arguments."),
            sortable = False,
            add_label = _("Add plugin"),
        ),
        forth = transform_mrpe_forth,
        back = transform_mrpe_back
    ),
    match = "list",
)


group = "agents/" + _("Automatic Updates")

def signature_key_choices():
    return sorted([(key["certificate"], key["alias"]) for key in config.agent_signature_keys.values()])



class CAorCAChain(UploadOrPasteTextFile):

    def __init__(self, **args):
        args.setdefault("title", _("Certificate Chain (Root / Intermediate Certificate)"))
        args.setdefault("file_title", _("CRT/PEM File"))
        UploadOrPasteTextFile.__init__(self, **args)


    def from_html_vars(self, varprefix):
        value = Alternative.from_html_vars(self, varprefix)
        if type(value) == tuple:
            value = value[2] # FileUpload sends (filename, mime-type, content)
        return value


    def validate_value(self, value, varprefix):
        try:
            self.analyse_cert(value)
        except Exception, e:
            # FIXME TODO: Cleanup this general exception catcher
            raise MKUserError(varprefix, _("Invalid certificate file"))


    def analyse_cert(self, value):
        from OpenSSL import crypto
        cert = crypto.load_certificate(crypto.FILETYPE_PEM, value)
        titles = {
            "C"  : _("Country"),
            "ST" : _("State or Province Name"),
            "L"  : _("Locality Name"),
            "O"  : _("Organization Name"),
            "CN" : _("Common Name"),
        }
        cert_info = {}
        for what, x509, title in [
            ( "issuer", cert.get_issuer(), _("Issuer") ),
            ( "subject", cert.get_subject(), _("Subject") ),
        ]:
            cert_info[what] = {}
            for key, val in x509.get_components():
                if key in titles:
                    cert_info[what][titles[key]] = val.decode("utf8")
        return cert_info


    def value_to_text(self, value):
        cert_info = self.analyse_cert(value)
        text = "<table>"
        for what, title in [
            ( "issuer", _("Issuer") ),
            ( "subject", _("Subject") ),
        ]:
            text += "<tr><td>%s:</td><td>" % title
            for title, value in sorted(cert_info[what].items()):
                text += "%s: %s<br>" % (title, value)
            text += "</tr>"
        text += "</table>"
        return text





# Move this to wato.py or valuespec.py as soon as we need this at least
# once again somewhere
def ListOfCAs(**args):
    args.setdefault("title", _("CAs to accept"))
    args.setdefault("help", _("The agent will only use HTTPs connections with a server certificate that "
                 "is signed with one of the CAs that are listed here. That way it is guaranteed "
                 "that it is communicating only with the authentic update server. "
                 "If you use self signed certificates for you server then enter that certificate "
                 "here. <b>Note:</b> The agent uses <b>only</b> the CAs that you upload."))
    args.setdefault("add_label", _("Add new CA certificate or chain"))
    args.setdefault("empty_text", _("You need to enter at least one CA. Otherwise no SSL connection can be made."))
    args.setdefault("allow_empty", False)
    return ListOf(
        CAorCAChain(),
        movable = False,
        **args
    )


register_rule(group,
    "agent_config:cmk_update_agent",
    Dictionary(
        title = _("Install agent updater (Linux, Windows)"),
        help = _("This rule set allows to deploy an agent plugin that updates "
                 "the Check_MK agent for Linux and Windows on a regular base. "
                 "The agent will look for new updates on a regular base and update "
                 "itself, when a new baked agent is available, released and signed. "
                 "<br><b>Note<sup>1</sup>:</b> This update mechanism is baked into the agent itself. "
                 "So after activating this feature you need at least one more manual "
                 "update of the agent, of course."
                 "<br><b>Note<sup>2</sup>:</b> After deploying this new plugin "
                 "you need to call it once manually for registerting the agent "
                 "at your update server. Call <tt>cmk-update-agent -H <i>HOST</i></tt>, "
                 "where <i>HOST</i> must be the name of that host in your monitoring. You "
                 "find the plugin in <tt>/usr/lib/check_mk_agent/plugins</tt> (Linux) and "
                 "in the subdirectory <tt>plugins</tt> of your agent installation directory (Windows)."),
        elements = [
            ( "activated",
              DropdownChoice(
                  title = _("Activation"),
                  help = _("Do not forget to activate the plugin in at least one of your rules. "
                           "It can be useful to create rules that are only partially filled out. "
                           "Since the rule execution is done on a <i>per parameter</i> base "
                           "you can for example create one rule at the top of your list that "
                           "just sets the activation to <i>no</i> for just some of your hosts without "
                           "setting any of the other parameters."),
                  choices = [
                      ( False,  _("No not deploy automatic agent update plugin") ),
                      ( True, _("Deploy plugin that updates agent automatically") ),
                  ]
            )),
            ( "interval",
              Age(
                 title = _("Interval for update check"),
                 help = _("In order to save ressources on the target host and on the update server "
                          "it is recommended to do the update checks not more than once every 10 minutes."),
                 display = [ "days", "hours", "minutes" ],
                 default_value = 3600,
            )),
            ( "server",
               Hostname(
                 title = _("DNS name or IP address of update server"),
                 help = _("This is the name/address at which the agent "
                          "can reach your central monitoring server via HTTPS or HTTP. "
                          "<b>Note</b>: If you are using HTTPS then this name must "
                          "exactly match the common name of the server certificate."),
            )),
            ( "site",
              ID(
                  title = _("Name of Check_MK site of update server"),
                  help = _("You should probably enter <tt>%s</tt> here.") % defaults.omd_site,
                  default_value = defaults.omd_site,
            )),
            ( "protocol",
              CascadingDropdown(
                  title = _("Protocol to use for fetching updates"),
                  help = _("HTTPS is the suggested setting here. It requires a valid SSL "
                           "setup of your Apache webserver on your monitoring server, however. "
                           "When using HTTP then confidential data that is contained in the baked "
                           "agents could be spied out (such as passwords for plugins)."),
                  choices = [
                    ( "http", _("HTTP") ),
                    ( "https", _("HTTPS with the following CAs"), ListOfCAs() ),
                  ],
                  default_value = ( "https", [] ),
            )),
            ( "signature_keys",
              ListChoice(
                  title = _("Signature keys the agent will accept"),
                  help = _("The agent will update itself only with packages that are signed "
                           "by one of these keys. You need to specify at least one key. Keys are "
                           "being create <a href='%s'>here</a>.") % "wato.py?mode=signature_keys",
                  choices = signature_key_choices,
                  no_elements_text = _("You have not created any agent <a href='%s'>signature keys</a>"
                                       " yet. This is neccessary.") % "wato.py?mode=signature_keys",
                  allow_empty = False,
            )),
        ],
    ),
    match = "dict",
)


group = "agents/" + _("Linux Agent")

register_rule(group,
    "agent_config:agent_user",
    ID(
        title = _("Run agent as non-root user (Linux)"),
        help = _("Here you can specify an alternative user that the agent should be run as. "
                 "Please note that only root has enough priviledges for all commands and plugins. "
                 "When run as another user then some information will be missing."),
        allow_empty = False,
        label = _("Linux-User:"),
        default_value = "root",
    ),
    match = "first")


register_rule(group,
    "agent_config:runas",
    ListOf(
        Tuple(
            orientation = "horizontal",
            elements = [
                DropdownChoice(
                    title = _("Script Type"),
                    choices = [
                        ( "plugin", _("Check_MK Agent Plugins-Directory" ) ),
                        ( "local", _("Check_MK Agent Local-Directory") ),
                        ( "mrpe", _("MK Remote Plugin Executor (MRPE)") ),
                    ]
                ),
                TextAscii(
                    title = _("Run as user (empty = root):"),
                    size = 20,
                ),
                TextAscii(
                    title = _("Filename / Directory"),
                    help = _("An absolute path to the filename (in case of MRPE) or to a directory "
                             "in case of local / plugin"),
                    regex = "^/",
                    regex_error = _("Please specify an absolute path name"),
                    size = 40,
                    allow_empty = False,
                )
            ]
        ),
        add_label = _("Add new user configuration"),
        sortable = False,
        title = _("Plugins, local checks and MRPE for non-root users"),
        help = _("This plugin allows you to let users keep their own plugins, local scripts "
                 "and MRPE configuration, which will be run by the agent with the permissions "
                 "of that user."),
    ),
    match = "list")

group = "agents/" + _("Windows Agent")

def get_custom_files_choices():
    choices = []
    custom_dir = custom_file_base_path()
    if not os.path.exists(custom_dir):
        return choices

    for entry in os.listdir(custom_dir):
        if os.path.isdir(custom_dir + entry):
            choices.append( (entry, entry) )
    return choices


register_rule(
    group,
    "agent_config:custom_files",
    Transform(
        ListChoice(
            title = _("Deploy custom files with agent (Windows)"),
            choices = get_custom_files_choices,
        ),
        help = _("This rule allows you to add your own files and folders to the agent package. All you have to do "
                 "is to place them under <i>%s/local/share/check_mk/agents/custom with</i> the following constraints. "
                 "Create a subdirectory for each package you want to deploy. Those packages will be shown in this rule "
                 "as list choice. Within a package folder you can create the following subfolders: <i>lib</i>, <i>bin</i>, "
                 "<i>var</i>, <i>config</i>. These folders are the logical target paths for the files. "
                 "For example, if you put <i>mypackage/lib/myfolder/myfile</i> into the <i>custom</i> directory. "
                 "This rule will show mypackage as list choice option. The subfolder <i>lib</i> will be resolved to "
                 "<i>/usr/lib/check_mk_agent</i> when using a linux agent with the default configuration. "
                 "In the end the file will be located at <i>/usr/lib/check_mk_agent/myfolder/myfile</i>. "
                 "Note: When using the windows agent, all logical paths point to the same location - "
                 "the agent installation dir. Multiple rules can apply, this means that the packages "
                 "of multiple rules are merged. Note: This feature will be available for Linux/Unix soon, hence"
                 "the documentation was written for linux (this example is more clear)." % defaults.omd_root
        ),
        # If a custom dir gets deleted, we also remove the configured path
        forth = lambda choices: [ x for x in choices if (x, x) in get_custom_files_choices() ],
    ),
    None,
    match = "list",
)


register_rule(group,
    "agent_config:crash_debug",
    Checkbox(
        title = _("Crash-Debugging"),
        label = _("Enable Crash-Debugging for Windows Agent"),
        help = _("Enable this option when you experience a sporadic crash "
                 "of the Check_MK agent for Windows. This option will make "
                 "the agent create a file called <tt>crash.log</tt> in case "
                 "of a crash. That file is located in the agent's installation "
                 "directory. It is a text file that will explain in detail "
                 "what was the last action of the agent and probably caused "
                 "the crash."),
        default_value = False,
    ),
    match = "first")

register_rule(group,
    "agent_config:win_exe_suffixes",
    Optional(
        ListOfStrings(
            default_value =  [ "exe", "bat", "vbs", "ps1" ],
            orientation = "horizontal",
            valuespec = TextAscii(
                size = 4,
                regex = "^[a-z0-9]{1,8}$",
                regex_error = _("Please specify the extensions in lower case and without dot."),
            ),
        ),
        title = _("Limit script types to execute"),
        help = _("If you want to run your own custom agent plugins on Windows, "
                  "simply put them into the subdirectory <tt>plugins</tt> "
                  "of your agent's installation directory. <i>Local checks</i> work just the "
                  "same, but are installed in <tt>local</tt>. Plugins that are deployed "
                  "with the agent bakery are installed by the MSI installer into the same "
                  "directory. This ruleset limits the file extensions that the agent "
                  "will run. This for example avoids starting a Notepad when a file with "
                  "the extention <tt>.txt</tt> is found. <b>Note:</b> If you remove some "
                  "of the default extensions, not all of the shipped agent plugins will "
                  "continue to work!<br><br>Specify the extensions in lower case and without "
                  "a leading dot."),
        label = _("Only execute plugins with the following extensions:"),
        none_label = _("Execute all files in the plugins directory"),
    ),
    match = "first")


register_rule(group,
    "agent_config:win_agent_sections",
    ListChoice(
        title = _("Agent sections to be executed"),
        help = _("The Check_MK Agent for Windows has a number of builtin modules "
                 "that provide functionality for various types of checks. Usually "
                 "all those sections are being executed, but here you can select "
                 "which sections you want the agent to compute. Disabling some of "
                 "the sections will make the agent faster but will result in less "
                 "different checks. The most CPU performance is needed by the "
                 "Eventlog monitoring and the performance counters."),
        choices = [
            ("check_mk",         _("General information, Agent version")),
            ("uptime",           _("System uptime")),
            ("systemtime",       _("System time (time synchronization)")),
            ("df",               _("Filesystems (volumes, drives)")),
            ("mem",              _("Memory and Pagefile")),
            ("ps",               _("Currently running processes")),
            ("services",         _("Installed, stopped and running services")),
            ("winperf",          _("Various performance counters")),
            ("logwatch",         _("Windows Event Logs")),
            ("logfiles",         _("Messages in text log files")),
            ("fileinfo",         _("Information about size, age and count of files")),
            ("plugins",          _("Execute plugins in general")),
            ("local",            _("Execute local scripts")),
            ("mrpe",             _("Execute legacy monitoring plugins")),
            ("spool",            _("Asynchronously spooled check results")),
            ("wmi_cpuload",      _("CPU Load via WMI")),
            ("msexch",           _("MS Exchange counters (various)")),
            ("webservices",      _("Web Services")),
            ("dotnet_clrmemory", _(".Net/CLR Memory")),
            ("ohm",              _("Hardware Sensors via OpenHardwareMonitor"))
        ],
        default_value = [
               "check_mk", "uptime", "df", "ps", "mem", "services", "winperf",
               "logwatch", "logfiles", "systemtime", "plugins", "local",
               "spool", "mrpe", "fileinfo", "wmi_cpuload", "msexch", "webservices",
               "dotnet_clrmemory", "ohm"
        ],
    ),
)

register_rule(group,
    "agent_config:win_script_async",
    DropdownChoice(
        title = _("Asynchronous execution of plugins"),
        help = _("Per default all plugins and local scripts are being "
                 "run sequentially. You choose to run all asynchronous plugins "
                 "in parallel. This can cut down the execution time of the agent "
                 "in situations where several complex scripts are being run "
                 "in the background - at the cost of more CPU usage."),
        choices = [
            ( "sequential", _("Run asynchronous plugins sequential") ),
            ( "parallel",   _("Run asynchronous plugins in parallel") ),
        ],
        default_value = "sequential",
    ),
)

register_rule(group,
    "agent_config:win_script_timeout",
    Dictionary(
        title = _("Set timeouts for plugins and local checks"),
        help = _("A timeout on plugins and local checks keeps those checks from staling the "
                 "whole agent. If no timeout is configured here, the default is 60 seconds."),
        elements = [
            ( "type", DropdownChoice(
                title = _("Type"),
                help = _("Choose if this rule applies to plugins or local checks"),
                choices = [
                    ( "plugin", _("Plugin" ) ),
                    ( "local", _("Local") ),
                ]
            )),
            ( "pattern", TextAscii(
                title = _("Script Pattern"),
                help = _("The pattern (wildcards supported) by which to select the affected "
                            "scripts."),
                allow_empty = False,
                default_value = "*",
            )),
            ( "timeout", Integer(
                title = _("Timeout"),
                unit = _("Seconds"),
                default_value = 60,
                help = _("Timeout in seconds before the script process is terminated")
            )),
        ],
        optional_keys = [],
    ),
    match = "all",
)

register_rule(group,
    "agent_config:win_script_cache_age",
    Dictionary(
        title = _("Set cache age for plugins and local checks"),
        help = _("A cache age limit on plugins and local checks. If set, the output of plugins "
                 "is cached and reused until this much time has passed."),
        elements = [
            ( "type", DropdownChoice(
                title = _("Type"),
                help = _("Choose if this rule applies to plugins or local checks"),
                choices = [
                    ( "plugin", _("Plugin" ) ),
                    ( "local", _("Local") ),
                ]
            )),
            ( "pattern", TextAscii(
                title = _("Script Pattern"),
                help = _("The pattern (wildcards supported) by which to select the affected "
                            "scripts."),
                allow_empty = False,
                default_value = "*",
            )),
            ( "cache_age", Integer(
                title = _("Max Cache Age"),
                unit = _("Seconds"),
                default_value = 0,
                help = _("If this is set, the plugin output will be cached and will only "
                            "be refreshed once the age of the cache has exceeded "
                            "this value."
                         )
            )),
        ],
        optional_keys = [],
    ),
    match = "all"
)

register_rule(group,
    "agent_config:win_script_retry_count",
    Dictionary(
        title = _("Set retry count for plugins and local checks"),
        help = _("If set, the plugin may be called repeately - up to this many "
                 "times or until it finishes successfully."),
        elements = [
            ( "type", DropdownChoice(
                title = _("Type"),
                help = _("Choose if this rule applies to plugins or local checks"),
                choices = [
                    ( "plugin", _("Plugin" ) ),
                    ( "local", _("Local") ),
                ]
            )),
            ( "pattern", TextAscii(
                title = _("Script Pattern"),
                help = _("The pattern (wildcards supported) by which to select the affected "
                            "scripts."),
                allow_empty = False,
                default_value = "*",
            )),
            ( "retry_count", Integer(
                title = _("Retry Count"),
                default_value = 0,
                help = _("Number of retries (not counting the initial regular run)")
            )),
        ],
        optional_keys = [],
    ),
    match = "all"
)


register_rule(group,
    "agent_config:win_script_execution",
    Dictionary(
        title = _("Set execution mode for plugins and local checks"),
        help = _("Some scripts are slow and can therefore stall the execution of the agent. "
                 "To remedy this, such scripts may be run asynchronously. With this rule "
                 "asynchronous execution can be set on a per-script basis."),

        elements = [
            ( "type", DropdownChoice(
                title = _("Type"),
                help = _("Choose if this rule applies to plugins or local checks"),
                choices = [
                    ( "plugin", _("Plugin") ),
                    ( "local", _("Local") ),
                ]
            )),
            ( "pattern", TextAscii(
                title = _("Script Pattern"),
                help = _("The pattern (wildcards supported) by which to select the affected "
                            "scripts."),
                allow_empty = False,
                default_value = "*",
            )),
            ( "execution", DropdownChoice(
                title = _("Execution Mode"),
                help = _("Selects how parallelizable scripts are executed"),
                choices = [
                    ( "sync", _("Synchronous") ),
                    ( "async", _("Asynchronous") ),
                ]
            )),
        ],
        optional_keys = [],
    ),
    match = "all"
)

register_rule(group,
    "agent_config:win_openhardwaremonitor",
    DropdownChoice(
        title = _("OpenHardwareMonitor (Windows)"),
        help = _("Adds a headless version of the OpenHardwareMonitor to the windows agent. "
                 "The agent will then automatically use this to provide readings of hardware "
                 "sensors (temperature, fans, ...) to check_mk."
                 "This does require .Net to be installed on the target system. Please leave this "
                 "disabled if you have a different way to monitor sensors. You also don't need "
                 "this if you are running the regular OpenHardwareMonitor software."),
        choices = [
            ( True, _("Deploy OpenHardwareMonitor (headless)") ),
            ( None, _("Do not deploy OpenHardwareMonitor (headless)") ),
        ]
    )
)


group = "agents/" + _("Agent Plugins")

register_rule(group,
    "agent_config:jar_signature",
    Alternative(
        title = _("Signatures of certificates in JAR files"),
        help = _("This plugin can be used to check the remaining life time "
                 "of SSL certificates that are contained in JAVA JAR files. "
                 "The tool <tt>jarsigner -verify</tt> is being called for "
                 "that purpose."),
        style = "dropdown",
        elements = [
            Tuple(
                title = _("Deploy plugin for JAR signatures"),
                elements = [
                    TextAscii(
                        title = _("<tt>JAVA_HOME</tt> - path to Java installation"),
                        size = 80,
                        allow_empty = False),
                    ListOfStrings(
                        title = _("Path-patterns where to search for Jar files"),
                        help = _("You can use <tt>*</tt> and <tt>?</tt> here."),
                        regex = "^/[^ \t]+$",
                        regex_error = _("File patterns must begin with <tt>/</tt> and must not contain spaces."),
                        size = 80,
                        allow_empty = False,
                    )
                ]
            ),
            FixedValue(None, title = _("Do not deploy plugin for JAR signatures"), totext = _("(disabled)")),
        ],
        default_value = (
            "/home/oracle/bin/jdk_latest_version",
            [ "/home/oracle/fmw/11gR2/as_1/forms/java/*.jar" ])
    )
)

register_rule(group,
    "agent_config:win_logwatch",
    Dictionary(
        title = _("Text logfiles (Windows)"),
        elements = [
            ( "logfiles",
                ListOfStrings(
                    title = _("Logfile Names"),
                    size = 50,
                    help  = _("Define textfiles to be monitored. The full path is necessary."),
                    allow_empty = False,
            )),
            ( "patterns",
                ListOf(
                    Tuple(
                        title = _("Patterns"),
                        elements = [
                            DropdownChoice(
                                choices = [
                                  ( "ok",       _("OK") ),
                                  ( "warn",     _("Warning") ),
                                  ( "crit",     _("Critical") ),
                                  ( "ignore",   _("Ignore") ),
                                ],
                            ),
                            TextUnicode( title = _("Matching Pattern") ),
                        ],
                        orientation = "horizontal",
                    ),
                    title = _("Patterns"),
                    help = _("Patterns for matching lines in the specified logfiles"),
            )),
        ],
        optional_keys = None,
    ),
    match = "all",
)


register_rule(group,
    "agent_config:mailman_lists",
    DropdownChoice(
        title = _("Mailman mailing lists queues (Linux)"),
        help = _("The plugin <tt>mailman_lists</tt> allows monitoring mailing lists of mailman."),
        choices = [
            ( {},   _("Deploy Mailman plugin") ),
            ( None, _("Do not deploy Mailman plugin") ),
        ]
    )
)

register_rule(group,
    "agent_config:windows_updates",
    DropdownChoice(
        title = _("Windows Updates"),
        help = _("The Windows Updates plugin checks for the number of pending updates"),
        choices = [
            ( True,   _("Deploy Windows Updates plugin") ),
            ( False, _("Do not deploy Windows Updates plugin") ),
        ]
    )
)

register_rule(group,
    "agent_config:isc_dhcpd",
    DropdownChoice(
        title = _("ISC DHCP-Daemon (Linux)"),
        help = _("The plugin <tt>isc_dhcpd</tt> collects information about a DHCP server daemon on Linux."),
        choices = [
            ( {},   _("Deploy ISC DHCP-Daemon plugin") ),
            ( None, _("Do not deploy ISC DHCP-Daemon plugin") ),
        ]
    )
)

register_rule(group,
    "agent_config:lnx_quota",
    DropdownChoice(
        title = _("User quotas on filesystems (Linux)"),
        help = _("Hosts configured via this rule get the <tt>lnx_quota</tt> plugin "
                 "deployed. This is only useful if you have filesystem quotas setup "
                 "on the host."),
        choices = [
            ( True, _("Deploy Linux filesystem quota plugin") ),
            ( None, _("Do not deploy Linux filesystem quota plugin") ),
        ]
    )
)

register_rule(group,
    "agent_config:mk_logins",
    DropdownChoice(
        title = _("Number of logged in users (Linux)"),
        help = _("Hosts configured via this rule get the <tt>mk_logins</tt> plugin "
                 "deployed."),
        choices = [
            ( True, _("Deploy logged in users plugin") ),
            ( None, _("Do not deploy logged in users plugin") ),
        ]
    )
)

register_rule(group,
    "agent_config:winperf",
    ListOf(
        Tuple(
            elements = [
                TextAscii(
                    title = _("Section name"),
                    help = _("Choose a name here. If you choose <tt>foo</tt> then section header and thus the "
                             "name of the required check plugin will be &lt;&lt;&lt;winperf_foo&gt;&gt;&gt;."),
                    regex = "^[a-z]+[a-z0-9_]*$",
                    regex_error = _("Use only lower case letters, digits, and underscores here."),
                    size = 20,
                ),
                TextUnicode(
                    title = _("Counter-object name or number"),
                    size = 40,
                    allow_empty = False,
                    forbidden_chars = "\\",
                )
            ]
        ),
        add_label = _("Add Counter Object"),
        title = _("Windows Performance-Counter objects"),
        help = _("Here you can configure which performance counter objects the Windows agent "
                 "should extract and provide to the Check_MK server. You can either specify "
                 "counter numbers or names. Note: numbers might be host-specific. Names are "
                 "language-specific. Also note that you need to specify names of counter <i>objects</i>, "
                 "not of single counters. In order to make use of the counters you need a "
                 "matching check plugin on the Check_MK server."),
     )
)

register_rule(group,
    "agent_config:win_eventlog",
    Dictionary(
        title = _("Finetune Windows Eventlog monitoring"),
        elements = [
            ( "logfiles",
              ListOf(
                  Tuple(
                      elements = [
                          OptionalDropdownChoice(
                              choices = [ ( "*", _("All Eventlogs") ) ],
                              otherlabel = _("Eventlog:"),
                              explicit = TextUnicode(
                                  allow_empty = False,
                                  size = 20,
                              ),
                          ),
                          DropdownChoice(
                              choices = [
                                  ( "warn", _("WARN/CRIT") ),
                                  ( "crit", _("CRIT") ),
                                  ( "all",  _("ALL") ),
                                  ( "off",  _("disable") ),
                              ],
                              default_value = "warn",
                          ),
                          DropdownChoice(
                              choices = [
                                  (True, _("with context")),
                                  (False, _("without context")),
                              ]
                          ),
                      ],
                      orientation = "horizontal",
                  ),
                  movable = True,
                  title = _("Configuration of individual Eventlogs"),
                  help = _("Here you can configure which messages should be discovered and "
                           "reported from which Eventlog. When you set the severity to <i>disabled</i> "
                           "then this Eventlog will totally be ignored. The switch <i>without context</i> "
                           "will disable sending so-called context messages. When at least one new relevant "
                           "message is being found in an Eventlog, the agent usually sends <i>all</i> "
                           "new messages from that check cycle. This makes it more convenient when viewing "
                           "the messages in Check_MK. In high-volume logfiles it might be neccessary "
                           "to switch the context off in order to save networking and disk ressources. "
                           "<b>Note</b>: When several configuration tuning entries match for a logfile "
                           "then only the first one is being executed. So it does not make sense to "
                           "create more then one entry for <i>All Eventlogs</i>."),
                  add_label = _("Add tuning configuration"),
              )
            ),
            ( "sendall",
              Checkbox(title = _("Historic messages"),
                       label = _("Always send <b>all</b> messages, even old ones (just for testing!)"),
                       help = _("This is just for testing! The agent will then always send "
                                "all historic messages, not just the new ones. This leads "
                                "to enormous amounts of data.")),
            ),
        ],
        optional_keys = False,
    )
)

register_rule(group,
    "agent_config:fileinfo",
    ListOfStrings(
        title = _("Count, size and age of files (Linux, Windows)"),
        size = 50,
        help = _("Here you can specify a list of filename patterns to be sent by the "
                 "agent in the section <tt>fileinfo</tt>. Use globbing patterns like "
                 "<tt>C:\\foo\\*.log</tt> or <tt>/var/*/*.log</tt> here. Per default each found file "
                 "will be monitored for size and age. By building groups you can alternatively "
                 "monitor a collection of files as an entity and monitor the count, total size, the largest, "
                 "smallest oldest or newest file. Note: if you specify more than one matching rule, then "
                 "<b>all</b> matching rules will be used for defining pattern - not just the first one."),
    ),
    match = "list")

register_rule(group,
    "agent_config:smart",
    DropdownChoice(
        title = _("SMART harddisk monitoring (Linux)"),
        help = _("Hosts configured via this rule get the <tt>smart</tt> plugin "
                 "deployed. Assuming you have installed <tt>smartmontools</tt>, "
                 "your local harddisks will be monitored for temperature and errors."),
        choices = [
            ( True, _("Deploy SMART plugin") ),
            ( None, _("Do not deploy SMART plugin") ),
        ]
    )
)

register_rule(group,
    "agent_config:vxvm",
    DropdownChoice(
        title = _("Veritas Volume Manger (Linux, HP-UX)"),
        help = _("This plugin monitors various aspects of VxVM."),
        choices = [
            ( True, _("Deploy VxVM plugin") ),
            ( None, _("Do not deploy VxVM plugin") ),
        ]
    )
)

register_rule(group,
    "agent_config:ad_replication",
    DropdownChoice(
        title = _("Active Directory Replication (Windows)"),
        help = _("This plugin checks the replication of Active Directory. "
                  "To be able to run this check you need appropriate credentials "
                  "in the target domain. Normally the Check_MK agent runs as sevice "
                  "with local system credentials which are not sufficient for this check. "
                  "To solve this problem you can do e.g. change the account the service "
                  "is being started with to a domain user account with enough "
                  "permissions on the DC."),
        choices = [
            ( True, _("Deploy AD-Replication plugin") ),
            ( None, _("Do not deploy AD-Replication plugin") ),
        ]
    )
)

register_rule(group,
    "agent_config:windows_tasks",
    DropdownChoice(
        title = _("Windows Task Scheduler"),
        help = _('This plugin checks the last return state of Windows task scripts'),
        choices = [
            ( True, _("Deploy plugin for Windows Task Scheduler") ),
            ( None, _("Do not deploy plugin for Windows Task Scheduler") ),
        ]
    )
)

register_rule(group,
    "agent_config:winperf_if",
    DropdownChoice(
        title = _("Network interfaces on Windows"),
        help = _("This plugin checks the status and performance of network interfaces on Windows. "
                 "Use the legacy plugin for Windows versions without Powershell"),
        choices = [
            ( "ps1", _("Deploy powershell plugin - recommended for Win2k8 upwards") ),
            ( "bat", _("Deploy batch file plugin - recommended for WinXP / Win2k upwards") ),
            ( False, _("Do not deploy plugin for Windows interfaces") ),
        ]
    )
)


register_rule(group,
    "agent_config:windows_broadcom_bonding",
    DropdownChoice(
        title = _("Broadcom bonding network interface on Windows"),
        help = _('This plugin checks the current state of a Windows broadcom bonding interface.'),
        choices = [
            ( True, _("Deploy plugin for Windows broadcom bonding") ),
            ( None, _("Do not deploy plugin for Windows broadcom bonding") ),
        ]
    )
)

mssql_authentication_choices = [
    ("system", _("System Authentication")),
    ("db", _("Database User Credentials"), Tuple(
        elements = [
            TextAscii(
                title = _("User ID"),
                default_value = "monitoring",
            ),
            Password(
                title = _("Password")
            ),
        ]
    )),
]

register_rule(group,
    "agent_config:mssql",
    Transform(
        Alternative(
            title = _("Microsoft SQL Server (Windows)"),
            help = _('This plugin can be used to collect information of all running MSSQL servers '
                     'on the local system. '
                     'The current implementation of the check uses the "trusted authentication" '
                     'where no user/password needs to be created in the MSSQL server instance by '
                     'default. Using this method, you needed to grant the user as which the Check_MK '
                     'windows agent service is running access to the MSSQL database. Otherwise you '
                     'can configure the credentials of a database user which has the permission to '
                     'read the needed information from the server instance.'),
            style = "dropdown",
            elements = [
                Dictionary(
                    title = _("Deploy MSSQL Server plugin"),
                    elements = [
                        ("auth_default", CascadingDropdown(
                            title = _("Authentication (defaults)"),
                            choices = mssql_authentication_choices,
                        )),
                        ("auth_instances", ListOf(
                            Tuple(
                                elements = [
                                    TextAscii(
                                        title = _("Instance ID"),
                                    ),
                                    CascadingDropdown(
                                        title = _("Authentication (defaults)"),
                                        choices = mssql_authentication_choices,
                                    ),
                                ],
                            ),
                            allow_empty = False,
                            title = _("Authentication (instance specific)"),
                        )),
                    ],
                    optional_keys = ["auth_instances"],
                ),
                FixedValue(None, title = _("Do not deploy plugin for Microsoft SQL Server"), totext = _("(disabled)")),
            ],
            default_value = {
                "auth_default": "system",
            }
        ),
        forth = lambda x: x == True and {"auth_default":"system"} or x,
    )
)

register_rule(group,
    "agent_config:citrix_licenses",
    DropdownChoice(
        title = _("Citrix Licenses (Windows)"),
        help = _("This plugin checks for Citrix Licenses on Window hosts."),
        choices = [
            ( True, _("Deploy plugin for Citrix licenses") ),
            ( None, _("Do not deploy plugin for Citrix licenses") ),
        ]
    )
)

register_rule(group,
    "agent_config:citrix_xenapp",
    DropdownChoice(
        title = _("Citrix XenApp (Windows)"),
        help = _("This plugin monitors Citrix sessions and server load on Window hosts."),
        choices = [
            ( True, _("Deploy plugin for Citrix XenApp") ),
            ( None, _("Do not deploy plugin for Citrix XenApp") ),
        ]
    )
)

register_rule(group,
    "agent_config:hyperv_vms",
    DropdownChoice(
        title = _("Hyper-V VMs (Windows)"),
        help = _("This plugin monitors the state of Hyper-V VMs on Window hosts."),
        choices = [
            ( True, _("Deploy plugin for Hyper-V VMs") ),
            ( None, _("Do not deploy plugin for Hyper-V VMs") ),
        ]
    )
)

register_rule(group,
    "agent_config:arcserve_backup",
    Alternative(
        title = _("Arcserve (German) backups (Windows)"),
        help = _("This plugin monitors Arcserve backups by deploying a plugin for the "
                 "Arcserve server on Windows. This only supports the German version of "
                 "Arcserve, since the log messages are in localized language."),
        style = "dropdown",
        elements = [
            Dictionary(
                title = _("Deploy Arcserve plugin"),
                elements = [
                    ( "sqlserver",
                      TextAscii(
                          title = _("SQL-Server to connect to"),
                          help = _("Put the name of the database here, e.g. SATURN\\ARCSERVE_DB"),
                          allow_empty = False,
                          regex = r"^[A-Za-z0-9_\\]+$",
                          regex_error = _("You have used an invalid character"),
                    )),
                ],
                optional_keys = False,
            ),
            FixedValue(None, title = _("Do not deploy the Arcserve plugin"), totext = _("(disabled)") ),
        ],
        default_value = { "sqlserver" : r"SATURN\ARCSERVE_DB" },
    )
)

register_rule(group,
    "agent_config:win_megaraid",
    Alternative(
        title = _("MegaRAID monitoring (Windows)"),
        help = _("This plugin uses the command line tool <tt>MegaCli.exe</tt> in order to provide "
                 "monitoring information of LSI RAID controllers and attached hard disks. You can "
                 "download this tool from <a href='http://www.lsi.com/'>LSI</a>."),
        style = "dropdown",
        elements = [
            Dictionary(
                title = _("Deploy MegaRAID plugin"),
                elements = [
                    ( "megacli",
                      TextAscii(
                          title = _("Path to <tt>MegaCLI.exe</tt>"),
                          size = 64,
                          allow_empty = False,
                    )),
                    ( "tempdir",
                      TextAscii(
                          title = _("Path to temporary directory (will be created)"),
                          allow_empty = False,
                    )),
                ],
                optional_keys = False,
            ),
            FixedValue(None, title = _("Do not deploy the MegaRAID plugin"), totext = _("(disabled)") ),
        ],
        default_value = {
            "megacli" : r"C:\Program Files\LSI Corporation\MegaCLI\MegaCli.exe",
            "tempdir" : r"C:\Temp",
        },
    )
)


register_rule(group,
    "agent_config:websphere_mq",
    DropdownChoice(
        title = _("IBM WebSphere MQ (Linux)"),
        help = _("This plugin enables checks for channels and queues of IBM WebSphere MQ."),
        choices = [
            ( True, _("Deploy IBM WebSphere MQ plugin") ),
            ( None, _("Do not deploy IBM WebSphere MQ plugin") ),
        ]
    )
)
register_rule(group,
    "agent_config:unitrends",
    DropdownChoice(
        title = _("Unitrends backup and replication plugins (Linux)"),
        help = _("This ruleset is for deploying two plugins for monitoring Unitrends software."),
        choices = [
            ( True, _("Deploy Unitrends plugins") ),
            ( None, _("Do not deploy Unitrends plugins") ),
        ]
    )
)

register_rule(group,
    "agent_config:lvm",
    DropdownChoice(
        title = _("LVM, Logical Volume Groups (Linux)"),
        choices = [
            ( True, _("Deploy LVM plugin") ),
            ( None, _("Do not deploy LVM plugin") ),
        ]
    )
)

register_rule(group,
    "agent_config:nfsexports",
    DropdownChoice(
        title = _("NFS4 exports (Linux)"),
        help = _("This plugin monitors the NFS version 4 exports of a Linux host."),
        choices = [
            ( True, _("Deploy NFS4 exports plugin") ),
            ( None, _("Do not deploy NFS4 exports plugin") ),
        ]
    )
)

register_rule(group,
    "agent_config:netstat",
    DropdownChoice(
        title = _("Established TCP/UDP connections"),
        help = _("This plugin uses the command <tt>netstat</tt> in order to output information "
                 "about established TCP (and UDP) connections. The according check then allows "
                 "you to make sure that certain TCP connections are always established."),
        choices = [
            ( True, _("Deploy netstat plugin") ),
            ( None, _("Do not deploy netstat plugin") ),
        ]
    )
)

register_rule(group,
    "agent_config:symantec_av",
    DropdownChoice(
        title = _("Symantec Anti Virus (Linux)"),
        help = _("Here you can deploy a plugin for monitoring the quarantine queue, tasks and updates of Symantec Anti Virus."),
        choices = [
            ( True, _("Deploy Symantec Anti Virus plugin") ),
            ( None, _("Do not deploy Symantec Anti Virus plugin") ),
        ]
    )
)

register_rule(group,
    "agent_config:kaspersky_av",
    DropdownChoice(
        title = _("Kaspersky Anti-Virus (Linux)"),
        help = _("Here you can deploy a plugin for monitoring the quarantine queue, tasks and updates of Kaspersky Anti Virus."),
        choices = [
            ( True, _("Deploy Kaspersky Anti-Virus plugin") ),
            ( None, _("Do not deploy Kaspersky Anti-Virus plugin") ),
        ]
    )
)

register_rule(group,
    "agent_config:win_dmidecode",
    DropdownChoice(
        title = _("System Information for Inventory via dmidecode (Windows)"),
        help = _("This plugin requires the executable <tt>C:\Programme\GnuWin32\sbin\dmidecode.exe</tt>. "
                 "This is the default location when installing this tool. You can download it for free "
                 "from <a href='http://gnuwin32.sourceforge.net/packages/dmidecode.htm'>here</a>. "),
        choices = [
            ( True, _("Deploy plugin for Windows System Info") ),
            ( None, _("Do not deploy plugin for Windows System Info") ),
        ]
    )
)

register_rule(group,
    "agent_config:win_dhcp_pools",
    DropdownChoice(
        title = _("DHCP pools (Windows)"),
        choices = [
            ( True, _("Deploy plugin for Windows DHCP pools") ),
            ( None, _("Do not deploy plugin for Windows DHCP pools") ),
        ]
    )
)

register_rule(group,
    "agent_config:mk_inventory",
    Transform(
        Alternative(
            title = _("Hardware/Software-Inventory (Linux, Windows)"),
            help = _("If you activate this option, the agent plugin <tt>mk_inventory</tt> will be deployed on "
                     "linux and windows hosts. It gathers information about installed hardware and software and makes the "
                     "information available in Multisite and for exporting to third-party software. <b>Note:</b> "
                     "in order to actually use the inventory for a host you also need to enable it in "
                     "the ruleset <a href='wato.py?varname=active_checks%3Acmk_inv&folder=&mode=edit_ruleset'>"
                     "Hardware/Software-Inventory / Do hardware/software Inventory</a>."),
            style = "dropdown",
            elements = [
                Dictionary(
                    title = _("Deploy the Check_MK Inventory plugin"),
                    elements = [
                         ( "interval",
                            Age(
                                title = _("Interval for collecting data"),
                         )),
                         ( "exe_paths",
                           ListOfStrings(
                               title = _("Directories to search for <tt>EXE</tt> files (Windows)"),
                               size = 64,
                         )),
                         ( "reg_paths",
                           ListOfStrings(
                               title = _("Registry Keys to search for software (Windows)"),
                               size = 64,
                         )),
                    ],
                    optional_keys = [ "exe_paths", "reg_paths" ],
                ),
                FixedValue(None, title = _("Do not deploy the Check_MK Inventory plugin"), totext = _("(disabled)") ),
            ],
            default_value = {
                "interval" : 14400,
                # "exe_paths" : [ r'', ],
                "reg_paths" : [
                    r'Software\Microsoft\Windows\CurrentVersion\Uninstall',
                    r'Software\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall',
                ],
            }
        ),
        forth = lambda conf: type(conf) == int and {
            "interval" : conf,
            #"exe_paths" : [ "" ],
            "reg_paths" : [
                r'Software\Microsoft\Windows\CurrentVersion\Uninstall',
                r'Software\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall',
            ],
        } or conf,
    ),
)

register_rule(group,
    "agent_config:mk_apt",
    Alternative(
        title = _("APT normal and security updates (Linux)"),
        help = _("This will deploy the agent plugin <tt>mk_apt</tt>. This will activate the "
                 "check <tt>apt</tt> on DEB-based hosts (like Debian and Ubuntu) and monitor pending normal and security updates."),
        style = "dropdown",
        elements = [
            Dictionary(
                title = _("Deploy the APT plugin"),
                elements = [
                    ( "interval",
                      Age(title = "Interval for checking for updates"),
                    ),
                    ( "method",
                      DropdownChoice(
                          title = _("Method"),
                          choices = [
                              ("upgrade", _("apt-get upgrade")),
                              ("dist-upgrade", _("apt-get dist-upgrade")),
                          ]
                    )),
                    ( "update",
                      Checkbox(
                          title = _("Update package database"),
                          label = _("Do an <tt>apt-get update</tt> before the check"),
                    )),
                ],
                optional_keys = False,
            ),
            FixedValue(None, title = _("Do not deploy the APT plugin"), totext = _("(disabled)")),
        ],
        default_value = { "interval": 86400, "method" : "upgrade", "update" : True },
    )
)

register_rule(group,
    "agent_config:mk_zypper",
    Alternative(
        title = _("Zypper normal and security updates (Linux)"),
        help = _("This will deploy the agent plugin <tt>mk_zypper</tt>. This will activate the "
                 "check <tt>zypper</tt> on SUSE Linux hosts and monitor normal and security updates."),
        style = "dropdown",
        elements = [
            Age(
                title = _("Deploy the Zypper plugin"),
                label = _("Interval for checking for updates:"),
            ),
            FixedValue(None, title = _("Do not deploy the Zypper plugin"), totext = _("(disabled)")),
        ],
        default_value = 14400,
    )
)

register_rule(group,
    "agent_config:plesk",
    Alternative(
        title = _("Plesk backups and domains (Linux)"),
        help = _("This will deploy the two agent plugins <tt>plesk_backup</tt> and <tt>plesk_domain</tt>, used "
                 "for monitoring Plesk."),
        style = "dropdown",
        elements = [
            Age(
                title = _("Deploy the plesk plugins"),
                label = _("Interval for checking Plesk backups:"),
            ),
            FixedValue(None, title = _("Do not deploy the plesk plugins"), totext = _("(disabled)")  ),
        ],
        default_value = 3600,
    )
)


register_rule(group,
    "agent_config:dnsclient",
    Alternative(
        title = _("Local DNS resolving (Linux, Unix)"),
        help = _("This plugin tests the local DNS resolver by looking up one "
                 "or several host names using <tt>nslookup</tt>. That tool is expected "
                 "to be installed on the target machine."),
        style = "dropdown",
        elements = [
            ListOfStrings(
                title = _("Hostnames to resolve"),
                allow_empty = False,
                valuespec = TextAscii(
                    regex = "^[-a-zA-Z0-9._]*$",
                    regex_error = _("Your hostname has an invalid format."),
                    allow_empty = False,
                )
            ),
            FixedValue(None, title = _("Do not deploy the DNS client plugin"), totext = _("(disabled)") ),
        ]
    )
)

register_rule(group,
    "agent_config:mk_tsm",
    Alternative(
        title = _("TSM - IBM Tivoli Storage Manager (Linux, Unix)"),
        help = _("This will deploy the agent plugin <tt>mk_tsm</tt>. It will provide several checks "
                 "concerning TSM."),
        style = "dropdown",
        elements = [
            Dictionary(
                title = _("Deploy the TSM plugin"),
                elements = [
                   ( "user",     TextAscii(title = _("Username for login"), allow_empty = False )),
                   ( "password", Password(title = _("Password for login"), allow_empty = False )),
                   ( "interval", Age(title = _("Run asynchronously"), label = _("Interval for collecting data"), default_value = 300 )),
                ],
                optional_keys = [ "interval" ],
            ),
            FixedValue(None, title = _("Do not deploy the TSM plugin"), totext = _("(disabled)") ),
        ]
    )
)

register_rule(group,
    "agent_config:mk_mysql",
    Alternative(
        title = _("MySQL Databases"),
        help = _("This will deploy the agent plugin <tt>mk_mysql</tt> on linux systems and "
                 "<tt>mk_mysql.vbs</tt> on windows systems for monitoring several aspects "
                 "of MySQL databases."),
        style = "dropdown",
        elements = [
            Dictionary(
                title = _("Deploy the MySQL plugin"),
                elements = [
                    ("credentials", Tuple(
                        title = _("Credentials to access the Database"),
                        elements = [
                            TextAscii(
                                title = _("User ID"),
                                default_value = "monitoring",
                            ),
                            Password(
                                title = _("Password")
                            ),
                        ]
                    )),
                    ("socket", TextAscii(
                                title=_("Socket"),
                                default_value = "/var/run/mysqld/mysqld.sock")),
                    ("interval", Age(
                        title = _("Run asynchronously"),
                        label = _("Interval for collecting data"),
                        default_value = 300
                    )),
                ],
            ),
            FixedValue(None, title = _("Do not deploy the MySQL plugin"), totext = _("(disabled)") ),
        ]
    )
)

register_rule(group,
    "agent_config:db2_mem",
    DropdownChoice(
        title = _("Memory usage of DB2 (Linux, AIX, Solaris)"),
        help = _("Hosts configured via this rule get the plugin <tt>db2_mem</tt> "
                 "deployed. This will create one service for the memory consumption "
                 "of every running DB2 instance found on the system."),
        choices = [
            ( True, _("Deploy DB2 memory plugin") ),
            ( None, _("Do not deploy DB2 memory plugin") ),
        ]
    )
)

register_rule(group,
    "agent_config:mk_postgres",
    DropdownChoice(
        title = _("PostgreSQL database and sessions (Linux)"),
        help = _("Deploy the agent plugin <tt>mk_postgres</tt>. This "
                 "will create information neccessary for the checks <tt>postgres_sessions</tt> "
                 "and <tt>postgres_stat_database</tt>."),
        choices = [
            ( True, _("Deploy PostgreSQL plugin") ),
            ( None, _("Do not deploy PostgreSQL plugin") ),
        ]
    )
)

register_rule(group,
    "agent_config:mk_mongodb",
    DropdownChoice(
        title = _("MongoDB (Linux)"),
        help = _("Deploy the agent plugin <tt>mk_mongodb</tt>."),
        choices = [
            ( True, _("Deploy MongoDB plugin") ),
            ( None, _("Do not deploy MongoDB plugin") ),
        ]
    )
)

# Used twice, therefore we define it here
mk_jolokia_elements = [
   ( "protocol",
     DropdownChoice(
         title = _("Protocol"),
         choices = [
            ( "http",  _("HTTP") ),
            ( "https", _("HTTPS") ),
         ],
         default_value = "http",
     )
   ),
   ( "server",
     IPv4Address(
         title = _("IP address of JVM webserver"),
         help = _("Use <tt>127.0.0.1</tt> should be correct in almost every situation. "
                  "It is possible - though uncommon - to do a remote monitoring here."),
         default_value = "127.0.0.1",
     )
   ),
   ( "port",
     Integer(
         title = _("TCP port for connection"),
         default_value = 8080,
         minvalue = 1,
         maxvalue = 65535,
     )
   ),
   ( "login",
       Tuple(
           title = _("Optional login (if required)"),
           elements = [
             TextAscii(
                 title = _("User ID for web login (if login required)"),
                 default_value = "monitoring",
             ),
             Password(
                 title = _("Password for this user")
             ),
             DropdownChoice(
                 title = _("Login mode"),
                 choices = [
                    ( "basic",  _("HTTP Basic Authentication") ),
                    ( "digest", _("HTTP Digest") ),
                 ]
             )
           ]
       )
   ),
   ( "suburi",
     TextAscii(
         title = _("relative URI under which Jolokia is visible"),
         default_value = "jolokia",
         size = 30,
     )
   ),
   ( "instance",
     TextUnicode(
         title = _("Name of the instance in the monitoring"),
         help = _("If you do not specify a name here, then the TCP port number "
                  "will be used as an instance name.")
     ),
   ),
]

register_rule(group,
    "agent_config:mk_jolokia",
    Alternative(
        title = _("JMX monitoring of Java JVMs using Jolokia"),
        help = _("This will deploy and configure the Check_MK agent plugin <tt>mk_jolokia</tt>. "
                 "In order to use this you need the Jolokia "
                 "<tt>.war</tt> file from the <a href='http://www.jolokia.org/index.html'>"
                 "Jolokia Project Homepage</a> deployed in your JVMs. Currently at least "
                 "Tomcat, JBoss and BEA Weblogic are supported. Please note: you need both "
                 "the Check_MK <tt>mk_jolokia</tt> plugin <b>and</b> the WAR file."),
        style = "dropdown",
        elements = [
           Dictionary(
               title = _("Deploy the Jolokia plugin"),
               elements = mk_jolokia_elements + [
                   ( "instances",
                      ListOf(
                        Dictionary(
                            elements = mk_jolokia_elements,
                        ),
                        title = _("Monitor multiple JMX instances on the host"),
                        add_label = _("Add instance to be monitored"),
                        allow_empty = False,
                        help = _("You just need to specify values that are different from "
                                 "the default values that you've specified above."),
                      )
                   )
               ]
           ),
           FixedValue(
               None,
               title = _("Do not deploy the Jolokia plugin"),
               totext = _("(disabled)")),
        ]
    )
)


register_rule(group,
    "agent_config:mk_sap",
    Alternative(
        title = _("SAP R/3 Monitoring plugin"),
        help = _("This ruleset will deploy the agent plugin <tt>mk_sap</tt> for (locally) monitoring "
                 "SAP R/3 instances. Note: you still need to manually deploy the SAP NetWeaver RFCSDK (nwrfcsdk) "
                 "and the Python module sapnwrfc. Both is explained in an <a href='http://mathias-kettner.de/checkmk_sap_r3.html'>article "
                 "in the Check_MK documentation</a>."),
        style = "dropdown",
        default_value = {
            "instances" : [{
                'ashost':   'localhost',
                'sysnr':    '00',
                'client':   '100',
                'user':     'cmk-user',
                'passwd':   'thiswontworkanyway',
                'trace':    '3',
                'lang':     'EN',
            }],
            "paths" : [
              'SAP BI Monitors/BI Monitor',
              'SAP BI Monitors/BI Monitor/*/Oracle/Performance',
              'SAP CCMS Monitor Templates/Operating System/OperatingSystem/CPU/*',
              'SAP CCMS Monitor Templates/Operating System/OperatingSystem/CPU/CPU_Utilization',
            ],
        },
        elements = [
           Dictionary(
               title = _("Deploy the SAP R/3 plugin"),
               optional_keys = False,
               elements = [
                   ( "instances",
                        ListOf(
                            Dictionary(
                                optional_keys = False,
                                elements = [
                                     ( 'ashost',
                                       TextAscii(title = _("Hostname"), default_value = "localhost", allow_empty = False ) ),
                                     ( 'sysnr',
                                       TextAscii(title = _("System Number"), size = 2, default_value = "00", regex = "^[0-9][0-9]$", regex_error = _("The system number is two digits"))),
                                     ( 'client',
                                       TextAscii(title = _("SAP-Client"), size = 3, default_value = "100", allow_empty = False )),
                                     ( 'user',
                                       TextAscii(title = _("User for login"), default_value = 'cmk-user', allow_empty = False)),
                                     ( 'passwd',
                                       Password(title = _("Password for login"), allow_empty = False)),
                                     ( 'trace',
                                       TextAscii(
                                           title = _("Trace level"),
                                           size = 1,
                                           default_value = '3',
                                           regex = "^[1-9]$",
                                           regex_error = _("Allowed is 1 ... 9"))),
                                     # ( 'loglevel': 'warn', ), # hard coded to "warn" currently. Found no docu about other levels
                                     ( 'lang',
                                       TextAscii(
                                           title = _("Language"),
                                           size = 2, default_value = "EN",
                                           regex = "^[A-Z][A-Z]$",
                                           regex_error = _("Specify two upper case letters like <tt>EN</tt> or <tt>DE</tt>."))),
                                ],
                            ),
                            title = _("Instances to monitor"),
                            add_label = _("Add instance to monitor"),
                            movable = False,
                        )
                    ),
                    ( "paths",
                      ListOfStrings(
                          title = _("CCMS paths to monitor"),
                          help = _("Specify the paths in CCMS that you want to monitor. Each entry must match the full path to one or "
                                  " several monitor objects. We use unix shell patterns during matching, so "
                                  " you can use several chars as placeholders:<ul>"
                                  "<li><tt>*      </tt> matches everything</li>"
                                  "<li><tt>?      </tt> matches any single character</li>"
                                  "<li><tt>[seq]  </tt> matches any character in seq</li>"
                                  "<li><tt>[!seq] </tt> matches any character not in seq</li>"
                                  "</ul>"),
                          size = 100,
                      )
                    ),
               ]
           ),
           FixedValue(None, title = _("Do not deploy the SAP R/3 plugin"), totext = _("disabled")),
        ]
    )
)

register_rule(group,
    "agent_config:mk_logwatch",
    Alternative(
        title = _("Text logfiles (Linux)"),
        help = _("The agent plugin <tt>mk_logwatch</tt> monitors a configured set "
                 "of text logfiles for new messages, classifies these according "
                 "to a set of rules and sends (only) relevant messages to the "
                 "Check_MK server for further processing. Here the messages can "
                 "further be (re-)classified using the "
                 "<a href='wato.py?varname=logwatch_rules&folder=&mode=edit_ruleset'>"
                 "Logwatch Ruleset</a>. <b>Note:</b> If you want to configure "
                 "several log files with different sets of patterns, then simply "
                 "create several rules. In this ruleset <b>all</b> matching rules "
                 "are being executed, not only the first one. "
                 "<b>Note (2):</b> This duplicate possibility for "
                 "classification is due to performance issues. It is always a good "
                 "idea that the agent forwards only messages that are potential "
                 "problems in order to cut down network traffic. Central reclassification "
                 "on the other hand is more conveniant because no agent update is "
                 "neccessary when rules need to be changed."),
        style = "dropdown",
        elements = [
           Dictionary(
               title = _("Deploy the Logwatch plugin and its configuration"),
               elements = [
                  ( "logfiles",
                    ListOfStrings(
                        title = _("Patterns for logfiles to monitor (allowing <tt>*</tt> and <tt>?</tt>)"),
                        valuespec = TextAscii(
                            size = 80,
                            regex = "^/[^ \t]+$",
                            regex_error = _("File patterns must begin with <tt>/</tt> and must not contain spaces."),
                       ),
                       allow_empty = False,
                    )
                  ),
                  ( "maxlinesize",
                    Integer(
                        title = _("Restrict the length of the lines"),
                        help = _("Lines longer then this number of characters will be truncated to the "
                                 "configured length before parsing. The word <tt>[TRUNCATED]</tt> will be "
                                 "appended to each such line."),
                        unit = _("Characters"),
                        minvalue = 1,
                        default_value = 500,
                    ),
                  ),
                  ( "maxfilesize",
                    Filesize(
                        title = _("Watch the total size of the log file"),
                        help = _("If the total size of the logfile exceeds this configured size "
                                 "then an artificial warning message will be introduced. That message "
                                 "will be repeated every time that the file has again grown by the configured size. "
                                 "That way you can detect a broken log file rotation or the like."),
                        minvalue = 1,
                        default_value = 100 * 1024 * 1024,
                    ),
                  ),
                  ( "maxlines",
                    Integer(
                        title = _("Restrict number of new messages per cycle"),
                        help = _("With this option activated at most <i>X</i> new messages "
                                 "per monitoring cycle are being transferred."),
                        minvalue = 1,
                        default_value = 50000,
                    ),
                  ),
                  ( "maxtime",
                    Age(
                        title = _("Restrict runtime of logfile parsing"),
                        help = _("With this option activated at most the configured time is being "
                                 "spent for each logfile in question. "),
                        minvalue = 1,
                        default_value = 10,
                    ),
                  ),
                  ( "overflow",
                    DropdownChoice(
                        title = _("In case of an overflow..."),
                        help = _("This option is only relevant when you limit the number of messages "
                                 "or the parsing time. It sets the behaviour in case of too many new "
                                 "messages in a logfile."),
                        choices = [
                            ( "C", _("Skip exceeding messages, create one critical message") ),
                            ( "W", _("Skip exceeding messages, create one warning message") ),
                            ( "I", _("Silently skip exceeding messages") ),
                        ]
                    )
                  ),
                  ( "context",
                    DropdownChoice(
                        title = _("Handling of context messages"),
                        help = _("If context messages are enabled then if at least one relevant message "
                                 "is being found (classified as WARN or CRIT), then all new messages "
                                 "of the current cycle are being transferred. This make logfile analysis "
                                 "on the monitoring server simpler, but can result much traffic on logfiles "
                                 "with frequent new messages."),
                       choices = [
                          ( True, _("Do transfer context") ),
                          ( False, _("Do not transfer context") ),
                       ]
                    )
                 ),
                 ( "patterns",
                   ListOf(
                       Tuple(
                           show_titles = True,
                           orientation = "horizontal",
                           elements = [
                              DropdownChoice(
                                title = _("State"),
                                choices = [
                                    ('C', _('CRITICAL')),
                                    ('W', _('WARNING')),
                                    ('O', _('OK')),
                                    ('I', _('IGNORE')),
                                    ('R', _('REWRITE')),
                                ],
                              ),
                              TextUnicode(
                                  title = _("Pattern (Regex)"),
                                  size  = 64,
                              ),
                           ]
                       ),
                       title = _("Regular expressions for message classification"),
                       help = _("Each new message runs through this list of rules. As soon as "
                                "a regular expression is found <i>in</i> the message, the state "
                                "is determined and the search terminated. If no rule matches "
                                "the message is classified as <i>IGNORE</i>. Note: <i>OK</i> "
                                "messages are being sent to the central Check_MK server, where "
                                "they can be reclassified. <i>IGNORE</i> messages are silently "
                                "dropped.<br><br><i>REWRITE</i> allows to change the content "
                                "a a message that has been matched by the <i>previous</i> rule. "
                                "You can specify a new text and use the placeholders <tt>\\0</tt> for "
                                "the original message, <tt>\\1</tt> for the first regex match group "
                                "of the match, <tt>\\2</tt> for the second and so on."),
                       add_label = _("Add message pattern"),
                   )
                 )
               ],
               required_keys = [ "logfiles", "overflow", "context", "patterns" ],
           ),
           FixedValue(True, title = _("Deploy the Logwatch plugin without configuration"),
                      help = _("The file <tt>/etc/check_mk/logwatch.cfg</tt> needs to "
                               "be created and maintained manually."),
                      totext = _("manually configure <tt>/etc/check_mk/logwatch.cfg</tt>")),
           FixedValue(None, title = _("Do not deploy the Logwatch plugin"), totext = _("(disabled)")),
        ]
    ),
    match = "all",
)



register_rule(group,
    "agent_config:nginx_status",
    CascadingDropdown(
        title = _("NGINX webservers (Linux)"),
        help = _("If you activate this option, then the agent plugin <tt>ngnix_status</tt> will be deployed. "
                 "For each configured or detected NGINX instance there will be one new service with detailed "
                 "statistics of the current number of clients and processes and their various states."),
        style = "dropdown",
        choices = [
            ( "autodetect", _("Autodetect instances, expect HTTPs on the following ports:"),
                ListOfIntegers(
                    default_value = [ 443 ],
                    minvalue = 1,
                    maxvalue = 65535,
                    orientation = "horizontal",
            )),
            ( "static", _("Specific list of instances"),
                ListOf(
                    Dictionary(
                        elements = [
                            ( "protocol",
                               DropdownChoice(
                                title = _("Protocol"),
                                choices = [
                                    ( "http", _("HTTP") ),
                                    ( "https", _("HTTPS") ),
                                ],
                            )),
                            ( "address", IPv4Address(
                                title = _("IP Address"),
                                default_value = "127.0.0.1",
                            )),
                            ( "port", Integer(
                                title = _("TCP Port Number"),
                                minvalue = 1,
                                maxvalue = 65535,
                                default_value = 80,
                            )),
                            ( "page", TextAscii(
                                title = _("URI (page name)"),
                                default_value = "nginx_status",
                                allow_empty = False,
                            )),
                        ],
                        optional_keys = [ "page" ],
                    ),
                    add_label = _("Add NGNIX instance"),
                ),
            ),
            ( None, _("Do not deploy the NGINX Status plugin") ),
        ]
    ),
)

register_rule(group,
    "agent_config:apache_status",
    CascadingDropdown(
        title = _("Apache webservers (Linux)"),
        help = _("If you activate this option, then the agent plugin <tt>apache_status</tt> will be deployed. "
                 "For each configured or detected Apache instance there will be one new service with detailed "
                 "statistics of the current number of clients and processes and their various states."),
        style = "dropdown",
        choices = [
            ( "autodetect", _("Autodetect instances, expect HTTPs on the following ports:"),
                ListOfIntegers(
                    default_value = [ 443 ],
                    minvalue = 1,
                    maxvalue = 65535,
                    orientation = "horizontal",
            )),
            ( "static", _("Specific list of instances"),
                ListOf(
                    Tuple(
                        elements = [
                            DropdownChoice(
                                title = _("Protocol"),
                                choices = [
                                    ( "http", _("HTTP") ),
                                    ( "https", _("HTTPS") ),
                                ],
                            ),
                            IPv4Address(
                                title = _("IP Address"),
                                default_value = "127.0.0.1",
                            ),
                            Alternative(
                                style = "dropdown",
                                elements = [
                                    FixedValue(None,
                                        title = _("Don't use custom port"),
                                        totext = _("Use default port"),
                                    ),
                                    Integer(
                                        title = _("TCP Port Number"),
                                        minvalue = 1,
                                        maxvalue = 65535,
                                        default_value = 80,
                                    ),
                                ]
                            ),
                        ]
                    ),
                ),
            ),
            ( None, _("Do not deploy the Apache Status plugin") ),
        ]
    ),
)

oracle_sections = [
    ( "instance",           "sync",  _("General instance status")),
    ( "performance",        "sync",  _("Performance")),
    ( "processes",          "sync",  _("Current number of processes")),
    ( "sessions",           "sync",  _("Current number of Sessions")),
    ( "longactivesessions", "sync",  _("Long active sessions")),
    ( "logswitches",        "sync",  _("Logswitches")),
    ( "undostat",           "sync",  _("Undo Statistics")),
    ( "recovery_area",      "sync",  _("Recovery Area")),
    ( "recovery_status",    "sync",  _("Recovery Status")),
    ( "dataguard_stats",    "sync",  _("Dataguard Statistics")),
    ( "tablespaces",        "async", _("Tablespaces")),
    ( "ts_quotas",          "async", _("TS Quotas")),
    ( "rman",               "async", _("RMAN Backups")),
    ( "jobs",               "async", _("Scheduled Jobs")),
    ( "resumable",          "async", _("Resumables")),
    ( "locks",              "async", _("Locks")),
    ( "asm:instance",       "sync",  _("ASM - General instance status")),
    ( "asm:processes",      "sync",  _("ASM - Processes")),
    ( "asm:asm_diskgroup",  "async", _("ASM - Disk Groups") ),
]

def oracle_section_choices():
    dict_elements = []
    for name, defval, title in oracle_sections:
        choices = [ ( "sync", _("Run synchronously" )) ]
        if name not in [ "instance", "asm:instance" ]:
            choices += [
              ( "async", _("Run asynchronously and cached" ) ),
              ( None, _("disable this section" ) ),
            ]
        dict_elements.append((
            name,
            DropdownChoice(
               title = title,
               choices = choices,
               default_value = defval,
            )
        ))
    return dict_elements

def oracle_auth_choices(title, asm):
    as_choices = []
    if not asm:
        as_choices.append( (None, _("normal connection")) )
    as_choices +=  [
        ("sysdba",  _("sysdba")),
        ("sysoper", _("sysoper")),
    ]
    if asm:
        as_choices.append( ("sysasm",  _("sysasm")) )

    return Dictionary(
        title = title,
        elements = [
            ( "auth",
              CascadingDropdown(
                  title = _("Authentication method"),
                  choices = [
                      ( "explicit", _("Login with the following credentials"),
                        Tuple(
                            elements = [
                                TextAscii(title = _("User"), size=30, allow_empty=False),
                                Password(title = _("Password"), size=16, forbidden_chars=":$'\"", allow_empty=False),
                             ]
                      )),
                      ( "wallet", _("Use manually created ORACLE password wallet") ),
                  ]
            )),
            ( "as",
              DropdownChoice(
                  title = _("login as role"),
                  choices = as_choices,
            )),
            ( "host",
              Hostname(
                  title = _("Hostname or IPv4 address for listener"),
                  default_value = "localhost",
            )),
            ( "port",
              Integer(
                  title = _("TCP-Port for Listener"),
                  default_value = 1521,
                  minvalue = 1,
                  maxvalue = 65535,
            )),
        ]
    )


register_rule(group,
    "agent_config:mk_oracle",
    Alternative(
        title = _("ORACLE Databases (Linux, AIX)"),
        help = _("By activating this option you will deploy the agent plugin <tt>mk_oracle</tt> on "
                 "your target system. Currently only Linux is supported, but AIX and Windows will follow."),
        style = "dropdown",
        elements = [
            Dictionary(
                title = _("Deploy the ORACLE Database plugin"),
                elements = [
                    ( "login",
                      oracle_auth_choices(_("Login Defaults"), asm=False),
                    ),
                    ( "login_exceptions",
                      ListOf(
                          Tuple(
                              elements = [
                                  ID(title = _("SID"), size=12),
                                  oracle_auth_choices(_("Authentication"), asm=False),
                              ]
                          ),
                          title = _("Login for selected databases"),
                    )),
                    ( "login_asm",
                      oracle_auth_choices(_("Login for ASM"), asm=True),
                    ),
                    ( "sids",
                      CascadingDropdown(
                          title = _("Instances to monitor"),
                          choices = [
                              ( None, _("Monitor all instances found") ),
                              ( "only", _("Only query the following SIDs (if found)"),
                                        ListOfStrings(size = 12, orientation = "horizontal")),
                              ( "exclude", _("Exclude the following SIDs (if found)"),
                                        ListOfStrings(size = 12, orientation = "horizontal")),
                          ],
                      ),
                    ),
                    ( "sections",
                      Dictionary(
                          title = _("Sections - data to collect"),
                          columns = 2,
                          elements = oracle_section_choices(),
                          optional_keys = None,
                    )),
                    ( "excluded_sections",
                      ListOf(
                          Tuple(
                              elements = [
                                  ID(title = _("SID"), size = 12, allow_empty = False ),
                                  ListChoice(
                                      title = _("Sections to exclude"),
                                      choices = [ (i[0], i[2])
                                                  for i in oracle_sections
                                                  if not i[0].startswith("asm:") and i[0] != "instance" ],
                                  ),
                              ],
                          ),
                          add_label = _("Add exclusion"),
                          title = _("Exclude some sections on certain instances"),
                      )
                    ),
                    ( "async_interval",
                      Age(
                          title = _("Cache age for background checks"),
                          help = _("The data for the background checks is recomputed at most in this interval."),
                          default_value = 600,
                      )
                    ),
                    ( "remote_instances",
                        ListOf(
                            Dictionary(
                                optional_keys = False,
                                elements = [
                                    ( "id",
                                       CascadingDropdown(
                                           title = _("Unique ID"),
                                           orientation = "horizontal",
                                           choices = [
                                              ( "piggyhost", _("Use monitoring host name") ),
                                              ( "sid",       _("Use the remote SID") ),
                                              ( "explicity", _("Use the following ID"), ID(size=12)),
                                           ]
                                    )),
                                    ( "piggyhost",
                                       Hostname(
                                           title = _("Monitoring host this database should be mapped to"),
                                           help = _("If you leave this empty then the database will appear on the host "
                                                    "where the <tt>mk_oracle</tt> plugin is running. In this case the "
                                                    "SIDs of all monitored databases must be unique."),
                                    )),
                                    ( "sid",
                                      ID(title = _("ORACLE SID of the remote database"), size=12, default_value = "OACL",
                                    )),
                                    ( "host",
                                       TextAscii(title = _("DNS Hostname or IP-address of database server"), allow_empty = False
                                    )),
                                    ( "port",
                                      Integer(title = _("TCP port number"), minvalue = 1, maxvalue = 65535, default_value = 1521,
                                    )),
                                    ( "release",
                                      DropdownChoice(
                                          title = _("ORACLE release of the remote database"),
                                          choices = [
                                              ( "9.2",  "9.2" ),
                                              ( "10.1", "10.1" ),
                                              ( "10.2", "10.2" ),
                                              ( "11.1", "11.1" ),
                                              ( "11.2", "11.2" ),
                                              ( "12.1", "12.1" ),
                                          ]
                                    )),
                                ],
                            ),
                            title = _("Remote instances"),
                            help = _("You can monitor database instances that are hosted on another server. This can be "
                                     "helpful if you want to monitoring ORACLE databases on Windows without deploying the "
                                     "mk_oracle plugin for Windows. For each remote instance you need to specify a unique "
                                     "ID. This can be the same as the SID. This ID is then being used as a reference "
                                     "in the other configuration - for example for the <i>Login for selected databases</i> "
                                     "and for <i>Sections to exclude</i>."),
                            add_label = _("Add remote instance"),
                            movable = False,
                    )),
                    ( "remote_oracle_home",
                      AbsoluteDirname(
                          title = _("<tt>ORACLE_HOME</tt> to use for remote access"),
                          help = _("Here you can specify an <tt>ORACLE_HOME</tt> for the access to the "
                                   "remote databases. Omitting this empty will fall back to the system "
                                   "<tt>ORACLE_HOME</tt>."),
                          size = 40,
                    )),
                ],
            ),
            FixedValue(None, title = _("Do not deploy the ORACLE plugin"), totext = _("disabled")),
        ],
    ),
    match = "dict")

#.
#   .--Bake Agents---------------------------------------------------------.
#   |       ____        _             _                    _               |
#   |      | __ )  __ _| | _____     / \   __ _  ___ _ __ | |_ ___         |
#   |      |  _ \ / _` | |/ / _ \   / _ \ / _` |/ _ \ '_ \| __/ __|        |
#   |      | |_) | (_| |   <  __/  / ___ \ (_| |  __/ | | | |_\__ \        |
#   |      |____/ \__,_|_|\_\___| /_/   \_\__, |\___|_| |_|\__|___/        |
#   |                                     |___/                            |
#   +----------------------------------------------------------------------+
#   |  WATO modes for baking the configured agents.                        |
#   '----------------------------------------------------------------------'


agent_package_types = {
    "linux_rpm"   : (_("Linux"),   "RPM",  ),
    "linux_deb"   : (_("Linux"),   "DEB",  ),
    "linux_tgz"   : (_("Linux"),   "TGZ",  ),
    "aix_tgz"     : (_("AIX"),     "TGZ",  ),
    "windows_msi" : (_("Windows"), "MSI",  ),
}


def is_affecting_baked_agents(rule_name):
    return rule_name.startswith("agent_config:") or \
       rule_name in [ "agent_ports" ]


last_bake_file = var_dir + "last_bake.mk"

def need_to_bake_agents():
    try:
        os.unlink(last_bake_file)
    except OSError:
        pass


def bake_agents():
    check_mk_local_automation('bake-agents')
    write_last_bake_file()


def write_last_bake_file():
    file(last_bake_file, 'w').write('%s\n' % defaults.check_mk_version)


def marked_need_to_bake_agents():
    try:
        return file(last_bake_file).read().strip() != defaults.check_mk_version
    except IOError:
        return True


def agents_need_update():
    if marked_need_to_bake_agents():
        return True

    if files_included_in_agents_changed():
        return True

    return False


# Hope this is performing good enough for being run on each agents_need_update() call.
def files_included_in_agents_changed():
    for aghash in agent_bakery.find_agents().keys():
        try:
            agconf = get_agent_configuration(aghash)
        except IOError:
            # Ignore missing configuration. This e.g. happens if a host has
            # been deleted but on old symlink still lies around. The next baking
            # will delete these orphans anyway.
            continue

        for (base_name, rel_path), fhash in agconf.get("file_hashes", {}).items():
            if base_name == 'custom':
                path = custom_file_base_path() + rel_path
            else:
                raise MKGeneralException("Don't know how to handle \"%s\" file hashes" % base_name)

            try:
                if fhash != file_hash(path):
                    return True
            except IOError, e:
                if e.errno == 2:
                    return True # A file seem to be deleted
                else:
                    raise
    return False


def custom_file_base_path():
    return defaults.omd_root + "/local/share/check_mk/agents/custom/"


# Code duplication with modules/agent_bakery.py. Sorry.
def file_hash(path):
    h = md5()
    f = file(path)
    buf = f.read(65536)
    while len(buf) > 0:
        h.update(buf)
        buf = f.read(65536)
    return h.hexdigest()


def vs_sign():
    choices = [
        (nr, key["alias"])
        for (nr, key)
        in sorted(config.agent_signature_keys.items())
    ]
    return Dictionary(
        title = _("Sign agents"),
        elements = [
            ( "key",
              DropdownChoice(
                  title = _("Key to sign with"),
                  choices = choices,
            )),
            ( "passphrase",
              Password(
                  title = _("Key passphrase"),
            )),
        ],
        render = "form",
        optional_keys = None,
    )

def mode_agents(phase):
    if phase == "title":
        return _("Monitoring Agents - Agent Bakery")

    elif phase == "buttons":
        home_button()
        html.context_button(_("Bake Agents"),
            make_action_link([("mode", "agents"),("_action", "bake")]), "agents", hot=agents_need_update())
        html.context_button(_("Rules"), folder_preserving_link([("mode", "rulesets"),
                ("group", "agents")]), "rulesets")
        html.context_button(_("Unpackaged Agents"), folder_preserving_link([("mode", "download_agents")]), "download_agents")
        html.context_button(_("Automatic Agent Updates"), folder_preserving_link([("mode", "agent_deployment")]), "deploy_agents")
        if config.may("wato.agent_deployment"):
            html.context_button(_("Signature Keys"), folder_preserving_link([("mode", "signature_keys")]), "signature_key")
        if config.may("wato.agent_deployment") and have_signature_keys():
            html.context_button(_("Sign All Agents"), html.makeuri([("_sign", "all")]), "signature_key")
        return

    elif phase == "action":
        if html.var("_do_sign") and html.check_transaction():
            v = vs_sign().from_html_vars("key")
            agent_key = config.agent_signature_keys[v["key"]]
            sign_all_agents(agent_key, v["passphrase"])
            log_audit(None, "sign-agents", _("Signed all agents with the key '%s'") % agent_key["alias"])
            html.del_var("_do_sign")

        if html.var("_action") == "bake" and html.check_transaction():
            try:
                bake_agents()
                return None, _("Successfully baked all needed agents.")
            except Exception, e:
                if config.debug:
                    raise
                raise MKUserError(None, _("Failed to bake the agents: %s") % e)
        return

    if html.var("_sign") or html.var("_do_sign"):
        html.begin_form("key", method="POST")
        html.prevent_password_auto_completion()
        vs_sign().render_input("key", {})
        html.set_focus("key_p_passphrase")
        html.button("create", _("Sign"))
        html.hidden_field("_do_sign", "1")
        html.hidden_fields()
        html.end_form()
        html.write("<br>")


    agents = agent_bakery.find_agents()
    table.begin(_("Available agent configurations"), searchable = False)
    for aghash, opsyses in agents.items():
        if not agent_config_available(aghash):
            continue # *.conf file for hash is missing, symlink exists (unclean state)

        table.row()

        # Configuration
        table.cell(_("Configuration"))
        try:
            html.write(describe_agent_configuration(aghash))
        except:
            import traceback
            html.write(_('Failed to process configuration:')+'<br>')
            html.write(traceback.format_exc().replace('\n', '<br>'))

        # Hash
        table.cell(_("Hash"), "<tt>%s</tt>" % aghash[:8])

        # Signatures
        if have_signature_keys():
            table.cell(_("Signed"))
            fully_signed, partially_signed = agent_bakery.find_agent_signed_certificates(aghash)
            for key_id in fully_signed:
                html.icon(_("All package types are signed with the key '%s'") %
                            config.agent_signature_keys[key_id]["alias"], "signature_key")
            for key_id in partially_signed:
                html.icon(_("Some but not all package types are signed with key '%s'") %
                          config.agent_signature_keys[key_id]["alias"], "signature_key_partial")


        hosts = set([])
        sorted_items = agent_package_types.items()
        sorted_items.sort()
        for opsys, (opsys_title, opsys_pactype) in sorted_items:
            hostnames = opsyses.get(opsys, [])
            url = "download_agent.py?" + html.urlencode_vars([("os", opsys), ("hash", aghash)])
            table.cell(opsys_title)
            if hostnames and agent_bakery.agent_exists(opsys, aghash):
                html.write("<a href='%s'>" % url)
                html.icon(_("Download %s package of agent for %s") % (opsys_pactype, opsys_title), "agents")
                html.write("%s</a>" % opsys_pactype)
                hosts.update(hostnames)
            else:
                html.write("<i>%s</i>" % _("not available"))

        if None in hosts:
            hosts.remove(None)

        hosts = list(hosts)
        hosts.sort()
        hosts_with_links = [
            '<a href="%s">%s</a>' % (folder_preserving_link([("mode", "agent_of_host"), ("host", host)]), host)
            for host in hosts ]
        if len(hosts) > 5:
            hosts_text = ", ".join(hosts_with_links[:5]) + " ... (%d %s)" % (len(hosts), _("Hosts"))
        else:
            hosts_text = ", ".join(hosts_with_links)

        table.cell(_("Hosts"), hosts_text)

    table.end()


def agent_config_available(aghash):
    return os.path.exists(defaults.var_dir + "/agents/linux_rpm/_PACKAGES/%s.conf" % aghash)


def get_agent_configuration(aghash):
    return eval(file(defaults.var_dir + "/agents/linux_rpm/_PACKAGES/%s.conf" % aghash).read())


def describe_agent_configuration(aghash):
    try:
        conf = get_agent_configuration(aghash)
    except (OSError, IOError):
        if config.debug:
            raise
        return "<i>%s</i>" % _("Configuration description missing or unreadable")

    output = ""
    for key, value in conf.items():
        # Some keys are contained in the vanilla setting. Display these only
        # if they are modified from the defaults.
        if key == "agent_port" and value != 6556:
            rulespec = g_rulespecs["agent_ports"]

        elif key == "agent_paths":
            default_value = {
                'bin':    '/usr/bin',
                'config': '/etc/check_mk',
                'lib':    '/usr/lib/check_mk_agent',
                'var':    '/var/lib/check_mk_agent'
            }
            if value == default_value:
                continue
            rulespec = g_rulespecs["agent_config:agent_paths"]

        elif "agent_config:" + key in g_rulespecs:
            rulespec = g_rulespecs["agent_config:" + key]

        elif key == "is_ipv6_primary" and value != False:
            rulespec = g_rulespecs["primary_address_family"]
            # Transform boolean value into valuespec matching value
            value = value and "ipv6" or "ipv4"

        else:
            continue

        if rulespec["match"] == "all":
            totext = ", ".join( rulespec["valuespec"].value_to_text(v) for v in value )
        else:
            totext = rulespec["valuespec"].value_to_text(value)

        title = rulespec["title"]
        output += "<tr><td>%s:</td><td>%s</td></tr>" % (title, totext)

    if output:
        return "<table>" + output + "</table>"
    else:
        return "<i>%s</i>" % _("Default configuration")



def mode_agent_of_host(phase):
    host_name = html.var("host")

    if phase == "title":
        return _("Download Check_MK Agent for host %s") % host_name

    elif phase == "buttons":
        html.context_button(_("All Agents"), folder_preserving_link([("mode", "agents")]), "agents")
        if host_name not in ( "GENERIC", "VANILLA" ):
            html.context_button(_("Host Properties"), folder_preserving_link([("mode", "edit_host"), ("host", host_name)]), "wato")
        return

    elif phase == "action":
        return

    aghash, package_types = agent_bakery.find_agents_of_host(host_name)
    if not package_types:
        html.write("This host has no Check_MK agents.")
        return

    html.write('<table class="data headerleft">')
    html.write('<tr><th>%s</th><td>%s</td>' % (_("Configuration"), describe_agent_configuration(aghash)))
    for package_type in sorted(package_types):
        os_name, package_name = agent_package_types[package_type]
        title = _("Agent for %s (%s)") % (os_name, package_name)
        file_name_pattern = "check_mk_agent" + agent_bakery.agent_package_types[package_type][1]

        file_name = file_name_pattern % { "version" : defaults.check_mk_version, "release" : aghash }
        file_size = agent_bakery.agent_file_size(aghash, package_type)
        url = "download_agent.py?" + html.urlencode_vars([("os", package_type), ("hash", aghash)])
        html.write('<tr><th>%s</th><td><a href="%s">%s</a> (%s)</td>' % (title, url, file_name, bytes_human_readable(file_size)))
    html.write('</table>')



#.
#   .--Deployment-Pages----------------------------------------------------.
#   |       ____             _                                  _          |
#   |      |  _ \  ___ _ __ | | ___  _   _ _ __ ___   ___ _ __ | |_        |
#   |      | | | |/ _ \ '_ \| |/ _ \| | | | '_ ` _ \ / _ \ '_ \| __|       |
#   |      | |_| |  __/ |_) | | (_) | |_| | | | | | |  __/ | | | |_        |
#   |      |____/ \___| .__/|_|\___/ \__, |_| |_| |_|\___|_| |_|\__|       |
#   |                 |_|            |___/                                 |
#   +----------------------------------------------------------------------+
#   |  WATO page for agent deployment.                                     |
#   '----------------------------------------------------------------------'

def mode_agent_deployment(phase):
    if phase == "title":
        return _("Automatic agent updates")

    elif phase == "buttons":
        global_buttons()
        html.context_button(_("Baked agents"), make_action_link([("mode", "agents")]), "agents")
        url = folder_preserving_link([("mode", "edit_ruleset"), ("varname", "agent_config:cmk_update_agent"), ("back", "foo")])
        html.context_button(_("Update Plugin"), url, "rulesets")
        html.context_button(_("Update Status"), "view.py?view_name=allhosts_deploy", "view")
        return

    elif phase == "action":
        if html.check_transaction():
            if html.var("_action") == "toggle":
                toggle_deployment_master_switch()
            elif html.var("change"):
                change_deployment_host_selection()
                return None, _("Changed host selection has been saved.")
        return

    check_deployment_prerequisites()
    deployment_host_selection()
    render_agent_deployment_status()


def toggle_deployment_master_switch():
    current_settings = load_configuration_settings()
    current_settings["agent_deployment_enabled"] = \
      not current_settings.get("agent_deployment_enabled", False)
    config.agent_deployment_enabled = current_settings["agent_deployment_enabled"] # make directly active
    save_configuration_settings(current_settings)
    log_audit(None, "agent-deployment-enabling", current_settings["agent_deployment_enabled"] and
                       _("Enabled agent updates") or _("Disabled agent updates"))


def change_deployment_host_selection():
    current_settings = load_configuration_settings()
    vs = vs_deployment_host_selection()
    new_settings = vs.from_html_vars("host_selection")
    vs.validate_value(new_settings, "host_selection")
    config.agent_deployment_host_selection = new_settings # make directly active
    current_settings["agent_deployment_host_selection"] = new_settings
    save_configuration_settings(current_settings)
    log_audit(None, "agent-deployment-host-selection", _("Changed host selection of agent updates"))


def check_deployment_prerequisites():
    # table.begin(title = _("Prerequisites"), searchable=False, sortable=False)
    table.begin(title = _("Prerequisites"), searchable=False, sortable=False)
    check_deployment_prerequisite_signature_keys()
    check_deployment_prerequisite_rules()
    check_deployment_prerequisite_plugins()
    check_deployment_prerequisite_signed_agents()
    check_deployment_prerequisite_registration()
    check_deployment_prerequisite_enabling()
    table.end()


def check_deployment_prerequisite_signature_keys():
    signature_keys = load_agent_signature_keys()
    url = "wato.py?mode=signature_keys"
    render_deployment_prerequisite(
        _("Signature keys"),
        len(signature_keys) > 0,
        _('You have created %d <a href="%s">signature keys</a> for signing agents') % (len(signature_keys), url),
        _('Signatures are required for cryptographically signing your agents. That '
          'makes sure that the nobody can inject faked agents. Sigantures are mandatory '
          'and need to be created as a first step'),
        url)


def check_deployment_prerequisite_rules():
    # Do we have any rule for deploying the update plugin?
    all_rulesets = load_all_rulesets(only_varname = "agent_config:cmk_update_agent")
    num_activated = 0
    for folder, rule in all_rulesets.get("agent_config:cmk_update_agent", []):
        if rule[0].get("activated"):
            num_activated += 1

    url = "wato.py?mode=edit_ruleset&varname=agent_config:cmk_update_agent"
    render_deployment_prerequisite(
        _("Configuration of update plugin"),
        num_activated > 0,
        _('You have %d <a href="%s">rules</a> for deploying the update plugin') % (num_activated, url),
        _('You need to create <a href="%s">rules</a> where you configure that the '
          'agent updater should be baked in the agents. There you also need to specify the access '
          'data to the update server.') % url,
        url)


def check_deployment_prerequisite_plugins():
    # Is there any baked agent containing that plugin?
    num_activated = 0
    for aghash, opsyses in agent_bakery.find_agents().items():
        if agent_config_available(aghash):
            for opsys, hostnames in opsyses.items():
                if hostnames and agent_bakery.agent_exists(opsys, aghash):
                    conf = get_agent_configuration(aghash)
                    if "cmk_update_agent" in conf and conf["cmk_update_agent"].get("activated"):
                        num_activated += 1
    render_deployment_prerequisite(
        _("Baked agents"),
        num_activated > 0,
        _("You have %d baked agents containing the update plugin") % num_activated,
        _("After creating a rule that deploys the agent updater you need to bake "
          "agents. On all hosts that should do automatic updates you need to install "
          "such a baked agent that contains the plugin. This needs to be done manually "
          "(for the last time)."),
        "wato.py?mode=agents",
     )


def check_deployment_prerequisite_registration():
    deployment_status = load_deployment_status()
    render_deployment_prerequisite(
        _("Registered agents"),
        len(deployment_status) > 0,
        _("Number of agents that have registered for updates: %d") % len(deployment_status),
        _("In order for your agents to do automatic updates you first need to install an agent "
          "package that contains the updater. Then you need to run "
          "<tt>cmk-update-agent -v</tt>. On Linux do this as <tt>root</tt> user. On Windows "
          "you find this script in the <tt>plugins</tt> subdirectory."))


def check_deployment_prerequisite_enabling():
    render_deployment_prerequisite(
        _("Master switch"),
        config.agent_deployment_enabled,
        config.agent_deployment_enabled and
        _("Agent updates are enabled") or
        _("Agent updates are currently disabled"),
        _("With the master switch you can immediately turn on or off the agent updates. When "
          "it is turned off then agents still can query their target configuration and register "
          "but cannot download packges for update."),
         html.makeactionuri([("_action", "toggle")]))


def check_deployment_prerequisite_signed_agents():
    have = agent_bakery.have_signed_agents()
    render_deployment_prerequisite(
        _("Signed agents"),
        have,
        have and
        _("You have signed some or all agent packages") or
        _("None of your agent packages is signed"),
        _("Agents will not accept packages that are not signed by at least one key that they accept."),
        "wato.py?mode=agents&_sign=all")


def render_deployment_prerequisite(test, state, details, help, url=None):
    table.row()
    table.cell(_("State"), css="buttons")
    if state:
        html.icon(_("Fullfilled Prerequisite"), "success")
    else:
        html.icon(_("Missing Prerequisite"), "failed")
    table.cell(_("Edit"), css="buttons")
    if url:
        html.icon_button(url, _("Fix this"), "edit")
    table.cell(_("Test"), test, css="narrow nowrap")
    table.cell(_("Details"))
    html.write(details)
    html.help(help)


def vs_deployment_host_selection():
    return Dictionary(
        title = _("Selection of hosts to activate agent updates for"),
        help = _("By activating a matching entry here you can <b>restrict</b> the hosts that "
                 "we should send updates to. <b>Note</b>: The properties from the hosts are being "
                 "taken from the <b>activated changes</b>, not from the current settings in WATO!"),
        elements = simple_host_rule_match_conditions(),
    )


def deployment_host_selection():
    html.write("<h3>%s</h3>" % _("Host Selection"))
    html.begin_form("host_selection", method="POST")
    forms.header(_("Activate update only on the selected hosts"))
    forms.section(_("Conditions"))
    vs_deployment_host_selection().render_input("host_selection", config.agent_deployment_host_selection)
    forms.section(_("Test with this host name"))
    html.text_input("test_selection", "")
    test_host_name = html.var("test_selection")
    if test_host_name:
        html.write(" &nbsp; ")
        try:
            agent_bakery.check_deployment_enabling_for_host(test_host_name)
            html.icon(_("Selected"), "success")
            html.write(" " + _("Updates are enabled for this host."))
        except MKGeneralException, e:
            html.icon(_("Not selected"), "failed")
            html.write(" " + _("Updates are <b>disabled</b> for this host: %s") % e)

    forms.section()
    forms.end()
    html.hidden_fields()
    html.button("change", _("Change Selection"), "submit")
    html.end_form()



def get_age_range_match(dict_key, from_age, to_age):
    def match_function(status):
        now = time.time()
        timestamp = status.get(dict_key)
        if timestamp == None:
            return False
        age = now - timestamp
        return age >= from_age and age <= to_age
    return match_function


deployment_status_groups = [
   ( "pending_update",
     _("Hosts that need an agent update"),
     _("For these hosts there is a new agent available that is not yet installed. This simply means "
       "that the target agent configuration hash does not match the configuration has that is currently "
       "installed."),
     agent_bakery.agent_needs_update,
     "",
     [ ("is_deployment_need_update", "1") ],
   ),
   ( "uptodate",
     _("Hosts that have an up-to-date agent"),
     _("These hosts have installed the correct version of their agent."),
     lambda status: status.get("target_aghash") and status["target_aghash"] == status.get("installed_aghash"),
     "",
     [ ("is_deployment_need_update", "0") ],
   ),
   ( "contact_hour",
     _("Hosts that contacted us during the last 65 minutes"),
     _("If you have selected 60 minutes as the interval for the update checks, then "
       "all hosts should be listed here."),
     get_age_range_match("last_contact", 0, 3900),
     "",
     [ ("deployment_last_contact_from", "60"), ("deployment_last_contact_from_range", "60") ],
   ),
   ( "contact_day",
     _("Hosts that contacted us during the last 24 hours"),
     _("These hosts have at least once in the last 24 hours connected the update server "
       "and asked for an update."),
     get_age_range_match("last_contact", 0, 86400),
     "",
     [ ("deployment_last_contact_from", "24"), ("deployment_last_contact_from_range", "3600") ],
   ),
   ( "contact_older",
     _("Hosts whose last contact is more than 24 hours ago"),
     _("The hosts listed here have contacted the update server in that past and thus had "
       "been correctly registered then. But they do not ask for updates anymore. If you remove "
       "these hosts from the monitoring, they are automatically unregistered and removed from "
       "this list."),
     get_age_range_match("last_contact", 86400, 2**32),
     "state1",
     [ ("deployment_last_contact_until", "24"), ("deployment_last_contact_until_range", "3600") ],
   ),
   ( "download_hour",
     _("Hosts that downloaded an agent during the last 65 minutes"),
     _("Here you get feedback about recent agent rollouts. If you e.g. restrict a new rollout "
       "to a known list of five hosts, then these five should appear here at latest after one "
       "configured update interval."),
     get_age_range_match("last_download", 0, 3900),
     "",
     [ ("deployment_last_download_from", "60"), ("deployment_last_download_from_range", "60") ],
   ),
   ( "never_download",
     _("Hosts that never downloaded an agent"),
     _("These host have correctly registered for updates but, never downloaded an agent. (nor "
       "installed one)"),
     lambda status: not agent_bakery.has_downloaded_agent(status),
     "state1",
     [ ("is_deployment_has_downloaded", "0") ],
   ),
   ( "no_agent_avail",
     _("Hosts without a baked agent"),
     _("Here you see hosts that have registered for updates but have no available agent. "
       "You need to bake agents again in order to create agents for them."),
     lambda status: not agent_bakery.has_baked_agent(status),
     "state1",
     [ ("is_deployment_has_agent", "0") ],
   ),
   ( "error",
     _("Hosts with an error"),
     _("Here you can see all hosts that have sent a negative status to the deployment "
       "server."),
     agent_bakery.host_has_error,
     "state2",
     [ ("is_deployment_host_has_error", "1") ],
   ),
   ( "total",
     _("Total number of registered hosts"),
     _("Only host are listed that are currently part of the active monitoring configuration and "
       "that have registered for updates."),
     lambda status: True,
     "",
     [],
   ),
]

def render_agent_deployment_status():
    deployment_status = load_deployment_status()
    counts = {}
    for host_name, status in deployment_status.items():
        for group_key, group_title, help, match_function, cssclass, view_filters in deployment_status_groups:
            if match_function(status):
                counts.setdefault(group_key, 0)
                counts[group_key] += 1

    table.begin(title=_("Update Status"), sortable=False, searchable=False)
    for group_key, group_title, help, match_function, cssclass, view_filters in deployment_status_groups:
        count = counts.get(group_key, 0)
        table.row()
        table.cell(_("Count"), count, css="number %s %s" % (count and cssclass or "", not count and "stale" or ""))
        table.cell(_("Show"), css="button")
        if view_filters != None and count > 0:
            url = html.makeuri_contextless([
                ("view_name", "allhosts_deploy"),
                ("filled_in", "filter")] + view_filters, filename="view.py")
            html.icon_button(url, _("Show the list of hosts in this category"), "view")
        table.cell(_("Class of hosts"), group_title, css="wide" + (not count and " stale" or ""))
        html.help(help)
    table.row()
    table.end()


def load_deployment_status():
    status = {}
    all_live_hosts = set(html.live.query_column_unique("GET hosts\nColumns: name"))
    dir = defaults.var_dir + "/agent_deployment"
    if os.path.exists(dir):
        for host_name in os.listdir(dir):
            if host_name in all_live_hosts:
                host_status = eval(file(dir + "/" + host_name).read())
                host_status["target_aghash"] = agent_bakery.target_agent_hash(host_name)
                status[host_name] = host_status
    return status


#.
#   .--Key Management------------------------------------------------------.
#   |             _  __            __  __                 _                |
#   |            | |/ /___ _   _  |  \/  | __ _ _ __ ___ | |_              |
#   |            | ' // _ \ | | | | |\/| |/ _` | '_ ` _ \| __|             |
#   |            | . \  __/ |_| | | |  | | (_| | | | | | | |_              |
#   |            |_|\_\___|\__, | |_|  |_|\__, |_| |_| |_|\__|             |
#   |                      |___/          |___/                            |
#   +----------------------------------------------------------------------+
#   | Managing of the keys that are used for signing the agents            |
#   '----------------------------------------------------------------------'

def mode_signature_keys(phase):
    if phase == "title":
        return _("Signature Keys for Signing Agents")

    elif phase == "buttons":
        html.context_button(_("Back"), folder_preserving_link([("mode", "agent_deployment")]), "back")
        html.context_button(_("Create Key"), folder_preserving_link([("mode", "edit_signature_key")]), "new")
        return

    signature_keys = load_agent_signature_keys()

    if phase == "action":
        if html.has_var("_delete"):
            key_id = int(html.var("_delete"))
            if key_id not in signature_keys:
                return

            key = signature_keys[key_id]
            if agent_signature_key_in_use(key):
                raise MKUserError("", _("This key is still used by at least one automatic agent updates rule."))
            title = _("Confirm deletion of agent signature key '%s'" % key["alias"])
            message = _("Are you sure you want to delete this signature key?<br><br>"
                        "<b>Beware:</b> Deleting this key "
                        "means that you will not be able to create signatures with the key. "
                        "Already deployed agents might not be able to update anymore. You might "
                        "need to update all agents manually!")
            if key["owner"] != config.user_id:
                message += _("<br><b>Note</b>: this key has created by user <b>%s</b>") % key["owner"]
            c = wato_confirm(title, message)
            if c:
                log_audit(None, "delete-signature-key", _("Deleted agent signature key '%s'") % (key["alias"]))
                del signature_keys[key_id]
                save_agent_signature_keys(signature_keys)
            elif c == False:
                return ""
            else:
                return
        return

    table.begin(title = _("Agent Signature Keys"))

    for key_id, key in sorted(signature_keys.items()):
        table.row()
        table.cell(_("Actions"), css="buttons")
        delete_url = html.makeactionuri([("_delete", key_id)])
        html.icon_button(delete_url, _("Delete this key"), "delete")
        table.cell(_("Description"), key["alias"])
        table.cell(_("Created"), date_human_readable(key["date"]))
        table.cell(_("By"), key["owner"])
    table.end()


vs_signature_key = \
    Dictionary(
        title = _("Properties"),
        elements = [
            ( "alias",
              TextUnicode(
                  title = _("Description or comment"),
                  size = 64,
                  allow_empty = False,
            )),
            ( "passphrase",
              Password(
                  title = _("Passphrase"),
                  help = _("The signature keys will be stored on disk encrypted with "
                           "this passphrase. The phrase will not be stored anywhere. "
                           "If it get's lost then you will not be able to sign any "
                           "agents with this key anymore. This can mean that you have "
                           "to update all agents manually with a new key. So beware!"),
                  allow_empty = False,
            )),
        ],
        optional_keys = False,
        render = "form",
    )


def mode_edit_signature_key(phase):
    if phase == "title":
        return _("Create Agent Signature Key")

    elif phase == "buttons":
        html.context_button(_("Back"), folder_preserving_link([("mode", "signature_keys")]), "back")
        return

    if phase == "action":
        if html.check_transaction():
            value = vs_signature_key.from_html_vars("key")
            vs_signature_key.validate_value(value, "key")
            create_agent_signature_key(value["alias"], value["passphrase"])
            log_audit(None, "create-signature-key", _("New agent signature key '%s' created") % value["alias"])
            return "signature_keys"
        return

    # Currently only "new" is supported
    html.begin_form("signature_key", method="POST")
    html.prevent_password_auto_completion()
    vs_signature_key.render_input("key", {})
    html.button("create", _("Create"))
    vs_signature_key.set_focus("key")
    html.hidden_fields()
    html.end_form()


def agent_signature_key_in_use(key):
    all_rulesets = load_all_rulesets(only_varname = "agent_config:cmk_update_agent")
    for folder, rule in all_rulesets.get("agent_config:cmk_update_agent", []):
        if key["certificate"] in rule[0].get("signature_keys", []):
            return True

    return False


def load_agent_signature_keys():
    filename = multisite_dir + "agent_signature_keys.mk"
    if not os.path.exists(filename):
        return {}

    try:
        variables = { "agent_signature_keys" : {} }
        execfile(filename, variables, variables)
        return variables["agent_signature_keys"]

    except (IOError, OSError, SyntaxError), e:
        if config.debug:
            raise
        return {}


def save_agent_signature_keys(signature_keys):
    make_nagios_directory(multisite_dir)
    out = create_user_file(multisite_dir + "agent_signature_keys.mk", "w")
    out.write("# Written by WATO\n# encoding: utf-8\n\n")
    out.write("agent_signature_keys.update(%s)\n\n" % pprint.pformat(signature_keys))


def have_signature_keys():
    return not not config.agent_signature_keys


def create_agent_signature_key(alias, passphrase):
    signature_keys = load_agent_signature_keys()

    new_id = 1
    for key_id in signature_keys:
        new_id = max(new_id, key_id + 1)

    signature_keys[new_id] = generate_agent_signature_key(alias, passphrase)
    save_agent_signature_keys(signature_keys)


def generate_agent_signature_key(alias, passphrase):
    from OpenSSL import crypto

    pkey = crypto.PKey()
    pkey.generate_key(crypto.TYPE_RSA, 2048)

    # create a self-signed cert
    cert = crypto.X509()
    cert.get_subject().O = "Check_MK Site %s" % defaults.omd_site
    cert.get_subject().CN = config.user_id
    cert.set_serial_number(1)
    cert.gmtime_adj_notBefore(0)
    cert.gmtime_adj_notAfter(30*365*24*60*60) # valid for 30 years.
    cert.set_issuer(cert.get_subject())
    cert.set_pubkey(pkey)
    cert.sign(pkey, 'sha1')

    return {
        "certificate" : crypto.dump_certificate(crypto.FILETYPE_PEM, cert),
        "private_key" : crypto.dump_privatekey(crypto.FILETYPE_PEM, pkey, "AES256", passphrase),
        "alias"       : alias,
        "owner"       : config.user_id,
        "date"        : time.time(),
    }


def sign_all_agents(key, passphrase):
    pkey = decrypt_private_key(key, passphrase)

    agents = agent_bakery.find_agents()
    for aghash, opsyses in agents.items():
        for opsys in opsyses:
            sign_agent(aghash, opsys, key["certificate"], pkey)


def sign_agent(aghash, opsys, certificate, pkey):
    from OpenSSL import crypto
    content = agent_bakery.read_agent(aghash, opsys)
    signed = crypto.sign(pkey, content, "sha1")
    signatures = agent_bakery.load_agent_signatures(aghash, opsys)
    signatures[certificate] = signed
    agent_bakery.save_agent_signatures(aghash, opsys, signatures)


def decrypt_private_key(key, passphrase):
    from OpenSSL import crypto
    try:
        return crypto.load_privatekey(crypto.FILETYPE_PEM, key["private_key"], passphrase)
    except:
        # FIXME TODO: Cleanup this general exception catcher
        raise MKUserError("key_p_passphrase", _("Invalid pass phrase"))


#.
#   .--Global Settings-----------------------------------------------------.
#   |  ____ _       _           _   ____       _   _   _                   |
#   | / ___| | ___ | |__   __ _| | / ___|  ___| |_| |_(_)_ __   __ _ ___   |
#   || |  _| |/ _ \| '_ \ / _` | | \___ \ / _ \ __| __| | '_ \ / _` / __|  |
#   || |_| | | (_) | |_) | (_| | |  ___) |  __/ |_| |_| | | | | (_| \__ \  |
#   | \____|_|\___/|_.__/ \__,_|_| |____/ \___|\__|\__|_|_| |_|\__, |___/  |
#   |                                                          |___/       |
#   +----------------------------------------------------------------------+
#   | Global WATO settings for agent deployment.                           |
#   '----------------------------------------------------------------------'

group = _("Automatic agent updates")
configvar_order()[group] = 35

register_configvar(group,
    "agent_deployment_enabled",
    Checkbox(
        title = _("Activation of automatic agent updates"),
        label = _("Enable automatic agent updates"),
        help = _("This is the master switch that allows you to immediately stop the agent updates "
                 "in case of some problem."),
    ),
    domain = "multisite",
    need_restart = False,
)

register_configvar(group,
    "agent_deployment_host_selection",
    vs_deployment_host_selection(),
    domain = "multisite",
    need_restart = False,
)


#.
#   .--Declarations--------------------------------------------------------.
#   |       ____            _                 _   _                        |
#   |      |  _ \  ___  ___| | __ _ _ __ __ _| |_(_) ___  _ __  ___        |
#   |      | | | |/ _ \/ __| |/ _` | '__/ _` | __| |/ _ \| '_ \/ __|       |
#   |      | |_| |  __/ (__| | (_| | | | (_| | |_| | (_) | | | \__ \       |
#   |      |____/ \___|\___|_|\__,_|_|  \__,_|\__|_|\___/|_| |_|___/       |
#   |                                                                      |
#   +----------------------------------------------------------------------+
#   |  Declaration of WATO module, WATO menu item and permissions.         |
#   '----------------------------------------------------------------------'

modules.append(
 ( "agents", _("Monitoring Agents"), "agents", "agents",
   _("Package customized monitoring agents for Windows, Linux and other OSes"))
)

modes.update({
    "agents"             : (["agents"], mode_agents),
    "agent_of_host"      : (["agents"], mode_agent_of_host),
    "agent_deployment"   : (["agents"], mode_agent_deployment),
    "signature_keys"     : (["agents", "agent_deployment"], mode_signature_keys),
    "edit_signature_key" : (["agents", "agent_deployment"], mode_edit_signature_key),
})

config.declare_permission("wato.agents",
    _("Create Monitoring Agents"),
    _("Create customized packaged monitoring agents for Linux, Windows and other operating systems"),
    [ "admin" ])

config.declare_permission("wato.download_agents",
    _("Download Monitoring Agents of Your Hosts"),
    _("Download monitoring agents that have been created with the agent bakery for hosts where "
      "you are a contact for.  Please note that these agents might contain "
      "confidential information such as passwords."),
    [ "admin", ])

config.declare_permission("wato.download_all_agents",
    _("Download All Monitoring Agents"),
    _("Download all monitoring agents that have been created with the agent bakery, even if you "
      "are not a contact for the host in question. "
      "Please note that these agents might contain confidential information such as passwords."),
    [ "admin" ])

config.declare_permission("wato.agent_deployment",
    _("Configuration of agent updates"),
    _("This permissions allows full control of the automatic agent updates."),
    [ "admin" ])


backup_domains.update({
    "agent_bakery" : {
        "group"   : _("Agent Bakery & Update"),
        "title"   : _("Baked agents including signatures"),
        "prefix"  : defaults.omd_root,
        "paths"   : [ ( "dir", "var/check_mk/agents" ) ],
    },
    "agent_deployment" : {
        "group"   : _("Agent Bakery & Update"),
        "title"   : _("Current state of agent updates"),
        "prefix"  : defaults.omd_root,
        "paths"   : [ ( "dir", "var/check_mk/agent_deployment" ) ],
    }
})
