#!/usr/bin/python
# -*- encoding: utf-8; py-indent-offset: 4 -*-
# .------------------------------------------------------------------------.
# |                ____ _               _        __  __ _  __              |
# |               / ___| |__   ___  ___| | __   |  \/  | |/ /              |
# |              | |   | '_ \ / _ \/ __| |/ /   | |\/| | ' /               |
# |              | |___| | | |  __/ (__|   <    | |  | | . \               |
# |               \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\              |
# |                                        |_____|                         |
# |             _____       _                       _                      |
# |            | ____|_ __ | |_ ___ _ __ _ __  _ __(_)___  ___             |
# |            |  _| | '_ \| __/ _ \ '__| '_ \| '__| / __|/ _ \            |
# |            | |___| | | | ||  __/ |  | |_) | |  | \__ \  __/            |
# |            |_____|_| |_|\__\___|_|  | .__/|_|  |_|___/\___|            |
# |                                     |_|                                |
# |                     _____    _ _ _   _                                 |
# |                    | ____|__| (_) |_(_) ___  _ __                      |
# |                    |  _| / _` | | __| |/ _ \| '_ \                     |
# |                    | |__| (_| | | |_| | (_) | | | |                    |
# |                    |_____\__,_|_|\__|_|\___/|_| |_|                    |
# |                                                                        |
# | mathias-kettner.com                                 mathias-kettner.de |
# '------------------------------------------------------------------------'
#  This file is part of the Check_MK Enterprise Edition (CEE).
#  Copyright by Mathias Kettner and Mathias Kettner GmbH.  All rights reserved.
#
#  Distributed under the Check_MK Enterprise License.
#
#  You should have  received  a copy of the Check_MK Enterprise License
#  along with Check_MK. If not, email to mk@mathias-kettner.de
#  or write to the postal address provided at www.mathias-kettner.de

# cmc_livestatus_threads       = 20

group = _("Monitoring Core")
configvar_order()[group] = 3


#   .--Performance---------------------------------------------------------.
#   |    ____            __                                                |
#   |   |  _ \ ___ _ __ / _| ___  _ __ _ __ ___   __ _ _ __   ___ ___      |
#   |   | |_) / _ \ '__| |_ / _ \| '__| '_ ` _ \ / _` | '_ \ / __/ _ \     |
#   |   |  __/  __/ |  |  _| (_) | |  | | | | | | (_| | | | | (_|  __/     |
#   |   |_|   \___|_|  |_|  \___/|_|  |_| |_| |_|\__,_|_| |_|\___\___|     |
#   |                                                                      |
#   +----------------------------------------------------------------------+
#   |  Settings for tuning the performance                                 |
#   '----------------------------------------------------------------------'

register_configvar(
    group,
    "cmc_check_helpers",
    Integer(
        title = _("Maximum concurrent active checks"),
	help = _("At start the CMC creates a fixed number of worker processes "
	         "that are used for executing classical active checks. This number is "
		 "configured here. If the number of processes is too low for "
		 "then not all checks can be executed in time. Checks that run "
		 "for a longer time block the workers longer. A larger "
		 "number of helpers will consume more memory."),
	default_value = 5,
	minvalue = 1,
	maxvalue = 500,
    ),
    need_restart = True)

register_configvar(
    group,
    "cmc_cmk_helpers",
    Integer(
        title = _("Maximum concurrent Check_MK checks"),
	help = _("For running Check_MK based checks via the Micro Core keeps "
                 "a pool of specialized helpers that speed up Check_MK by avoiding "
                 "process creations. The size of this pool is configured here. "
                 "This is identical with the maximum number of concurrent "
                 "Check_MK checks."),
	default_value = 20,
	minvalue = 1,
	maxvalue = 500,
    ),
    need_restart = True)


register_configvar(
    group,
    "cmc_livestatus_threads",
    Integer(
        title = _("Maximum concurrent Livestatus connections"),
	help = _("The maximum number of concurrent Livestatus connections is limited by this setting. Usually the "
                 "default value should be fine. If you have a larger number of users and use persistent connections "
                 "then you could run out of Livestatus connections. This results in hanging Apache processes and "
                 "browsers running into timeouts. Increase this number in such a case. Also consider using the "
                 "Livestatus proxy."),
        default_value = 20,
        minvalue = 1,
        maxvalue = 2000,
    ),
    need_restart = True)


register_configvar(
    group,
    "cmc_statehist_cache",
    Optional(
        Dictionary(
            elements = [
                ( "horizon",
                  Age(title = _("Time horizon to keep in cache"),
                      help = _("You cannot query for time ranges that are before "
                               "the horizon."),
                      default_value = 3600 * 24 * 365 * 2,
                      minvalue = 1,
                  ),
                ),
                ( "max_core_downtime",
                  Age(title = _("Ignore core restarts shorter than"),
                      help = _("When it comes to the computation of unmonitored time spans, "
                               "core stops and starts that last shorter then this period will be "
                               "ignored. A time of 0 will honor every core downtime."),
                      default_value = 30,
                  ),
                ),
                ( "tarpit",
                  Integer(title = _("Artificial slowdown of logfile parsing (for testing!)"),
                          help = _("This option is just for testing. It will be removed in future. Do "
                                   "not use it unless you are told to do so!"),
                          unit = _("ms / entry"),
                          default_value = 0,
                  ),
                ),
            ],
            optional_keys = [ "tarpit" ],
        ),
        title = _("In-memory cache for availability data"),
        help = _("This option enables a new implementation of the Livestatus table "
                 "<tt>statehist</tt> that makes use of a in-memory cache of state "
                 "transitions of hosts and services. This dramatically speeds up "
                 "availability queries for large installations."),
        label = _("Enable in-memory cache for availability"),
        none_label = _("disabled"),
    ),
    need_restart = True)


register_configvar(
    group,
    "cmc_livestatus_logcache_size",
    Integer(
        title = _("Maximum number of cached log messages"),
	help = _("In order to speed up queries for historic data, the core keeps an "
                 "in-memory cache of log file message. This number can be configured "
                 "here. A larger number needs more RAM. Note: even if you set this to 0 "
                 "there might be some cases where messages need to be cached anyway."),
        minvalue = 0,
        maxvalue = 100000000,
        default_value = 500000,
    ),
    need_restart = True)



register_configvar(
    group,
    "cmc_timeperiod_horizon",
    Integer(
        title = _("Horizon for timeperiod computation"),
        help = _("In order to keep the core simple, Check_MK pre-computes all "
                 "transitions for all timeperiods in advance. Since that cannot "
                 "be done until infinity you need to specify the number of days "
                 "for the precomputation. After that time all periods remain "
                 "in their last state. Make sure that you <i>Activate Changes</i> "
                 "before that time."),
        minvalue = 7,
        maxvalue = 365 * 30,
        unit = _("days"),
    ),
    need_restart = True)



register_configvar(
    group,
    "cmc_state_retention_interval",
    Age(
        title = _("State retention interval"),
        help = _("For sake of optimal performance Check_MK Micro Core keeps all current "
                 "state data in the main memory of the core. Before a regular shut down "
                 "this state is saved to disk. In order to prevent data loss in a case of "
                 "a system crash the state is also saved to disk on a regular base. The "
                 "interval for this is configured here. Smaller intervals keep the amount "
                 "of lost data smaller but create a larger disk IO."),
        default_value = 600,
    ),
    need_restart = True)

register_configvar(
    group,
    "cmc_initial_scheduling",
    Dictionary(
        title = _("Initial Scheduling"),
        help = _("These settings control how the <i>initial</i> scheduling of new active "
                 "checks should be done. If you just add a few new checks then those are "
                 "scheduled immediately. Then you add a larger number of services then their "
                 "initial checks are spread over the complete check interval."),
        optional_keys = False,
        elements = [
            ( "burst",
              Integer(
                  title = _("Immediately scheduled checks"),
                  help = _("This number of new checks is scheduled immediately"),
                  minvalue = 0,
                  default_value = 10,
           )),
           ( "spread_cmk",
             Integer(
                 title = _("Time between new checks of type Check_MK"),
                 help = _("This interval is applied to all new checks that go beyond the number "
                          "of immediately scheduled checks. Note: the maximum delay is five times "
                          "the normal check interval"),
                 unit = "ms",
                 default_value = 1200,
             )
           ),
           ( "spread_generic",
             Integer(
                 title = _("Time between new classical active checks"),
                 help = _("This interval is applied to all new non-Check_MK active checks that go beyond the number "
                          "of immediately scheduled checks. Note: the maximum delay is five times "
                          "the normal check interval"),
                 unit = "ms",
                 default_value = 150,
             )
           ),
        ]
    ),
    need_restart = True
)

register_configvar(
    group,
    "cmc_smartping_tuning",
    Dictionary(
        title = _("Tuning of Smart PING"),
        elements = [
          ( "omit_payload",
            Checkbox(
                title = _("Omit ICMP payload"),
                label = _("Omit payload in ICMP packets"),
                help = _("The command line <tt>ping</tt> command defaults to sending ICMP packets "
                         "with a payload of 40 bytes. Together with the IP and ICMP header that "
                         "sums up to 64 Bytes. For detecting the reachability of a host no payload "
                         "is neccessary, so Smart PING can omit the payload and send packets of "
                         "24 bytes. Some firewalls seem to interprete this as some kind of bogus attack "
                         "and block such packets."),
                default_value = False,
          )),
          ( "num_sockets",
            Integer(
                title = _("Number of ICMP sockets"),
                help = _("In environments with a large number of hosts a single raw socket "
                         "does not have enough buffer space to output the required ICMP bandwidth. "
                         "For that reason we use several sockets in parallel."),
                default_value = 8,
                minvalue = 1,
                maxvalue = 64,
          )),
          ( "ignore_rst",
            Checkbox(
                title = _("Ignore TCP Reset Packets"),
                label = _("Ignore TCP RST packets when determining host state"),
                help = _("Some routers/firewalls send a TCP RST for a host which is known to be down, "
                         "using the source IP of the host in question. Enabling this flag prevents "
                         "Smart PING from considering such a host as being up."),
                default_value = False,
          )),
          ( "throttling",
            Tuple(
                title = _("Limit rate of PINGs"),
                help = _("Some firewalls are not capable of dealing with a few thousand PING packets "
                         "in one millisecond. With this setting you can force a small wait time "
                         "between two bulks of PINGs being sent. Per default we send with the full "
                         "possible rate"),
                elements = [
                Integer(
                    title = _("Wait time between PING bulks:"),
                    default_value = 10,
                    unit = "micro seconds",
                    size = 10,
                  ),
                  Integer(
                       title = _("Send that many PINGs in one bulk:"),
                       help = _("If you enable a wait time then this number of PINGs will always be sent "
                                "in one piece without waiting. This reduces the overhead."),
                       default_value = 1000,
                       minvalue = 1,
                  ),
            ]
        )),
        ],
        optional_keys = [ "throttling" ],
    ),
    need_restart = True)

#.
#   .--Logging-------------------------------------------------------------.
#   |                _                      _                              |
#   |               | |    ___   __ _  __ _(_)_ __   __ _                  |
#   |               | |   / _ \ / _` |/ _` | | '_ \ / _` |                 |
#   |               | |__| (_) | (_| | (_| | | | | | (_| |                 |
#   |               |_____\___/ \__, |\__, |_|_| |_|\__, |                 |
#   |                           |___/ |___/         |___/                  |
#   '----------------------------------------------------------------------'

register_configvar(
    group,
    "cmc_log_level",
    DropdownChoice(
        title = _("Log level"),
	help = _("This setting decides which types of messages to log into "
                 "the daemon log of the Check_MK Micro Core. The object "
                 "history (service alerts, events, etc.) is <b>not</b> affected "
                 "by this setting."),
        choices = [
                (0, _("Emergency") ),
                (1, _("Alert") ),
                (2, _("Critical") ),
                (3, _("Error") ),
                (4, _("Warning") ),
                (5, _("Notice") ),
                (6, _("Informational") ),
                (7, _("Debug") ),
        ],
        default_value = 5,
    ),
    need_restart = True)


register_configvar(
    group,
    "cmc_debug_notifications",
    Checkbox(
        title = _("Logging of the notification mechanics"),
        label = _("Enable notification logging"),
	help = _("This options creates additional log messages in various phases "
                 "of the notification mechanics. Each time a notification is suppressed "
                 "due to settings a log message will explain the exact reason."),
        default_value = False,
    ),
    need_restart = True)


register_configvar(
    group,
    "cmc_livestatus_debug",
    DropdownChoice(
        title = _("Logging of Livestatus queries"),
	help = _("Here you can enable debugging for Livestatus queries."),
        choices = [
                (0, _("off - no debugging") ),
                (1, _("moderate") ),
                (2, _("log full queries") ),
        ],
        default_value = 0,
    ),
    need_restart = True)


register_configvar(
    group,
    "cmc_log_rrdcreation",
    DropdownChoice(
        title = _("Logging of RRD creation"),
        help = _("Here you can configure whether and how the creation of RRD files "
                 "is being logged (in the log file var/log/cmk-create-rrd.log)"),
        default_value = None,
        choices = [
           ( None,    _("No logging") ),
           ( "terse", _("Terse") ),
           ( "full",  _("Full - Including RRA configuration") ),
        ]
    ),
    need_restart = True)


register_configvar(
    group,
    "cmc_log_cmk_helpers",
    Dictionary(
        title = _("Logging of Check_MK helpers"),
	help = _("Here you can specify logging and debugging options of the Check_MK "
                 "check helpers."),
        elements = [
            ("log_level", DropdownChoice(
                title = _("Log level"),
                choices = [
                        (0, _("off - no extra logging") ),
                        (1, _("moderate") ),
                        (2, _("full detailed logging") ),
                ],
                default_value = 0,
            )),
            ("debug", Checkbox(
                title = _("Debug mode"),
                label = _("Enable debug mode"),
                help = _("When enabled all exceptions occuring during the helper execution "
                         "will raise through and directly result in termination of the helper "
                         "process."),
                default_value = False,
            )),
        ],
        optional_keys = [],
    ),
    need_restart = True)


register_configvar(
    group,
    "cmc_log_microtime",
    Checkbox(
        title = _("Add micro seconds to timestamps in Logfiles"),
	help = _("Normally the resolution of the daemon logfile is in seconds. This "
                 "option enable micro second precision."),
        default_value = False,
    ),
    need_restart = True)


register_configvar(
    group,
    "cmc_log_rotation_method",
    DropdownChoice(
        title = _("Rotation of history log file"),
        help = _("In the history file all events like notifications and alerts "
                 "are saved. This file is being renamed and restarted on a regular "
                 "basis, which can be configured here."),
        default_value = 3,
        choices = [
            (0, _("Rotate once a minute (only for testing!)")),
            (1, _("Rotate every full hour")                  ),
            (2, _("Rotate every day at 00:00 local time")    ),
            (3, _("Rotate every monday at 00:00 local time") ),
            (4, _("Do not rotate based on the time")         ),
        ],
    ),
    need_restart = True)


register_configvar(
    group,
    "cmc_livestatus_lines_per_file",
    Integer(
        title = _("Maximum number of parsed lines per log file"),
	help = _("In order to avoid large timeouts in case of oversized history logfiles "
                 "the core limits the number of lines read from history logfiles. The limit "
                 "is on a per-file-base and can be configred here. Exceeding lines are simply "
                 "ignored and an error is logged in the the CMC daemon logfile."),
        minvalue = 10,
        maxvalue = 2000000000,
        default_value = 1000000,
    ),
    need_restart = True)


register_configvar(
    group,
    "cmc_log_limit",
    Filesize(
        title = _("Limit of the size of the history log file"),
        help = _("Regardless of the setting <i>Rotation of history log file</i> "
                 "a rotation of the log file will be forced whenever its size exceeds "
                 "the limit that is configured here. Note: The actual size of the "
                 "log file can exceed this size by a few percent in situation with "
                 "a high logging volume."),
        default_value = 20 * 1024 * 1024,
        minvalue = 100 * 1024,
        maxvalue = 1 * 1024 * 1024 * 1024,
    ),
    need_restart = True)


register_configvar(
    group,
    "cmc_dump_core",
    Checkbox(
        title = _("Enable core dumps"),
	help = _("In case of an illegal exception the Micro Core can dump core. This helps "
                 "post-mortem debugging but can create a file with several MB in size. The "
                 "core dumps are stored in <tt>var/check_mk/core</tt> by default, but this "
                 "might vary depending on your system settings."),
        default_value = False,
    ),
    need_restart = True)

#.
#   .--Monitoring----------------------------------------------------------.
#   |          __  __             _ _             _                        |
#   |         |  \/  | ___  _ __ (_) |_ ___  _ __(_)_ __   __ _            |
#   |         | |\/| |/ _ \| '_ \| | __/ _ \| '__| | '_ \ / _` |           |
#   |         | |  | | (_) | | | | | || (_) | |  | | | | | (_| |           |
#   |         |_|  |_|\___/|_| |_|_|\__\___/|_|  |_|_| |_|\__, |           |
#   |                                                     |___/            |
#   +----------------------------------------------------------------------+
#   |  Settings that affect the logic of the monitoring                    |
#   '----------------------------------------------------------------------'

register_configvar(
    group,
    "cmc_authorization",
    Dictionary(
        title = _("Authorization settings"),
        help = _("These settings control under which circumstances a user "
                 "is allowed to see a host or service in the user interface. <b>Note:</b> "
                 "This has <b>no</b> impact on notifications."),
        optional_keys = False,
        elements = [
            ( "host",
              DropdownChoice(
                  title = _("Hosts"),
                  choices = [
                    ( 0, _("Loose - Contacts for hosts see all services") ),
                    ( 1, _("Strict - Must be explicit contact of a service") ),
                  ],
                  default_value = 0,
             )
           ),
            ( "group",
              DropdownChoice(
                  title = _("Host-/Servicegroups"),
                  choices = [
                    ( 0, _("Loose - Sufficient to be contact for one member") ),
                    ( 1, _("Strict - Must be contact of all members") ),
                  ],
                  default_value = 0,
             )
           ),
        ]
    ),
    need_restart = True
)

register_configvar(
    group,
    "cmc_check_timeout",
    Age(
        title = _("Check plugin timeout"),
	help = _("After the configured time active host- and service checks are being "
                 "interrupted and evaluate to the UNKNOWN state."),
        default_value = 60,
        minvalue = 1,
        maxvalue = 7200,
    ),
    need_restart = True)

register_configvar(
    group,
    "cmc_flap_settings",
    Tuple(
        title = _("Tuning of flap detection"),
        help = _("These settings tune, how sensitive the core is to state changes "
                 "when it decides whether the state of an object is <b>flapping</b>. Each object "
                 "keeps are value called <i>flappiness</i>. This value is increased by 1.0 each "
                 "time its state changes. Every minute the value is being decreased by a certain "
                 "part that is relative to its current amount. When the flappiness reaches the "
                 "<i>high threshold</i> the object is set to <i>flapping</i>. When it later "
                 "falls below the <i>low threshold</i> the flapping state ends."),
        elements = [
            Float(
                title = _("low threshold"),
                minvalue = 0.1,
                default_value = 3.0,
            ),
            Float(
                title = _("high threshold"),
                minvalue = 1.0,
                default_value = 5.0,
            ),
            Transform(
                Percentage(
                    minvalue = 1,
                    maxvalue = 100,
                    default_value = 10,
                    unit = _("% / min"),
                    size = 5,
                ),
                title = _("decay"),
                forth = lambda x: x * 100,
                back = lambda y: y / 100,
           )
        ]),
    need_restart = True)


register_rule(
    "monconf/" + _("Host Checks"),
    "cmc_smartping_settings",
    Dictionary(
        title = _("Settings for host checks via Smart PING"),
        help = _("Smart PING is the default method for host checks when using the Check_MK Micro Core. "
                 "The Core sends out ICMP echo requests (PING) at the rate configured with "
                 "<i>Normal check interval for host checks</i> "
                 "(or <i>Retry check interval for host checks</i> while the host is in soft non-UP state). "),
        optional_keys = False,
        elements = [
            ( "timeout",
            Alternative(
                 title = _("Expected packet rate"),
                 help = _("If no ICMP or TCP-SYN/RST packet is being seen from the target host for a certain amount "
                 "of time then the host is considered down. This time is configured here. It defaults to "
                 "2.5 times of the host check interval. <b>Note</b>: This setting is <i>not</i> measured from "
                 "the point of time when the PING packet is being sent but from the time the last packet "
                 "from the host was being seen."),
                 elements = [
                     Float(
                         title = _("In relation to normal check interval"),
                         label = _("at least one packet every"),
                         default_value = 2.5,
                         unit = _("times the interval")),
                     Integer(
                         title = _("Fixed rate"),
                         label = _("at least one packet every"),
                         unit = _("sec"),
                         default_value = 20),
                 ],
                 default_value = 2.5,
            ))
        ]
    ),
    match = 'dict'
)


register_rule(
    "agent/" + _("SNMP"),
    "snmp_limit_oid_range",
    Tuple(
        elements = [
            DropdownChoice(
                title = _("Check Type"),
                choices = [
                    ("if64_tplink",       _("SNMP interface check for TP-Link Switches")),
                    ("if64",              _("SNMP interface check with 64 bit counters (using v2c)")),
                    ("if64adm",           _("SNMP interface check with 64 bit counters using "
                                            "admin states (using v2c)")),
                    ("if",                _("SNMP interface check with 32 bit counters (v1)")),
                    ("if_lancom",         _("SNMP interface check for LANCOM devices")),
                    ("emc_isilon_quota",  _("SNMP check for EMC Isilon quotas")),
                ],
                sorted = True,
            ),
            Optional(
                ListOf(
                    CascadingDropdown(
                        choices = [
                            ("first", _("from the beginning"), Integer(
                                minvalue = 1,
                                unit = _("entries"),
                            )),
                            ("mid",  _("range in the middle"), Tuple(
                                elements = [
                                    Integer(
                                        title = _("Starting with entry"),
                                        minvalue = 1,
                                    ),
                                    Integer(
                                        title = _("Number of entries"),
                                        minvalue = 1,
                                    ),
                                ]
                            )),
                            ("last", _("from the end"), Integer(
                                minvalue = 1,
                                unit = _("entries"),
                            )),
                        ],
                        orientation = "horizontal",
                    ),
                ),
                label = _("Only fetch specific OIDs"),
                none_label = _("Fetch all available sub OIDs"),
            ),
        ],
    title = _("Limit SNMP OID Ranges for Bulkwalks"),
    help = _("When performing walks on an OID during checking, Check_MK fetches all "
             "sub OIDs regardless whether these values are needed or not. For example, "
             "if you monitor a single network interface of a switch with hundrets of "
             "ports, the check will fetch the information for all the interfaces, while "
             "only one is used. This rule can be used to reduce the amount of fetched "
             "OIDs, but please note, this is only useful for special cases like "
             "the example above and only used when you perform bulkwalks."))
)


register_configvar(
    group,
    "cmc_real_time_checks",
    Alternative(
        title = _("Enable handling of Real-Time Checks"),
	help = _("The Check_MK Agent supports a feature called Real-Time Checks where it sends status updates "
                 "of specific monitoring information more often than the regular check interval of the host, "
                 "for example in an interval of 1 second."
                 "This operation mode requires the communication between the agent and the monitoring server "
                 "to be extended so that the agent can send the status updates to the monitoring server "
                 "as UDP packets. The monitoring core needs to listen on this port and needs to know the "
                 "secrect which has been used to encrypt the sent data to be able to receive and decrypt "
                 "it."),
        elements = [
            FixedValue(None,
                title = _("Don't handle Real-Time Checks"),
                totext = _("Don't handle Real-Time Checks"),
            ),
            Dictionary(
                title = _("Handle Real-Time Checks"),
                elements = [
                    ("port", Integer(
                        title = _("UDP port to listen to"),
                        help = _("Specify the UDP port to listen on for Real-Time Check packets. "
                                 "It needs to match the port configured in the "
                                 "<a href=\"wato.py?mode=edit_ruleset&varname=agent_config:real_time_checks\">agent</a>."),
                        default_value = 6559,
                        minvalue = 1025, # don't allow well known ports
                        maxvalue = 65535,
                    )),
                    ("secret", Password(
                        title = _("Secret to decrypt the packets"),
                        allow_empty = False,
                        help = _("This secret is shared between the agent and the Check_MK server and "
                                 "used to decrypt the Real-Time packet contents. It needs to match "
                                 "the secret configured in the "
                                 "<a href=\"wato.py?mode=edit_ruleset&varname=agent_config:real_time_checks\">agent</a>."),
                    )),
                ],
                optional_keys = [],
            ),
        ],
        style = "dropdown",
    ),
    need_restart = True)

#.
#   .--Graphing------------------------------------------------------------.
#   |               ____                 _     _                           |
#   |              / ___|_ __ __ _ _ __ | |__ (_)_ __   __ _               |
#   |             | |  _| '__/ _` | '_ \| '_ \| | '_ \ / _` |              |
#   |             | |_| | | | (_| | |_) | | | | | | | | (_| |              |
#   |              \____|_|  \__,_| .__/|_| |_|_|_| |_|\__, |              |
#   |                             |_|                  |___/               |
#   '----------------------------------------------------------------------'

register_configvar(
    group,
    "cmc_pnp_update_delay",
    Age(
        title = _("Update delay for PNP meta information"),
	help = _("PNP4Nagios needs meta information about hosts and services. These information "
                 "is stored in XML files. Updating these files for each new check results "
                 "produces lots of I/O. Delaying the update for up to one hour saves this I/O."
                 "The price is, that WARN/CRIT levels displayed in graphs might not be uptodate for "
                 "a while after a configuration change. Select <tt>0</tt> for no delay."),
        default_value = 3600,
    ),
    need_restart = True)


register_configvar(
    group,
    "cmc_pnp_update_on_restart",
    Checkbox(
        title = _("Force update of PNP meta information on restart / reload"),
	help = _("PNP4Nagios needs meta information about hosts and services. These information "
                 "is stored in XML files. Updating these files for on restart or reload of the monitoring "
                 "core might cause a peak of I/O short time after the restart. This might be no problem "
                 "for smaller installations. You can enable this to have the meta information updated when "
                 "processing the result of the first check after the restart."),
        default_value = False,
    ),
    need_restart = True)



def migrate_cmc_graphite(v):
    if v == None:
        return []
    elif type(v) == dict:
        return [v]
    else:
        return v


register_configvar(
    group,
    "cmc_graphite",
    Transform(
        ListOf(
            Dictionary(
                elements = [
                    ( "host",
                      TextAscii(
                          title = _("Hostname / IP address of Graphite server"),
                          default_value = "127.0.0.1")
                    ),
                    ( "port",
                      Integer(
                          title = _("TCP port of Carbon line receiver"),
                          default_value = 2003,
                          minvalue = 1,
                          maxvalue = 65535,
                      ),
                    ),
                    ( "prefix",
                      TextAscii(
                          title = _("Optional variable prefix"),
                          help = _("The text entered here well be prefixed to the hostname. Ending the prefix "
                                   "with a period will create a separate folder in Graphite"),
                          allow_empty = True,
                      )
                    ),
                ],
                optional_keys = False,
            ),
            title = _("Send metrics to Graphite / InfluxDB"),
            help = _("When this option is enabled then all performance data (check metrics) will "
                     "<i>in addition</i> be sent to an external Graphite (Carbon) installation of your choice. "
                     "Graphite graphs will be named with the schema <i>HOST / SERVICE / VARIABLE</i>."),
            label = _("Activate export to Graphite"),
            add_label = _("Add new Graphite / InfluxDB connection"),
            text_if_empty = _("No connections defined"),
            movable = False,
        ),
        forth = migrate_cmc_graphite,
    ),
    need_restart = True)



def validate_rra_config(value, varprefix):
    rras = value["rras"]
    last_steps = 0
    nr = 0
    for interval, steps, rows in rras:
        nr += 1
        if steps <= last_steps:
            raise MKUserError(varprefix, _("RRA entry number %d: the number of steps must be greater than that of the previous entry.") % nr)
        last_steps = steps


rrd_config_elements = [
            ( "format",
              DropdownChoice(
                  title = _("RRD storage format"),
                  choices = [
                      ( "cmc_single",   _("One RRD per host/service (saves disk IO, only with CMC)")),
                      ( "pnp_multiple", _("Multiple RRDs per host/service (compatible with PNP4Nagios)")),
                  ]
            )),
            ( "cfs",
              ListChoice(
                  title = _("Consolidation functions"),
                  allow_empty = False,
                  choices = [
                    ( "MIN",     _("Minimum") ),
                    ( "MAX",     _("Maximum") ),
                    ( "AVERAGE", _("Average") ),
                  ],
                  default_value = [ "MIN", "MAX", "AVERAGE" ],
               ),
            ),
            ( "step",
              Integer(
                  title = _("Step (precision)"),
                  help = _("This setting controls the resolution of the RRD in seconds. The smaller the step "
                           "the more precise the timing will be, but the larger your RRD will be. It does not "
                           "make sense to have the step smaller then your normal check interval for a service. "
                           "The default is a value of 60 seconds - which matches the default check interval."),
                  minvalue = 1,
                  default_value = 60,
                  unit = _("sec."),
              ),
            ),
            ( "rras",
              ListOf(
                  Tuple(
                      elements = [
                          Percentage(
                              title = _("Percentage of points below which an interval is <i>unknown</i>. "),
                              help = _("This is relevant during periods where the according check cannot "
                                       "be executed for any reason and thus no metrics data is being generated. "
                                       "The RRD distinguishes between an unknown value and a zero value. "
                                       "If metrics are missing for just a short period of time then RRD makes "
                                       "an interpolation and does not mark that period as unknown. This setting defines "
                                       "who many percent of the time metrics need to be available for not making "
                                       "the period unknown. This is relevant for aggregations that are much larger "
                                       "than one minute."),
                              default_value = 50),
                          Integer(
                              title = _("Number of steps aggregated into one data point"),
                              help = _("The size of one step is usually one minute (defined in this same rule set). "
                                       "If you use e.g. 10 here and keep the default setting of 60 seconds, then RRD "
                                       "will keep one value for every 10 minutes of time."),
                              minvalue = 1),
                          Integer(
                              title = _("Number of data points"),
                              help = _("If you multiply the number of data points with the number of steps aggregated then "
                                       "you get the time range that is stored in this archive. Each data point needs 8 bytes "
                                       "for each aggregation (MIN, MAX, AVERAGE) that you have selected."),
                              minvalue = 1,
                              defaut_value = 3600),
                      ]
                  ),
                  title = _("RRA configuration"),
                  help = _("Please select the levels of aggregations. The RRD will automatically compress metrics as they get "
                           "older in order to create a good compromise between needed disk space and precision. The default "
                           "is to keep a one minute precision for the first 2880 minutes (48 hours), then aggregate to "
                           "5 minutes per value for 10 days, then into 30 minute value for 90 days and last one value per "
                           "6 hours for 4 years. When this aggregation takes place then e.g. 5 values need to be converted "
                           "into one. Here RRD needs to decide whether to use MIN, MAX or AVERAGE. Per default all three "
                           "aggregations are done in parallel."),
                  allow_empty = False,
                  default_value = [
                      ( 50.0,   1, 2880 ),
                      ( 50.0,   5, 2880 ),
                      ( 50.0,  30, 4320 ),
                      ( 50.0, 360, 5840 ),
                  ]
              )
            ),
       ]

def transform_rrd_config(d):
    if "format" in d:
        return d
    else:
        return dict(d.items() + [ ("format", "pnp_multiple")])

register_rule(
    "monconf/" + _("Various"),
    "cmc_service_rrd_config",
    Transform(
        Dictionary(
            title = _("Configuration of RRD databases of services"),
            help = _("This configures how many datapoints will be stored of the performance values of "
                     "services. Please note, that these settings only apply for <i>new</i> services. "
                     "Existing RRDs cannot be changed."),
            optional_keys = False,
            elements = rrd_config_elements,
            validate = validate_rra_config,
        ),
        forth = transform_rrd_config,
    ),
    match = 'first',
    itemtype = "service",
)

register_rule(
    "monconf/" + _("Various"),
    "cmc_host_rrd_config",
    Transform(
        Dictionary(
            title = _("Configuration of RRD databases of hosts"),
            help = _("This configures how many datapoints will be stored of the performance values of "
                     "hosts. Note: hosts checked via Smart PING do not create any performance data."),
            optional_keys = False,
            elements = rrd_config_elements,
            validate = validate_rra_config,
        ),
        forth = transform_rrd_config,
    ),
    match = 'first',
)


#.
#   .--Debugging-----------------------------------------------------------.
#   |          ____       _                       _                        |
#   |         |  _ \  ___| |__  _   _  __ _  __ _(_)_ __   __ _            |
#   |         | | | |/ _ \ '_ \| | | |/ _` |/ _` | | '_ \ / _` |           |
#   |         | |_| |  __/ |_) | |_| | (_| | (_| | | | | | (_| |           |
#   |         |____/ \___|_.__/ \__,_|\__, |\__, |_|_| |_|\__, |           |
#   |                                 |___/ |___/         |___/            |
#   +----------------------------------------------------------------------+
#   |  These settings are currently remove from the GUI                    |
#   '----------------------------------------------------------------------'

if False:
    register_configvar(
        group,
        "cmc_turbo_mode",
        Checkbox(
            title = _("Enable Turbo Scheduling mode"),
            help = _("With this option enabled the core schedules all checks "
                      "<b>as fast as possible</b>. As soon as a check is finished "
                      "it is being scheduled for the earliest possible time. This "
                      "is useful for performance-measurement. You probably won't "
                      "want to do this in a productive environment."),
            default_value = False,
        ),
        need_restart = True)

    register_configvar(
        group,
        "cmc_livestatus_delay",
        Optional(
            Dictionary(
                optional_keys = False,
                elements = [
                    ( "delay",
                      Integer(title = _("Delay Livestatus responses"),
                              default_value = 500,
                              unit = _("ms")),
                    ),
                    ( "randomize",
                      Integer(title = _("Random additional delay"),
                              default_value = 0,
                              unit = _("ms"))
                    ),
                ]
            ),
            title = _("Artificial Livestatus Delay"),
            help = _("This option is just for testing and can be used in "
                     "tests where long round trip times should be simulated. "
                     "Never use this option in production mode."),
        ),
        need_restart = True)

#.
#   .--Various-------------------------------------------------------------.
#   |                __     __         _                                   |
#   |                \ \   / /_ _ _ __(_) ___  _   _ ___                   |
#   |                 \ \ / / _` | '__| |/ _ \| | | / __|                  |
#   |                  \ V / (_| | |  | | (_) | |_| \__ \                  |
#   |                   \_/ \__,_|_|  |_|\___/ \__,_|___/                  |
#   |                                                                      |
#   '----------------------------------------------------------------------'

register_configvar(
    group,
    "cmc_import_nagios_state",
    Checkbox(
        title = _("Import state from Nagios core"),
	help = _("When switching from the Nagios core to CMC with <tt>omd config</tt> "
                 "with this option enabled, then during the configuration generation for the CMC "
                 "the current state of Nagios comments, downtimes and switches from "
                 "<i>Master Control</i> will be imported into CMC. <b>Note:</b> This implies "
                 "that any previous state of the CMC will be <i>lost</i>. The import will always "
                 "take place when the age of the Nagios state file is younger than that of the CMC. "
                 "Therefore once you are using CMC in production we propose switching this "
                 "option to <b>False</b>."),
        default_value = True,
    ),
    need_restart = False)


#.
#   .--Recurring Downtimes-------------------------------------------------.
#   |            ____                           _                          |
#   |           |  _ \ ___  ___ _   _ _ __ _ __(_)_ __   __ _              |
#   |           | |_) / _ \/ __| | | | '__| '__| | '_ \ / _` |             |
#   |           |  _ <  __/ (__| |_| | |  | |  | | | | | (_| |             |
#   |           |_| \_\___|\___|\__,_|_|  |_|  |_|_| |_|\__, |             |
#   |                                                   |___/              |
#   |         ____                      _   _                              |
#   |        |  _ \  _____      ___ __ | |_(_)_ __ ___   ___  ___          |
#   |        | | | |/ _ \ \ /\ / / '_ \| __| | '_ ` _ \ / _ \/ __|         |
#   |        | |_| | (_) \ V  V /| | | | |_| | | | | | |  __/\__ \         |
#   |        |____/ \___/ \_/\_/ |_| |_|\__|_|_| |_| |_|\___||___/         |
#   |                                                                      |
#   '----------------------------------------------------------------------'

recurring_downtimes_types = {
    1: _("hour"),
    2: _("day"),
    3: _("week"),
    4: _("second week"),
    5: _("fourth week"),
    6: _("same nth weekday (from beginning)"),
    7: _("same nth weekday (from end)"),
    8: _("same day of the month"),
    # 999: _("5 minutes (just for testing)"),
}

group = "monconf/" + _("Notifications")

def RecurringDowntime(what_title):
    return Dictionary(
        title = _("Recurring downtimes for %s") % what_title,
        help = _("This rule set allows you to create downtimes for %s that are "
                 "repeated on a regular base.") % what_title,
        elements =  [
            ( "comment",
                TextUnicode(
                    title = _("Comment for the downtime"),
                    allow_empty = False,
                    size = 60,
            )),
            ( "start_time",
                AbsoluteDate(
                    title = _("First time the downtime starts"),
                    include_time = True,
            )),
            ( "interval",
                DropdownChoice(
                    title = _("Repeat this downtime"),
                    choices = sorted(recurring_downtimes_types.items()),
                    default_value = 3,
            )),
            ( "duration",
              Age(
                  title = _("Duration"),
                  default_value = 7200,
                  display = [ "hours", "minutes" ],
            )),
            ( "flex_length",
              Age(
                  title = _("Flexible start time"),
                  label = _("Maximum delay until downtime starts:"),
                  default_value = 7200,
                  display = [ "hours", "minutes" ],
            )),
        ],
        optional_keys = [ "flex_length" ],
    )

register_rule(group,
    "host_recurring_downtimes",
    RecurringDowntime(_("hosts")),
    match = "all",
)

register_rule(group,
    "service_recurring_downtimes",
    RecurringDowntime(_("services")),
    itemtype = "service",
    match = "all",
)


#.
#   .--Alert Handlers------------------------------------------------------.
#   |     _    _           _     _   _                 _ _                 |
#   |    / \  | | ___ _ __| |_  | | | | __ _ _ __   __| | | ___ _ __ ___   |
#   |   / _ \ | |/ _ \ '__| __| | |_| |/ _` | '_ \ / _` | |/ _ \ '__/ __|  |
#   |  / ___ \| |  __/ |  | |_  |  _  | (_| | | | | (_| | |  __/ |  \__ \  |
#   | /_/   \_\_|\___|_|   \__| |_| |_|\__,_|_| |_|\__,_|_|\___|_|  |___/  |
#   |                                                                      |
#   +----------------------------------------------------------------------+
#   |  WATO module for configuration of rule based alert handling.         |
#   '----------------------------------------------------------------------'

def load_alert_handler_rules():
    filename = wato_root_dir + "alert_handlers.mk"
    if not os.path.exists(filename):
        return []
    try:
        vars = { "alert_handler_rules" : [] }
        execfile(filename, vars, vars)
        return vars["alert_handler_rules"]
    except Exception, e:
        if config.debug:
            raise MKGeneralException(_("Cannot read configuration file %s: %s" %
                          (filename, e)))
        return []

def save_alert_handler_rules(rules):
    make_nagios_directory(wato_root_dir)
    file(wato_root_dir + "alert_handlers.mk", "w").write("alert_handler_rules += %s\n" % pprint.pformat(rules))


config.declare_permission("wato.alert_handlers",
    _("Alert handler configuration"),
   _("This permission is needed for creating alert handlers, which automatically "
     "execute custom scripts upon monitoring alerts."),
   [ "admin", ])




def mode_alert_handlers(phase):
    if phase == "title":
        return _("Alert handler configuration")

    elif phase == "buttons":
        global_buttons()
        html.context_button(_("New Rule"), html.makeuri_contextless([("mode", "alert_handler_rule")]), "new")
        return

    rules = load_alert_handler_rules()
    num_rules_before_action = len(rules)

    if phase == "action":
        result = generic_rule_list_actions(rules, "alert_handler",
                                           _("alert handler rule"),  save_alert_handler_rules)
        if len(rules) == 0 and num_rules_before_action > 0:
            log_pending(SYNCRESTART, None, "delete-last-alert-rule", _("Delete last alert handler rule"))

        return result

    if not rules:
        html.message(_("You have not created any rules yet."))
        return

    table.begin(title = _("Alert Handler Rules"), limit = None)

    for nr, rule in enumerate(rules):
        table.row()

        delete_url = make_action_link([("mode", "alert_handlers"), ("_delete", nr)])
        top_url    = make_action_link([("mode", "alert_handlers"), ("_move", nr), ("_where", 0)])
        bottom_url = make_action_link([("mode", "alert_handlers"), ("_move", nr), ("_where", len(rules)-1)])
        up_url     = make_action_link([("mode", "alert_handlers"), ("_move", nr), ("_where", nr-1)])
        down_url   = make_action_link([("mode", "alert_handlers"), ("_move", nr), ("_where", nr+1)])
        edit_url   = html.makeuri_contextless([("mode", "alert_handler_rule"), ("edit", nr)])
        clone_url  = html.makeuri_contextless([("mode", "alert_handler_rule"), ("clone", nr)])

        table.cell(_("Actions"), css="buttons")
        html.icon_button(edit_url, _("Edit this alert handler rule"), "edit")
        html.icon_button(clone_url, _("Create a copy of this alert handler rule"), "clone")
        html.icon_button(delete_url, _("Delete this alert handler rule"), "delete")
        if not rule is rules[0]:
            html.icon_button(top_url, _("Move this alert_handler rule to the top"), "top")
            html.icon_button(up_url, _("Move this alert_handler rule one position up"), "up")
        else:
            html.empty_icon_button()
            html.empty_icon_button()

        if not rule is rules[-1]:
            html.icon_button(down_url, _("Move this alert_handler rule one position down"), "down")
            html.icon_button(bottom_url, _("Move this alert_handler rule to the bottom"), "bottom")
        else:
            html.empty_icon_button()
            html.empty_icon_button()

        table.cell("", css="narrow")
        if rule.get("disabled"):
            html.icon(_("This rule is currently disabled and will not be applied"), "disabled")
        else:
            html.empty_icon_button()

        alert_handler_name, alert_handler_params = rule["alert_handler"]
        table.cell(_("Alert Handler"), alert_handler_name, css="narrow nowrap")
        table.cell(_("Parameters"), " ".join(["<tt>%s</tt>" % p for p in alert_handler_params]))

        table.cell(_("Description"))
        url = rule.get("docu_url")
        if url:
            html.icon_button(url, _("Context information about this rule"), "url", target="_blank")
            html.write("&nbsp;")
        html.write(html.attrencode(rule["description"]))

        table.cell(_("Conditions"))
        num_conditions = len([key for key in rule if key.startswith("match_")])
        if num_conditions:
            html.write(_("%d conditions") % num_conditions)
        else:
            html.write("<i>%s</i>" % _("(no conditions)"))

    table.end()

def mode_alert_handler_rule(phase):
    edit_nr = int(html.var("edit", "-1"))
    clone_nr = int(html.var("clone", "-1"))

    new = edit_nr < 0

    if phase == "title":
        if new:
            return _("Create new alert handler rule")
        else:
            return _("Edit alert handler rule %d") % edit_nr

    elif phase == "buttons":
        html.context_button(_("All Rules"), html.makeuri_contextless([("mode", "alert_handlers")]), "back")
        return

    rules = load_alert_handler_rules()

    if new:
        if clone_nr >= 0 and not html.var("_clear"):
            rule = {}
            rule.update(rules[clone_nr])
        else:
            rule = {}
    else:
        rule = rules[edit_nr]

    vs = vs_alert_handler_rule()

    if phase == "action":
        if not html.check_transaction():
            return "alert_handlers"

        num_rules_before_action = len(rules)

        rule = vs.from_html_vars("rule")
        vs.validate_value(rule, "rule")

        if new and clone_nr >= 0:
            rules[clone_nr:clone_nr] = [ rule ]
        elif new:
            rules[0:0] = [ rule ]
        else:
            rules[edit_nr] = rule

        save_alert_handler_rules(rules)

        if new:
            log_what = "new-alert_handler-rule"
            log_text = _("Created new alert handler rule")
        else:
            log_what = "edit-alert_handler-rule"
            log_text = _("Changed alert handler rule %d") % edit_nr

        if len(rules) > 0 and num_rules_before_action == 0:
            sync_method = SYNCRESTART
        else:
            sync_method = SYNC
        log_pending(sync_method, None, log_what, log_text)
        return "alert_handlers"


    html.begin_form("rule", method = "POST")
    vs.render_input("rule", rule)
    vs.set_focus("rule")
    forms.end()
    html.button("save", _("Save"))
    html.hidden_fields()
    html.end_form()
    return

def vs_alert_handler_rule():
    # TODO: move this into a function
    matching_conditions = alert_handler_conditions()
    matching_keys = [ t[0] for t in matching_conditions ]

    return Dictionary(
        title = _("General Properties"),
        elements = rule_option_elements() +
        [
            ("alert_handler", vs_alert_handler())
        ] +
        alert_handler_conditions(),
        headers = [
            # General
            ( _("General Properties"), [ "description", "comment", "disabled", "docu_url", ] ),
            # Alert Handler
            ( _("Alert Handler"), [ "alert_handler", ]),
            # Matching:
            ( _("Matching"),  matching_keys ),
        ],
        optional_keys = matching_keys,
        render = "form",
        form_narrow = True,
    )

def vs_alert_handler():
    return Tuple(
        title = _("Alert Handler"),
        elements = [
            TextAscii(
                title = _("Name of the script"),
                help = _("Create an executable script in the directory <tt>%s</tt> and give its name here without "
                         "the directory.") % (defaults.omd_root + "/local/share/check_mk/alert_handlers"),
                size = 32,
                allow_empty = False,
                regex = "[a-zA-Z0-9_. ]",
                regex_error = _("Your script name contains invalid characters."),
            ),
            ListOfStrings(
                 title = _("Call with the following parameters:"),
                 valuespec = TextUnicode(size = 24),
                 orientation = "horizontal",
            )
        ],
    )


def alert_handler_conditions():
    return generic_rule_match_conditions()[:8] + \
           event_rule_match_conditions(flavour="alerts") + \
           [
               ("match_attempt",
                   Tuple(
                       title = _("Restrict to n<sup>th</sup> to m<sup>th</sup> check attempt"),
                       orientation = "float",
                       elements = [
                           Integer(
                               label = _("from"),
                               help = _("Handle alerts if the current check attempt has at least this number. The "
                                        "first attempt always has number 1."),
                               default_value = 1,
                               minvalue = 1,
                               maxvalue = 999999,
                           ),
                           Integer(
                               label = _("to"),
                               help = _("Handle alerts if the current check attempt has at most this number. The "
                                        "first attempt always has number 1."),
                               default_value = 999999,
                               minvalue = 1,
                               maxvalue = 999999,
                           ),
                     ],
               )),
           ]


modules += [
    ( "alert_handlers",    _("Alert Handlers"), "alert_handlers", "alert_handlers",
      _("Rules for automatic execution of scripts upon alerts"), ),
]

modes.update({
    "alert_handlers"      : (["alert_handlers"], mode_alert_handlers),
    "alert_handler_rule"  : (["alert_handlers"], mode_alert_handler_rule),
})

group = _("Alert Handlers")
configvar_order()[group] = 16

register_configvar(
    group,
    "cmc_debug_alerts",
    Checkbox(
        title = _("Logging of the alert processing in the core"),
	help = _("This options creates additional log messages in <tt>var/log/cmc.log</tt> when alerts are being "
                 "sent (or not being sent) to the alert handler."),
        default_value = False,
    ),
    need_restart = True)


register_configvar(
    group,
    "alert_logging",
    DropdownChoice(
        title = _("Alert handler log level"),
        help = _("You can configure the alert handler mechanism to log more details about "
                 "the events into the alert log file. This information is logged "
                 "into the file <tt>%s</tt>") % site_neutral_path(defaults.log_dir + "/alert.log"),
        choices = [
            ( 1, _("Normal logging")),
            ( 2, _("Full dump of all variables"))
        ],
        default_value = 1,
    ),
    domain = "check_mk")


register_configvar(
    group,
    "alert_handler_timeout",
    Age(
        title = _("Timeout for the execution of alert handlers"),
        help = _("When an event handler does not finish after the time configured here, "
                 "it is killed with a normal termination signal. If it does not terminate after "
                 "that, then after another full length timeout it is definitely killed with signal "
                 "9 (KILL)"),
        default_value = 60,
        min_value = 1,
    ),
    domain = "check_mk")


register_configvar(
    group,
    "alert_handler_event_types",
    ListChoice(
        title = _("Types of events that are being processed"),
        help = _("Per default setting only state changes (alerts) trigger any action. You can "
                 "add normal check executions here. Please note that this can lead to a huge "
                 "CPU utilization. Use inline-handlers in such a case in order to avoid process "
                 "creations."),
        choices = [
            ( "statechange", _("State changes of hosts / services") ),
            ( "checkresult", _("All check executions!") ),
        ],
        default_value = [ "statechange", "checkresult" ],
        allow_empty = False,
    ),
    domain = "check_mk")

