#!/usr/bin/python
# -*- encoding: utf-8; py-indent-offset: 4 -*-
# .------------------------------------------------------------------------.
# |                ____ _               _        __  __ _  __              |
# |               / ___| |__   ___  ___| | __   |  \/  | |/ /              |
# |              | |   | '_ \ / _ \/ __| |/ /   | |\/| | ' /               |
# |              | |___| | | |  __/ (__|   <    | |  | | . \               |
# |               \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\              |
# |                                        |_____|                         |
# |             _____       _                       _                      |
# |            | ____|_ __ | |_ ___ _ __ _ __  _ __(_)___  ___             |
# |            |  _| | '_ \| __/ _ \ '__| '_ \| '__| / __|/ _ \            |
# |            | |___| | | | ||  __/ |  | |_) | |  | \__ \  __/            |
# |            |_____|_| |_|\__\___|_|  | .__/|_|  |_|___/\___|            |
# |                                     |_|                                |
# |                     _____    _ _ _   _                                 |
# |                    | ____|__| (_) |_(_) ___  _ __                      |
# |                    |  _| / _` | | __| |/ _ \| '_ \                     |
# |                    | |__| (_| | | |_| | (_) | | | |                    |
# |                    |_____\__,_|_|\__|_|\___/|_| |_|                    |
# |                                                                        |
# | mathias-kettner.com                                 mathias-kettner.de |
# '------------------------------------------------------------------------'
#  This file is part of the Check_MK Enterprise Edition (CEE).
#  Copyright by Mathias Kettner and Mathias Kettner GmbH.  All rights reserved.
#
#  Distributed under the Check_MK Enterprise License.
#
#  You should have  received  a copy of the Check_MK Enterprise License
#  along with Check_MK. If not, email to mk@mathias-kettner.de
#  or write to the postal address provided at www.mathias-kettner.de

import config

try:
    mknotifyd_enabled = config.mknotifyd_enabled
except:
    # Non OMD-users: must enable this explicitely, sorry
    mknotifyd_enabled = False

mknotifyd_config_dir = defaults.default_config_dir + "/mknotifyd.d/wato/"

if mknotifyd_enabled:
    replication_paths.append(( "dir",  "mknotify",  mknotifyd_config_dir))
    backup_paths.append(( "dir",  "mknotify",  mknotifyd_config_dir))

    group = _("Notifications")

    # Check_MK var
    register_configvar(group,
        "notification_spooling",
        Transform(
            DropdownChoice(
                title = _("Notification Spooling"),
                choices = [
                    ( "off",    _("Direct local delivery without spooling") ),
                    ( "local",  _("Asynchronous local delivery by notification spooler") ),
                    ( "remote", _("Forward to remote site by notification spooler") ),
                    ( "both",   _("Asynchronous local delivery plus remote forwarding" ) ),
                ],
                default_value = False,
            ),
            help = _("This option configures how notifications will be processed. Without the "
                     "notification spooler (<tt>mknotifyd</tt>) only direct local delivery "
                     "is possible. Long lasting or excessive notifications might slow down "
                     "the monitoring. If you select remote forwarding, make sure that "
                     "your notification spooler is correctly set up for incoming and/or "
                     "outgoing connections."),
            forth = lambda x: (x == False and "off" or (x == True and "local" or x)),
        ),
        domain = "check_mk",
    )


    # Daemon var
    register_configvar_domain("mknotifyd", mknotifyd_config_dir)
    register_configvar(group,
        "config",
        Dictionary(
            title = _("Notification Spooler Configuration"),
            elements = [
                ( "log_level",
                  DropdownChoice(
                      title = _("Verbosity of logging"),
                      choices = [
                        ( 0, _("Normal logging (only startup, shutdown and errors)") ),
                        ( 1, _("Verbose logging (also spooled notifications)") ),
                        ( 2, _("Debugging (log every single action)") ),
                      ],
                      default_value = 0,
                )),
                ( "deferred_cooldown",
                  Age(
                      title = _("Cooldown time before retrying delivery"),
                      minvalue = 1,
                      maxvalue = 86400,
                      default_value = 180,
                )),
                ( "deferred_force_retransmit",
                  Age(
                      title = _("Delay for forcing retransmission"),
                      help = _("The notification spooler will defer retrying a delivery on the same TCP "
                               "connection for a notification that has already been queued on that "
                               "connection. This is because TCP does not retransmit on its own and does "
                               "not lose any data as long as the connection is valid."),
                      minvalue = 1,
                      default_value = 1800,
                )),
                ( "incoming",
                  Dictionary(
                      columns = 2,
                      title = _("Accept incoming TCP connections"),
                      elements = [
                          ( "listen_port",
                            Integer(
                                title = _("TCP Port"),
                                minvalue = 1024,
                                maxvalue = 65535,
                                default_value = 6555,
                          )),
                          ( "heartbeat_interval",
                            Age(
                                title = _("Interval at which to <b>send</b> regular heart beats"),
                                minvalue = 1,
                                default_value = 10,
                          )),
                      ],
                      optional_keys = None,
                )),
                ( "outgoing",
                  ListOf(
                      Dictionary(
                          columns = 2,
                          elements = [
                              ( "address",
                                Hostname(
                                    title = _("DNS name or IP address of target notification spooler"),
                              )),
                              ( "port",
                                Integer(
                                    title = _("TCP Port"),
                                    minvalue = 1,
                                    maxvalue = 65535,
                                    default_value = 6555,
                              )),
                              ( "cooldown",
                                Age(
                                    title = _("Cooldown time before trying to reconnect after failure"),
                                    minvalue = 1,
                                    default_value = 20,
                              )),
                              ( "heartbeat_interval",
                                Age(
                                    title = _("Interval at which to <b>expect</b> regular heart beats"),
                                    minvalue = 1,
                                    default_value = 10,
                              )),
                              ( "heartbeat_timeout",
                                Age(
                                    title = _("Maximum expected run time for heart beat packet"),
                                    minvalue = 1,
                                    default_value = 3,
                              )),
                          ],
                          optional_keys = None,
                      ),
                      title = _("Connect to remote sites"),
                      add_label = _("Add connection"),
                )),

            ],
            optional_keys = [ "incoming", ],
        ),
        domain = "mknotifyd",
    )
