#!/usr/bin/python
# -*- encoding: utf-8; py-indent-offset: 4 -*-
# .------------------------------------------------------------------------.
# |                ____ _               _        __  __ _  __              |
# |               / ___| |__   ___  ___| | __   |  \/  | |/ /              |
# |              | |   | '_ \ / _ \/ __| |/ /   | |\/| | ' /               |
# |              | |___| | | |  __/ (__|   <    | |  | | . \               |
# |               \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\              |
# |                                        |_____|                         |
# |             _____       _                       _                      |
# |            | ____|_ __ | |_ ___ _ __ _ __  _ __(_)___  ___             |
# |            |  _| | '_ \| __/ _ \ '__| '_ \| '__| / __|/ _ \            |
# |            | |___| | | | ||  __/ |  | |_) | |  | \__ \  __/            |
# |            |_____|_| |_|\__\___|_|  | .__/|_|  |_|___/\___|            |
# |                                     |_|                                |
# |                     _____    _ _ _   _                                 |
# |                    | ____|__| (_) |_(_) ___  _ __                      |
# |                    |  _| / _` | | __| |/ _ \| '_ \                     |
# |                    | |__| (_| | | |_| | (_) | | | |                    |
# |                    |_____\__,_|_|\__|_|\___/|_| |_|                    |
# |                                                                        |
# | mathias-kettner.com                                 mathias-kettner.de |
# '------------------------------------------------------------------------'
#  This file is part of the Check_MK Enterprise Edition (CEE).
#  Copyright by Mathias Kettner and Mathias Kettner GmbH.  All rights reserved.
#
#  Distributed under the Check_MK Enterprise License.
#
#  You should have  received  a copy of the Check_MK Enterprise License
#  along with Check_MK. If not, email to mk@mathias-kettner.de
#  or write to the postal address provided at www.mathias-kettner.de



#   .--Base Class----------------------------------------------------------.
#   |            ____                    ____ _                            |
#   |           | __ )  __ _ ___  ___   / ___| | __ _ ___ ___              |
#   |           |  _ \ / _` / __|/ _ \ | |   | |/ _` / __/ __|             |
#   |           | |_) | (_| \__ \  __/ | |___| | (_| \__ \__ \             |
#   |           |____/ \__,_|___/\___|  \____|_|\__,_|___/___/             |
#   |                                                                      |
#   +----------------------------------------------------------------------+
#   |  Abstract base class of all package modes                            |
#   '----------------------------------------------------------------------'
class ModeMKPManager(WatoMode):
    def __init__(self):
        WatoMode.__init__(self)
        self._load_package_info()


    def title(self):
        return _("Extension Packages")


    def buttons(self):
        global_buttons()


    def _load_package_info(self):
        package_info = self._get_all_packages_info()
        self._installed_packages = package_info["installed"]
        self._unpackaged_files = package_info["unpackaged"]


    def _get_all_packages_info(self):
        return check_mk_local_automation("get-package-info", [])


    def _button_create_package(self):
        html.context_button(_("Create Package"), html.makeuri_contextless([("mode", "mkps_create")]), "new_mkp")

    @staticmethod

    def _package_parts():
        return {
            'agents'        : _("Agents"),
            'checkman'      : _("Check Manuals"),
            'checks'        : _("Check Plugins"),
            'doc'           : _("Documentation"),
            'inventory'     : _("HW/SW-Inventory"),
            'notifications' : _("Notifications"),
            'pnp-templates' : _("PNP Templates"),
            'web'           : _("GUI Extensions"),
            'bin'           : _("Binaries"),
            'lib'           : _("Libraries"),
        }

    @staticmethod
    def _package_part_title(part):
        return ModeMKPManager._package_parts()[part]


#.
#   .--Installed Packages--------------------------------------------------.
#   |                ___           _        _ _          _                 |
#   |               |_ _|_ __  ___| |_ __ _| | | ___  __| |                |
#   |                | || '_ \/ __| __/ _` | | |/ _ \/ _` |                |
#   |                | || | | \__ \ || (_| | | |  __/ (_| |                |
#   |               |___|_| |_|___/\__\__,_|_|_|\___|\__,_|                |
#   |                                                                      |
#   |              ____            _                                       |
#   |             |  _ \ __ _  ___| | ____ _  __ _  ___  ___               |
#   |             | |_) / _` |/ __| |/ / _` |/ _` |/ _ \/ __|              |
#   |             |  __/ (_| | (__|   < (_| | (_| |  __/\__ \              |
#   |             |_|   \__,_|\___|_|\_\__,_|\__, |\___||___/              |
#   |                                        |___/                         |
#   +----------------------------------------------------------------------+
#   |  Mode for listing all packages. This is the module entry page.       |
#   '----------------------------------------------------------------------'
class ModeMKPManagerPackages(ModeMKPManager):
    def __init__(self):
        ModeMKPManager.__init__(self)


    def title(self):
        return ModeMKPManager.title(self) + " - " + _("Installed Packages")


    def buttons(self):
        ModeMKPManager.buttons(self)
        html.context_button(_("Unpackaged Files"), "wato.py?mode=mkps_unpackaged", "unpackaged_files")
        html.context_button(_("Upload Package"), html.makeuri_contextless([("mode", "mkps_upload")]), "upload")
        self._button_create_package()


    def action(self):
        config.need_permission("wato.manage_mkps")
        if not html.transaction_valid():
            return

        if html.var("_remove"):
            self._action_remove()

        elif html.var("_release"):
            self._action_release()


    def _action_remove(self):
        package_name = html.var("_remove")
        c = wato_confirm(_("Confirm removal of package"),
                        _("Do you really want to remove the package <tt>%s</tt> from your system?")
                        % html.attrencode(package_name))
        if c:
            check_mk_local_automation("remove-package",  [ package_name ])
            log_pending(SYNCRESTART, None, "remove-package", _("Removed extension package %s") % package_name)
            self._load_package_info()
        elif c == False:
            return ""
        else:
            return


    def _action_release(self):
        package_name = html.var("_release")
        c = wato_confirm(_("Confirm dissolving package"),
                        _("This operation will release the package <tt>%s</tt> from your system and "
                          "release all its packaged files into unpackaged state. The files are then "
                          "free to be packaged into another package or to be deleted manually.")
                        % html.attrencode(package_name))
        if c:
            check_mk_local_automation("release-package",  [ package_name ])
            log_pending(SYNCRESTART, None, "release-package", _("Dissolved extension package %s") % package_name)
            self._load_package_info()
        elif c == False:
            return ""
        else:
            return


    def page(self):
        table.begin("packages", title = _("Installed Extension Packages"))
        for package_name, package in self._installed_packages.items():
            table.row()
            self._package_action_buttons(package_name, package)
            table.cell(_("Name"), "<tt>%s</tt>" % html.attrencode(package_name))
            table.cell(_("Version"), html.attrencode(package["version"]), css="number")
            table.cell(_("Alias"), html.attrencode(package["title"]))
            table.cell(_("Author"), html.attrencode(package["author"]))
            table.cell(_("Req. Version"), html.attrencode(package["version.min_required"]), css="number")
            table.cell(_("Contents"))
            html.write(self._package_content_overview(package))
        table.end()


    def _package_action_buttons(self, package_name, package):
        table.cell(_("Actions"), css="buttons")
        download_url = html.makeuri([("package", package_name)], filename = "download_mkp.py")
        html.icon_button(download_url, _("Download this package as an MKP file"), "download")
        remove_url = html.makeactionuri([("_remove", package_name)])
        html.icon_button(remove_url, _("Uninstall and remove this package"), "delete")
        release_url = html.makeactionuri([("_release", package_name)])
        html.icon_button(release_url, _("Dissolve the package, release its files into unpackaged state"), "release_mkp")
        edit_url = html.makeuri_contextless([("mode", "mkps_edit"), ("package", package_name)])
        html.icon_button(edit_url, _("View and edit the details of this package"), "edit")
        if package["download_url"]:
            html.icon_button(package["download_url"], _("Go to the web page where this package came from"), "url")


    def _package_content_overview(self, package):
        parts = []
        for part_name, part_title in sorted(self._package_parts().items()):
            count = len(package["files"].get(part_name, []))
            if count:
                parts.append("%s: %d" % (part_title, count))
        return ", ".join(parts)


#.
#   .--Unpackaged Files----------------------------------------------------.
#   |       _   _                        _                        _        |
#   |      | | | |_ __  _ __   __ _  ___| | ____ _  __ _  ___  __| |       |
#   |      | | | | '_ \| '_ \ / _` |/ __| |/ / _` |/ _` |/ _ \/ _` |       |
#   |      | |_| | | | | |_) | (_| | (__|   < (_| | (_| |  __/ (_| |       |
#   |       \___/|_| |_| .__/ \__,_|\___|_|\_\__,_|\__, |\___|\__,_|       |
#   |                  |_|                         |___/                   |
#   |                         _____ _ _                                    |
#   |                        |  ___(_) | ___  ___                          |
#   |                        | |_  | | |/ _ \/ __|                         |
#   |                        |  _| | | |  __/\__ \                         |
#   |                        |_|   |_|_|\___||___/                         |
#   |                                                                      |
#   +----------------------------------------------------------------------+
#   |  Mode for listing unpackaged files.                                  |
#   '----------------------------------------------------------------------'
class ModeMKPManagerUnpackagedFiles(ModeMKPManager):
    def __init__(self):
        ModeMKPManager.__init__(self)


    def buttons(self):
        ModeMKPManager.buttons(self)
        html.context_button(_("Installed Packages"), "wato.py?mode=mkps", "mkps")
        self._button_create_package()


    def title(self):
        return ModeMKPManager.title(self) + " - " + _("Unpackaged Files")


    def action(self):
        config.need_permission("wato.manage_mkps")
        if html.var("_delete"):
            part = html.var("part")
            file_name = html.var("file")
            c = wato_confirm(_("Confirm deletion of unpackaged file"),
                            _("Do you really want to delete the unpackaged file <b>%s: <tt>%s</tt></b> from your system?")
                            % (self._package_part_title(part), html.attrencode(file_name)))
        if c:
            check_mk_local_automation("remove-unpackaged-file",  [ part, file_name ])
            log_pending(SYNCRESTART, None, "remove-unpackaged-file", _("Removed unpackaged extension file %s: %s") %
                (self._package_part_title(part), file_name))
            self._load_package_info()
        elif c == False:
            return ""
        else:
            return


    def page(self):
        if not self._have_unpackaged_files():
            html.message(_("No unpackaged files found. If you want to create new extension packages "
                           "then you need to create files below the directory <tt>%s/local</tt> of your "
                           "Check_MK instance.") % defaults.omd_root)
            return

        for part_name, filelist in self._unpackaged_files.items():
            if filelist:
                self._show_unpackaged_files(part_name, filelist)


    def _have_unpackaged_files(self):
        for part_name, filelist in self._unpackaged_files.items():
            if filelist:
                return True
        return False


    def _show_unpackaged_files(self, part_name, filelist):
        part_title = self._package_parts().get(part_name, part_name)
        table.begin("files", title=part_title)
        for file_name in filelist:
            table.row()
            table.cell(_("Actions"), css="buttons")
            delete_url = html.makeactionuri([("_delete", "1"), ("part", part_name), ("file", file_name)])
            html.icon_button(delete_url, _("Delete this unpackaged file"), "delete")
            table.cell(_("Files"), file_name)
        table.end()


#.
#   .--Create Package------------------------------------------------------.
#   |                     ____                _                            |
#   |                    / ___|_ __ ___  __ _| |_ ___                      |
#   |                   | |   | '__/ _ \/ _` | __/ _ \                     |
#   |                   | |___| | |  __/ (_| | ||  __/                     |
#   |                    \____|_|  \___|\__,_|\__\___|                     |
#   |                                                                      |
#   |                ____            _                                     |
#   |               |  _ \ __ _  ___| | ____ _  __ _  ___                  |
#   |               | |_) / _` |/ __| |/ / _` |/ _` |/ _ \                 |
#   |               |  __/ (_| | (__|   < (_| | (_| |  __/                 |
#   |               |_|   \__,_|\___|_|\_\__,_|\__, |\___|                 |
#   |                                          |___/                       |
#   +----------------------------------------------------------------------+
#   |  Mode for creating new packages from unpackaged files                |
#   '----------------------------------------------------------------------'

class ModeMKPManagerCreatePackage(ModeMKPManager):
    def __init__(self):
        ModeMKPManager.__init__(self)
        self._package = { "files" : {} }


    def title(self):
        return ModeMKPManager.title(self) + " - " + _("Create new package")


    def buttons(self):
        ModeMKPManager.buttons(self)
        html.context_button(_("Back"), "wato.py?mode=mkps", "back")


    def action(self):
        config.need_permission("wato.manage_mkps")
        package_info = self._get_package_info()

        try:
            check_mk_local_automation("create-package", [ package_info["name"] ], package_info)
            message = _("Created new extension package %s") % package_info["name"]
            log_pending(SYNCRESTART, None, "create-package", message)
            return "mkps", message
        except Exception, e:
            if config.debug:
                raise
            raise MKUserError(None, _("Could not create package: %s") % e)

        return "mkps"


    def page(self):
        if self._check_unpackaged_files():
            html.begin_form("package", method="POST")
            self._vs_package().render_input("", self._package)
            self._vs_package().set_focus("")
            html.button("_save", _("Save changes"))
            html.hidden_fields()
            html.end_form()


    def _check_unpackaged_files(self):
        for part, rel_paths in self._unpackaged_files.items():
            if rel_paths:
                return True
        html.message(_("You can only create a new package if there are unpackaged files."))
        return False


    def _get_package_info(self):
        package_info = self._vs_package().from_html_vars("")
        self._vs_package().validate_value(package_info, "")
        package_info["version.packaged"] = defaults.check_mk_version
        return package_info


    def _vs_package(self):
        return Dictionary(
            title = _("Package properties"),
            render = "form",
            elements = [
                ( "name",
                  ID(title = _("Package Name"),
                     allow_empty = False,
                )),
                ( "version",
                  TextAscii(
                      title = _("Package Version"),
                      regex = "[0-9.]+",
                      regex_error = _("Only digits and dots are allowed in the version number."),
                      size = 10,
                      allow_empty = False,
                )),
                ( "version.min_required",
                  CheckMKVersion(
                      title = _("Minimum Required Check_MK Version"),
                      default_value = "1.2.8",
                )),
                ( "title",
                  TextUnicode(
                      title = _("Title"),
                      allow_empty = False,
                      size = 80,
                )),
                ( "author",
                  TextUnicode(
                      title = _("Author"),
                      size = 80,
                      default_value = config.user_alias,
                )),
                ( "download_url",
                  HTTPUrl(
                      title = _("Homepage / Download - URL of this package"),
                      size = 80,
                )),
                ( "description",
                  TextAreaUnicode(
                      title = _("Description"),
                      cols = 80,
                      rows = 10,
                )),
                ( "files",
                   self._vs_packaged_files()
                ),
            ],
            optional_keys = False,
        )



    # Convert from package's way to list files into DualListChoice-compatible
    # and back.
    def _vs_packaged_files(self):
        def transform_forth(parts):
            return [ c[0] for c in self._dict_to_files_choices(parts) ]

        def transform_back(selection):
            parts = {}
            for entry in selection:
                part, rel_path = entry.split(":", 1)
                parts.setdefault(part, []).append(rel_path)
            return parts


        return Transform(
            DualListChoice(
                title = _("Packaged files"),
                choices = self._files_choices(),
                size = 80,
                rows = 10,
                allow_empty = False,
            ),
            forth = transform_forth,
            back = transform_back,
        )


    def _files_choices(self):
        return self._packaged_files_choices() + \
               self._unpackaged_files_choices()


    def _packaged_files_choices(self):
        return self._dict_to_files_choices(self._package["files"])


    def _unpackaged_files_choices(self):
        return self._dict_to_files_choices(self._unpackaged_files)


    def _dict_to_files_choices(self, parts):
        choices = []
        for part, rel_paths in parts.items():
            for rel_path in rel_paths:
                choices.append(("%s:%s" % (part, rel_path), "%s: %s" % (self._package_part_title(part), rel_path)))
        return choices



#.
#   .--Edit Package--------------------------------------------------------.
#   |       _____    _ _ _     ____            _                           |
#   |      | ____|__| (_) |_  |  _ \ __ _  ___| | ____ _  __ _  ___        |
#   |      |  _| / _` | | __| | |_) / _` |/ __| |/ / _` |/ _` |/ _ \       |
#   |      | |__| (_| | | |_  |  __/ (_| | (__|   < (_| | (_| |  __/       |
#   |      |_____\__,_|_|\__| |_|   \__,_|\___|_|\_\__,_|\__, |\___|       |
#   |                                                    |___/             |
#   +----------------------------------------------------------------------+
#   |  Mode for modifying existing installed packages.                     |
#   '----------------------------------------------------------------------'

class ModeMKPManagerEditPackage(ModeMKPManagerCreatePackage):
    def __init__(self):
        ModeMKPManagerCreatePackage.__init__(self)
        self._package_name = html.var("package")
        if self._package_name not in self._installed_packages:
            raise MKUserError("name", _("This package does not exist."))
        self._package = self._installed_packages[self._package_name]


    def title(self):
        return ModeMKPManager.title(self) + " - " + _("Package %s") % self._package_name


    def action(self):
        config.need_permission("wato.manage_mkps")
        package_info = self._get_package_info()
        try:
            check_mk_local_automation("edit-package", [ self._package_name ], package_info)
            message = _("Changed properties of installed extension package %s") % self._package_name
            log_pending(SYNCRESTART, None, "edit-package", message)
            return "mkps", message
        except Exception, e:
            if config.debug:
                raise
            raise MKUserError(None, _("Could not save changes: %s") % e)

        return "mkps"


    def _check_unpackaged_files(self):
        return True


#.
#   .--Upload Package------------------------------------------------------.
#   |                   _   _       _                 _                    |
#   |                  | | | |_ __ | | ___   __ _  __| |                   |
#   |                  | | | | '_ \| |/ _ \ / _` |/ _` |                   |
#   |                  | |_| | |_) | | (_) | (_| | (_| |                   |
#   |                   \___/| .__/|_|\___/ \__,_|\__,_|                   |
#   |                        |_|                                           |
#   |                ____            _                                     |
#   |               |  _ \ __ _  ___| | ____ _  __ _  ___                  |
#   |               | |_) / _` |/ __| |/ / _` |/ _` |/ _ \                 |
#   |               |  __/ (_| | (__|   < (_| | (_| |  __/                 |
#   |               |_|   \__,_|\___|_|\_\__,_|\__, |\___|                 |
#   |                                          |___/                       |
#   +----------------------------------------------------------------------+
#   |  Mode for uploading and installing MKP files                         |
#   '----------------------------------------------------------------------'

class ModeMKPManagerUploadPackage(ModeMKPManager):
    def __init__(self):
        ModeMKPManager.__init__(self)


    def title(self):
        return ModeMKPManager.title(self) + " - " + _("Upload Extension Package")


    def buttons(self):
        ModeMKPManager.buttons(self)
        html.context_button(_("Back"), "wato.py?mode=mkps", "back")


    def action(self):
        config.need_permission("wato.manage_mkps")
        if html.var("_do_upload"):
            if html.check_transaction():
                file_info = self._vs_upload().from_html_vars("_upload")
                self._vs_upload().validate_value(file_info, "_upload")
                file_name, mime_type, content = file_info
                self._upload_mkp(content)
                return "mkps"
            else:
                return


    def page(self):
        html.begin_form("package", method="POST")
        forms.header(_("Upload extension package"))
        forms.section(_("Select file"))
        self._vs_upload().render_input("_upload", None)
        forms.end()
        html.hidden_fields()
        html.button("_do_upload", _("Upload and install package"))
        html.end_form()


    def _vs_upload(self):
        return FileUpload(
            allow_empty = False,
            allow_empty_content = False,
            allowed_extensions = [ ".mkp" ],
        )


    def _upload_mkp(self, content):
        try:
            package_info = check_mk_local_automation('install-package', [], stdin_data = content)
        except Exception, e:
            if config.debug:
                raise
            raise MKUserError("upload", _("This package cannot be installed: %s") % e)
        message = _("Installed extension package %s-%s (%s)") % \
              (package_info["name"], package_info["version"], package_info["title"])
        log_pending(SYNCRESTART, None, "install-package", message)
        return "mkps", message


#.
#   .--Download Package----------------------------------------------------.
#   |            ____                      _                 _             |
#   |           |  _ \  _____      ___ __ | | ___   __ _  __| |            |
#   |           | | | |/ _ \ \ /\ / / '_ \| |/ _ \ / _` |/ _` |            |
#   |           | |_| | (_) \ V  V /| | | | | (_) | (_| | (_| |            |
#   |           |____/ \___/ \_/\_/ |_| |_|_|\___/ \__,_|\__,_|            |
#   |                                                                      |
#   |                ____            _                                     |
#   |               |  _ \ __ _  ___| | ____ _  __ _  ___                  |
#   |               | |_) / _` |/ __| |/ / _` |/ _` |/ _ \                 |
#   |               |  __/ (_| | (__|   < (_| | (_| |  __/                 |
#   |               |_|   \__,_|\___|_|\_\__,_|\__, |\___|                 |
#   |                                          |___/                       |
#   +----------------------------------------------------------------------+
#   |  Downloading MKPs is no WATO module but a separate page handler.     |
#   '----------------------------------------------------------------------'

def page_download_mkp():
    config.need_permission("wato.mkps")
    package_name = html.var("package")
    if "/" in package_name:
        raise MKUserError("package", _("Invalid package name"))
    package_info, package_mkp = check_mk_local_automation('get-package', [ package_name ])

    html.req.content_type = "application/x-mkp"
    html.req.headers_out['Content-Disposition'] = "attachment; filename=\"%s-%s.mkp\"" % (package_info["name"], package_info["version"])
    html.write(package_mkp)



#.
#   .--Declarations--------------------------------------------------------.
#   |       ____            _                 _   _                        |
#   |      |  _ \  ___  ___| | __ _ _ __ __ _| |_(_) ___  _ __  ___        |
#   |      | | | |/ _ \/ __| |/ _` | '__/ _` | __| |/ _ \| '_ \/ __|       |
#   |      | |_| |  __/ (__| | (_| | | | (_| | |_| | (_) | | | \__ \       |
#   |      |____/ \___|\___|_|\__,_|_|  \__,_|\__|_|\___/|_| |_|___/       |
#   |                                                                      |
#   +----------------------------------------------------------------------+
#   |  Integrate all that stuff into WATO                                  |
#   '----------------------------------------------------------------------'

modules.append(
    ( "mkps", _("Extension Packages"), "mkps", "mkps",
   _("Manage and create Check_MK extension packages (MKPs)."))
)


config.declare_permission("wato.mkps",
    _("Visit Extension Package Manager"),
    _("This permission gives read access to the Check_MK MKP manager. It allows to view "
      "and download installed packages. MKPs contain extensions to Check_MK such as new "
      "check plugins or GUI extensions."),
    [ "admin" ])


config.declare_permission("wato.manage_mkps",
    _("Manage Extension Packages (MKPs)"),
    _("Allows to upload, install, deinstall, create and modify extension packages "
      "(MKPs). Note: MKPs contain executable code."),
    [ "admin" ])


modes.update({
    "mkps"               : (["mkps"], ModeMKPManagerPackages),
    "mkps_unpackaged"    : (["mkps"], ModeMKPManagerUnpackagedFiles),
    "mkps_edit"          : (["mkps"], ModeMKPManagerEditPackage),
    "mkps_create"        : (["mkps"], ModeMKPManagerCreatePackage),
    "mkps_upload"        : (["mkps"], ModeMKPManagerUploadPackage),
})

